/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp.h,v 1.30 2003/03/03 02:16:32 tgucio Exp $
 */


#ifndef _FTP_H
#define _FTP_H

#include <sys/types.h>
/* local headers */
#include <defaults.h>
#include <ftp_object.h>

#if !(__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4))
#define __attribute__
#endif
	

typedef struct ftp_event ftp_event;
typedef struct ftp_request ftp_request;
typedef struct ftp_transfer ftp_transfer;
typedef struct _ftp_item _ftp_item;

#define FTP_CALLBACK(x) void (*x)()
#define FTP_EVENT_CALLBACK(x)	int (*x)()
#define FTP_CMD_WRAPPER(x, args...)                                 \
                   ftp_request *x(_ftp_item *ftp,                   \
		           int rpos, ftp_request *mreq,                     \
			       int condition          __attribute__((unused)) , \
				   FTP_CALLBACK(callback) __attribute__((unused)) , \
                   ##args)
#define FTP_CMD_WITH_REQ(x, args...) ftp_request *x(_ftp_item *ftp, ftp_request *mreq , ##args)
#define FTP_CMD_WITH_CB(x, args...) ftp_request *x(_ftp_item *ftp, FTP_CALLBACK(callback) , ##args)
#define FTP_CMD_PLAIN(x, args...) ftp_request *x(_ftp_item *ftp , ##args)

/* callback return values */
#define FTP_EVENT_NONBLOCK	0x1	/* only used for event handlers.*/
#define FTP_EVENT_SUCCESS	0x2
#define FTP_EVENT_FAILURE	0x3


/* some return values for ftp_readline function. */
#define FTP_CONN_CLOSED		0x1
#define FTP_CONN_ERR		0x2
#define FTP_READ_FAILURE	0x3
#define FTP_READ_CACHED		0x4
#define FTP_READ_SUCCESS	0x5

/* io definitions */
#define IO_WATCH_READ	0x1
#define IO_WATCH_WRITE	0x2
#define IO_WATCH_HUP	0x4


#define	_LIST_FILE	0x1
#define _LIST_DIR	0x2

#define J_ISDIR(x)	((x) == (_LIST_DIR))
#define J_ISFILE(x)	((x) == (_LIST_FILE))
#define J_ISLINK(x)	((x) == ((_LIST_DIR) | (_LIST_FILE)))

#define	MODE_PASV	1
#define	MODE_ACTV	2
#define	MODE_BINR	4
#define	MODE_ASCI	8

/* list methods */
#define	FTP_LIST_DEFAULT	0x0
#define FTP_LIST_PLAIN		0x1
#define FTP_LIST_CUSTOM		0x2

#define  TRFR_DEFAULT 0
#define  TRFR_OVERWRT 2
#define  TRFR_RESUME  4
#define  TRFR_NOACT   8
#define  TRFR_ASK     16

struct _list {
	int type;
	struct _list *next;
	struct _list *child;
	struct _list *parent;
	char *file;
	char *perm;
	char *owner;
	char *group;
	long size;
	time_t mtime;
	char *mtime_string;	// in case mtime is 0 (not possible to parse)
};


struct buffering {
	char *buffer;
	size_t buffer_cached;
};


/*
#define _TRFR_NONE		0
#define _TRFR_UL		1
#define _TRFR_DL		2
struct transfer {
	int mode;
	long transferred_bytes;
	long restored_bytes;
	long size;
};
*/


struct ftp_dir_info {
	char *pwd;
	struct _list *list_dir;
};

/*
 * void ui_update_transfer(unsigned long bytes, unsigned long size);
 */
#define CONN_FTP	1
#define CONN_DATA	2

#define FTP_STATE_NONE		0x0
#define FTP_STATE_CONNECTED	0x1
#define FTP_STATE_GREETED	0x2
#define FTP_STATE_LOGGED_IN	0x3
struct _ftp_item {
	char *addr;
	char *user;
	char *pass;
	int fd;
	int port;
	int type;
	char state;
	double transfer_limit;		/* transfer limit in bytes/second */
	enum { active, passive } mode;
	struct buffering bf;
	ftp_transfer *transfer;
	struct _ftp_item *dataconn;
	struct ftp_dir_info *dirinfo;
	int n_requests;			/* number of reqs yet to be processed (sent). */
	time_t last;			/* time when last ftp request was sent (for anti idler?).  */
	ftp_request *requests;
	ftp_event *event_handler;
	FTP_OBJECT_DECLARE;
};

typedef struct {
	char *ip;
	int port;
} t_ftpinfo;



/* function prototypes. */
_ftp_item *ftp_new();
void ftp_free(_ftp_item **);
int ftp_set_server(_ftp_item *, const char *, int);
int ftp_set_user(_ftp_item *, const char *, const char *pass);
int ftp_set_type(_ftp_item *, int);
int ftp_set_mode(_ftp_item *, int);

#endif
