/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp_request.h,v 1.13 2003/02/24 15:51:15 thepin Exp $
 */

#ifndef _FTP_REQUEST_H
#define _FTP_REQUEST_H
#include <ftp.h>
#include <ftp_object.h>


#define RP_END		0x0	/* end of tree, default */
#define RP_NEXT		0x1	/* into the next entry of main req */
#define RP_TREE_NEXT	0x2
/* Request Types. */
#define REQ_SEND	0x0	/* default, send a message */
#define REQ_CMD		0x1	/* run a command, not send a message */

/* States for requests */
#define R_QUEUED	(1)		/* command is waiting for being sent. */
#define R_SENT		(1 << 1)	/* command has been sent, waiting */
#define R_SUCCESS	(1 << 2)	/* done, succesfull. */
#define R_FAILURE	(1 << 3)	/* done, failed. */
#define R_READY		(1 << 4)	/* ready for transfer (expect another reply) */
#define R_ALWAYS	(R_SUCCESS | R_FAILURE | R_READY)
#define R_FINISH	(R_SUCCESS | R_FAILURE)

struct ftp_request {
	char *cmd;
	char *args;
	
	/*
	 * type: REQ_SEND (default), REQ_CMD
	 */
	int type;
	
	/*
	 * state, R_QUEUED, R_SENT, R_SUCCESS, R_ERROR, R_FAILURE
	 */
	int state;
	
	int condition;	/* state of the previous entry in tree, if same, the callback here is run  */
	FTP_CALLBACK(callback);	/* function to call on success. */
	FTP_OBJECT_DECLARE;
	struct ftp_request *tree_prior, *tree_next;
	struct ftp_request *prior, *next;
};

/* function prototypes. */
ftp_request *ftp_request_find_entry(ftp_request *, int, int);
void ftp_request_tree_free_backwards(ftp_request *);
void ftp_request_free(ftp_request *);
void ftp_request_tree_free(ftp_request **);
int ftp_request_iteration(_ftp_item *);
ftp_request *ftp_request_add(_ftp_item *, int rpos, ftp_request *, int, FTP_CALLBACK(x), const char *, ...);
int ftp_request_move(_ftp_item *, ftp_request *, ftp_request *);
int ftp_request_set_type(ftp_request *, int); 
ftp_request *ftp_request_tree_debug(ftp_request *, int);


#endif
