/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp_transfer.h,v 1.4 2003/02/24 15:51:16 thepin Exp $
 */


#ifndef _FTP_TRANSFER_H
#define _FTP_TRANSFER_H

#include <ftp.h>
#include <ftp_object.h>
#include <jtimer.h>

/* definitions */
#define FTP_TRANSFER_NONE	0x0
#define	FTP_TRANSFER_UPLOAD	0x1
#define FTP_TRANSFER_DOWNLOAD	0x2

/* macro */
#define FTP_TRANSFER(x)		((x) ? ((x->transfer)) : (NULL))


/* main structure */ 
struct ftp_transfer {
	FTP_OBJECT_DECLARE;
	int state;			/* FTP_TRANSFER_x							*/
	unsigned int s_transferred;	/* total transferred bytes of file					*/
	unsigned int s_resume_pos;	/* the position at which we started					*/
	unsigned int s_size;		/* total size of the file being transmitted				*/
	jtimer *timer;			/* timer that keeps track from the beginning of the transmission.	*/
	double v_max;			/* the speed limit  / if <= 0, then config value is used.		*/
};

/* function prototypes. */
ftp_transfer *ftp_transfer_new();
ftp_transfer *ftp_transfer_free(ftp_transfer *);
void ftp_transfer_reset(ftp_transfer *);
void ftp_transfer_set_state(ftp_transfer *, int);
int ftp_transfer_get_state(ftp_transfer *);
void ftp_transfer_set_resume_position(ftp_transfer *, size_t);
size_t ftp_transfer_get_resume_position(ftp_transfer *);
void ftp_transfer_set_position(ftp_transfer *, size_t);
size_t ftp_transfer_get_position(ftp_transfer *);
void ftp_transfer_set_size(ftp_transfer *, size_t);
size_t ftp_transfer_get_size(ftp_transfer *);
void ftp_transfer_setup(ftp_transfer *, int);
double ftp_transfer_get_sleep_time(ftp_transfer *);
void ftp_transfer_end(ftp_transfer *);
double ftp_transfer_get_average_speed(ftp_transfer *);
double ftp_transfer_get_current_speed(ftp_transfer *);
void ftp_transfer_update(ftp_transfer *, size_t);

#endif
