/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_theme.h,v 1.5 2003/02/24 15:51:38 thepin Exp $
 */

#ifndef _GUI_THEME_H
#define _GUI_THEME_H

#include <gtk/gtk.h>
/* local headers */
#include <defaults.h>

/* from tar.txt (wotsit.org) */
#define  RECORDSIZE  512
#define  NAMSIZ      100
#define  TUNMLEN      32
#define  TGNMLEN      32

union record {
	char charptr[RECORDSIZE];
	struct header {
		char name[NAMSIZ];
		char mode[8];
		char uid[8];
		char gid[8];
		char size[12];
		char mtime[12];
		char chksum[8];
		char linkflag;
		char linkname[NAMSIZ];
		char magic[8];
		char uname[TUNMLEN];
		char gname[TGNMLEN];
		char devmajor[8];
		char devminor[8];
	} header;
};

/* The checksum field is filled with this while the checksum is computed. */
#define CHKBLANKS    "        "        /* 8 blanks, no null */

/* The magic field is filled with this if uname and gname are valid. */
#define TMAGIC    "ustar  "        /* 7 chars and a null */

/* The magic field is filled with this if this is a GNU format dump entry */
#define GNUMAGIC  "GNUtar "        /* 7 chars and a null */

/* The linkflag defines the type of file */
#define  LF_OLDNORMAL '\0'       /* Normal disk file, Unix compatible */
#define  LF_NORMAL    '0'        /* Normal disk file */
#define  LF_LINK      '1'        /* Link to previously dumped file */
#define  LF_SYMLINK   '2'        /* Symbolic link */
#define  LF_CHR       '3'        /* Character special file */
#define  LF_BLK       '4'        /* Block special file */
#define  LF_DIR       '5'        /* Directory */
#define  LF_FIFO      '6'        /* FIFO special file */
#define  LF_CONTIG    '7'        /* Contiguous file */

/* Further link types may be defined later. */

/* Bits used in the mode field - values in octal */
#define TSUID    04000        /* Set UID on execution */
#define TSGID    02000        /* Set GID on execution */
#define TSVTX    01000        /* Save text (sticky bit) */

/* File permissions */
#define TUREAD   00400        /* read by owner */
#define TUWRITE  00200        /* write by owner */
#define TUEXEC   00100        /* execute/search by owner */
#define TGREAD   00040        /* read by group */
#define TGWRITE  00020        /* write by group */
#define TGEXEC   00010        /* execute/search by group */
#define TOREAD   00004        /* read by other */
#define TOWRITE  00002        /* write by other */
#define TOEXEC   00001        /* execute/search by other */

/* junkie structures  */

typedef struct gui_theme_header gui_theme_header;
typedef struct gui_theme_stat gui_theme_stat;
typedef struct gui_theme_entry gui_theme_entry;

struct gui_theme_stat {
	char *file;
	size_t size;
	int type;		/* LF_NORMAL, LF_DIR etc. */
};

struct gui_theme_entry {
	union record rec;
	gui_theme_stat info;
	long pos;		/* position of file in the archive. */
	char *data;		/* preload the data? */
	struct gui_theme_entry *next;
};

struct gui_theme_header {
	gui_theme_entry *entry;
	char *theme;
};


// #define T_PRELOAD	0x1	/* Preload the file */
// #define T_LOAD		0x2	/* Load the file every time it needs to be accessed. */

typedef struct gui_theme_files gui_theme_files;
struct gui_theme_files {
	char *file;
	enum { T_PRELOAD, T_LOAD, T_RCFILE, T_CONFIG } type;
};

/* prototypes. */

gui_theme_header * gui_theme_get_current(void);
void gui_theme_set_current(gui_theme_header *);
int gui_theme_load(const char *);
GdkPixbuf *gui_theme_pixbuf_get(const char *);
GtkWidget *gui_theme_image_get(const char *);
void gui_theme_close(void);


#endif
