/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: queue.h,v 1.24 2003/03/03 02:16:32 tgucio Exp $
 */

#ifndef _QUEUE_H
#define _QUEUE_H

#include <stdio.h>
#include <jio.h>
#include <jlist.h>
#include <ftp.h>
#include <ftp_request.h>
#include <ftp_event_callback.h>

#define Q_NONE 0
#define Q_QUEUED    1
#define Q_TRANSFERRING  2
#define Q_TRANSFERED 3

#define Q_RECURSE_NONE 0
#define Q_RECURSE_ALL  1


struct _queue_list {
	int type;                 /* _LIST_DIR | _LIST_FILE | link */
	struct _queue_list *next;
	struct _queue_list *child;
	struct _queue_list *parent;
	char *remote_path;
	char *local_path;
	char *site;
	char *user;
	char *pass;
	int port;
	int size;
	int t_mode; /* TRFR_UL | TRFR_DL                                 */
	int status; /* Q_NONE | Q_QUEUED | Q_TRANSFERRING | Q_TRANSFERED */
};
typedef struct _queue_list _queue_item;

_queue_item * queue_init(char *);
char *        queue_parse_path(char **);
_queue_item * queue_parse_item(char *);
void          queue_save(_queue_item *, char *);
void          queue_save_printf(_queue_item **, t_iter *, int, jio *);

void          queue_free      (_queue_item *);
void          queue_free_entry(_queue_item *, int);
_queue_item * queue_item_new(_ftp_item *, struct _list *, 
		                     char *, char *, 
							 char *, 
							 int);
_queue_item * queue_dir_local_new (_ftp_item *, char *, char *, char *, int);
_queue_item * queue_dir_remote_new(_ftp_item *, char *, char *, char *, int);
char * queue_full_path(char *, char *, char *);

_queue_item * queue_recurse_local (_ftp_item *, _queue_item *, int, 
		                           char *, char *, char *, char *);
void          queue_recurse_remote(_ftp_item *, ftp_request *, 
		                           _queue_item **, _queue_item *, char *);
FTP_CB_CMD(queue_recurse_list_cb);

void   queue_submit(_ftp_item *, jslist *);
FTP_CB_CMD(queue_submit_cb);

jslist * queue_list_append(jslist *, _queue_item *, int);

void     queue_debug_list(_queue_item *);


#endif

