/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sound.h,v 1.14 2003/02/24 15:51:46 thepin Exp $
 */

#include <misc.h>

#ifndef _SOUND_H
#define _SOUND_H

/* sound activion */
#define SOUND_OFF	0
#define SOUND_ESD	1
#define SOUND_SPEAKER	2

/* snd defaults */
#define SOUND_SPEAKER_DEV	"/dev/speaker"

#define SOUND_DEBUG(x)	printf("SOUND DEBUG: %s\n", x)
#define SND_ERR(x)      printf("ERR SND: %s\n", x)
#define SND_WARN(x)	printf("WARN SND: %s\n", x)


enum {
	SOUND_STARTUP,			/* sound when starting up junkie. */
	SOUND_QUIT,			/* sound when quitting junkie */
	SOUND_LOGIN,			/* sound when logging in */
	SOUND_TRANSFER_COMPLETE,		/* sound when a transfer has been finished. */
	SOUND_SEGFAULT
};

/* this is for listing sounds in sound.c */
struct sound_list_default {
	int type;		/* one of the types above. */
	char *file;
	enum {S_PRELOAD, S_LOAD} load;
};


/* here is the true list to be used in the prog. */
struct sound_list {
	struct sound_list *next;
	char *path;
	int type;
	char *handler;
	int esd_id;
	int load;
};


void sound_list_add(char *, char *, int, int, int);
int sound_list_load();
struct sound_list *sound_list_find_by_type(int);
int sound_init(void);
int sound_shutdown(void);
void sound_stop();
void sound_play_event(int);
void sound_play_path(const char *, int, cb);
void sound_stop();
#endif
