/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: anti_idler.c,v 1.5 2003/02/24 04:10:37 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <time.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>
/* local headers */
#include <ftp.h>
#include <ftp_cmd.h>
#include <ftp_event.h>
#include <ftp_misc.h>
#include <ftp_net.h>
#include <ftp_request.h>
#include <ftp_transfer.h>
#include <config.h>
#include <cache.h>
#include <error.h>
#include <log.h>
#include <anti_idler.h>


/*
** This function should be called with an interval as specified by the anti-idler settings, if the anti
** idler support is on.
*/
void
anti_idle_iterator(_ftp_item *ftp)
{
	int anti_idle;
	
	if (!ftp)
		return;

	/* Get the anti idle time from the config lib. */
	anti_idle = config_read_num("anti-idler");

	/* If there is a transfer, or the anti idle time has not been reached, then do nothing */
	if (ftp->last <= 0 || ftp_transfer_get_state(FTP_TRANSFER(ftp)) != FTP_TRANSFER_NONE
	    || anti_idle <= 0 || ((ftp->last + anti_idle) < time(NULL)))
		return;

	/* Send the noop request. */
	ftp_noop(ftp);

	return;
}


