/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: config.c,v 1.43 2003/03/07 17:48:11 thepin Exp $
 */

#define _GNU_SOURCE

#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <err.h>

/* local includes */
#include <config.h>
#include <jio.h>
#include <misc.h>
#include <mem.h>
#include <debug.h>

#define CHECK_INIT()						\
if (config_inited != 1) {					\
	DEBUG_MSG(DBG_ERROR, "WARNING: config_inited != 1");			\
	return (-1);						\
}

static int config_inited = 0;
static int config_fd;
static struct _conf_list *config_head;

struct _conf_item _conf_items[] = {
	{ "defuser", T_STR, "" },
	{ "anonpass", T_STR, "junkie@junkie.org" },
	{ "anonconn", T_BOOL, "yes" },
	{ "usepassive", T_BOOL, "yes" },
	{ "usebinary", T_BOOL, "yes" },
	{ "defport", T_NUM, "21" },
	{ "cache-enable", T_BOOL, "no" },
	{ "cache-session", T_BOOL, "yes" },
	{ "cache-days", T_NUM, "2" },
	{ "imgeditor", T_STR, "gimp %f" },
	{ "txteditor", T_STR, "xterm -e vim %f" },
	{ "theme", T_STR, "ya" },
	{ "imgviewer", T_STR, "gimp %f" },
	{ "txtviewer", T_STR, "xterm -e vim %f" },
	{ "transfer-limit", T_NUM, "0" },
	{ "file-exist", T_NUM, "4" },
	{ "list_method", T_NUM, "0" },
	{ "list_method_custom", T_STR, "-a" },
	{ "anti-idler", T_NUM, "300" },
	{ "hammering", T_NUM, "60" },
	{ "sound", T_BOOL, "yes" },
	{ "sound_startup_enable", T_BOOL, "yes" },
	{ "sound_startup_path", T_STR, DATADIR "/junkie/sound/junkie.xm" },
	{ "show_player", T_BOOL, "yes" },
	{ "encrypt", T_BOOL, "no" },
	{ "id", T_STR, "" },
	{ "queue-keep", T_BOOL, "yes" },
	{ "logwin_colour", T_STR, "" },
	{ "logwindow_colour_error", T_STR, "" },
	{ "logwindow_colour_ftp_misc", T_STR, "" },
	{ "logwindow_colour_ftp_sent", T_STR, "" },
	{ "logwindow_colour_ftp_recv", T_STR, "" },
	{ "logwindow_colour_misc", T_STR, ""},
	{ "theme_logwin_colour", T_STR, "" },
	{ "theme_logwindow_colour_error", T_STR, "" },
	{ "theme_logwindow_colour_ftp_misc", T_STR, "" },
	{ "theme_logwindow_colour_ftp_sent", T_STR, "" },
	{ "theme_logwindow_colour_ftp_recv", T_STR, "" },
	{ "theme_logwindow_colour_misc", T_STR, ""},
	{ NULL, 0, NULL },
};

int
config_getvars(char *file)
{
	struct _conf_list *cl;
	int found;
	char *confline, *p, *q; 

	config_fd = open(file, O_RDWR);
	if (config_fd == -1)
		return (-1);

	while ((confline = readline(config_fd)) != NULL) {
		p = strchr(confline, '=');
		if (p == NULL) {
			DEBUG_MSG(DBG_WEIRD, "unknown config line - " \
				"skipping: %s", confline);
			continue;
		}
		*p = '\0';
		/* sanity check */
		found = 0;
		cl = config_head;
		while (cl->cur != NULL) {
			if (!strcmp(cl->cur->name, confline)) {
				found = 1;
				break;
			}
			cl = cl->next;
		}
		if (!found) {
			DEBUG_MSG(DBG_WEIRD, "unknown config option - " \
				"skipping: %s", confline);
			continue;
		}
		p++;
		if (*p == '\0') {
			DEBUG_MSG(DBG_WEIRD, "no setting for config option? " \
				"skipping: %s", confline);
			continue;
		}
		while (*p == ' ' || *p == '\t')
			p++;
		if (*p == '\"' || *p == '\'') {
			p++;
			q = strrchr(p, *(p - 1));
			if (q != NULL)
				*q = '\0';
		}
		config_set_value(confline, cl->cur->type, p);
		j_free(confline);
	}
	close(config_fd);

	return (0);
}

int
config_getvars_str(char *str)
{
	struct _conf_list *cl;
	int found;
	char *confline, *p, *q, *pnt, *epnt; 

	if (str == NULL) {
		DEBUG_MSG(DBG_ERROR, "str == NULL");
		return (-1);
	}

	pnt = str - 1;
	do {
		pnt++;
		if (!*pnt)
			break;

		epnt = pnt;
		while (*epnt != '\n' && *epnt != '\0')
			epnt++;
		confline = j_malloc((epnt - pnt) + 1);
		memcpy(confline, pnt, epnt - pnt);
		confline[epnt - pnt] = '\0';
		pnt = epnt;
		p = strchr(confline, '=');
		if (p == NULL) {
			DEBUG_MSG(DBG_WEIRD, "unknown config line - " \
				"skipping: %s", confline);
			continue;
		}
		*p = '\0';
		/* sanity check */
		found = 0;
		cl = config_head;
		while (cl->cur != NULL) {
			if (!strcmp(cl->cur->name, confline)) {
				found = 1;
				break;
			}
			cl = cl->next;
		}
		if (!found) {
			DEBUG_MSG(DBG_WEIRD, "unknown config option - " \
				"skipping: %s", confline);
			continue;
		}
		p++;
		if (*p == '\0') {
			DEBUG_MSG(DBG_WEIRD, "no setting for config option? " \
				"skipping: %s", confline);
			continue;
		}
		while (*p == ' ' || *p == '\t')
			p++;
		if (*p == '\"' || *p == '\'') {
			p++;
			q = strrchr(p, *(p - 1));
			if (q != NULL)
				*q = '\0';
		}
		config_set_value(confline, cl->cur->type, p);
		j_free(confline);
	} while (*pnt != '\0');

	return (0);
}

int
config_init()
{
	struct _conf_list *cl;
	int i;
	char *buf;

	if (getenv("HOME") != NULL) {
		buf = j_asprintf("%s/.junkie/cfg", getenv("HOME"));
	} else {
		DEBUG_MSG(DBG_WEIRD, "$HOME isn't set, trying .");
		buf = j_asprintf("./.junkie/cfg");
	}
	if (buf == NULL)
		goto malloc_error;
	config_head = j_malloc(sizeof(struct _conf_list));
	if (config_head == NULL)
		goto malloc_error;
	cl = config_head;
	i = 0;
	while (_conf_items[i].name != NULL) {
		cl->cur = j_malloc(sizeof(struct _conf_item));
		if (cl->cur == NULL)
			goto malloc_error;
			
		cl->cur->name = j_strdup(_conf_items[i].name);
		if (cl->cur->name == NULL)
			goto malloc_error;
			
		cl->cur->type = _conf_items[i].type;
		cl->cur->value = j_strdup(_conf_items[i].value);
		if (cl->cur->value == NULL)
			goto malloc_error;
			
		cl->next = j_malloc(sizeof(struct _conf_list));
		if (cl->next == NULL)
			goto malloc_error;
		cl = cl->next;
		cl->cur = NULL;
		cl->next = NULL;
		i++;
	}
	config_inited = 1;	/* move it here for config_set_value() */
				/* no real failure cases below anyway  */
	if (config_getvars(buf) == -1) {	/* load the default config */
		if (errno == ENOENT) {
			DEBUG_MSG(DBG_INFO, "no config file found, creating " \
				"a default one in %s", buf);
			i = config_sync();
			j_free(buf);
			if (i == 0)
				config_inited = 1;
			return (i);
		} else if (config_fd == -1) {
			DEBUG_MSG(DBG_WEIRD, "couldn't open config file - %s",
				strerror(errno));
			j_free(buf);
			config_inited = -1;
			return (-1);
		}
	}
	j_free(buf);
	return (0);

malloc_error:
	config_inited = -1;
	if (buf != NULL)
		j_free(buf);
	return (-1);
}

int
config_sync()
{
	struct _conf_list *cl = NULL;
	struct stat sb;
	int n;
	char *buf = NULL, *path = NULL;
	jio *config_fh = NULL;

	if (getenv("HOME") != NULL) {
		buf = j_asprintf("%s/.junkie", getenv("HOME"));
	} else {
		DEBUG_MSG(DBG_WEIRD, "$HOME isn't set, trying .");
		buf = j_asprintf("./.junkie");
	}
	if (buf == NULL) 
		return (-1);
	
	n = stat(buf, &sb);
	if (n == -1) {
		DEBUG_MSG(DBG_INFO, "trying to create %s", buf);
		n = mkdir(buf, 0755);
		if (n == -1) {
			DEBUG_MSG(DBG_INFO, "WARNING: mkdir(): %s", strerror(errno));
			j_free(buf);
			config_inited = -1;
			return (-1);
		}
	}
	
	path = j_asprintf("%s/cfg", buf);
	config_fh = j_open(path, O_RDWR | O_CREAT | O_TRUNC, 0600);

	if (!config_fh) {
		DEBUG_MSG(DBG_ERROR, "failed to create config file %s - %s",
			path, strerror(errno));
		config_inited = -1;
		j_free(path); j_free(buf);
		return (-1);
	}
	
	j_free(buf);
	j_free(path);
	
	cl = config_head;
	while (cl->cur != NULL) {
		switch (cl->cur->type) {
		case T_BOOL:
			buf = j_asprintf("%s=%s\n", cl->cur->name,
			     (char *)cl->cur->value);
			j_write(config_fh, buf, strlen(buf));
			j_free(buf);
			break;
		case T_STR:
			buf = j_asprintf("%s=\"%s\"\n", cl->cur->name,
			    (char *)cl->cur->value);
			j_write(config_fh, buf, strlen(buf));
			j_free(buf);
			break;
		case T_NUM:
			buf = j_asprintf("%s=%s\n", cl->cur->name,
			    (char *)cl->cur->value);
			j_write(config_fh, buf, strlen(buf));
			j_free(buf);
			break;
		default:
			DEBUG_MSG(DBG_ERROR, "corrupt config array!");
			config_inited = -1;
			return (-1);
		}
		cl = cl->next;
	}
	
	j_close(config_fh);
	return (0);
}

int
config_read_value(char *name, int type, void **dest)
{
	struct _conf_list *cl;

	CHECK_INIT();
	cl = config_head;

	while (cl->cur != NULL) {
		if (!strcmp(cl->cur->name, name))
			break;
		cl = cl->next;
	}
	if (cl->cur == NULL) {
		DEBUG_MSG(DBG_ERROR, "called with unknown config var %s",
			name);
		*dest = NULL;
		return (-1);
	}
	if (cl->cur->type != type) {
		DEBUG_MSG(DBG_ERROR, "type mismatch of %s", name);
		*dest = NULL;
		return (-1);
	}
	
// ??	*dest = j_strdup(cl->cur->value);
	if (cl->cur->value)
		*dest = j_strdup(cl->cur->value);
		
	if (*dest == NULL) 
		return (-1);
	
	return (0);
}

int
config_set_value(char *name, int type, void *value)
{
	struct _conf_list *cl;
	void *temp;

	CHECK_INIT();
	cl = config_head;
	while (cl->cur != NULL) {
		if (!strcmp(cl->cur->name, name))
			break;
		cl = cl->next;
	}
	if (cl->cur == NULL) {
		DEBUG_MSG(DBG_ERROR, "called with unknown config var %s",
			name);
		return (-1);
	}
	if (cl->cur->type != type) {
		DEBUG_MSG(DBG_ERROR, "type mismatch of %s", name);
		return (-1);
	}
	
	temp = cl->cur->value;
	cl->cur->value = j_strdup(value);
	if (cl->cur->value == NULL) {
		cl->cur->value = temp;
		return (-1);
	}
	
	j_free(temp);
	return (0);
}

int
config_read_bool(char *name)
{
	int n;
	char *buf;

	n = config_read_value(name, T_BOOL, (void **)&buf);
	if (n == -1) {
		DEBUG_MSG(DBG_ERROR, "config_read_value() failed");
		return (-1);
	}
	if (!strcmp(buf, "yes") || !strcmp(buf, "true") || !strcmp(buf, "on") ||
	    !strcmp(buf, "1")) {
		j_free(buf);
		return (1);
	} else if (!strcmp(buf, "no") || !strcmp(buf, "false") ||
	    !strcmp(buf, "off") || !strcmp(buf, "0")) {
		j_free(buf);
		return (0);
	} else {
		DEBUG_MSG(DBG_ERROR, "unknown boolean value %s for %s", buf, name);
		j_free(buf);
		return (0);
	}
}

int
config_set_bool(char *name, int val)
{
	int n;

	if (val) {
		n = config_set_value(name, T_BOOL, "yes");
		if (n == -1) {
			DEBUG_MSG(DBG_ERROR, "config_set_value() failed");
			return (-1);
		}
	} else {
		n = config_set_value(name, T_BOOL, "no");
		if (n == -1) {
			DEBUG_MSG(DBG_ERROR, "config_set_value() failed");
			return (-1);
		}
	}
	return (0);
}

int
config_read_num(char *name)
{
	int n, num;
	char *buf;

	n = config_read_value(name, T_NUM, (void **)&buf);
	if (n == -1) {
		DEBUG_MSG(DBG_ERROR, "config_read_value() failed");
		return (-1);
	}
	num = (int)strtol(buf, (char **)NULL, 10);
	j_free(buf);
	return (num);
}

int
config_set_num(char *name, int val)
{
	int n;
	char *buf;

	buf = j_asprintf("%d", val);
	n = config_set_value(name, T_NUM, buf);
	if (n == -1) {
		DEBUG_MSG(DBG_ERROR, "config_set_value() failed");
		return (-1);
	}
	return (0);
}
