/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: error.c,v 1.6 2003/03/07 17:48:12 thepin Exp $
 */

#define _GNU_SOURCE

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <err.h>
#include <errno.h>
/* local headers */
#include <misc.h>
#include <error.h>
#include <mem.h>


int
error_report(int mode, int action, const char *comment)
{
	char *buf = NULL;
	
	switch (mode) {
		case MODE_FTP:
			buf = j_asprintf("FTP Error: %s", comment);
			break;
		case MODE_GUI:
			buf = j_asprintf("GUI Error: %s", comment);
			break;
		case MODE_SYS:
			buf = j_asprintf("SYS Error: %s%s%s", 
					comment,
					errno ? " " : "",
					errno ? strerror(errno) : "");
			break;
		case MODE_MEM:
			/*
			 * is it wise to allocate memory on a memory error? :)
			 */
			buf = j_asprintf("MEM Error: %s%s%s",
					comment,
					errno ? " " : "",
					errno ? strerror(errno) : "");
			break;
		default:
			/* Unknown error type. */
			buf = j_asprintf("Unknown error? (%s)", comment);
			break;
	}
		
	
	if ((action & ERR_SHOW) && (buf != NULL)) {
		printf("%s\n", buf);
	} else if ((action & ERR_SHOW) && (buf == NULL)) {
		printf("Out of memory while handling error!\n");
	}
	
	/*
	 * those modes are currently not available.
	 * 
	if (action & ERROR_LOG)
		log("%s\n", buf);
	if (action & ERROR_FATAL)
		exit(1)
	*/
	
	j_free(buf);
		
	return 1;
}

