/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp.c,v 1.14 2003/02/24 04:10:38 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>
/* local headers */
#include <ftp_transfer.h>
#include <ftp_event.h>
#include <ftp_request.h>
#include <ftp.h>
#include <ftp_misc.h>
#include <ftp_net.h>
#include <cache.h>
#include <error.h>
#include <mem.h>
#include <log.h>


/*
 * allocate a new ftp_item
 */
_ftp_item *
ftp_new()
{
	_ftp_item *ftp = NULL;

	if ( (ftp = j_malloc(sizeof(_ftp_item))) == NULL) {
		MEM_ERR("malloc", ERR_SHOW);
		return NULL;
	}

	memset(ftp, 0, sizeof(_ftp_item));
	ftp->transfer = ftp_transfer_new();
	ftp->state = FTP_STATE_NONE;
   
	return ftp;
}

/*
 * free an ftp item
 */
void
ftp_free(_ftp_item **ftp)
{
	if (!ftp || !(*ftp))
		return;

	(*ftp)->transfer = ftp_transfer_free(FTP_TRANSFER((*ftp)));
	j_free((*ftp)->addr);
	j_free((*ftp)->user);
	j_free((*ftp)->pass);
	if ((*ftp)->dataconn)
		ftp_free(&(*ftp)->dataconn);
		
	if ((*ftp)->dirinfo) {
		free((*ftp)->dirinfo->pwd);
		
		if ((*ftp)->dirinfo->list_dir)
			ftp_free_list((*ftp)->dirinfo->list_dir);
	}
	
	j_free(*ftp);
	(*ftp) = NULL;

/*
XXX fix for 0.2
	if (ftp->requests)
		ftp_request_free(ftp->requests);
	if (ftp->event_handler)
		ftp_event_free(ftp->event_handler);
	if (FTP_OBJECT(ftp))
		ftp_object_free(FTP_OBJECT(ftp));
*/
	return;
}



/*
 * Set the server information (host address and port) for an ftp item.
 */
int
ftp_set_server(_ftp_item *ftp, const char *host, int port)
{
	if (!ftp)
		return -1;

	j_free(ftp->addr);
	ftp->addr = j_strdup(host);
	ftp->port = port;

	return 1;
}


/*
 * Set the user information (username and password) for an ftp item.
 */
int
ftp_set_user(_ftp_item *ftp, const char *user, const char *pass)
{
	if (!ftp)
		return -1;

	j_free(ftp->user);
	j_free(ftp->pass);
	ftp->user = j_strdup(user);
	ftp->pass = j_strdup(pass);

	return 1;
}


/*
 * Set the type CONN_FTP/CONN_DATA for an ftp item.
 * XXX for 0.4, obsolete?? consider killing
 */
int
ftp_set_type(_ftp_item *ftp, int type)
{
	if (!ftp)
		return -1;
		
	ftp->type = type;
	
	return 1;
}


/*
 * Set the mode (active(PORT) or passive (PASV)) information for an ftp item.
 */
int
ftp_set_mode(_ftp_item *ftp, int mode)
{
	if (!ftp)
		return -1;

	ftp->mode = mode;

	return 1;
}

