/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp_cmd.c,v 1.51 2003/03/07 17:48:12 thepin Exp $
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>

#ifdef IPV6
#include <arpa/inet.h> /* for inet_ntop */
#endif

/* local headers */
#include <gui.h> //XXX die
#include <ftp_event.h>
#include <ftp_request.h>
#include <ftp.h>
#include <ftp_misc.h>
#include <ftp_cmd.h>
#include <ftp_net.h>
#include <misc.h>
#include <ftp_object.h>
#include <cache.h>
#include <config.h>
#include <error.h>
#include <log.h>
#include <local.h>
#include <mem.h>

/* QUIT */
FTP_CMD_WRAPPER(ftp_quit_wrapper)
{
	if (ftp == NULL)
		return NULL;
	
	FTP_MISC_LOG(ftp, "    Disconnecting from %s:%d\n",
					ftp->addr, ftp->port);
	
	return ftp_request_add(ftp, rpos, mreq, condition, callback, "QUIT");
}

FTP_CMD_WITH_REQ(ftp_quit_with_req)
{
	return ftp_quit_wrapper(ftp, RP_END, mreq, 0, NULL);
}
	
FTP_CMD_WITH_CB(ftp_quit_with_cb)
{
	return ftp_quit_wrapper(ftp, RP_END, NULL, 0, callback); 
}	

FTP_CMD_PLAIN(ftp_quit)
{
	return ftp_quit_wrapper(ftp, RP_END, NULL, 0, NULL);
}

/* MOVE */
FTP_CMD_WRAPPER(ftp_move_wrapper, char *file, char *new)
{
	ftp_request *req = NULL;
	
	if (ftp == NULL)
		return NULL;
	
	req = ftp_request_add(ftp, rpos, mreq, condition, NULL, "RNFR %s", file);
	req = ftp_request_add(ftp, rpos, req,  condition, callback, "RNTO %s", new);
	free(file); free(new);
	
	return req;
}

/* DELETE */
FTP_CMD_WRAPPER(ftp_remove_wrapper, char *file)
{
	ftp_request *req = NULL;

	if(ftp == NULL)
		return NULL;
		
	req = ftp_request_add(ftp, rpos, mreq, condition, callback, "DELE %s", file);
	/* free(file); */
	
	return req;
}

FTP_CMD_PLAIN(ftp_remove, char *file)
{
	return ftp_remove_wrapper(ftp, RP_END, NULL, 0, NULL, file);
}

/* RMDIR */

FTP_CB_CMD(ftp_event_remove_dir)
{
	ftp_request *req = NULL;
	char *dir = j_strdup(ftp_object_get_data(obj, "directory"));

	req = ftp_object_get_data(obj, "request");

	req = ftp_list_wrapper(ftp, (req ? RP_NEXT : RP_END), req, 0, ftp_event_remove_dir_listed, 1, dir);
	ftp_object_set_data(FTP_OBJECT(req), "directory", dir);
	ftp_object_set_data(FTP_OBJECT(req), "request", req);
	
	return;
}

FTP_CB_CMD(ftp_event_remove_dir_listed)
{
	struct _list *list = NULL;
	char *file = NULL, *dir = strdup(ftp_object_get_data(obj, "directory"));
	ftp_request *req = NULL;

	req = ftp_object_get_data(obj, "request");

	list = ftp->dirinfo->list_dir;
	while(list) {
		if(!strcmp(list->file, ".") || !strcmp(list->file, ".."))
			continue;
		else {
			file = j_asprintf("%s/%s", dir, list->file);
			DEBUG_MSG(DBG_INFO, "file/dir: %s scheduled for removal", file);

			if (J_ISFILE(list->type)) {
				req = ftp_remove_wrapper(ftp, (req ? RP_NEXT : RP_END), req, 0, NULL, file);
			} else {
				req = ftp_remdir_wrapper(ftp, (req ? RP_NEXT : RP_END), req, 0, NULL, file);
			}

			j_free(file);
		}
		list = list->next;
	}
	
	ftp_request_add(ftp, (req ? RP_NEXT : RP_END), req, 0, NULL, "RMD %s", dir);
	
	return;
}

FTP_CMD_WRAPPER(ftp_remdir_wrapper, char *file)
{
	ftp_request *req = NULL;

	if(ftp == NULL)
		return NULL;

	req = ftp_request_add(ftp, rpos, mreq, R_FAILURE, ftp_event_remove_dir, "RMD %s", file);
	/* req = ftp_request_add(ftp, rpos, req, R_FAILURE, ftp_event_remove_dir, 0); */
	ftp_object_set_data(FTP_OBJECT(req), "directory", file);
	ftp_object_set_data(FTP_OBJECT(req), "request", req);

	return req;
}

FTP_CMD_PLAIN(ftp_remdir, char *file)
{
	return ftp_remdir_wrapper(ftp, RP_END, NULL, 0, NULL, file);
}

/* MKDIR */
FTP_CMD_WRAPPER(ftp_mkdir_wrapper, const char *path)
{
	if(ftp == NULL)
		return NULL;

	return ftp_request_add(ftp, rpos, mreq, condition, callback, "MKD %s", path);
}

FTP_CMD_WITH_REQ(ftp_mkdir_with_req, const char *path)
{
	return ftp_mkdir_wrapper(ftp, RP_END, mreq, 0, NULL, path);
}

FTP_CMD_WITH_CB(ftp_mkdir_with_cb, const char *path)
{
	return ftp_mkdir_wrapper(ftp, RP_END, NULL, 0, callback, path);
}

FTP_CMD_PLAIN(ftp_mkdir, const char *path)
{
	return ftp_mkdir_wrapper(ftp, RP_END, NULL, 0, NULL, path);
}	

/* move this to misc? */
ftp_request *
ftp_mkpath(_ftp_item *ftp, ftp_request *mreq, const char *file)
{
	register int i = 1;
	int len = 0;
	char *path = strdup(file);
	len = strlen(path);
	
	while(i < len ) {
		for(; path[i] != '\0' && path[i] != '/'; i++);
		if(i >= len)	break;
		path[i] = '\0';
		DEBUG_MSG(DBG_INFO, "requesting MKPATH for [%s]", path);
		mreq = ftp_mkdir_with_req(ftp, mreq, path);
		path[i] = '/';
		i++;	
	}
	
	free(path);

	return mreq;
}

/* TYPE */
/* XXX need any other functions? with callback etc? */
FTP_CMD_WRAPPER(ftp_type, int type)
{
	ftp_request *subr = NULL;
	
	if (ftp == NULL)
		return NULL;

	switch (type) {
		case MODE_ASCI:
			subr = ftp_request_add(ftp, rpos, mreq, condition, callback, "TYPE A");
			break;
		case MODE_BINR:
			subr = ftp_request_add(ftp, rpos, mreq, condition, callback, "TYPE I");
			break;
	}
	
	return subr;
}


/* open up a dataconnection. PORT/PASV */
FTP_CMD_WRAPPER(ftp_open_dataconn)
{
	ftp_request *subreq = NULL;

	switch(ftp->mode) {
		case active:
			subreq = ftp_port(ftp, rpos, mreq, 0,  callback);
			break;
		case passive:
			subreq = ftp_passive(ftp, rpos, mreq, 0, callback);
			break;
	}

	return subreq;

}

/* CWD */
FTP_CMD_WRAPPER(ftp_cwd_wrapper, const char *path)
{
	if (ftp == 0)
		return NULL;
	
	if (!strcmp(path, "..")) {
		/* XXX perhaps try CWD .. also, no? */
		/* XXX make also possible to use failure condition. */
		return ftp_request_add(ftp, RP_END, mreq, condition, callback, "CDUP");
	}
	
	DEBUG_MSG(DBG_INFO, "adding request(CWD %s) to %p", path, ftp);
	return ftp_request_add(ftp, rpos, mreq, condition, callback, "CWD %s", path);
	
}

FTP_CMD_WITH_REQ(ftp_cwd_with_req, const char *path)
{
	return ftp_cwd_wrapper(ftp, RP_END, mreq, 0, NULL, path);
}

FTP_CMD_WITH_CB(ftp_cwd_with_cb, const char *path)
{
	return ftp_cwd_wrapper(ftp, RP_END, NULL, 0, callback, path);
}

FTP_CMD_PLAIN(ftp_cwd, const char *path)
{
	return ftp_cwd_wrapper(ftp, RP_END, NULL, 0, NULL, path);
}


/* PWD */
FTP_CMD_WRAPPER(ftp_pwd_wrapper)
{
	return ftp_request_add(ftp, rpos, mreq, condition, callback, "PWD");
}


FTP_CMD_WITH_REQ(ftp_pwd_with_req)
{
	return ftp_pwd_wrapper(ftp, RP_END, mreq, 0, NULL);
}


FTP_CMD_WITH_CB(ftp_pwd_with_cb)
{
	return ftp_pwd_wrapper(ftp, RP_END, NULL, 0, callback);
}

FTP_CMD_PLAIN(ftp_pwd)
{
	return ftp_pwd_wrapper(ftp, RP_END, NULL, 0, NULL);
	
}

#ifdef IPV6
/* PORT */
FTP_CMD_WRAPPER(ftp_port)
{
	ftp_request *req = NULL;
	register int port;
	int port_fd;
	struct sockaddr_in6 sin;
	int n = sizeof(struct sockaddr_in6);
	char addrbuf[INET6_ADDRSTRLEN];
	_ftp_item *dataconn = NULL;

/*	dataconn = &(ftp->dataconn);
	if (*dataconn != NULL) {
		free(*dataconn);
		*dataconn = NULL;
	}*/

	dataconn = ftp_new();
	
	/* get the port from which we connect and add 1 to it. */
	if (getsockname(ftp->fd, (struct sockaddr *)&sin, &n) == -1) {
		SYS_ERR("getsockname", ERR_SHOW);
		return NULL;
	}
	port = ntohs(sin.sin6_port) + 1;
	while (port < 65535) {
		DEBUG_MSG(DBG_INFO, "testing port: %d", port);
		if ((port_fd = bind6_to(port,sin.sin6_family)) != -1)
			break;
		port++;
	}

	DEBUG_MSG(DBG_INFO, "bound the data fd to %d (port: %d)", port_fd, port);
	if (port > 65535) {
		SYS_ERR("failed to bind port", ERR_SHOW);
		return NULL;
	}


	dataconn->fd = port_fd;
	dataconn->port = port;

	DEBUG_MSG(DBG_INFO, "bound the data fd to %d (port: %d) (%d:%d)", port_fd, port, dataconn->fd, dataconn->port);
	if (port > 65535) {
		SYS_ERR("failed to bind port", ERR_SHOW);
		return NULL;
	}

	DEBUG_MSG(DBG_INFO, "adding the port command to queue");

	if (sin.sin6_family==AF_INET) { /* use the old PORT command */
		inet_ntop(sin.sin6_family,&sin.sin6_addr,
				addrbuf,sizeof(addrbuf));
		addrbuf[strcspn(addrbuf, ".")]=',';
		addrbuf[strcspn(addrbuf, ".")]=',';
		addrbuf[strcspn(addrbuf, ".")]=',';
		req = ftp_request_add(ftp, rpos, mreq, 0, NULL, 
				"PORT %d,%d,%d", addrbuf,
				(port >> 8) & 0xff, port & 0xff);
	} else { /* use EPRT instead of PORT */
		req = ftp_request_add(ftp, rpos, mreq, 0, NULL, 
				"EPRT |2|%s|%d|", 
				inet_ntop(sin.sin6_family,&sin.sin6_addr, 
					addrbuf,sizeof(addrbuf)),
				dataconn->port);
	}
	ftp_object_set_data(FTP_OBJECT(req), "dataconn", (void *)dataconn);
	
	return req;
}
#else
/* PORT */
FTP_CMD_WRAPPER(ftp_port)
{
	ftp_request *req = NULL;
	register int port;
	int port_fd;
	struct sockaddr_in sin;
	int n = sizeof(struct sockaddr_in);
	_ftp_item *dataconn = NULL;

/*	dataconn = &(ftp->dataconn);
	if (*dataconn != NULL) {
		free(*dataconn);
		*dataconn = NULL;
	}*/

	dataconn = ftp_new();
	
	/* get the port from which we connect and add 1 to it. */
	if (getsockname(ftp->fd, (struct sockaddr *)&sin, &n) == -1) {
		SYS_ERR("getsockname", ERR_SHOW);
		return NULL;
	}

	port = ntohs(sin.sin_port) + 1;
	while (port < 65535) {
		DEBUG_MSG(DBG_INFO, "testing port: %d", port);
		if ( (port_fd = bind_to(port)) != -1) 
			break;
		port ++;
	}

	DEBUG_MSG(DBG_INFO, "bound the data fd to %d (port: %d)", port_fd, port);
	if (port > 65535) {
		SYS_ERR("failed to bind port", ERR_SHOW);
		return NULL;
	}


	dataconn->fd = port_fd;
	dataconn->port = port;

	DEBUG_MSG(DBG_INFO, "bound the data fd to %d (port: %d) (%d:%d)", port_fd, port, dataconn->fd, dataconn->port);
	if (port > 65535) {
		SYS_ERR("failed to bind port", ERR_SHOW);
		return NULL;
	}

	DEBUG_MSG(DBG_INFO, "adding the port command to queue");

	req = ftp_request_add(ftp, rpos, mreq, 0, NULL, "PORT %d,%d,%d,%d,%d,%d", 
			sin.sin_addr.s_addr & 0xff,
			(sin.sin_addr.s_addr >> 8) & 0xff,
			(sin.sin_addr.s_addr >> 16) & 0xff,
		      	(sin.sin_addr.s_addr >> 24) & 0xff,
			(port >> 8) & 0xff,
			port & 0xff);
	
	ftp_object_set_data(FTP_OBJECT(req), "dataconn", (void *)dataconn);
	
	return req;
}
#endif /* ifdef IPV6 -> else. */


/* PASV */
FTP_CMD_WRAPPER(ftp_passive)
{
#ifdef IPV6
	/* XXX/FIXME: server will choose protocol that matches the 
	 * controlconnection, that might not always be what you want
	 * in fxp-mode. IE. ipv6-client connected to a ipv6+ipv4 server
	 * fxp:ing to a ipv4-only server.
	 * Syntax: EPSV[<space><net-prt>]
	 */
	struct sockaddr_in6 sin;
	int n = sizeof(struct sockaddr_in6);
	/* get the port from which we connect and add 1 to it. */
       
	if (getsockname(ftp->fd, (struct sockaddr *)&sin, &n) == -1) {
		SYS_ERR("getsockname", ERR_SHOW);
		return NULL;
	}

	if (sin.sin6_family == AF_INET) 
		return ftp_request_add(ftp, rpos, mreq, 0, NULL, "PASV");

	return ftp_request_add(ftp,rpos,mreq,0,NULL,"EPSV");
#else
	return ftp_request_add(ftp, rpos, mreq, 0, NULL, "PASV");
#endif
}



/* LIST */
/*
 * NULL from this function is not bad neccessarily.
 */

FTP_CB_CMD(ftp_list_cache_callback)
{
	FTP_CALLBACK(cb);
	_ftp_item *site = NULL;
	ftp_request *req = NULL;
	struct _list *list_dir = NULL;
	char *path = NULL;
	int condition;

	DEBUG_MSG(DBG_INFO, "--ftp_list_cache_callback--");
	if (!ftp || !ftp->dirinfo) 
		return;

	site = ftp_object_get_data(obj, "ftp");
	condition = (int)ftp_object_get_data(obj, "condition");
	cb = ftp_object_get_data(obj, "callback");
	req = ftp_object_get_data(obj, "request");
	path = ftp_object_get_data(obj, "path");

	DEBUG_MSG(DBG_INFO, "fanki shit");
	if ( (list_dir = cache_list(ftp, path ? path : ftp->dirinfo->pwd)) == NULL) {
		DEBUG_MSG(DBG_INFO, "not found fanki shit %s or %s", path, ftp->dirinfo->pwd);
		req = ftp_list_wrapper((site ? site : ftp), RP_END, req, condition, cb, 1, path);
		ftp_object_copy_data(FTP_OBJECT(req), obj);
		free(path);
		
		return;
	}
	
	ftp->dirinfo->list_dir = list_dir;
	free(path);

	if (cb) {
		FTP_MISC_LOG(ftp, "    Giving cached directory structure of %s\n", ftp->dirinfo->pwd);
		cb(site ? site : ftp, 0, R_SUCCESS, NULL, NULL, NULL, obj);
	}

	return;
}

FTP_CMD_WRAPPER(ftp_list_wrapper, int refresh, char *path)
{
	ftp_request *subreq = NULL;
	char arg[MED_SIZE + 1];
	int list_method;
	char *list_method_args = NULL;
	int have_space;

	if (ftp == NULL)
		return NULL;

	if ((config_read_bool("cache-enable") || config_read_bool("cache-session")) && !refresh) {
		/* check if the entry is available in the cache. */
		subreq = ftp_request_add(ftp, RP_END, mreq, condition, ftp_list_cache_callback, NULL);
		
		ftp_object_set_data(FTP_OBJECT(subreq), "ftp", ftp);
		ftp_object_set_data(FTP_OBJECT(subreq), "condition", (void *)condition);
		ftp_object_set_data(FTP_OBJECT(subreq), "callback", callback);
		ftp_object_set_data(FTP_OBJECT(subreq), "request", mreq);
		if (path)
			ftp_object_set_data(FTP_OBJECT(subreq), "path", strdup(path));
			
		ftp_request_set_type(subreq, REQ_CMD);

		DEBUG_MSG(DBG_INFO, "** Added cache request");
		
		return subreq;
	}

 	list_method = config_read_num("list_method");
	if (list_method == FTP_LIST_PLAIN) {
		list_method_args = NULL;
		have_space = 0;
	} else if (list_method == FTP_LIST_CUSTOM) {
		have_space = 1;
		config_read_value("list_method_custom", T_STR, (void **)&list_method_args);
	} else {
		/* default */
		have_space = 1;
		list_method_args = j_strdup("-al");
	}
	
	snprintf(arg, MED_SIZE, "LIST%s%s",
				(have_space) ? (" ") : ((path) ? " " : ""),
				path ? path : (list_method_args ? list_method_args : ""));

	j_free(list_method_args);
 
	subreq = ftp_open_dataconn(ftp, rpos, mreq, condition, callback);
	return ftp_request_add(ftp, RP_TREE_NEXT, subreq, condition, callback, "%s", arg);
}

FTP_CMD_WITH_REQ(ftp_list_with_req, int refresh, char *path)
{
	return ftp_list_wrapper(ftp, RP_END, mreq, 0, NULL, refresh, path);
}

FTP_CMD_WITH_CB(ftp_list_with_cb, int refresh, char *path)
{
	return ftp_list_wrapper(ftp, RP_END, NULL, 0, callback, refresh, path);
}

FTP_CMD_PLAIN(ftp_list, int refresh, char *path)
{
	return ftp_list_wrapper(ftp, RP_END, NULL, 0, NULL, refresh, path);
}

/* Login, USER/PASS */
FTP_CMD_PLAIN(ftp_login)
{
	ftp_request *rpnt = 0;

	DEBUG_MSG(DBG_INFO, "adding login requests");
	rpnt = ftp_request_add(ftp, RP_END, NULL, 0, NULL, "USER %s", ftp->user);
	return ftp_request_add(ftp, RP_END, rpnt, 0, NULL, "PASS %s", ftp->pass);
}

/* RETR */
FTP_CMD_WRAPPER(ftp_retr, struct _list *ent, char *localfile, int state)
{
	struct stat _stat;
	ftp_request *subreq = 0;

	if (state == TRFR_DEFAULT)
		 state = config_read_num("file-exist");
	
	DEBUG_MSG(DBG_INFO, "STATE is.............:  %d", state);
	
	if (ent == NULL)
		return NULL;

	if(!localfile) {
		localfile = ent->file;
	}

	DEBUG_MSG(DBG_INFO, "retr file :) %s", ent->file);

	/* request a data connection. */
	if ( (subreq = ftp_open_dataconn(ftp, rpos, mreq, 0, NULL)) == NULL)
		return NULL;
		

	/* TYPE I. */
	subreq = ftp_type(ftp, RP_END, subreq, 0, NULL, MODE_BINR);
	_stat.st_size = 0;
	if (stat(localfile, &_stat) != -1) {
		/* The file exist, check how we want to handle it. */
		if (state == TRFR_ASK)
			state = gui_dialog_file_exist(localfile);
		if (state == TRFR_RESUME) {
			FTP_MISC_LOG(ftp, "    %s: Resuming transfer (starting at %d/%d).\n", 
						localfile, _stat.st_size, ent->size);
			subreq = ftp_request_add(ftp, RP_END, mreq, condition, 0, 
						"REST %d", _stat.st_size);
		}
		else if (state == TRFR_OVERWRT)
			unlink(localfile);
		else if (state == TRFR_NOACT)
			return NULL;
	}

	/* RETR file. */
	subreq = ftp_request_add(ftp, RP_END, subreq, condition, callback, "RETR %s", ent->file);

	/* setup arguments for the callback. */
	ftp_object_set_data(FTP_OBJECT(subreq), "ftp_transfer_size", (void *)ent->size);
	ftp_object_set_data(FTP_OBJECT(subreq), "ftp_transferred_bytes",
			state == TRFR_RESUME ? (void *)((long)_stat.st_size) : 0);
	ftp_object_set_data(FTP_OBJECT(subreq), "ftp_restored_bytes",
			state == TRFR_RESUME ? (void *)((long)_stat.st_size) : 0);
	ftp_object_set_data(FTP_OBJECT(subreq), "file", strdup(localfile));
	ftp_object_set_data(FTP_OBJECT(subreq), "state", (void *) state);

	DEBUG_MSG(DBG_INFO, "the retr req is %p", subreq);
	return subreq;
}

/* STOR */
FTP_CB_CMD(ftp_stor_size_callback)
{
	FTP_CALLBACK(cb);
	ftp_request *req = NULL, *subreq = NULL, *sreq = NULL;
	struct _list fent;
	char *pnt = NULL, *file = NULL;
	int size = 0;

	req = ftp_object_get_data(obj, "ftp_stor_subreq");
	cb = ftp_object_get_data(obj, "ftp_stor_callback");
	file = ftp_object_get_data(obj, "ftp_stor_file");
	if (status & R_SUCCESS) {
		/* Parse the reply. */
		if ( (pnt = strchr(resp, ' ')) == NULL)
			return;
	
		size = atol(pnt + 1);
		if (size > 0)
			subreq = ftp_request_add(ftp, RP_NEXT, req, 0, NULL, "REST %d", size);
	}
	

	memset(&fent, 0, sizeof(struct _list));
	fent.size = (int)ftp_object_get_data(obj, "ftp_stor_size");
	fent.file = file;

	DEBUG_MSG(DBG_INFO, "req: %p, cb: %p", req, cb);
	
	sreq = ftp_stor(ftp, RP_NEXT, (subreq ? subreq : req), 0, cb, 
			(struct _list *)&fent, 
			ftp_object_get_data(obj, "ftp_stor_localfile"), 
			subreq ? 1 : 1);
	ftp_object_copy_data(FTP_OBJECT(sreq), obj);
	ftp_object_set_data(FTP_OBJECT(sreq), "ftp_transfer_rest", (void *)size);

	DEBUG_MSG(DBG_INFO, "widget: %p", ftp_object_get_data(FTP_OBJECT(sreq), "widget"));
	DEBUG_MSG(DBG_INFO, "--ftp_stor_size_callback");
	DEBUG_MSG(DBG_INFO, "status: %d", status);
	DEBUG_MSG(DBG_INFO, "response: %s", resp);
	DEBUG_MSG(DBG_INFO, "size: %d file size: %d", (int)size, (int)fent.size);
	
	return;
}

FTP_CB_CMD(ftp_stor_rm_callback)
{
	FTP_CALLBACK(cb);
	ftp_request *req = NULL, *sreq = NULL;
	char *file;
	struct _list fent;
	req  = ftp_object_get_data(obj, "ftp_stor_subreq");
	cb   = ftp_object_get_data(obj, "ftp_stor_callback");
	file = ftp_object_get_data(obj, "ftp_stor_file");
	DEBUG_MSG(DBG_INFO, "GG***********************************@#$!@#$");
	memset(&fent, 0, sizeof(struct _list));
	fent.size = (int)ftp_object_get_data(obj, "ftp_stor_size");
	fent.file = file;

	sreq = ftp_stor(ftp, RP_NEXT, req, 0, cb,
			(struct _list *)&fent,
			ftp_object_get_data(obj, "ftp_stor_localfile"),
			status & R_SUCCESS ? 1 : 8);

	return;
}

FTP_CB_CMD(ftp_stor_skip_callback)
{
	FTP_CALLBACK(cb);
	ftp_request *req = NULL, *sreq = NULL;
	char *file;
	struct _list fent;
	req  = ftp_object_get_data(obj, "ftp_stor_subreq");
	cb   = ftp_object_get_data(obj, "ftp_stor_callback");
	file = ftp_object_get_data(obj, "ftp_stor_file");

	memset(&fent, 0, sizeof(struct _list));
	fent.size = (int)ftp_object_get_data(obj, "ftp_stor_size");
	fent.file = file;

	sreq = ftp_stor(ftp, RP_NEXT, req, 0, cb,
			(struct _list *)&fent,
			ftp_object_get_data(obj, "ftp_stor_localfile"),
			1);

	return;
}

FTP_CMD_WRAPPER(ftp_stor, struct _list *ent, char *localfile, int state)
{
	struct stat _stat;
	ftp_request *subreq = NULL;

	if (state == TRFR_DEFAULT)
		state = config_read_num("file-exist");  
	
	DEBUG_MSG(DBG_INFO, "state is %d", state);
	
	if (ent == NULL)
		return NULL; 

	if (!localfile)
		localfile = ent->file;
	
	if (stat(localfile, &_stat) == -1) {
		DEBUG_MSG(DBG_ERROR, "stat(%s...) error: %s", localfile, strerror(errno));
		return NULL;
	}

	if (state == TRFR_ASK)
		state = gui_dialog_file_exist(localfile);

	if (state == TRFR_RESUME ||  state == TRFR_NOACT) {
		if(state == TRFR_RESUME)
			subreq = ftp_request_add(ftp, RP_END, mreq,
					                (R_SUCCESS | R_FAILURE),
									ftp_stor_size_callback, 
									"SIZE %s", ent->file);
//		else if(state == TRFR_OVERWRT)
//			subreq = ftp_remove_wrapper(ftp, RP_END, mreq,
//					                   (R_SUCCESS | R_FAILURE),
//									   ftp_stor_rm_callback, 
//									   ent->file);
		else if(state == TRFR_NOACT)
			subreq = ftp_request_add(ftp, RP_END, mreq,
										R_FAILURE,
										ftp_stor_skip_callback,
										"SIZE %s", ent->file);
		
		ftp_object_set_data(FTP_OBJECT(subreq), "ftp_stor_req", mreq);
		ftp_object_set_data(FTP_OBJECT(subreq), "ftp_stor_subreq", subreq);
		ftp_object_set_data(FTP_OBJECT(subreq), "ftp_stor_callback", callback);
		ftp_object_set_data(FTP_OBJECT(subreq), "ftp_stor_file", ent->file);
		ftp_object_set_data(FTP_OBJECT(subreq), "ftp_stor_localfile", localfile);
		ftp_object_set_data(FTP_OBJECT(subreq), "ftp_stor_size", (void *)ent->size);

		return subreq;
	}

	DEBUG_MSG(DBG_INFO, "request sent");
	
	//ftp_mkpath(ftp, NULL, ent->file);
	
	if ( (subreq = ftp_open_dataconn(ftp, rpos, mreq, 0, NULL)) == NULL)
		return NULL;

        subreq = ftp_type(ftp, rpos, subreq, 0, NULL, MODE_BINR);
					
	subreq = ftp_request_add(ftp, rpos, subreq, condition, callback, "STOR %s", ent->file);
	ftp_object_set_data(FTP_OBJECT(subreq), "ftp_transfer_size", (void *)ent->size);
	ftp_object_set_data(FTP_OBJECT(subreq), "ftp_transfer_file", strdup(localfile));

	DEBUG_MSG(DBG_INFO, "done");
	
	return subreq;
}


/*
** NOOP - no operation, useful for anti idlers
*/
FTP_CMD_PLAIN(ftp_noop)
{
	return ftp_request_add(ftp, RP_END, NULL, 0, NULL, "NOOP");
}
