/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp_event.c,v 1.24 2003/03/07 17:48:12 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>
/* temporary */
#include <gui_ftp.h>
/* local headers */
#include <ftp.h>
#include <ftp_object.h>
#include <ftp_event.h>
#include <ftp_event_callback.h>
#include <ftp_request.h>
#include <ftp_net.h>
#include <defaults.h>
#include <error.h>
#include <gui_toolkit.h>	/* NOO */
#include <gui_ftp.h>		/* NOOOO :((( XXX */
#include <mem.h>


void
ftp_event_setup_default(_ftp_item *ftp)
{
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "CWD", ftp_event_callback_cwd);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "LIST", ftp_event_callback_list);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "PASS", ftp_event_callback_pass);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "PWD", ftp_event_callback_pwd);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "PASV", ftp_event_callback_pasv);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "PORT", ftp_event_callback_port);
#ifdef IPV6
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "EPSV", ftp_event_callback_epsv);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "EPRT", ftp_event_callback_port);
#endif
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "RETR", ftp_event_callback_retr);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "STOR", ftp_event_callback_stor);
	ftp_event_add_handler(ftp, FTP_EVENT_IRRUPT, (void *)FTP_EVENT_IRRUPT_QUIT, ftp_event_irrupt_disconnect);
}

int
rfc_code_interpreter(int code)
{
	if (code == 1)
		return R_READY;
	else if (code == 2)
		return R_SUCCESS;
	else if (code == 3)
		return R_SUCCESS; /* needs more data.. */
	else if (code == 4)
		return R_FAILURE;
	else if (code == 5)
		return R_FAILURE;
	
	return 0;
}


int
ftp_event_add_handler(_ftp_item *ftp, int type, void *arg, FTP_EVENT_CALLBACK(callback))
{
	ftp_event **epnt = NULL;

	if (ftp == NULL)
		return 0;
	
	epnt = &ftp->event_handler;
	while (*epnt)
		epnt = &(*epnt)->next;

	if ( (*epnt = j_malloc(sizeof(ftp_event))) == NULL) {
		MEM_ERR("malloc", ERR_SHOW);
		return -1;
	}
	
	memset(*epnt, 0, sizeof(ftp_event));
	if (arg != NULL) {
		switch (type) {
			case FTP_EVENT_CMD:
				(*epnt)->arg = strdup(arg);
				break;
			default:
				(*epnt)->arg = arg;
				break;
		}
	}

	(*epnt)->type = type;
	(*epnt)->objects = NULL;
	(*epnt)->callback = callback;
	(*epnt)->next = NULL;

	return 1;
}

ftp_event *
ftp_event_find_entry(ftp_event *root, int type, void *arg)
{
	ftp_event *eptr = NULL;

	if (arg == NULL)
		return NULL;
	
	eptr = root;
	while (eptr) {
		if (eptr->type == type && eptr->arg) {
			if (type == FTP_EVENT_CMD) {
				if (!strcasecmp((char *)eptr->arg, (char *)arg))
					return eptr;
			} else {
				if (eptr->arg == arg)
					return eptr;
			}
			
		}
		
		eptr = eptr->next;
	}

	return NULL;
}




/*
** generate a data structure with all the values
*/
struct ftp_event_cb_info *
ftp_event_setup_info_data(_ftp_item *ftp, int estat_index, ftp_request *rpnt, int code, int status, char *resp, ftp_object **obj)
{
	struct ftp_event_cb_info *einfo = NULL;

	if (!(einfo = j_malloc(sizeof(struct ftp_event_cb_info))))
		return NULL;
	
	einfo->ftp = ftp;
	einfo->estat_index = estat_index;
	einfo->rpnt = rpnt;
	einfo->code = code;
	einfo->state = status;
	if (resp)
		einfo->buf = strdup(resp);

	einfo->obj = obj;

	return einfo;
}
	
/*
** a nonblocked function should call this before finishing, very important.
*/
void
ftp_event_nonblock_finished(_ftp_item *ftp, struct ftp_event_handler_status *estat, int entry_index, struct ftp_event_cb_info *einfo)
{
	int i;
	ftp_request *rpnt = NULL;

	if (!ftp || !estat || !einfo)
		return;

	/* this little prelude is to ensure all nonblocked event handlers are finished before continuing.*/
	estat[entry_index].status = FTP_EVENT_SUCCESS;
	for (i = 0; estat[i].rpnt; i++) {
		if (estat[i].status == FTP_EVENT_NONBLOCK) {
			return;
		}
	}

	j_free(estat);
	
	rpnt = einfo->rpnt;
	/* call the defined callback, if it was defined */
	rpnt->state = einfo->state;
	if (rpnt && rpnt->callback != 0 && (einfo->state & rpnt->condition)) {
		rpnt->callback(ftp, einfo->code, einfo->state, rpnt->cmd, rpnt->args, einfo->buf, FTP_OBJECT(rpnt));
	}

	j_free(einfo->buf);
	j_free(einfo);
	/*
	** Turn the request sender on again after, if there are any requests to be sent.   
	** They were disabled while processing the callbacks/event handlers.
	*/
	if (ftp->n_requests > 0)
		gui_ftp_setup_write(ftp, 1);

	gui_ftp_setup(ftp, 1);
	/* call the ftp event handler in case there was some data cached earlier and not handled yet. */
	ftp_handle_events(&ftp);

	return;
}

void
ftp_handle_events(_ftp_item **ftp)
{
	char buf[MAX_SIZE + 1];
	int code, first, state, ret;
	ftp_request *rpnt = NULL;
	ftp_event *eptr = NULL;
	struct ftp_event_handler_status *estat = NULL;
	int estat_entries;
	void *ptr;
	
	if ( (*ftp) == NULL)
		return;

	memset(buf, 0, sizeof(buf));
	/* loop till there is no more data available. */
	DEBUG_MSG(DBG_INFO, "ftp_handle_events");

	while ( (code = ftp_get_resp(*ftp, buf, MAX_SIZE)) > 0 && code != FTP_READ_CACHED) {
		DEBUG_MSG(DBG_INFO, "!!! got resp: %s (code: %d)", buf, code);
		if ((*ftp)->n_requests > 0)
			gui_ftp_setup_write(*ftp, 1);

		if (code == 220) {
			if ((*ftp)->state == FTP_STATE_CONNECTED)
				(*ftp)->state = FTP_STATE_GREETED;
				
			continue;
		}

		/* detect state by first digit of code see RFC. */
		first = (code - (code % 100)) / 100;
		state = rfc_code_interpreter(first);

		/* find the appropriate request entry. */
		rpnt = ftp_request_find_entry((*ftp)->requests, R_SENT | R_READY, 0);
		if (rpnt == NULL)
			continue;

		/* does it meet the requirements of the condition? */
		if (!(rpnt->state & R_SENT) && !(rpnt->state & R_READY)) 
			continue;

		/*
		** If there are any requests waiting to be sent, prevent them from being sent by
		** preventing the request sender being called.  Nothing should be sent while processing
		** those callbacks and event handlers.
		*/
		if ((*ftp)->n_requests > 0)
			gui_ftp_setup_write(*ftp, 0);
			

		ftp_object_set_data(FTP_OBJECT(rpnt), "ftp_request", rpnt);
		/* find & run the default cmd handlers if they are found. */
		eptr = ftp_event_find_entry((*ftp)->event_handler, FTP_EVENT_CMD, rpnt->cmd);
		estat = NULL; estat_entries = 0;
		while (eptr) {
			if (eptr && eptr->callback != 0) {
				ftp_object_set_data(FTP_OBJECT(rpnt), "ftp_event_entry", (void *)estat_entries);
				ret = eptr->callback(*ftp, code, state, rpnt->cmd, rpnt->args, buf, FTP_OBJECT(rpnt));
				
				if (ret == FTP_EVENT_NONBLOCK) {
					/* nonblocking one. */
					if (!(ptr = realloc(estat, (++estat_entries + 1) * sizeof(struct ftp_event_handler_status)))) {
						free(estat);
						return;
					}
					estat = ptr;

					estat[estat_entries-1].rpnt = rpnt;
					estat[estat_entries-1].status = FTP_EVENT_NONBLOCK;
					estat[estat_entries].rpnt = NULL;
				}
			}
			
			eptr = ftp_event_find_entry(eptr->next, FTP_EVENT_CMD, rpnt->cmd);
		}

		if (estat_entries > 0) {
			ftp_object_set_data(FTP_OBJECT(rpnt), "ftp_event_estat", estat); // lame :)
			gui_ftp_setup(*ftp, 0);
			break;
		} else {
			/* all the event handlers finished processing (no nonblocking ones). */

			/* call the defined callback, if it was defined */
			if (rpnt->callback != 0 && (state & rpnt->condition)) {
				rpnt->callback(*ftp, code, state, rpnt->cmd, rpnt->args, buf, FTP_OBJECT(rpnt));
			}

			/* update the state of the appropriate request entry. */
			rpnt->state = state;

			/*
			** Turn the request sender on again after, if there are any requests to be sent.   
			** They were disabled while processing the callbacks/event handlers.
			*/
			if ((*ftp)->n_requests > 0)
				gui_ftp_setup_write(*ftp, 1);
		}

		memset(buf, 0, sizeof(buf));
	}

	if (code == 0) {
		/*
		** Connection was closed, call all the interrupt quit handlers.
		*/
		eptr = ftp_event_find_entry((*ftp)->event_handler, FTP_EVENT_IRRUPT, (void *)FTP_EVENT_IRRUPT_QUIT);
		while (eptr) {
			if (eptr && eptr->callback) {
				eptr->callback(*ftp, 0, 0, NULL, NULL, NULL, NULL);
			}

			eptr = ftp_event_find_entry(eptr->next, FTP_EVENT_IRRUPT, (void *)FTP_EVENT_IRRUPT_QUIT);
		}
	}

	return;
}

