/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp_event_callback.c,v 1.52 2003/03/07 17:48:12 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <glib.h> /* nooo :( */

#ifdef IPV6
#include <arpa/inet.h> /* inet_ntop */
#endif

/* local headers */
#include <ftp_event.h>
#include <ftp_event_callback.h>
#include <ftp_request.h>
#include <ftp_transfer.h>
#include <ftp_net.h>
#include <ftp_misc.h>
#include <ftp.h>
#include <ftp_net.h>
#include <ftp_cmd.h>
#include <cache.h>
#include <config.h>
#include <error.h>
#include <ftp_parse.h>
#include <telnet.h>
#include <log.h>
#include <medium.h>
#include <local.h>
#include <mem.h>
#include <defaults.h>


FTP_EVENT_CB_CMD(ftp_event_callback_cwd)
{
	if (code != 250)
		return FTP_EVENT_FAILURE;

	/*
	 * Get the current working directory
	 * List the directory.
	 */
	return FTP_EVENT_SUCCESS;
}


FTP_EVENT_CB_CMD(ftp_event_callback_pwd)
{
	char *pwd = 0, *pnt = 0;
	struct ftp_dir_info **dirinfo = NULL;
	
	/*
	 * Parse the current working directory,
	 * and store it.
	 */
	if (ftp == NULL)
		return FTP_EVENT_FAILURE;

	DEBUG_MSG(DBG_INFO, "PWD callback, cmd: %s, args: %s resp: %s", cmd, args, resp);

	dirinfo = &ftp->dirinfo;
	if (*dirinfo == NULL) {
		(*dirinfo) = j_malloc(sizeof(struct ftp_dir_info));
		if (*dirinfo == NULL) 
			return FTP_EVENT_FAILURE;
		
		memset(*dirinfo, 0, sizeof(struct ftp_dir_info));
	}

	if ( (pwd = strchr(resp, '"')) == NULL)
		return FTP_EVENT_FAILURE;
	pwd += 1;
	
	if ( (pnt = strrchr(pwd, '"')) == NULL)
		return FTP_EVENT_FAILURE;
	
	*pnt = 0;	/* XXX try to avoid this. */

	j_free((*dirinfo)->pwd);
	(*dirinfo)->pwd = NULL;
	if (pwd)
		(*dirinfo)->pwd = strdup(pwd);

	*pnt = '"';	/* XXX this then should not be needed either. */

	DEBUG_MSG(DBG_INFO, "in dir: %s", (*dirinfo)->pwd);

	medium_cmd(M_PATHS_UPDATE, NULL, ftp, (*dirinfo)->pwd);
	
	return FTP_EVENT_SUCCESS;
}


/*
	returns 1 when it wants more data, 0 when finished.
*/
int
ftp_event_list_read_callback(int fd __attribute__((unused)), void *data)
{
	struct ftp_event_cb_info *einfo = data;
	_ftp_item *ftp = NULL;
	struct _list **list_pnt = NULL;
	struct ftpparse fp;
	unsigned char buf[GOOD_SIZE + 1];
	char /**buf = NULL, */ *pnt = NULL;
	int ret, readmore = 1;

	if (!einfo) {
		DEBUG_MSG(DBG_INFO, "no einfo");
		return 0;
	}
	
	ftp = einfo->ftp;
        einfo->estat = ftp_object_get_data(einfo->obj, "ftp_event_estat");
		
	list_pnt = &ftp->dirinfo->list_dir;
	while (*list_pnt) 
		list_pnt = &(*list_pnt)->next;

	while ((ret = ftp_readline(ftp->dataconn, buf, GOOD_SIZE, readmore)) == FTP_READ_SUCCESS) {
		pnt = strchr(buf, '\r');
		if (pnt)
			*pnt = 0;
		
		if (readmore)
			readmore = 0;

		// DEBUG_MSG(DBG_INFO, "buffer read: \"%s\" (%d)\n", buf, strlen(buf));
		/* a normal file or a directory. */
		if (ftpparse(&fp, buf, strlen(buf)) == 0) {
			DEBUG_MSG(DBG_WEIRD, "ftpparse returned 0");
			continue;
		}

		if (!strncmp(fp.name, ".", fp.namelen) ||
		!strncmp(fp.name, "..", fp.namelen))
			continue;

		if ( (*list_pnt = (struct _list *)j_malloc(sizeof(struct _list))) == NULL) 
			return 0;

		(*list_pnt)->next  = NULL;
		(*list_pnt)->child = NULL;
		(*list_pnt)->perm  = NULL;
		(*list_pnt)->owner = NULL;
		(*list_pnt)->group = NULL;
		if (fp.perm != NULL && fp.permlen > 0) {
			if ( ((*list_pnt)->perm = (char *)j_malloc(sizeof(char)*fp.permlen+1)) == NULL) 
				return 0;

			strncpy((*list_pnt)->perm, fp.perm, fp.permlen);
			(*list_pnt)->perm[fp.permlen] = 0;
		}
		if (fp.owner != NULL && fp.ownerlen > 0) {
			if ( ((*list_pnt)->owner = (char *)j_malloc(sizeof(char)*fp.ownerlen+1)) == NULL) 
				return 0;
				
			strncpy((*list_pnt)->owner, fp.owner, fp.ownerlen);
			(*list_pnt)->owner[fp.ownerlen] = 0;
		}
		if (fp.group != NULL && fp.grouplen > 0) {
			if ( ((*list_pnt)->group = (char *)j_malloc(sizeof(char)*fp.grouplen+1)) == NULL) 
				return 0;
				
			strncpy((*list_pnt)->group, fp.group, fp.grouplen);
			(*list_pnt)->group[fp.grouplen] = 0;
		}
		if (fp.name != NULL && fp.namelen > 0) {
			if ( ((*list_pnt)->file = (char *)j_malloc(sizeof(char)*fp.namelen+1)) == NULL) 
				return 0;

			strncpy((*list_pnt)->file, fp.name, fp.namelen);
			(*list_pnt)->file[fp.namelen] = 0;
		}


		/* get the date of the file. */
		if (fp.mtimetype == FTPPARSE_MTIME_STRING) {
			DEBUG_MSG(DBG_INFO, "got a string returned, in \"%s\" (%d)\n", fp.datestring, fp.datelen);
			(*list_pnt)->mtime = 0;
			if (fp.datestring != NULL && fp.datelen > 0) {
				(*list_pnt)->mtime_string = j_malloc(fp.datelen + 1);
				strncpy((*list_pnt)->mtime_string, fp.datestring, fp.datelen);
				(*list_pnt)->mtime_string[fp.datelen] = 0;
				DEBUG_MSG(DBG_INFO, "aha, mtime string is \"%s\"\n", (*list_pnt)->mtime_string);
			}
		} else {
			(*list_pnt)->mtime = fp.mtime;
			(*list_pnt)->mtime_string = NULL;
		}
			
		(*list_pnt)->size = fp.size;

		(*list_pnt)->type = 0;
		if (fp.flagtrycwd == 1 && fp.flagtryretr == 1) {
			(*list_pnt)->type = _LIST_DIR | _LIST_FILE;
		} else if (fp.flagtrycwd == 1)
			 (*list_pnt)->type = _LIST_DIR;
		else if (fp.flagtryretr == 1)
			(*list_pnt)->type = _LIST_FILE;

		list_pnt = (struct _list **)&((*list_pnt)->next);
	}

	if (ret == FTP_CONN_CLOSED) {
		DEBUG_MSG(DBG_INFO, "dataconn closed!");
		close(ftp->dataconn->fd);
		if ((config_read_bool("cache-enable") || config_read_bool("cache-session")) && ftp->dirinfo && ftp->dirinfo->pwd) {
			/* Update the cache. */
			cache_update(ftp, ftp->dirinfo->pwd, ftp->dirinfo->list_dir);
		}

		ftp_event_nonblock_finished(ftp, einfo->estat, einfo->estat_index, einfo);
		
		return 0;
	}
	
	return 1;
}

FTP_EVENT_CB_CMD(ftp_event_callback_list)
{
	int ret;
	struct ftp_event_cb_info *einfo = NULL;
	/* XXX Should be called on state READY. */
	if (code != 150)
		return FTP_EVENT_SUCCESS;
	
	DEBUG_MSG(DBG_INFO, "ftp_event_callback_list, LIST response");
	
	if (ftp == NULL || ftp->dataconn == NULL) {
		DEBUG_MSG(DBG_INFO, "failure, ftp == %p ftp->dataconn == %p", ftp, (ftp != NULL ? ftp->dataconn : NULL));
		return FTP_EVENT_FAILURE;
	}

	if (ftp->mode == active) { 
		ftp->dataconn->fd = accept_dataport(ftp->dataconn->fd);
		if (ftp->dataconn->fd < 0) {
			FTP_MISC_LOG(ftp, "    PORT failed\n");
			return FTP_EVENT_FAILURE;
		}
	}
	
	einfo = ftp_event_setup_info_data(ftp,  (int)ftp_object_get_data(obj, "ftp_event_entry"), ftp_object_get_data(obj, "ftp_request"), code, status, resp, obj);

	/* add a callback that will call ftp_event_list_read_callback when there is data available there. */
	if (medium_cmd(M_IO_WATCH, &ret, ftp->dataconn->fd, IO_WATCH_READ|IO_WATCH_HUP, ftp_event_list_read_callback, einfo)) {
		ftp->dataconn->type = CONN_DATA;
		ftp->dirinfo->list_dir = NULL;
		return FTP_EVENT_NONBLOCK;
	}


	return FTP_EVENT_SUCCESS;
}

FTP_EVENT_CB_CMD(ftp_event_callback_pass)
{
	if (code != 230)
		return FTP_EVENT_FAILURE;

	ftp->state = FTP_STATE_LOGGED_IN;
		
	return FTP_EVENT_SUCCESS;
}


void
ftp_event_callback_pasv_lookup_cb(int fd, int ret, void *data)
{
	_ftp_item *ftp = NULL;

	ftp = data;
	if (fd <= 0 || ret || !ftp) 
		return;

	/* set the fd for the dataconn if available.*/
	if (ftp->dataconn) 
		ftp->dataconn->fd = fd;

	return;
}


FTP_EVENT_CB_CMD(ftp_event_callback_pasv)
{
	_ftp_item **dataconn = NULL;
	t_ftpinfo *info = NULL;

	DEBUG_MSG(DBG_INFO, "ftp_event_callback_pasv");
	
	dataconn = &(ftp->dataconn);
	if (*dataconn != 0) {
		j_free(*dataconn);
		*dataconn = 0;
	}

	if ( ((*dataconn) = ftp_new()) == NULL)
		return FTP_EVENT_FAILURE;

	if (!(info = ftp_parse_pasv_mode(resp))) {
		FTP_ERR("ftp_parse_pasv_mode", ERR_SHOW);
		return FTP_EVENT_FAILURE;
	}

	ftp_set_server(*dataconn, info->ip, info->port);
	if (connect_to(info->ip, info->port, ftp_event_callback_pasv_lookup_cb, ftp))
		return FTP_EVENT_FAILURE;

	j_free(info->ip);
	j_free(info);

	return FTP_EVENT_SUCCESS;
}


FTP_EVENT_CB_CMD(ftp_event_callback_port)
{
	_ftp_item *dataconn = NULL;
	
	if (!ftp || ftp->mode != active)
		return FTP_EVENT_FAILURE;

	dataconn = ftp_object_get_data(obj, "dataconn");
	if (!dataconn)
		return FTP_EVENT_FAILURE;
	
	if (ftp->dataconn != NULL) {
		j_free(ftp->dataconn);
		ftp->dataconn = NULL;
	}
	
	ftp->dataconn = dataconn;

	return FTP_EVENT_SUCCESS;
}

#ifdef IPV6
FTP_EVENT_CB_CMD(ftp_event_callback_epsv)
{
	_ftp_item **dataconn = 0;
	int port,len;
	char *portptr,ip[INET6_ADDRSTRLEN];
	struct sockaddr_in6 sin6;

	dataconn = &(ftp->dataconn);
	if (*dataconn != 0) {
		j_free(*dataconn);
		*dataconn = 0;
	}

	if ( ((*dataconn) = ftp_new()) == NULL)
		return FTP_EVENT_FAILURE;

	/* some really ugly parsing! */
	portptr = strrchr(resp,'(');
	portptr += strspn(portptr,"(|");
	/* now we have "1234|)" let atoi do the rest of the job */
	port=atoi(portptr);

	/* lets find out the ip */
	memset(ip,0,sizeof(ip));
	len=sizeof(sin6);
	if (getpeername(ftp->fd,(struct sockaddr*)&sin6,&len)!=0) {
		DEBUG_MSG(DBG_ERROR, "getpeername failed!");
		return FTP_EVENT_FAILURE;
	}

	if(inet_ntop(sin6.sin6_family,&(sin6.sin6_addr),ip,sizeof(ip)) == NULL) {
		DEBUG_MSG(DBG_ERROR, "inet_ntop failed!");
		return FTP_EVENT_FAILURE;
	}
	// DEBUG_MSG(DBG_INFO, "Connecting to %s (%s) on port %d!", ip, sin6.sin6_family == AF_INET6 ? "ipv6" : "ipv4", port);

	ftp_set_server(*dataconn, ip, port);
//	(*dataconn)->fd = connect_to(ip, port);
	if (connect_to(ip, port, ftp_event_callback_pasv_lookup_cb, ftp))
		return FTP_EVENT_FAILURE;

	return FTP_EVENT_SUCCESS;
}
#endif



int ftp_event_retr_nonblock_callback(int, void *);

void
ftp_event_retr_limit_timeout(void *data)
{
	ftp_object *obj = NULL;
        struct ftp_event_cb_info *einfo = NULL;
	int sock, ret;

	if (!(obj = data))
		return;

	sock = (int)ftp_object_get_data(&obj, "sock");
	einfo = ftp_object_get_data(&obj, "einfo");

	medium_cmd(M_IO_WATCH, &ret, sock, IO_WATCH_READ|IO_WATCH_HUP, ftp_event_retr_nonblock_callback, einfo);
	ftp_object_free(&obj);

	return;
}

/*
** handles retrieving the data for downloads.
*/
int
ftp_event_retr_nonblock_callback(int sock, void *data)
{
        struct ftp_event_cb_info *einfo = data;
	_ftp_item *ftp = NULL;
	char buf[MAX_SIZE + 1];
	int ret = 0, file_fd = -1;
	double sleep_time;

	if (!einfo)
		return 0;
	
	ftp = einfo->ftp;
	einfo->estat = ftp_object_get_data(einfo->obj, "ftp_event_estat");
	file_fd = (int)ftp_object_get_data(einfo->obj, "ftp_transfer_file_fd");
	
	ret = recv(sock, buf, MAX_SIZE, 0);
	if (ret < 0)
		DEBUG_MSG(DBG_INFO, "ret < 0: ret: %d, errno: %d EAGAIN: %d", ret, errno, EAGAIN);
	if (ret == 0 || (ret == -1 && errno == EBADF)) {
		/* EOF */
		jtimer *utimer = NULL;
		
		ftp_transfer_end(FTP_TRANSFER(ftp));
		j_free(ftp_object_get_data(FTP_OBJECT(ftp), "ftp_transfer_file"));
		ftp_object_unlink_tag(FTP_OBJECT(ftp), "ftp_transfer_file");
		utimer = ftp_object_get_data(FTP_OBJECT(ftp), "transfer_update_timer");
		utimer = jtimer_kill(utimer);
		ftp_object_unlink_tag(FTP_OBJECT(ftp), "transfer_update_timer");

		if (errno != EBADF)
			close(ftp->dataconn->fd);

		close(file_fd);
		medium_cmd(M_TRFR_UPDATE, NULL, ftp);
		ftp_event_nonblock_finished(ftp, einfo->estat, einfo->estat_index, einfo);
		return 0;
	} else if (ret > 0) {
		/* handle the data. */
		write(file_fd, buf, ret);
		ftp_transfer_update(FTP_TRANSFER(ftp), ret);
		sleep_time = ftp_transfer_get_sleep_time(FTP_TRANSFER(ftp));
		if (sleep_time > 0) {
			int ret;
			ftp_object *obj = NULL;

			ftp_object_set_data(&obj, "sock", (void *)sock);
			ftp_object_set_data(&obj, "einfo", einfo);
			medium_cmd(M_TIMEOUT_ADD, &ret, (int)(sleep_time * 1000), ftp_event_retr_limit_timeout, (void *)obj);
			return 0;
		}
		{
			/* visual update of transfer stats, updated with some interval. */
			jtimer *utimer = NULL;

			if (!(utimer = ftp_object_get_data(FTP_OBJECT(ftp), "transfer_update_timer"))) {
				utimer = jtimer_new();
				ftp_object_set_data(FTP_OBJECT(ftp), "transfer_update_timer", utimer);
				jtimer_start(utimer);
				medium_cmd(M_TRFR_UPDATE, NULL, ftp);
			}

			if (jtimer_get_elapsed_time(utimer) > 1.0) {
				medium_cmd(M_TRFR_UPDATE, NULL, ftp);
				jtimer_start(utimer);
			}
		}
	}

	return 1;
}

FTP_EVENT_CB_CMD(ftp_event_callback_retr)
{
	struct ftp_event_cb_info *einfo = NULL;
	int file_fd = 0;
	struct stat _stat;
	char *file = NULL;
	int trunc = O_TRUNC, state;
	int ret;

	file  = ftp_object_get_data(obj, "file");
	if (ftp == NULL || file == NULL || !ftp->dataconn || code != 150)
		return FTP_EVENT_FAILURE;

	if (ftp->mode == active) {
		ftp->dataconn->fd = accept_dataport(ftp->dataconn->fd);
		if (ftp->dataconn->fd < 0) {
			FTP_MISC_LOG(ftp, "    PORT failed\n");
			return FTP_EVENT_FAILURE;
		}
	}

	state = (int)ftp_object_get_data(obj, "state");
	if (stat(file, &_stat) != -1) {
		if(state == TRFR_RESUME) {
			DEBUG_MSG(DBG_INFO, "retr: [%s] file exists (RESUMING)", file);
			trunc = O_APPEND;
		}
	}

	/* Make sure the directory exists to which we are dling.*/
	local_mkpath(file);
	
	/* Open the file fd. */
	if ( (file_fd = open(file, O_RDWR | O_CREAT | trunc, 0600)) == -1) {
		DEBUG_MSG(DBG_INFO, "retr [%s] file_fd = open.. == -1, file_fd: %d", file, file_fd);
		DEBUG_MSG(DBG_INFO, "errno = %d ENOENT = %d", errno, ENOENT);
		file_fd = -1;
	}

	ftp_object_set_data(FTP_OBJECT(ftp), "ftp_transfer_file", j_strdup(file));

	/* setup transfer information */
	ftp_transfer_setup(FTP_TRANSFER(ftp), FTP_TRANSFER_DOWNLOAD);
	ftp_transfer_set_size(FTP_TRANSFER(ftp), (unsigned int)ftp_object_get_data(obj, "ftp_transfer_size"));
	ftp_transfer_set_resume_position(FTP_TRANSFER(ftp), (unsigned int)ftp_object_get_data(obj, "ftp_restored_bytes"));
	ftp_transfer_set_position(FTP_TRANSFER(ftp), (unsigned int)ftp_object_get_data(obj, "ftp_transferred_bytes"));

	einfo = ftp_event_setup_info_data(ftp,  (int)ftp_object_get_data(obj, "ftp_event_entry"),
						ftp_object_get_data(obj, "ftp_request"), code, status, resp, obj);
		
	//einfo->estat = ftp_object_get_data(obj, "ftp_event_estat");
	
	/* add a callback that will call ftp_event_list_read_callback when there is data available there. */
	if (medium_cmd(M_IO_WATCH, &ret, ftp->dataconn->fd, IO_WATCH_READ|IO_WATCH_HUP, ftp_event_retr_nonblock_callback, einfo)) {
		ftp_object_set_data(obj, "ftp_transfer_file_fd", (void *)file_fd);
		ftp->dataconn->type = CONN_DATA;
		j_free(file);
		return FTP_EVENT_NONBLOCK;
	}

	/* reached only if the medium handler was not called. */
	close(file_fd);
	j_free(file);

	return FTP_EVENT_SUCCESS;
}

/* stor */

int ftp_event_stor_nonblock_callback(int, void *);

void
ftp_event_stor_limit_timeout(void *data)
{
	ftp_object *obj = NULL;
        struct ftp_event_cb_info *einfo = NULL;
	int sock, ret;

	if (!(obj = data))
		return;

	sock = (int)ftp_object_get_data(&obj, "sock");
	einfo = ftp_object_get_data(&obj, "einfo");

	medium_cmd(M_IO_WATCH, &ret, sock, IO_WATCH_READ|IO_WATCH_HUP, ftp_event_stor_nonblock_callback, einfo);
	ftp_object_free(&obj);

	return;
}

int
ftp_event_stor_nonblock_callback(int fd, void *data)
{
        struct ftp_event_cb_info *einfo = data;
	_ftp_item *ftp = NULL;
	char buf[MAX_SIZE + 1];
	int ret, tmp, tmp2, file_fd = -1;
	double sleep_time;
	jtimer *utimer = NULL;

	if (!einfo)
		return 0;
		
	ftp = einfo->ftp;
	einfo->estat = ftp_object_get_data(einfo->obj, "ftp_event_estat");
	file_fd = (int)ftp_object_get_data(einfo->obj, "ftp_transfer_stor_file_fd");
		
	ret = read(file_fd, buf, MAX_SIZE);
	if (ret == 0) {
		/* EOF */
		ftp_transfer_end(FTP_TRANSFER(ftp));
		j_free(ftp_object_get_data(FTP_OBJECT(ftp), "ftp_transfer_file"));
		ftp_object_unlink_tag(FTP_OBJECT(ftp), "ftp_transfer_file");
		utimer = ftp_object_get_data(FTP_OBJECT(ftp), "transfer_update_timer");
		utimer = jtimer_kill(utimer);
		ftp_object_unlink_tag(FTP_OBJECT(ftp), "transfer_update_timer");
		
		close(ftp->dataconn->fd);
		close(file_fd);
		medium_cmd(M_TRFR_UPDATE, NULL, ftp);
		ftp_event_nonblock_finished(ftp, einfo->estat, einfo->estat_index, einfo);
		return 0;
	} else if (ret > 0) {
		tmp = send(fd, buf, ret, 0);
		if (tmp < ret && tmp > 0) {
			/* if not all the data was sent, spin back in the file. */
			tmp2 = tmp;
			tmp -= ret;
			ret = tmp2;
			lseek(file_fd, tmp, SEEK_CUR);
		} else if (tmp < 0) {
			/* EOF (might be due to ABORT). */
			ftp_transfer_end(FTP_TRANSFER(ftp));
			j_free(ftp_object_get_data(FTP_OBJECT(ftp), "ftp_transfer_file"));
			ftp_object_unlink_tag(FTP_OBJECT(ftp), "ftp_transfer_file");

			close(ftp->dataconn->fd);
			close(file_fd);
			medium_cmd(M_TRFR_UPDATE, NULL, ftp);
			ftp_event_nonblock_finished(ftp, einfo->estat, einfo->estat_index, einfo);
			return 0;
		}

		ftp_transfer_update(FTP_TRANSFER(ftp), ret);
		
		sleep_time = ftp_transfer_get_sleep_time(FTP_TRANSFER(ftp));
		if (sleep_time > 0) {
			int ret;
			ftp_object *obj = NULL;

			ftp_object_set_data(&obj, "sock", (void *)fd);
			ftp_object_set_data(&obj, "einfo", einfo);
			medium_cmd(M_TIMEOUT_ADD, &ret, (int)(sleep_time * 1000), ftp_event_stor_limit_timeout, (void *)obj);
			return 0;
		}
		
		/* visual update of transfer stats, updated with some interval. */
		if (!(utimer = ftp_object_get_data(FTP_OBJECT(ftp), "transfer_update_timer"))) {
			utimer = jtimer_new();
			ftp_object_set_data(FTP_OBJECT(ftp), "transfer_update_timer", utimer);
			jtimer_start(utimer);
			medium_cmd(M_TRFR_UPDATE, NULL, ftp);
		}

		if (jtimer_get_elapsed_time(utimer) > 1.0) {
			medium_cmd(M_TRFR_UPDATE, NULL, ftp);
			jtimer_start(utimer);
		}
	}

	return 1;
}

FTP_EVENT_CB_CMD(ftp_event_callback_stor)
{
	struct ftp_event_cb_info *einfo = NULL;
	char *file = NULL;
	struct stat stats;
	int file_fd, ret;
	
	file = ftp_object_get_data(obj, "ftp_transfer_file");
	if (ftp == NULL || file == NULL || !ftp->dataconn || (code != 150))
		return FTP_EVENT_FAILURE;

        if (ftp->mode == active) {
		ftp->dataconn->fd = accept_dataport(ftp->dataconn->fd);
		DEBUG_MSG(DBG_INFO, "ftp_event_callback_stor: ftp->dataconn->fd: %d", ftp->dataconn->fd);

		if (ftp->dataconn->fd < 0) {
			FTP_MISC_LOG(ftp, "    PORT failed\n");
			return FTP_EVENT_FAILURE;
		}
	}

	if (stat(file, &stats) == -1) 
		return FTP_EVENT_FAILURE;

	if ( (file_fd = open(file, O_RDONLY)) == -1) {
		FTP_MISC_LOG(ftp, "    failed to open %s for reading\n", file);
		return FTP_EVENT_FAILURE;
	}
	
	ftp_object_set_data(FTP_OBJECT(ftp), "ftp_transfer_file", j_strdup(file));

	einfo = ftp_event_setup_info_data(ftp,  (int)ftp_object_get_data(obj, "ftp_event_entry"),
				ftp_object_get_data(obj, "ftp_request"), code, status, resp, obj);

	/* add a callback that will call ftp_event_list_read_callback when there is data available there. */
	if (medium_cmd(M_IO_WATCH, &ret, ftp->dataconn->fd, IO_WATCH_WRITE|IO_WATCH_HUP, ftp_event_stor_nonblock_callback, einfo)) {
		ftp_transfer_setup(FTP_TRANSFER(ftp), FTP_TRANSFER_UPLOAD);
		ftp_transfer_set_size(FTP_TRANSFER(ftp), (int)ftp_object_get_data(obj, "ftp_transfer_size"));
		ftp_transfer_set_position(FTP_TRANSFER(ftp), (int)ftp_object_get_data(obj, "ftp_transfer_rest"));
		ftp_transfer_set_resume_position(FTP_TRANSFER(ftp), ftp_transfer_get_position(FTP_TRANSFER(ftp)));

		lseek(file_fd, ftp_transfer_get_position(FTP_TRANSFER(ftp)), SEEK_SET);
		ftp_object_set_data(obj, "ftp_transfer_stor_file_fd", (void *)file_fd);

		j_free(file);
		return FTP_EVENT_NONBLOCK;
	}
	
	/* failure, no medium was found*/
	ftp_transfer_reset(FTP_TRANSFER(ftp));
	close(ftp->dataconn->fd);
	close(file_fd);
	j_free(file);
	medium_cmd(M_TRFR_UPDATE, NULL, ftp);

	return FTP_EVENT_SUCCESS;
}


FTP_EVENT_CB_CMD(ftp_event_irrupt_disconnect)
{
	ftp->state = FTP_STATE_NONE;
	if (!config_read_bool("cache-enable") && config_read_bool("cache-session"))
		cache_free_site(ftp);
	
	// ftp_free(ftp);
		
	return FTP_EVENT_SUCCESS;
}

