/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp_misc.c,v 1.5 2003/02/24 04:10:38 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>
/* local headers */
#include <ftp.h>
#include <error.h>
#include <log.h>
#include <mem.h>


/*
** Parse the output from a PASV (passive mode) request.
** We need to parse the host address and port number that the server will yield on success.
** also converts the server ip and port into a suitable format that can be used elsewhere.
*/
t_ftpinfo *
ftp_parse_pasv_mode(const char *message)
{
	int i;
	char *p = NULL, *pnt = NULL, *ptr = NULL, *h[6];
	t_ftpinfo *info = NULL;

	if ( !(ptr = j_strdup(message)) )
		return NULL;

	/* delimit the (x,x,x,x,x) section, from left to right */
	if ( !(p = strchr (ptr, ',')) )
		return NULL;

	p--;
	while (*p >= '0' && *p <= '9')
		p--;
		
	*(p++) = 0;

	/* delimit the (x,x,x,x,x,x) section, from the right to left. */
	if ( !(pnt = strrchr(p, ',')))
		return NULL;
		
	pnt++;
	while (*pnt >= '0' && *pnt <= '9')
		pnt++;
	
	*pnt = 0;

	if ( !(info = j_malloc(sizeof(t_ftpinfo))) ) {
		return NULL;
	}

	for (i = 0; (h[i + 1] = strchr(p, ',')); i++) {
		*h[i + 1] = 0;
		h[i] = p;
		p = h[i + 1] + 1;
	}

	h[i] = p;

	/* ip: xxx.xxx.xxx.xxx\0 = 3 * 4 + 1 + 3 */
	if (i != 6) {
		if(! (info->ip = j_malloc(3 * 4 + 4)) ) {
			return NULL;
		}
		snprintf(info->ip, 3 * 4 + 4, "%s.%s.%s.%s", h[0], h[1], h[2], h[3]);
		info->port = (atoi(h[4]) << 8) | atoi(h[5]);
		j_free(ptr);
		return info;	/* SUCCESS */
	}

	return NULL;
}

/*
** Return a row specified by the number (will return the n'th row (n: int row))
** If n > no of entries, NULL will be returned.
*/
struct _list *
ftp_list_row(struct _list *head, int row)
{
	register int i;
	struct _list *lpnt = NULL;

	i = 0;  lpnt = head;
	while (lpnt) {
		if (i++ == row)
			return lpnt;
		
		lpnt = lpnt->next;
	}

	return NULL;
}

/*
** Simply frees (deallocates) all elements of a (struct _list) linked list.
*/
void
ftp_free_list(struct _list *list_start)
{
	struct _list *lptr = NULL, *lptr2 = NULL;

	lptr = list_start;
	while (lptr) {
		lptr2 = lptr;
		lptr = lptr->next;
		if (lptr2->file) j_free(lptr2->file);
		if (lptr2->perm) j_free(lptr2->perm);
		if (lptr2->owner) j_free(lptr2->owner);
		if (lptr2->group) j_free(lptr2->group);
		lptr2->file = lptr2->perm = lptr2->owner = lptr2->group = NULL;
		j_free(lptr2);
	}

	return;
}


