/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp_object.c,v 1.15 2003/03/07 17:48:12 thepin Exp $
 */

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>
/* local headers */
#include <ftp_object.h>
#include <defaults.h>
#include <error.h>
#include <mem.h>

int
ftp_object_copy_data(ftp_object **dest, ftp_object **src)
{
	ftp_object **opnt = NULL;

	if (!dest || !src || !(*src))
		return 0;
	
	opnt = src;
	while (*opnt) {
		ftp_object_set_data(dest, (*opnt)->tag, (*opnt)->data);
		opnt = &(*opnt)->next;
	}
   return 1;
}

void
ftp_object_free(ftp_object **root)
{
	ftp_object *pnt = NULL, *pnt2 = NULL;
	
	if (!root || !(*root))
		return;

	pnt = *root;
	while (pnt) {
		pnt2 = pnt;
		pnt = pnt->next;

		/* free the pnt2 entry */
		j_free(pnt2->tag);
		j_free(pnt2);
	}

	return;
}

int
ftp_object_set_data(ftp_object **root, const char *tag, void *data)
{
	ftp_object **opnt = NULL;
	
	if (!tag || !root)
		return 0;

	opnt = root;
	while (*opnt) {
		if (!strcasecmp((*opnt)->tag, tag)) 
			break;
		
		opnt = &(*opnt)->next;
	}

	if ((*opnt) == NULL) {
		if ( ((*opnt) = j_malloc(sizeof(ftp_object))) == NULL) {
			MEM_ERR("malloc", ERR_SHOW);
			return -1;
		}
		
		(*opnt)->next = NULL;
	} else {
		j_free((*opnt)->tag);
	}

	(*opnt)->tag = j_strdup(tag);
	(*opnt)->data = data;

	return 1;
}

int
ftp_object_unlink_tag(ftp_object **root, const char *tag)
{
	ftp_object *opnt = NULL, *prior = NULL;

	if (!root || !(*root) || !tag)
		return 0;

	opnt = *root;
	while (opnt) {		
		if (!strcasecmp(opnt->tag, tag))
			break;

		prior = opnt;
		opnt = opnt->next;
	}

	if (opnt == NULL)
		return 0;	// not found.
	
	// fix the entry before prior to this one.
	if (prior)
		prior->next = opnt->next;
	else {
		// the first entry of *root
		*root = (*root)->next;
		
	}

	// free the entry
	j_free(opnt->tag);
	j_free(opnt);
		
	return 1;
}


void *
ftp_object_get_data(ftp_object **root, const char *tag)
{
	ftp_object *opnt = NULL;

	if (!root || !tag || !(*root))
		return NULL;

	opnt = *root;
	while (opnt) {
		if (!strcasecmp(tag, opnt->tag))
			return opnt->data;
		
		opnt = opnt->next;
	}

	return NULL;
}


