
/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * junkie - gui ftp client
 * Copyright (C) 2002 Gunnsteinn Hall, Joel Thomas, Martin Faxer
 *
 * $Id: gui_dialog.c,v 1.57 2003/03/07 17:48:12 thepin Exp $
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
/* local headers */
#ifdef HAVE_CONFIG_H
#include <configure.h>
#endif
#include <gui.h>
#include <ftp_request.h>
#include <ftp.h>
#include <ftp_cmd.h>
#include <gui_misc.h>
#include <gui_ftp.h>
#include <gui_ftp_callback.h>
#include <gui_ftp_lookup.h>
#include <gui_toolkit.h>
#include <gui_widgets.h>
#include <gui_tree_model.h>
#include <gui_qwin.h>
#include <gui_io.h>
#include <config.h>
#include <misc.h>
#include <error.h>
#include <mem.h>

t_toggle * toggle_new(int *var, int value)
{
	t_toggle *tog = j_malloc(sizeof(t_toggle));
	tog->var = var;
	tog->value = value;
	return tog;
}

void toggle_cb(GtkObject* radio __attribute__((unused)), gpointer func_data)
{
        t_toggle* tog = (t_toggle*)func_data;
        *(tog->var) = tog->value;
}


void check_cb(GtkObject* check, gpointer func_data)
{
		t_toggle *tog = (t_toggle*)func_data;
		*(tog->var) = 
			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check)) ?
			tog->value : 0;
}

/* set up the connect dialog for anonymous mode or user mode */
int set_user_mode(GtkObject *check __attribute__((unused)), gpointer func_data)
{
	GtkWidget **upnt = (GtkWidget **)func_data;
	GtkWidget *loginentry = upnt[0];
	GtkWidget *passwdentry = upnt[1];
	GtkWidget *anon_check = upnt[2];
	int mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(anon_check));
	int n;
	char *defpass, *defuser;

	if (mode == TRUE) {		/* anonymous */
		config_set_bool("anonconn", 1);
		gtk_entry_set_text(GTK_ENTRY(loginentry), "anonymous");
		gtk_editable_set_editable(GTK_EDITABLE(loginentry), FALSE);
		gtk_entry_set_visibility(GTK_ENTRY(passwdentry), TRUE);
		n = config_read_value("anonpass", T_STR, (void **)&defpass);
		if (n != -1) {
			gtk_entry_set_text(GTK_ENTRY(passwdentry), defpass);
			j_free(defpass);
		}
	} else if (mode == FALSE) {	/* user mode */
		config_set_bool("anonconn", 0);
		gtk_entry_set_visibility(GTK_ENTRY(passwdentry), FALSE);
		gtk_entry_set_text(GTK_ENTRY(passwdentry), "");
		gtk_editable_set_editable(GTK_EDITABLE(loginentry), TRUE);
		n = config_read_value("defuser", T_STR, (void **)&defuser);
		if (n != -1) {
			gtk_entry_set_text(GTK_ENTRY(loginentry), defuser);
			j_free(defuser);
		} else {
			gtk_entry_set_text(GTK_ENTRY(loginentry), "");
		}
	}

	return (0);
}


void
gui_dialog_connected_cb(_ftp_item *ftp)
{
	GtkWidget *rtview = NULL;

	DEBUG_MSG(DBG_INFO, "gui_dialog_connected_cb");
	if (!ftp) {
		DEBUG_MSG(DBG_INFO, "ftp not set");
        	gui_connect_button_set_connect();
		return;
	}

	rtview = gui_lookup_widget("right_tview");
	gui_ftp_lookup_add(ftp, rtview, FTP_MODE_FTP, FTP_STATE_CONNECTED);
	gui_tree_model_set(ftp, GTK_TREE_VIEW(rtview));
	gui_qwin_model_set(ftp, GTK_TREE_VIEW(gui_qwin_treeview()));
	gui_connect_update_button();

	
	DEBUG_MSG(DBG_INFO, "sending list...");
	gui_ftp_list(ftp, gui_ftp_callback_list, 0, NULL, "widget", rtview);
	DEBUG_MSG(DBG_INFO, "done with this stuff");

	return;
}

int
gui_connect_dialog()
{
	gint result;
	gchar *e_host = FTP_LOCALHOST;
	gchar *e_login = FTP_ANON_LOGIN;
	gchar *e_passwd = FTP_ANON_PASSWD;
	gchar *e_port = "21";
	GtkAdjustment *adj;
	GtkWidget *dialog = 0, *hbox = 0, *vbox;
	GtkWidget *label = 0;
	GtkWidget *hostentry = 0;
	GtkWidget *portentry = 0, *loginentry = 0, 
	          *passwdentry = 0;
// GtkWidget *rtview = NULL;
	GtkWidget *radio[2];
	GtkWidget *anon_check;
	GtkWidget *upnt[3];
	GSList *grp_mode = NULL;
	int mode = config_read_bool("usepassive");
	t_toggle *stoggle1, *stoggle2;
//	_ftp_item *ftp = 0;
	/*ftp_request *req = 0;*/

	/* update the connect button. */
	gui_connect_button_set_disconnect();
	
	dialog = gtk_dialog_new_with_buttons(
		"Connect to:",
		NULL,
		GTK_DIALOG_MODAL,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
		NULL);

	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
	gtk_container_set_border_width(GTK_CONTAINER(dialog), 2);
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 5);

	hbox = gtk_hbox_new(FALSE, 5);
	label = gtk_label_new("  Remote Host (localhost):");
	hostentry = gtk_entry_new_with_max_length(30);
	gtk_widget_set_size_request(hostentry, 110, -1);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), hostentry, FALSE, FALSE, 0);

	label = gtk_label_new("  Port (21):");
	adj = (GtkAdjustment *) gtk_adjustment_new (21.0, 0.0, 65536.0, 1.0, 100.0, 0.0);
	portentry = gtk_spin_button_new (adj, 0, 0);
	gtk_widget_set_size_request(portentry, 60, -1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(portentry),
	    (gdouble)config_read_num("defport"));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), portentry);
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox);

	hbox = gtk_hbox_new(FALSE, 5);
	
	vbox = gtk_vbox_new(FALSE, 5);
	label = gtk_label_new("User name (anonymous):        ");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
	label = gtk_label_new("  Password (junkie@developers.org):");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
	
	vbox = gtk_vbox_new(FALSE, 5);
	loginentry = gtk_entry_new_with_max_length(20);
	gtk_widget_set_size_request(loginentry, 80, -1);
	gtk_box_pack_start(GTK_BOX(vbox), loginentry, FALSE, FALSE, 0);
	passwdentry = gtk_entry_new_with_max_length(20);
	gtk_widget_set_size_request(passwdentry, 80, -1);
	gtk_box_pack_start(GTK_BOX(vbox), passwdentry, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);

	anon_check = gtk_check_button_new_with_label("Anonymous connection?");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(anon_check),
	    config_read_bool("anonconn") ? TRUE : FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), anon_check, FALSE, FALSE, 0);
	upnt[0] = loginentry;
	upnt[1] = passwdentry;
	upnt[2] = anon_check;
	gtk_signal_connect(GTK_OBJECT(anon_check), "toggled",
	    G_CALLBACK(set_user_mode), upnt);

	set_user_mode(GTK_OBJECT(anon_check), upnt);

	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 0);

	stoggle1 = toggle_new(&mode, passive);
	radio[0] = gtk_radio_button_new_with_label(grp_mode, "Passive mode (PASV)");
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), radio[0]);
	grp_mode = gtk_radio_button_group(GTK_RADIO_BUTTON(radio[0]));
	gtk_signal_connect(GTK_OBJECT(radio[0]), "toggled",
				G_CALLBACK(toggle_cb), stoggle1);
	stoggle2 = toggle_new(&mode, active);
	radio[1] = gtk_radio_button_new_with_label(grp_mode, "Active mode (ACTV)");
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), radio[1]);
	/*grp_mode = gtk_radio_button_group(GTK_RADIO_BUTTON(radio[1]));*/
	gtk_signal_connect(GTK_OBJECT(radio[1]),"toggled",
				G_CALLBACK(toggle_cb), stoggle2);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mode == passive ? radio[0] : radio[1]), 1);

	gtk_widget_grab_focus(hostentry);
	
	g_signal_connect(G_OBJECT(dialog), "key_press_event",
			G_CALLBACK(gui_dialog_connect_commit), dialog);
	
	gtk_widget_show_all(dialog);
	
	if((result = gtk_dialog_run(GTK_DIALOG(dialog))) == GTK_RESPONSE_ACCEPT) {
		e_host      = g_strdup((gchar*)gtk_entry_get_text(GTK_ENTRY(hostentry)));
		e_login     = (gui_convert_locale(gtk_entry_get_text(GTK_ENTRY(loginentry))));
		e_passwd    = (gui_convert_locale(gtk_entry_get_text(GTK_ENTRY(passwdentry))));
		e_port      = g_strdup((gchar*)gtk_entry_get_text(GTK_ENTRY(portentry)));
	}
	j_free(stoggle1); j_free(stoggle2);
	gtk_widget_destroy(dialog);

	if (result != GTK_RESPONSE_ACCEPT) {
		gui_connect_button_set_connect();
		return 0;
	}

	if (!gui_ftp_connect((*e_host ? e_host : FTP_LOCALHOST), (*e_port ? atol(e_port) : 21), mode,
			      (*e_login ? e_login : FTP_ANON_LOGIN), (*e_passwd ? e_passwd : FTP_ANON_PASSWD),
			      gui_dialog_connected_cb)) {
		gui_connect_button_set_connect();
	}

	g_free(e_host);
	g_free(e_login);
	g_free(e_passwd);
	g_free(e_port);

	return 0;
}

int 
gui_dialog_connect_commit(GtkWidget *widget __attribute__((unused)), 
		                  GdkEventKey *event, gpointer data )
{
	GtkDialog *dialog = data;
	if(event->keyval != 65293)	return FALSE;
	gtk_dialog_response(dialog, GTK_RESPONSE_ACCEPT);
	return FALSE;
}

void
gui_about_dialog()
{
	GtkWidget *dialog;
	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget  *logo;
   
	dialog = gtk_dialog_new_with_buttons(
		"About us:", NULL, GTK_DIALOG_MODAL,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL, NULL, NULL);

	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
	gtk_container_set_border_width(GTK_CONTAINER(dialog), 2);
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 5);
	label = gtk_label_new(	
            		"Junkie FTP (version " VERSION "), Copyright (C) 2002\n");
	
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), label);
	hbox = gtk_hbox_new(FALSE, 0);
	label = gtk_label_new(
				"  Programmers:\n"
				"        Gunnsteinn Hall\n"
				"        Joel Thomas\n"
				"        Martin Faxer\n\n"
				"  Contributors:\n"
				"        Mitja Horvat\n"
				"        Andreas Henriksson\n"
				"        Droo <Drood1e@yahoo.fr>\n\n"
				"  Music:\n"
				"        Tomasz Gucio <gustavo@wpk.p.lodz.pl>\n\n"
				"  Special Greetings:\n"
				"        Antti Kuusisto (wisekki@IRCnet)\n\n");
	
	gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
	logo = gtk_image_new_from_file(DATADIR "/junkie/" "logo.png");
	gtk_box_pack_start_defaults(GTK_BOX(hbox), logo);
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox);
	label = gtk_label_new(
        		 "You may contact us @ \"joel.thomas@t-online.de\"");
	
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), label);
   
	//gtk_misc_set_alignment(GTK_MISC(label), 0, 1.0);

	gtk_widget_show_all(dialog);

	gtk_dialog_run(GTK_DIALOG(dialog));

	gtk_widget_destroy(dialog);
	return;
}

#ifdef FUN
void
gui_yomomma_dialog(void) {
	GtkWidget *WYoMomma = NULL;
	GtkWidget *LYoMomma = NULL;
	GtkWidget *next = NULL;

	WYoMomma = gtk_dialog_new_with_buttons(
		"funny joke m4n", NULL, GTK_DIALOG_MODAL,
		NULL);

	LYoMomma = gtk_label_new(NULL);
	gui_yomomma_set_label(0, LYoMomma);
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(WYoMomma)->vbox), LYoMomma);
	next = gtk_button_new_with_label("Next Yo Momma");
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(WYoMomma)->vbox), next);
	
	g_signal_connect (G_OBJECT (next), "clicked", 
			G_CALLBACK (gui_yomomma_set_label), LYoMomma);

	gtk_widget_show_all(WYoMomma);

	if (gtk_dialog_run(GTK_DIALOG(WYoMomma)))
		gtk_widget_destroy(WYoMomma);

	return;
}

void
gui_yomomma_set_label(GtkButton *but __attribute__((unused)), GtkWidget *label)
{
	int rnd;
	char buf[GOOD_SIZE];
	FILE *fhd = NULL;

	if ( (fhd = fopen(DATADIR "/junkie/" "YoMomma", "r")) == NULL)
		return;

	rnd = random() % 197;	// XXX 197???

	while (rnd--)
		fgets(buf, GOOD_SIZE, fhd);

	fclose(fhd);
 
	gtk_label_set_text(GTK_LABEL(label), buf);
	return;
}

void 
gui_cute_dialog(void)
{
	GtkWidget *win = NULL;

//	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
//	gtk_window_set_title(GTK_WINDOW(win), "cute things m4n");
	win = gui_setup_popup_window("cute things m4n", 0, 1);
	
	gui_cute_set_img(win, NULL);

//	gtk_window_set_policy(GTK_WINDOW(win), FALSE, FALSE, TRUE);
//	gtk_window_set_resizable(GTK_WINDOW(win), FALSE);
	gtk_widget_set_events(win, GDK_BUTTON_PRESS_MASK);
	g_signal_connect(G_OBJECT(win), "button_press_event",
	                 G_CALLBACK(gui_cute_set_img), NULL);
	gtk_widget_show_all(win);

}

void
gui_cute_set_img(GtkWidget *win, GdkEventButton *event)
{
	char *images[] = {	DATADIR "/junkie/cute/" "coco.jpg", 
						DATADIR "/junkie/cute/" "chubb.jpg",
						DATADIR "/junkie/cute/" "cute-baby.jpg",
						DATADIR "/junkie/cute/" "ed.jpg",
						DATADIR "/junkie/cute/" "pickles.jpg",
						DATADIR "/junkie/cute/" "sleepyone.jpg",
						DATADIR "/junkie/cute/" "tubbercat.jpg"};
	int rnd;
	GtkWidget *img = NULL;
	GList *children = NULL;
//	GtkRequisition req;

	if (event == NULL || event->button == 1) {
		rnd = random() % 7;
		if (rnd > 6)
			rnd = 3;
		if( (children = gtk_container_get_children(GTK_CONTAINER(win))) )
			gtk_container_remove(GTK_CONTAINER(win), children->data);
		img = gtk_image_new_from_file(images[rnd]);
		gtk_container_add(GTK_CONTAINER(win), img);
		gtk_widget_show(img);
	//	gtk_widget_size_request(win, &req);
	//	gtk_window_resize(GTK_WINDOW(win), req.width, req.height);
	} else if (event != NULL && event->button == 3) {
		/* in case of the right mouse button, kill the window */
		gtk_widget_destroy(win);
	}
	return;
}
#endif

int
gui_dialog_file_exist(char *file)
{
	int cval;
	char *lab;
	GtkWidget *vbox, *label;
	GtkWidget *options, *menu, *menu_item;
	GtkWidget *dialog = gtk_dialog_new_with_buttons(
			"file exists", NULL, GTK_DIALOG_MODAL,
			GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	vbox = GTK_DIALOG(dialog)->vbox;
	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
	 gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
	gtk_container_set_border_width(GTK_CONTAINER(dialog), 2);
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 5);
	

	
	lab = j_asprintf("%s already exists.\nselect action:", file);
	label = gtk_label_new(lab);
	j_free(lab);

	menu = gtk_menu_new();
	menu_item = gtk_menu_item_new_with_label("skip");
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	menu_item = gtk_menu_item_new_with_label("resume");
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	menu_item = gtk_menu_item_new_with_label("overwrite");
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	options = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(options), menu);
	

	gtk_box_pack_start(GTK_BOX(vbox), label,   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), options, FALSE, FALSE, 0);

	gtk_widget_show_all(vbox);
	
	gtk_dialog_run(GTK_DIALOG(dialog));
	cval = gtk_option_menu_get_history(GTK_OPTION_MENU(options));
	gtk_widget_destroy(dialog);
	
	return cval == 2 ? TRFR_OVERWRT : cval == 1 ? TRFR_RESUME : TRFR_NOACT;
}
