/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_dnd.c,v 1.7 2003/03/07 17:48:12 thepin Exp $
 */

#define _GNU_SOURCE

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* local headers */
#include <gui_dnd.h>
#include <misc.h>
#include <error.h>
#include <mem.h>
	
dnd_item * 
gui_dnd_item_new() 
{
	dnd_item *item = j_malloc(sizeof(dnd_item));
	if(!item) {
		MEM_ERR("couldn't alloc item", ERR_SHOW);
		return NULL;
	}

	item->begin_cb	      = NULL;
	item->end_cb          = NULL;
	item->data_delete_cb  = NULL;
	item->motion_cb       = NULL;
	item->data_request_cb = NULL;
	return item;
}

dnd_actions *
gui_dnd_actions_new()
{
	dnd_actions *item = j_malloc(sizeof(dnd_actions));
	item->actions_cb = NULL;
	return item;
}

void
gui_dnd_setup(GtkWidget *widget,
		        gpointer drag_begin_cb,
				  gpointer drag_end_cb,
				  gpointer drag_delete_cb,
				  gpointer drag_data_cb,
				  gpointer drag_motion_cb)
{
	dnd_item *dnd;
	dnd = g_object_get_data(G_OBJECT(widget), "dnd");

	if(drag_begin_cb)		dnd->begin_cb        = drag_begin_cb;
	if(drag_end_cb)		dnd->end_cb          = drag_end_cb;
	if(drag_delete_cb)   dnd->data_delete_cb  = drag_delete_cb;
	if(drag_data_cb)		dnd->data_request_cb = drag_data_cb;

	if(drag_motion_cb)	dnd->motion_cb       = drag_motion_cb;
}

void
gui_dnd_enable(GtkWidget *widget)
{
	dnd_item *dnd;
	GtkTargetEntry target_entry[3];
	target_entry[0].target = DRAG_TAR_NAME_0;
	target_entry[0].flags  = 0;
	target_entry[0].info   = DRAG_TAR_INFO_0;
	target_entry[1].target = DRAG_TAR_NAME_1;
	target_entry[1].flags  = 0;
	target_entry[1].info   = DRAG_TAR_INFO_1;
	target_entry[2].target = DRAG_TAR_NAME_2;
	target_entry[2].flags  = 0;
	target_entry[2].info   = DRAG_TAR_INFO_2;
	
	dnd = gui_dnd_item_new();
	if(!dnd)
		return;
	g_object_set_data(G_OBJECT(widget), "dnd", dnd);
	
	gtk_widget_realize(widget);

	gtk_drag_dest_set(widget,
							  GTK_DEST_DEFAULT_MOTION    |
							  GTK_DEST_DEFAULT_HIGHLIGHT |
							  GTK_DEST_DEFAULT_DROP,
							  target_entry,
							  sizeof(target_entry)/sizeof(GtkTargetEntry),
							  GDK_ACTION_MOVE | GDK_ACTION_COPY);
	gtk_drag_source_set(widget,
							  GDK_BUTTON1_MASK,
							  target_entry,
							  sizeof(target_entry)/sizeof(GtkTargetEntry),
							  GDK_ACTION_MOVE | GDK_ACTION_COPY);
	
	gtk_signal_connect(GTK_OBJECT(widget), "drag_motion",
							 GTK_SIGNAL_FUNC(gui_dnd_drag_motion_cb),        dnd);
	gtk_signal_connect(GTK_OBJECT(widget),"drag_begin", 
							 GTK_SIGNAL_FUNC(gui_dnd_drag_begin_cb),         dnd);
	gtk_signal_connect(GTK_OBJECT(widget),"drag_end",
							 GTK_SIGNAL_FUNC(gui_dnd_drag_end_cb),           dnd);
	gtk_signal_connect(GTK_OBJECT(widget),"drag_data_get",
							 GTK_SIGNAL_FUNC(gui_dnd_drag_data_request_cb),  dnd);
	gtk_signal_connect(GTK_OBJECT(widget),"drag_data_received",
							 GTK_SIGNAL_FUNC(gui_dnd_drag_data_received_cb), dnd);
	gtk_signal_connect(GTK_OBJECT(widget),"drag_data_delete",
							 GTK_SIGNAL_FUNC(gui_dnd_drag_data_delete_cb),   dnd);
}

void
gui_dnd_interaction(GtkWidget *source, GtkWidget *target,
						  gpointer drag_action_cb, GdkDragAction action)
{
	char index[15];
	dnd_item *tgt;
	dnd_actions *actions;
	tgt = g_object_get_data(G_OBJECT(target), "dnd");
	
	actions = gui_dnd_actions_new();
	actions->action     = action;
	actions->actions_cb = drag_action_cb;
// XXX sNprintf?
	sprintf(index, "%p", source);
	g_dataset_set_data(tgt, index, actions);
}



void
gui_dnd_drag_begin_cb(GtkWidget *widget, GdkDragContext *dc, gpointer data)
{
	dnd_item *dnd = data;
	if(dnd->begin_cb)	dnd->begin_cb(widget, dc, dnd);
}

void
gui_dnd_drag_end_cb(GtkWidget *widget, GdkDragContext *dc, gpointer data)
{
	dnd_item *dnd = data;
	if(dnd->end_cb) dnd->end_cb(widget, dc, dnd);
}

gboolean
gui_dnd_drag_motion_cb(GtkWidget *widget, GdkDragContext *dc,
							  gint x, gint y, guint t, gpointer data)
{
	GtkWidget *source;
	GdkDragAction action;
	dnd_item *dnd;
	dnd_actions *actions;
	char index[15];

	source = gtk_drag_get_source_widget(dc);
	dnd = data;
	
	/* set action */
// XXX sNprintf?
	sprintf(index, "%p", source);
	actions = g_dataset_get_data(dnd, index);
	
	if(actions)	action = actions->action;
	else        action = 0;
	
	
	
	if(dnd->motion_cb)	
		if(!dnd->motion_cb(widget, dc, x, y, t, dnd))
			action = 0;

	gdk_drag_status(dc, action, t);
	
	return TRUE;
}

void
gui_dnd_drag_data_request_cb(GtkWidget *widget, GdkDragContext *dc,
									  GtkSelectionData *selectdata,
									  guint info, guint t, gpointer data)
{
	char *hack;
	dnd_item *dnd = data;

	/* send out the data */
	/* XXX evil hack XXX */
	hack = j_asprintf("%d", dnd->data_request_cb ? 
			(int)dnd->data_request_cb(widget, dc, selectdata, info, t, dnd) : 0);
	gtk_selection_data_set(selectdata, GDK_SELECTION_TYPE_STRING, 8, /* bits */
								  hack, strlen(hack));
}

void 
gui_dnd_drag_data_received_cb(GtkWidget *widget, GdkDragContext *dc,
										gint x, gint y,
										GtkSelectionData *selectdata,
										guint info __attribute__((unused)), 
										guint t, gpointer data)
{
	GtkWidget *source;
	dnd_item *dnd;
	dnd_actions *actions;
	void *ptr;
	char index[15];
	cb func;
	
	dnd = data;
	ptr = (void *)atol(selectdata->data);
	
	source = gtk_drag_get_source_widget(dc);
// XXX sNprintf?
	sprintf(index, "%p", source);
	actions = g_dataset_get_data(dnd, index);
	func = actions->actions_cb;
	if(func)	func(widget, dc, x, y, t, ptr);
}

void
gui_dnd_drag_data_delete_cb(GtkWidget *widget, GdkDragContext *dc,
									  gpointer data)
{
	dnd_item *dnd = data;
	if(dnd->data_delete_cb)	dnd->data_delete_cb(widget, dc, dnd);
}
	
	
