/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_dnd_tview.c,v 1.13 2003/03/07 17:48:12 thepin Exp $
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* local headers */
#include <gui_dnd.h>
#include <gui_dnd_tview.h>
#include <gui_widgets.h>
#include <gui_toolkit.h>
#include <gui_treestore.h>
#include <mem.h>

dnd_tview_item *
gui_dnd_tview_item_new()
{
	dnd_tview_item *item = j_malloc(sizeof(dnd_tview_item));
	memset(item, 0, sizeof(dnd_tview_item));
	return item;
}

void
gui_dnd_tview_enable(GtkTreeView *tview)
{
	GtkTreeModel   *model;
	dnd_tview_item *dnd;
	
	gui_dnd_enable(GTK_WIDGET(tview));
	
	dnd = gui_dnd_tview_item_new();
	g_object_set_data(G_OBJECT(tview), "dnd_tview", dnd);
   model = gtk_tree_view_get_model(tview);
	g_object_set_data(G_OBJECT(model), "view", tview);
	
	gui_dnd_setup(GTK_WIDGET(tview),
					  gui_dnd_tview_begin_cb,
					  NULL,
					  NULL,
					  gui_dnd_tview_data_request_cb,
					  gui_dnd_tview_motion_cb);
}



void
gui_dnd_tview_scroll_set(GtkTreeView *tview, gboolean b)
{
	dnd_tview_item* dnd = g_object_get_data(G_OBJECT(tview), "dnd_tview");
	dnd->scroll = b;
}

void
gui_dnd_tview_highlight_add(GtkTreeView *tview, int id, char *c, int mode)
{
	GdkColor *color = j_malloc(sizeof(GdkColor));
	
	dnd_tview_item* dnd = g_object_get_data(G_OBJECT(tview), "dnd_tview");
	
	gdk_color_parse(c, color);
	gdk_colormap_alloc_color(gdk_colormap_get_system(), color, 0, 0);
	dnd->hilights[id] = color;
	dnd->hilight[id] = mode;
}

void
gui_dnd_tview_interaction(GtkTreeView *source, GtkTreeView *target,
								  gpointer drag_action_cb, GdkDragAction action)
{
	gui_dnd_interaction(GTK_WIDGET(source), GTK_WIDGET(target), 
							  drag_action_cb, action);
}



void
gui_dnd_tview_begin_cb(GtkWidget *widget, GdkDragContext *dc, dnd_item *dnd)
{
	GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));	
	gtk_tree_selection_selected_foreach(selection,
			(GtkTreeSelectionForeachFunc)gui_dnd_tview_begin_selected, dc);
}

void
gui_dnd_tview_begin_selected(GtkTreeModel *model, 
		                       GtkTreePath *path, 
									  GtkTreeIter *iter,
									  gpointer data)
{

	GdkDragContext *dc = data;
	GdkPixmap *pixmap = NULL;
	GdkBitmap *mask   = NULL;
	GdkPixbuf *icon   = NULL;
	int w = 16, h = 16;

	gtk_tree_model_get(model, iter, 0, &icon, -1);
	gdk_pixbuf_render_pixmap_and_mask(icon, &pixmap, &mask, 32 );
	gdk_window_get_size((GdkWindow *)pixmap, &w, &h);
	gtk_drag_set_icon_pixmap(dc, gdk_colormap_get_system(),
									 pixmap, mask, w/2, h/2);
	//	gtk_drag_set_icon_default(dc);
}

int
gui_dnd_tview_motion_cb(GtkWidget *widget, GdkDragContext *dc,
								gint x, gint y, guint t, dnd_item *dnd)
{
	dnd_tview_item *dnd_tview;
	t_item *abs;
	GdkRectangle rect, rectw;
	GdkWindow *window;
	GtkTreePath *path = NULL;
	GdkColor *color;
	GdkColor white;
	GdkGC *gc = NULL;
	void *list;
	dnd_row dndrow;
	int *indices, depth;
	static int removed;
	static int row,    rowindex;
	static int rowold, rowoldindex, typeold;
	char prow[10];
	
	dnd_tview = g_object_get_data(G_OBJECT(widget), "dnd_tview");
	
	gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(widget), &rect);
	
	if( (y < 15) || (((rect.height -y) < 3) && (y = 70)) )
		if(dnd_tview->scroll)
			gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
													x, y, &path, NULL, 0, 0);
	if(path) {
		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(widget),
											  path, NULL, TRUE, 0.1, 0);
		g_free(path);
	}
	else {
		
		gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
				x, y, &path, NULL, 0, 0);
		if(!path)	return 1;
		
		gtk_tree_path_prev(path);
		depth   = gtk_tree_path_get_depth(path);
		indices = gtk_tree_path_get_indices(path);
		dndrow.depth = depth;
		dndrow.rows  = indices;
		list = gui_treestore_get_list_row(gtk_tree_view_get_model(GTK_TREE_VIEW(widget)), &dndrow);
		if(!list)	return 0;
		row = indices[0];
		abs = (t_item *) list;
		if(!dnd_tview->hilight)	return 1;
		if(!dnd_tview->hilight[abs->type]) {	
			if(removed)	return 1;
			else	removed = 0;
		}

		window = gtk_tree_view_get_bin_window(GTK_TREE_VIEW(widget));
		gc = gdk_gc_new(window);
		
		color = dnd_tview->hilights[abs->type];
		if(color) {
			gdk_gc_set_foreground(gc, color);
			
			gtk_tree_view_get_cell_area(GTK_TREE_VIEW(widget),
				path, gtk_tree_view_get_column(GTK_TREE_VIEW(widget), 0), &rectw);
			
			if     (dnd_tview->hilight[abs->type] == DNDTVIEW_HIRECT)
				gdk_draw_rectangle(window, gc, 0, 
									    0, rectw.y, rect.width, rectw.height);
			else if(dnd_tview->hilight[abs->type] == DNDTVIEW_HILINE) {
			
				if(y > (rectw.y + rectw.height / 2))	rowindex = 1;
				else                                   rowindex = 0;
				gdk_draw_line(window, gc, 
						0,          rowindex ? rectw.y + rectw.height : rectw.y, 
						rect.width, rowindex ? rectw.y + rectw.height : rectw.y);
			}
		}
		
		/* check if we have to erase the last hilight */
		if(rowold != row || rowoldindex != rowindex) {
			sprintf(prow, "%d", rowold);
			path = gtk_tree_path_new_from_string(prow);
			gdk_color_parse("white", &white);
			gdk_colormap_alloc_color(gdk_colormap_get_system(), &white, 0, 0);
			gdk_gc_set_foreground(gc, &white);
			
			gtk_tree_view_get_cell_area(GTK_TREE_VIEW(widget),
				path, gtk_tree_view_get_column(GTK_TREE_VIEW(widget), 0), &rectw);
			
			if     (typeold == DNDTVIEW_HIRECT)
				gdk_draw_rectangle(window, gc, 0,
				    	             0, rectw.y, rect.width, rectw.height);

			else if(typeold == DNDTVIEW_HILINE)
				gdk_draw_line(window, gc,
						0,          rowoldindex ? rectw.y + rectw.height : rectw.y,
						rect.width, rowoldindex ? rectw.y + rectw.height : rectw.y);

			typeold     = dnd_tview->hilight[abs->type];
			rowold      = row;
			rowoldindex = rowindex;
		}
	}
	return 1;
}


void *
gui_dnd_tview_data_request_cb(GtkWidget *widget, GdkDragContext *dc,
									   GtkSelectionData *selectdata,
										guint info, guint t, dnd_item *dnd)
{
	GtkTreeSelection *selection;
	dnd_tview_item *dnd_tview;

	dnd_tview  = g_object_get_data(G_OBJECT(widget), "dnd_tview");
	g_slist_free(g_object_get_data(G_OBJECT(widget), "data_request"));
	g_object_set_data(G_OBJECT(widget), "data_request", NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
	gtk_tree_selection_selected_foreach(selection,
			(GtkTreeSelectionForeachFunc)gui_dnd_tview_data_request_selected,
			dnd_tview);
	return g_object_get_data(G_OBJECT(widget), "data_request");
}

void
gui_dnd_tview_data_request_selected(GtkTreeModel *model,
		                              GtkTreePath *path,
												GtkTreeIter *iter,
												gpointer data)
{
	GtkWidget *widget;
	GSList *glist = NULL;
	void *list;
	dnd_row dndrow;
	gint *indices, depth;
	widget = g_object_get_data(G_OBJECT(model), "view");
	glist = g_object_get_data(G_OBJECT(widget), "data_request");
	
	depth   = gtk_tree_path_get_depth(path);
	indices = gtk_tree_path_get_indices(path);
	dndrow.depth = depth;
	dndrow.rows  = indices;
	list = gui_treestore_get_list_row(gtk_tree_view_get_model(GTK_TREE_VIEW(widget)), &dndrow);
	DEBUG_MSG(DBG_INFO, "adding to the request list: %p", list);
	if(!list)
		return;
	g_object_set_data(G_OBJECT(widget), "data_request", 
			            g_slist_append(glist, list));
}


	
