/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_ftp.c,v 1.35 2003/03/07 17:48:12 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
/* local headers */
#include <ftp_event.h>
#include <ftp_event_callback.h>
#include <gui.h>
#include <gui_widgets.h>
#include <gui_ftp_callback.h>
#include <gui_ftp_lookup.h>
#include <gui_statusbar.h>
#include <gui_paths.h>
#include <gui_logwin.h>
#include <gui_io.h>
#include <gui_timeout.h>
#include <ftp_request.h>
#include <ftp_event.h>
#include <ftp.h>
#include <ftp_cmd.h>
#include <ftp_net.h>
#include <cache.h>
#include <config.h>
#include <anti_idler.h>
#include <gui_anti_idler.h>
#include <medium.h>
#include <error.h>
#include <log.h>
#include <local.h>
#include <mem.h>
#include <defaults.h>

int gui_ftp_uninit(_ftp_item **);
extern int connected;

gboolean
gui_ftp_event_handler(GIOChannel *source, GIOCondition condition, gpointer data)
{
	if (condition & G_IO_IN) 
		ftp_handle_events((data ? (_ftp_item **)&data : NULL));

	if (condition & G_IO_OUT) 
		ftp_request_iteration(data);
	
	return TRUE;
}


int
gui_ftp_setup_events(_ftp_item *ftp)
{
	ftp_event_add_handler(ftp, FTP_EVENT_IRRUPT, (void *)FTP_EVENT_IRRUPT_QUIT, gui_ftp_irrupt_cb_disconnect);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "USER", gui_ftp_event_user_sbar_logging_in);
	ftp_event_add_handler(ftp, FTP_EVENT_CMD, "PASS", gui_ftp_event_pass_sbar_logged_in);
	return 1;
}

/*
** Setup the medium functions for the GUI.
*/
void
gui_medium_setup(void)
{
	medium_set_function(M_PATHS_UPDATE, gui_medium_paths_text);
	medium_set_function(M_SELECT_WRITE, gui_io_select_write);
	medium_set_function(M_TRFR_UPDATE, gui_medium_statusbar_trfr_update);
	medium_set_function(M_SELECT_WAIT, gui_io_select_wait);
	medium_set_function(M_TIMEOUT_ADD, gui_timeout_add);
	medium_set_function(M_TIMEOUT_REMOVE, gui_timeout_remove);
	medium_set_function(M_IO_WATCH, gui_io_watch);
	return;
}

int
gui_ftp_setup(_ftp_item *ftp, int setup)
{
	int ior;

	DEBUG_MSG(DBG_INFO, "gui_ftp_setup, ftp, %p (fd: %d), setup: %d", ftp, ftp->fd, setup);
	ior = (int)ftp_object_get_data(FTP_OBJECT(ftp), "gui_ftp_event_ior");
	if (setup == 1) {
		if (ior == 0) {
			GIOChannel *chan = g_io_channel_unix_new(ftp->fd);
			DEBUG_MSG(DBG_INFO, "ftp: %p &: %p", ftp, &ftp);
			DEBUG_MSG(DBG_INFO, "setting up gui_ftp_setup, ftp->fd is currently: %d", ftp->fd);
			ior = g_io_add_watch(chan, G_IO_IN, gui_ftp_event_handler, ftp);
			ftp_object_set_data(FTP_OBJECT(ftp), "gui_ftp_event_ior", (void *)ior);
			g_io_channel_unref(chan);
		}
	} else if (ior > 0) {
		DEBUG_MSG(DBG_INFO, "deinitializing the ior");
		g_source_remove(ior);
		ftp_object_unlink_tag(FTP_OBJECT(ftp), "gui_ftp_event_ior");
	}

	return 1;
}


int
gui_ftp_setup_write(_ftp_item *ftp, int setup)
{
	int dide;
	
	dide = (int)ftp_object_get_data(FTP_OBJECT(ftp), "gui_ftp_event_write_dide");	
	if (setup == 1 && dide == 0) {
		if (ftp->fd > 0) {
			GIOChannel *chan = g_io_channel_unix_new(ftp->fd);
			dide = g_io_add_watch(chan, G_IO_OUT, gui_ftp_event_handler, ftp);
 			ftp_object_set_data(FTP_OBJECT(ftp), "gui_ftp_event_write_dide", (void *)dide);
			g_io_channel_unref(chan);
		}
	} else if (dide != 0 && setup == 0) {
		/* unsetup id */
		g_source_remove(dide);
		ftp_object_unlink_tag(FTP_OBJECT(ftp), "gui_ftp_event_write_dide");
	}
	
	return 1;
}

void
gui_ftp_connected_cb(_ftp_item *ftp, void *data)
{
	cb callback;
	ftp_request *req=NULL;

	DEBUG_MSG(DBG_INFO, "gui_ftp_connected_cb");
	callback = data;
	if (!ftp) {
		callback(NULL);
		return;
	}
		
	/* setup log stuff. */
	gui_log_ftp_setup_buffer(ftp);
	gui_log_ftp_show_buffer(ftp);

	/* Will display that we are connected. */
	gui_statusbar_update_connect(1);

	/* Next step is to log in. */
	if (!(req = ftp_login(ftp))) {
		if (callback)
			callback(NULL);
			
		return;
	}

	/* will show, logging into site on the statusbar. */
	gui_statusbar_update_connect(2);

	/*
	** The default (gui ftp) event handler for USER will display logging in, on the statusbar.
	** The default (gui ftp) event handler for PASS will then handle displaying on the statusbar: logged in on success while 
	** emptying it on failure.
	*/

	/* send a PWD request after logging in, if login is successful. */
	ftp_pwd_with_req(ftp, req);

	/* setup the gtk socket handlers.  They'll work with the ftplib. */
	gui_ftp_setup(ftp, 1);
	gui_ftp_setup_events(ftp);

	/* if the anti idler is enabled in the config file, enable it. */
	if (config_read_num("anti-idler"))
		gui_anti_idler_toggle(ftp, TRUE);		

	DEBUG_MSG(DBG_INFO, "eof gui_ftp_conenct");
	if (callback)
		callback(ftp);

	return;
}

/*
** Connects to an ftp server.
** Also adds a login request and will send a pwd if the login was successful.
*/
_ftp_item *
gui_ftp_connect(const char *host, int port, int mode, const char *user, const char *passwd, cb callback)
{
	/*
	** Simply connect, ftp_connect should handle any error messages.
	*/	
	return ftp_connect(host, port, mode, user, passwd, gui_ftp_connected_cb, (void *)callback);
}


/*
** GUI FTP mkdir frontend to the ftplib, allows the user to send a request and associate one data object with it
** with a single command.
** Safes some writing in other parts of the gui. :)
*/
ftp_request *
gui_ftp_mkdir(_ftp_item *ftp, FTP_CALLBACK(cib), char *dir, const char *data, void *data_value)
{
	ftp_request *req = NULL;

	req = ftp_mkdir_with_cb(ftp, cib, dir);
	if (data)
		ftp_object_set_data(FTP_OBJECT(req), data, data_value);
	
	return req;
}

ftp_request *
gui_ftp_list(_ftp_item *ftp, FTP_CALLBACK(callback), int refresh, char *path, const char *data, void *data_value)
{
	ftp_request *req = NULL;

	DEBUG_MSG(DBG_INFO, "gui_ftp_listm adding a req");
	/* XXX fix , tie right to the end of the last req? */
	req = ftp_list_with_cb(ftp, callback, refresh, path);
	if (data)
		ftp_object_set_data(FTP_OBJECT(req), data, data_value);

	return req;
}

ftp_request *
gui_ftp_list_with_req(_ftp_item *ftp, ftp_request *preq, FTP_CALLBACK(callback), int refresh, char *path, const char *data, void *data_value)
{
	ftp_request *req = NULL;
		        
	req = ftp_list_wrapper(ftp, RP_END, preq, 0, callback, refresh, path);
	if (data)
		ftp_object_set_data(FTP_OBJECT(req), data, data_value);
		
	return req;
}


/*
** GUI FTP retr frontend.  Allows to retrieve files and associate an object data with the request through this one
** command.
*/
ftp_request *
gui_ftp_retr(_ftp_item *ftp, struct _list *fent, FTP_CALLBACK(callback), const char *data, void *data_value)
{
	ftp_request *req = NULL;
	struct stat _stat;

	if (stat(fent->file, &_stat) != -1) {
                /* The file exist, check how we want to handle it. */
		DEBUG_MSG(DBG_WEIRD, "The file to be retrieved already exists");		
	}

	req = ftp_retr(ftp, RP_END, NULL, 0, callback, fent, 0, TRFR_DEFAULT);

	if (data)
		ftp_object_set_data(FTP_OBJECT(req), data, data_value);
	
	return req;
}

ftp_request *
gui_ftp_stor(_ftp_item *ftp, struct _list *fent, FTP_CALLBACK(callback), const char *data, void *data_value)
{
	ftp_request *req = NULL;

	req = ftp_stor(ftp, RP_END, NULL, 0, callback, fent, NULL, 0);
	if (data)
		ftp_object_set_data(FTP_OBJECT(req), data, data_value);

	/* unfinished business? XXX */
	return req;
}

_ftp_item *
gui_ftp_current_connection()
{
	gui_ftp_lookup *lookup =
		gui_ftp_lookup_widget(gui_lookup_widget("right_tview"));

	if (lookup)
		return lookup->site;

	return NULL;
}


