/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_ftp_callback.c,v 1.42 2003/03/07 17:48:12 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
/* local headers */
#include <configure.h>
#include <ftp_event.h>
#include <ftp_event_callback.h>
#include <gui.h>
#include <gui_ftp.h>
#include <gui_ftp_callback.h>
#include <gui_ftp_lookup.h>
#include <gui_toolkit.h>
#include <gui_statusbar.h>
#include <gui_widgets.h>
#include <gui_tree_model.h>
#include <gui_qwin.h>
#include <queue.h>
#include <ftp_request.h>
#include <ftp.h>
#include <ftp_net.h>
#include <ftp_cmd.h>
#include <ftp_misc.h>
#include <ftp_object.h>
#include <cache.h>
#include <error.h>
#include <log.h>
#include <local.h>
#include <gui_treestore.h>
#include <gui_anti_idler.h>
#include <gui_tview.h>
#include <defaults.h>
#include <mem.h>
#ifdef HAVE_SOUND
#include <sound.h>
#endif

/* FTP CMD callbacks */

/* For link dirs */
FTP_CB_CMD(gui_ftp_link_handler_callback)
{
	GtkWidget *rtview = NULL, *ltview = NULL;
	ftp_request *req = NULL;
	/*gui_ftp_lookup *lookup = NULL;*/
	struct _list fent;
	char *file = NULL;
	int size;

	file = ftp_object_get_data(obj, "link_file");
	size = (int)ftp_object_get_data(obj, "link_file_size");

	rtview = gui_lookup_widget("right_tview");
	ltview = gui_lookup_widget("left_tview");
		
	if (status & R_FAILURE) {
		if (!file || size <= 0)
			return;
			
		memset(&fent, 0, sizeof(struct _list));
		fent.file = file;
		fent.size = size;

		req = ftp_retr(ftp, RP_END, NULL, 0, gui_ftp_callback_local_list, (struct
					_list *)&fent, fent.file, TRFR_DEFAULT);
		ftp_object_set_data(FTP_OBJECT(req), "widget", ltview);
		
		return;
	}

        ftp_pwd(ftp);
	gui_ftp_list(ftp, gui_ftp_callback_list, 1, NULL, "widget", rtview);
	
	free(file);

	return;
}


FTP_CB_CMD(gui_ftp_callback_quit)
{
	GtkWidget *tree;
	struct _list *list = NULL;
	
	DEBUG_MSG(DBG_INFO, "gnganagnagnagnaganga moo :)");
	if (!ftp) 
		return;		
		
	DEBUG_MSG(DBG_INFO, "gnigniginginignigigi uhu? :)");
	tree = ftp_object_get_data(obj, "widget"); 
	list = g_object_get_data(G_OBJECT(tree), "list");
	ftp_free_list(list);
	g_object_set_data(G_OBJECT(tree), "list", NULL);
	return;
}


/*
** This callback, upon calling will get a local list of the current local directory and fill the treeview specified
** by "widget" in the object data for the request.
** Useful when retrieving files when u need to refresh the local file list after receiving the file.
*/
FTP_CB_CMD(gui_ftp_callback_local_list)
{
	struct _list *list = NULL;
	GtkWidget *widget = NULL;
	if ( (widget = ftp_object_get_data(obj, "widget")) != NULL) {
		list = local_list();
		// gui_tview_fill(GTK_TREE_VIEW(widget), list, ftp_free_list, gui_fill_tview_cb);
		gui_tree_model_fill(gtk_tree_view_get_model(GTK_TREE_VIEW(widget)), list, ftp_free_list, gui_tree_model_fill_cb);
	}

	return;
}


/*
** This callback is useful after issuing a list request, when u need the treeview to be filled with the list of files.
** example, gui_ftp_list(ftp, 1, NULL, "widget", treeview_to_fill_with_files); :)
*/
FTP_CB_CMD(gui_ftp_callback_list)
{
	GtkTreeModel *model = NULL;
	gui_ftp_lookup *lookup = NULL;

	DEBUG_MSG(DBG_INFO, "gui_ftp_callback_list");
	if (!(status & R_SUCCESS))
		return;
	/*
	** check if the lookup widget exists for this ftp, in order to avoid problems with the min/maximizing, otherwise
	** a user could cwd to a dir on one site and then maximize  another site whilst still processing the former LIST
	** and then it would display it in the tview, for the other site.   However,  next time the site is maximized it 
	** should show the directory, if it has finished retrieving it.
	*/
	model = ftp_object_get_data(FTP_OBJECT(ftp), "gui_tree_model");
	if (!ftp || !model)
		return;
		
	if (ftp->dirinfo) {
		DEBUG_MSG(DBG_INFO, "ftp->dirinfo and lookup were found");
		gui_tree_model_fill(GTK_TREE_MODEL(model), ftp->dirinfo->list_dir,
			ftp_free_list, gui_tree_model_fill_cb);
		
		lookup = gui_ftp_lookup_site(ftp);
		if (lookup && lookup->widget) 
			gtk_tree_view_columns_autosize(GTK_TREE_VIEW(lookup->widget));
	}
	
	return;
}

/*
** This callback will simply request a file list of the current directory
** It will request a "refresh" meaning that it will not use the cache even if it
** is turned on.
*/
FTP_CB_CMD(gui_ftp_callback_get_fresh_list)
{
	ftp_request *req = NULL;

	req = ftp_list_with_cb(ftp, gui_ftp_callback_list, 1, 0);
	ftp_object_set_data(FTP_OBJECT(req), "widget", ftp_object_get_data(obj, "widget"));
	
	return;
}

/*
 | this callback will just add the given uploaded file to the remote tview
 | so to not require an LIST each time a file gets uploaded
 */
FTP_CB_CMD(gui_ftp_callback_add_file)
{
	struct _list *list = NULL, *new = NULL;
	GtkTreeModel *model = NULL;
//	GtkTreeView    *tree = NULL;
//	tree = (GtkTreeView *)ftp_object_get_data(obj, "widget");

	model = ftp_object_get_data(FTP_OBJECT(ftp), "gui_tree_model");
	if (!model && !ftp) {
		DEBUG_MSG(DBG_WEIRD, "gui_ftp_callback_add_file.. ftp == %p. model == %p", ftp, model);
		return;
	}
	
	list = ftp_object_get_data(obj, "item");

	new = j_malloc(sizeof(struct _list)); memset(new, 0, sizeof(struct _list));
	new->type = list->type; 
	new->file = j_strdup(list->file); 
	new->size = list->size;
	
	gui_treestore_append_list_model(model, new, "");
	list = gui_tree_model_list_sort(model, gui_treestore_get_list(model));
	gtk_tree_store_clear(GTK_TREE_STORE(model));
//	gui_treestore_fill(model, list, NULL, gui_fill_tview_cb);
	gui_treestore_fill(model, list, NULL, gui_tree_model_fill_cb);
	
}
/*
** GUI FTP EVENT HANDLERS
*/

/*
** FTP Event CMD callbacks.
*/



/*
** Default gui handler for USER.
*/

/*
** Default GUI FTP event callback handler for USER.
** Will only display, logging into ... on the statusbar.
**/
FTP_EVENT_CB_CMD(gui_ftp_event_user_sbar_logging_in)
{
	
	gui_statusbar_update_connect(2);


	return FTP_EVENT_SUCCESS;
}

/*
** Default GUI FTP event callback handler for PASS.
** Only purpose is to display either:
**	"Logged into..." on success (dipslying it on the statusbar), or just emptying 
**      the message on the statusbar on failure.
**      Also plays a sound for login when the user has successfully logged in.
*/
FTP_EVENT_CB_CMD(gui_ftp_event_pass_sbar_logged_in)
{
	if (status & R_SUCCESS) {
		gui_statusbar_update_connect(3);
#ifdef HAVE_SOUND
		sound_play_event(SOUND_LOGIN);
#endif

		 /* setup the queue site's specific */
		{
			_queue_item *head;
			GtkTreeModel *model;
			model = gui_qwin_model_get(ftp); 
			head = queue_init(ftp->addr);
			if(head)    gui_treestore_fill(model, head, NULL, gui_qwin_fill_cb);
		}			
	}

	
	return FTP_EVENT_SUCCESS;
}


/*
** FTP InteRRUPT callbacks.
** Will be called (if the disconnect interrupt is configured to call this function (is by default))
** This is supposed to deinitialise the treeview and everything concerning this ftp site.
*/
FTP_EVENT_CB_CMD(gui_ftp_irrupt_cb_disconnect)
{
	/*GtkWidget *tview = NULL; */
	GtkTreeModel *model = NULL;
	gui_ftp_lookup *lookup = NULL;

	DEBUG_MSG(DBG_INFO, "gui_ftp_irrupt_cb_disconnect");
	if (!ftp) 
		return FTP_EVENT_FAILURE;

	/* tview = ftp_object_get_data(FTP_OBJECT(ftp), "gui_ftp_treeview"); fix for 0.3 XXX */
	/*
	if ((rtview = gui_lookup_widget("right_tview"))) {
		model = gtk_tree_view_get_model(GTK_TREE_VIEW(rtview));
		gtk_tree_store_clear(GTK_TREE_STORE(model));
		if ((lookup = gui_ftp_lookup_widget(rtview)))
			gui_ftp_lookup_remove(lookup);
	}*/
	if ((lookup = gui_ftp_lookup_site(ftp))) {
		DEBUG_MSG(DBG_INFO, "removing the lookup stuff");
		
		if (lookup->widget) {
			/* XXX start recognizing them by model instead of treeview.. (in the TODO already for 0.3.1)*/
			model = gtk_tree_view_get_model(GTK_TREE_VIEW(lookup->widget));
			gtk_tree_store_clear(GTK_TREE_STORE(model));
		}
		
		DEBUG_MSG(DBG_INFO, "b4 gui_ftp_lookup_remove");
		gui_ftp_lookup_remove(lookup);
		DEBUG_MSG(DBG_INFO, "removed it!");
	}

	gui_anti_idler_toggle(ftp, FALSE);	/* turn the anti idler timer off */
	gui_ftp_setup(ftp, 0);
	gui_ftp_setup_write(ftp, 0);
	gui_connect_update_button();
	gui_statusbar_update(ftp);

	queue_save(gui_treestore_get_list(gui_qwin_model_get(ftp)), ftp->addr);
	queue_free(gui_treestore_get_list(gui_qwin_model_get(ftp)));
	gtk_tree_store_clear(GTK_TREE_STORE(gui_qwin_model_get(ftp)));
	g_object_set_data(G_OBJECT(gui_qwin_model_get(ftp)), "list", NULL);
	
	return FTP_EVENT_SUCCESS;
}


