/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_ftp_lookup.c,v 1.8 2003/03/03 02:16:33 tgucio Exp $
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
/* local headers */
#include <ftp.h>
#include <ftp_request.h>
#include <ftp_object.h>
#include <gui.h>
#include <gui_ftp.h>
#include <gui_ftp_lookup.h>
#include <mem.h>

gui_ftp_lookup *gui_ftp_lookup_head = NULL;


/*
 | return the gui_ftp_lookup head
 */
gui_ftp_lookup *
gui_ftp_lookup_get_head()
{
	return gui_ftp_lookup_head;
}

/*
** add a new entry to the gui_ftp_lookup_head linked list.. 
**/
gui_ftp_lookup *
gui_ftp_lookup_add(_ftp_item *ftp, GtkWidget *tview, int type, int state)
{
	gui_ftp_lookup **gpnt = NULL;

	/* go to the back of the list. */
	gpnt = &gui_ftp_lookup_head;
	while (*gpnt) {
		if ((*gpnt)->site == ftp)
			break;
			
		gpnt = &(*gpnt)->next;
	}

	if (!*gpnt) {
		/* allocate the entry. */
		if (!(*gpnt = j_malloc(sizeof(gui_ftp_lookup)))) 
			return NULL;
	}

	(*gpnt)->site = ftp;
	(*gpnt)->widget = tview;
	(*gpnt)->state = state;
	(*gpnt)->mode  = type;
	(*gpnt)->next = NULL;

	return *gpnt;
}

/*
** delete an entry from the gui_ftp_lookup linked list.
*/
int
gui_ftp_lookup_remove(gui_ftp_lookup *target)
{
	gui_ftp_lookup *gpnt = NULL, *prior = NULL;

	if (!target)
		return 0;

	DEBUG_MSG(DBG_INFO, "gui_ftp_lookup_remove");
	if (target->site)
		DEBUG_MSG(DBG_INFO, "deleting info for %s", target->site->addr);
	/* find the prior entry. */
	gpnt = gui_ftp_lookup_head;
	while (gpnt) {
		if (gpnt->next == target) {
			prior = gpnt;
			break;
		}
			
		gpnt = gpnt->next;
	}
	
	/* unlink this entry. */
	if (prior) /* middle or last item. */
		prior->next = target->next;
	else if (target == gui_ftp_lookup_head) {
		/* first item */
		DEBUG_MSG(DBG_INFO, "removing first entry of gui_ftp_lookup_head, gui_ftp_lookup_head = gui_ftp_lookup_head->next");
		gui_ftp_lookup_head = gui_ftp_lookup_head->next;
	}

	memset(target, 0, sizeof(gui_ftp_lookup));
	free(target);
		
	return 1;
	
}

/*
** look for a widget, return the corresponding gui_ftp_lookup entry if found.  NULL otherwise.
*/
gui_ftp_lookup *
gui_ftp_lookup_widget(GtkWidget *target)
{
	gui_ftp_lookup *gpnt = NULL;

	gpnt = gui_ftp_lookup_head;
	while (gpnt) {
		if (gpnt->widget == target)
			return gpnt;

		gpnt = gpnt->next;
	}

	return NULL;
}


/*
** look up by site, return the corresponding gui_ftp_lookup entry, if found, NULL otherwise.
*/
gui_ftp_lookup *
gui_ftp_lookup_site(_ftp_item *site)
{
	gui_ftp_lookup *gpnt = NULL;

	gpnt = gui_ftp_lookup_head;
	while (gpnt) {
		if (gpnt->site == site)
			return gpnt;

		gpnt = gpnt->next;
	}

	return NULL;
}

