/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_hammering.c,v 1.3 2003/03/03 02:16:33 tgucio Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <math.h>
/* local headers */
#include <gui.h>
#include <gui_ftp.h>
#include <gui_ftp_callback.h>
#include <gui_ftp_lookup.h>
#include <gui_statusbar.h>
#include <gui_paths.h>
#include <gui_logwin.h>
#include <gui_io.h>
#include <gui_timeout.h>
#include <gui_toolkit.h>
#include <gui_anti_idler.h>
#include <ftp.h>
#include <ftp_misc.h>
#include <ftp_object.h>
#include <medium.h>
#include <hammering.h>
#include <error.h>
#include <config.h>
/* icons */
#include <icons/connect_after.xpm> 


void
gui_connect_button_set_unhammer()
{
	DEBUG_MSG(DBG_INFO, "setting unhammer button");
	gui_connect_button_set("Unhammer", "connect_after.png", connect_after_xpm);
	return;
}

void
gui_hammering_start(_ftp_item *ftp)
{
	DEBUG_MSG(DBG_INFO, "starting out, gui_hammering_start");
	gui_connect_button_set_unhammer();
	gui_ftp_setup(ftp, 0);	gui_ftp_setup_write(ftp, 0);
	gui_anti_idler_toggle(ftp, FALSE);
	return;
}

void
gui_hammering_connected(_ftp_item *ftp)
{
	gui_ftp_lookup *lookup = NULL;
	
	/* setup the gtk socket handlers.  They'll work with the ftplib. */
	gui_ftp_setup(ftp, 1); // gui_ftp_setup_write(ftp, 1);
//	gui_ftp_setup_events(ftp);

	/* if the anti idler is enabled in the config file, enable it. */
	if (config_read_num("anti-idler"))
		gui_anti_idler_toggle(ftp, TRUE);

/*
	rtview = gui_lookup_widget("right_tview");
	gui_ftp_lookup_add(ftp, rtview, FTP_MODE_FTP, FTP_STATE_CONNECTED);
	DEBUG_MSG(DBG_INFO, "sending list...");
	gui_ftp_list(ftp, gui_ftp_callback_list, 0, NULL, "widget", rtview);
*/

//	ftp_object_set_data(FTP_OBJECT(ftp), "hammering_enabled", (void *)0);
	ftp_object_unlink_tag(FTP_OBJECT(ftp), "hammering_enabled");
	lookup = gui_ftp_lookup_site(ftp);
	if (lookup) {
		DEBUG_MSG(DBG_INFO, "lookie lookie");
		DEBUG_MSG(DBG_INFO, "widget is %p", lookup->widget);
		DEBUG_MSG(DBG_INFO, "model is %p", ftp_object_get_data(FTP_OBJECT(ftp), "gui_tree_model"));
		//gui_ftp_lookup_add(ftp, lookup->widget, FTP_MODE_FTP, FTP_STATE_CONNECTED);
		DEBUG_MSG(DBG_INFO, "lookup->widget is %p", lookup->widget);
		gui_ftp_list(ftp, gui_ftp_callback_list, 0, NULL, "widget", lookup->widget);
	}

	return;
	
}


void
gui_hammering_stop(_ftp_item *ftp)
{
	gui_ftp_lookup *lookup = NULL;
	GtkTreeModel *model = NULL;
	struct _list *list = NULL;

        DEBUG_MSG(DBG_INFO, "gui_hammering_stop");
	lookup = gui_ftp_lookup_site(ftp);
	if (lookup) {
		DEBUG_MSG(DBG_INFO, "lookup found (%p)", lookup);
		list = g_object_get_data(G_OBJECT(lookup->widget), "list");
		ftp_free_list(list);
		g_object_set_data(G_OBJECT(lookup->widget), "list", NULL);

		model = gtk_tree_view_get_model(GTK_TREE_VIEW(lookup->widget));
		gtk_tree_store_clear(GTK_TREE_STORE(model));
	        gui_ftp_lookup_remove(lookup);
	}

	gui_ftp_setup(ftp, 0);
	gui_ftp_setup_write(ftp, 0);
	ftp_free(&ftp);
	gui_connect_update_button();

	return;
}

int
gui_hammering_setup()
{
	hammering_signal_connect(HAMMERING_STOP, gui_hammering_stop);
	hammering_signal_connect(HAMMERING_START, gui_hammering_start);
	hammering_signal_connect(HAMMERING_CONNECTED, gui_hammering_connected);
	return 0;
}
