/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_io.c,v 1.29 2003/03/03 02:16:33 tgucio Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <gtk/gtk.h>
#include <sys/socket.h>
#include <netinet/in.h>
/* local headers */
#include <ftp_event.h>
#include <ftp_request.h>
#include <ftp_net.h>
#include <ftp_cmd.h>
#include <ftp.h>
#include <gui.h>
#include <gui_statusbar.h>
#include <gui_anti_idler.h>
#include <gui_ftp.h>
#include <gui_paths.h>
#include <hammering.h>
#include <medium.h>
#include <config.h>
#include <error.h>
#include <log.h>
#include <jtimer.h>
#include <mem.h>

/* 
 * medium M_SELECT_WAIT function
 * takes 1 argument
 */
int
gui_io_select_wait(MEDIUM_ARGS)
{
   int ret;
   struct timeval tv;
   double *wait = NULL;
   jtimer *timer = NULL;

   
   wait = MEDIUM_ARG(0);
   if (!wait)
	   return 0;

	DEBUG_MSG(DBG_INFO, "gonna sleep some %0.4f", *wait);

   timer = jtimer_new();
   jtimer_start(timer);
   
   while (1) {
      tv.tv_sec = 0;
      tv.tv_usec = 100000;
      if ( (ret = select(0, NULL, NULL, NULL, &tv)) < 0)
	    return -1;

      if (jtimer_get_elapsed_time(timer) > *wait)
	      break;

      while(gtk_events_pending())
         gtk_main_iteration();
   }
   jtimer_kill(timer);

   return 0;
}



/*
 * medium M_SELECT_WRITE function
 * takes 1 argument (fd)
 */
int
gui_io_select_write(MEDIUM_ARGS)
{
	int *fd = NULL, ret;
	struct timeval tv;
	fd_set fds;

	fd = (int *)MEDIUM_ARG(0);

	while (1) {
		tv.tv_sec = 0;
		tv.tv_usec = 10000;

		if (*fd < 0)
			return 1;
			
		FD_ZERO(&fds);
		FD_SET(*fd, &fds);

		if ( (ret = select (*fd + 1, NULL, &fds, NULL, &tv)) < 0)
			return 1;
			
		if (ret && FD_ISSET(*fd, &fds))
			break;
			
		while(gtk_events_pending())
			gtk_main_iteration();
	}
	
	return 0;
}


gboolean
gui_io_event_handler(GIOChannel *source, GIOCondition condition, gpointer data)
{
	struct gui_io_data {
		int fd;
		int (*callback)();
		void *data;
	} *pass = data;
	int ret;

	if (!pass->callback)
		return FALSE;

	ret = pass->callback(pass->fd, pass->data);
	if (ret == FALSE) 
		free(pass);

	return ret;
}
	


int
gui_io_watch(MEDIUM_ARGS)
{
	struct gui_io_data {
		int fd;
		int (*callback)();
		void *data;
	} *pass = NULL;
	int flags;
	GIOCondition g_flags;
	GIOChannel *ioc = NULL;
//	GError *err = NULL;

	if (!(pass = j_malloc(sizeof(struct gui_io_data))))
		return 0;
	
	pass->fd = (int)MEDIUM_ARG(0);
	flags = (int)MEDIUM_ARG(1);
	pass->callback = MEDIUM_ARG(2);
	pass->data = (void *)MEDIUM_ARG(3);


	if (!pass->callback)
		return 0;

	g_flags = 0;
	if (flags & IO_WATCH_READ)
		g_flags |= G_IO_IN;
	if (flags & IO_WATCH_WRITE)
		g_flags |= G_IO_OUT;
	if (flags & IO_WATCH_HUP)
		g_flags |= G_IO_HUP | G_IO_NVAL;
	
	ioc = g_io_channel_unix_new(pass->fd);
//	g_io_channel_set_encoding(ioc, NULL, &err);
//	g_io_channel_set_buffered(ioc, FALSE);
	g_io_add_watch_full(ioc, G_PRIORITY_DEFAULT, g_flags, gui_io_event_handler, pass, NULL);
	DEBUG_MSG(DBG_INFO, "flags: %d", g_flags);
	DEBUG_MSG(DBG_INFO, "buffered? %d", g_io_channel_get_buffered(ioc));
	DEBUG_MSG(DBG_INFO, "added a watch for %d", pass->fd);
	g_io_channel_unref(ioc);
	
	return 1;
}



void
gui_io_disconnect()
{
	/* XXX */
}


