/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_logwin.c,v 1.17 2003/03/03 02:16:33 tgucio Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
/* local headers */
#include <gui.h>
#include <gui_widgets.h>
#include <gui_logwin.h>
#include <gui_tview.h>
#include <gui_io.h>
#include <gui_misc.h>
#include <log.h>
#include <ftp.h>
#include <mem.h>
#include <config.h>
#include <error.h>

GtkTextBuffer *buffer = NULL;
GtkWidget *text_view;

char *
gui_log_config_get_color(const char *name)
{
	char *colour = NULL;

	if (!name)
		return NULL;
		
	config_read_value((char *)name, T_STR, (void *)&colour);
	if (!colour)
		return NULL;
	
	return colour;
}


void gui_logwin_config(GtkWidget *sw)
{
	GdkColor color;// = { 0, 0xd6d6, 0xd6d6, 0xd6d6 };
	GdkWindow *win;
	GtkWidget *view;
	GtkStyle *style;
	char *colour_bg = NULL;

	if (!(colour_bg = gui_log_config_get_color("logwin_colour")) || strlen(colour_bg) <= 3)
		colour_bg = gui_log_config_get_color("theme_logwin_colour");
	
	style = gtk_widget_get_style(GTK_WIDGET(gui_lookup_widget("mainwin")));
	if (colour_bg && strlen(colour_bg) > 3) {
		gdk_color_parse(colour_bg, &color);
		j_free(colour_bg);
		gdk_colormap_alloc_color(gdk_colormap_get_system(), &color, 0, 1);
	} else 
		color = style->bg[GTK_STATE_NORMAL];

	view = gtk_bin_get_child(GTK_BIN(sw));
	gtk_widget_realize(GTK_WIDGET(view));
	win = gtk_text_view_get_window(GTK_TEXT_VIEW(view),GTK_TEXT_WINDOW_TEXT);
//	gdk_colormap_alloc_color(gdk_colormap_get_system(), &color, 0, 1);
	gdk_window_set_background(win, &color);
	
	return;
}	
	
GtkWidget *
gui_setup_log_window(GtkWidget *window)
{
	GtkWidget *sw = NULL;

	buffer = gtk_text_buffer_new(NULL);
	gui_log_init_tags(buffer);

	sw = gui_setup_scrolled_window(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	text_view = gtk_text_view_new_with_buffer(buffer);
	gtk_widget_set_name(GTK_WIDGET(text_view), "log_window");

	gtk_container_set_border_width(GTK_CONTAINER(sw), 0);
	
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(text_view), FALSE);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);

	gtk_container_add(GTK_CONTAINER(sw), text_view);
	gtk_widget_show(text_view);

	return sw;
}


void
gui_log_ftp_show_buffer(_ftp_item *ftp)
{
	GtkTextBuffer *tb = NULL;
	GtkTextIter start, end;
	GtkTextMark *mark = NULL;

	if (ftp) {
		tb = ftp_object_get_data(FTP_OBJECT(ftp), "gui_log_textview_buffer");
		if (!tb)
			return;
	} else {
		tb = buffer;
	}
	
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(text_view), tb);
	gtk_text_buffer_get_bounds (tb, &start, &end);
	mark = gtk_text_buffer_create_mark (tb, NULL, &end, /* right grav */ FALSE);
	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (text_view), mark, 0, TRUE, 0.0, 1.0);
	gtk_text_buffer_delete_mark (buffer, mark);
	
	return;
}

void
gui_log_ftp_setup_buffer(_ftp_item *ftp)
{
	GtkTextBuffer *tb = NULL;

	if (!ftp)
		return;
		
	if (gui_lookup_widget("default_textview_buffer_ftp")) {
		tb = gtk_text_buffer_new(NULL);
		gui_log_init_tags(tb);
	} else {
		tb = buffer;
		gui_insert_widget("default_textview_buffer_ftp", ftp);
	}

	ftp_object_set_data(FTP_OBJECT(ftp), "gui_log_textview_buffer", tb);

	return;
}




static GtkTextTagTable *
gui_text_buffer_get_table (GtkTextBuffer *buffer)
{
	if (buffer->tag_table == NULL)
		buffer->tag_table = gtk_text_tag_table_new ();
	
	return buffer->tag_table;
}


void
gui_log_setup_tag(GtkTextBuffer *buffer, const char *tag, const char *confoption, const char *themeoption, const char *defcolor)
{
	GtkTextTag *texttag = NULL;
	char *colstr = NULL;

	/* tag error colour */
	texttag = gtk_text_tag_new(tag);
	if ((colstr = gui_log_config_get_color(confoption)) && strlen(colstr) > 3) 
		g_object_set(G_OBJECT(texttag), "foreground", colstr, NULL);
	else if ((colstr = gui_log_config_get_color(themeoption)) && strlen(colstr) > 3)
		g_object_set(G_OBJECT(texttag), "foreground", colstr, NULL);
	else 
		g_object_set(G_OBJECT(texttag), "foreground", defcolor, NULL);
	
	j_free(colstr);
	gtk_text_tag_table_add(gui_text_buffer_get_table(buffer), texttag);
	return;
}


void
gui_log_init_tags(GtkTextBuffer *tb)
{
	GtkTextIter iter;
	
	/*
	 * startup: black
	 * ftp to us: blue
	 * us to ftp: green
	 * errors: red
	 */
	if (tb == NULL) 
		return;
	
	gtk_text_buffer_get_iter_at_offset(tb, &iter, 0);

	/* tag error colour */
	gui_log_setup_tag(tb, "tag_error",    "logwindow_colour_error",    "theme_logwindow_colour_error",    "red");
	gui_log_setup_tag(tb, "tag_ftp_misc", "logwindow_colour_ftp_misc", "theme_logwindow_colour_ftp_misc", "black");
	gui_log_setup_tag(tb, "tag_ftp_sent", "logwindow_colour_ftp_sent", "theme_logwindow_colour_ftp_sent", "green");
	gui_log_setup_tag(tb, "tag_ftp_recv", "logwindow_colour_ftp_recv", "theme_logwindow_colour_ftp_recv", "blue");
	gui_log_setup_tag(tb, "tag_misc",     "logwindow_colour_misc",     "theme_logwindow_colour_misc",     "grey");
	
	
/*	
	gtk_text_buffer_create_tag(tb, "tag_error",
		"foreground", "red", NULL);
	gtk_text_buffer_create_tag(tb, "tag_ftp_misc",
		"foreground", "black", NULL);
	gtk_text_buffer_create_tag(tb, "tag_ftp_sent",
		"foreground", "green", NULL);
	gtk_text_buffer_create_tag(tb, "tag_ftp_recv",
		"foreground", "blue", NULL);
	gtk_text_buffer_create_tag(tb, "tag_misc",
		"foreground", "grey", NULL);
*/
	gtk_text_buffer_create_tag(tb, "not_editable",
		"editable", FALSE, NULL);

	return;
}




int
gui_log(GtkTextBuffer *tb, const gchar *buf, const gchar *t1, const gchar *t2, const gchar *t3, const gchar *t4)
{
	GtkTextIter start, end;
	GtkTextMark *mark = NULL;
	gchar *gna = NULL;
	int num = 1;
	
	if (tb == NULL || t1 == NULL)
		return 0;

	if (t2 != NULL) {
		num++;
		if (t3 != NULL) {
			num++;
			if (t4 != NULL) {
				num++;
			}
		}
	}

	gna = gui_convert_utf8(buf);
	if(gna)	buf = gna;
	
	gtk_text_buffer_get_bounds (tb, &start, &end);
	mark = gtk_text_buffer_create_mark (tb, NULL, &end, FALSE);
	switch (num) {
		case 1:
			gtk_text_buffer_insert_with_tags_by_name(tb, &end, 
					buf, -1, t1, 0);
			break;
		case 2:
			gtk_text_buffer_insert_with_tags_by_name(tb, &end,
					buf, -1, t1, t2, 0);
			break;
		case 3:
			gtk_text_buffer_insert_with_tags_by_name(tb, &end,
					buf, -1, t1, t2, t3, 0);
			break;
		case 4:
			gtk_text_buffer_insert_with_tags_by_name(tb, &end,
					buf, -1, t1, t2, t3, t4, 0);
			break;
	}

	/* scroll down. */
	if (gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view)) == tb) 
		gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(text_view), mark, 0, TRUE, 0.0, 1.0);
	
	gtk_text_buffer_delete_mark (tb, mark);

	g_free(gna);
	return 1;
}



void
gui_log_wrapper(_ftp_item *ftp, int mode, char *format, ...)
{
	GtkTextBuffer *tb = NULL;
	char buf[GOOD_SIZE + 1], *ptr = 0;
	va_list val;

	DEBUG_MSG(DBG_INFO, "gui_log_wrapper, ftp: %p", ftp);
	if (ftp) {
		tb = ftp_object_get_data(FTP_OBJECT(ftp), "gui_log_textview_buffer");
		if (!tb) 
			return;
	} else 
		tb = buffer;

	memset(buf, 0, sizeof(buf));
	va_start(val, format);
	vsnprintf(buf, GOOD_SIZE, format, val);
	va_end(val);

	ptr = strchr(buf, '\r');
	if (ptr != NULL) {
		*(ptr) = '\n';
		if (*(ptr+1) == '\n')
			*(ptr+1) = 0;
	}

	switch (mode) {
		case MODE_FTP_SENT:
			GUI_FTP_SENT_LOG(tb, buf);
			break;
		case MODE_FTP_RECV:
			GUI_FTP_RECV_LOG(tb, buf);
			break;
		case MODE_FTP_MISC:
			GUI_FTP_MISC_LOG(tb, buf);
			break;
		case MODE_ERROR:
			GUI_ERROR_LOG(tb, buf);
			break;
		case MODE_MISC:
			GUI_MISC_LOG(tb, buf);
			break;
	}

	return;
} 
