/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_main.c,v 1.51 2003/03/03 02:16:33 tgucio Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
#include <pthread.h>
/* local headers */
#include <configure.h>
#include <gui.h>
#include <gui_startup.h>
#include <gui_menu.h>
#include <gui_popup.h>
#include <gui_treestore.h>
#include <gui_tview.h>
#include <gui_tview_popup.h>
#include <gui_tree_model.h>
#include <gui_minimizer.h>
#include <gui_toolkit.h>
#include <gui_player.h>
#include <gui_widgets.h>
#include <gui_statusbar.h>
#include <gui_paths.h>
#include <gui_logwin.h>
#include <gui_hammering.h>
#include <gui_qwin.h>
#include <gui_ftp.h>
#include <gui_io.h>
#include <gui_theme.h>
#include <gui_toolbar.h>
#include <gui_misc.h>
#include <ftp.h>
#include <ftp_misc.h>
#include <local.h>
#include <config.h>
#include <cache.h>
#include <error.h>
#include <defaults.h>
#include <medium.h>
#include <log.h>
#include <mem.h>
#ifdef HAVE_SOUND
#include <sound.h>
#endif

gchar*
gui_convert_utf8(const char* string)
{
	gchar* utf8;
	GError *error = NULL;
	utf8 = g_locale_to_utf8(string, -1, NULL, NULL, &error);
	if(!utf8) {
		DEBUG_MSG(DBG_ERROR, "UTF8 locale *** %s", error->message);
		utf8 = g_convert(string, strlen(string), "UTF-8", "ISO-8859-1", 
				NULL, NULL, &error);
	}
	
	if(!utf8)
		DEBUG_MSG(DBG_ERROR, "UTF8 latin-1 *** %s", error->message);

	return utf8;
}

gchar*
gui_convert_locale(const char* string)
{
	gchar* ascii;
	GError *error = NULL;

	ascii = g_convert(string, strlen(string), "ISO-8859-1",
	    "UTF-8", NULL, NULL, &error);
	if(!ascii)
		DEBUG_MSG(DBG_ERROR, "UTF8 latin-1 *** %s", error->message);

	return ascii;
}

GtkWidget *
gui_setup()
{
	GtkWidget *window = NULL;
	GtkWidget *panedup = NULL, *paned = NULL;//, *panedlist = NULL;
	GtkWidget *menu = NULL, *menubar = NULL, *main_box = NULL;
	GtkWidget *tools = NULL;
	GtkWidget *hbox = NULL, *scrolled_left = NULL, *scrolled_right = NULL;
	GtkWidget *statusbar = NULL, *vbox_left = NULL, *vbox_right = NULL;
	GtkWidget *logwindow = NULL, *paths_right = NULL, *paths_left = NULL;
	GtkWidget *lview = NULL, *rview = NULL;
	char *wd = 0;
	GtkWidget *qwin = 0;
	GtkWidget *hbox_sub = NULL, *minimizer = NULL;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gui_insert_widget("mainwin",window);

	/* quit the program when the main window is closed */
	g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gui_quit), NULL);

	gtk_window_set_title(GTK_WINDOW(window), "junkie - GUI FTP client");
	gtk_widget_set_size_request(GTK_WIDGET(window), gdk_screen_width() * 0.5, gdk_screen_height() * 0.5);
	gtk_window_set_default_size(GTK_WINDOW(window), gdk_screen_width() * 0.8, gdk_screen_height() * 0.8);

	/* setup a box to pack the menu into. */
	main_box = gtk_vbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(main_box), 1);
	gtk_container_add(GTK_CONTAINER(window), main_box);
      
	/* setup the menu and show. */
	menu = gui_menu_setup(window);
	gtk_box_pack_start(GTK_BOX(main_box), menu, FALSE, TRUE, 0);
	gtk_widget_show(menu);

	/*setup the tool vbox */
	tools = gtk_hbox_new(0, 0);
	gtk_box_pack_start(GTK_BOX(main_box), tools, FALSE, FALSE, 0);
	gtk_widget_show(tools);
	
	/* setup the menubar and show */
	menubar = gui_toolbar_new();
	gtk_box_pack_start(GTK_BOX(tools), menubar, TRUE, TRUE, 0);
	gtk_widget_realize(menubar);
	gtk_widget_show(menubar);
	gui_menubar_setup(menubar);

#ifdef HAVE_SOUND
	/* setup the player bar */
	gui_player_setup(menubar);
#endif
	
	/* setup the paned widget */
	panedup = gtk_vpaned_new();
	gtk_box_pack_start(GTK_BOX(main_box), panedup, TRUE, TRUE, 0);
	gtk_widget_show(panedup);
	
	/* setup the log window */
	logwindow = gui_setup_log_window(window);
	gtk_widget_set_size_request(logwindow,  -1, 125);
//	gtk_box_pack_start(GTK_BOX(main_box), logwindow, FALSE, FALSE, 0);
	gtk_paned_add1(GTK_PANED(panedup), logwindow);
	gtk_widget_realize(logwindow);
	gui_logwin_config(logwindow);
	gtk_widget_show(logwindow);

	
	/* setup the paned widget */
	paned = gtk_vpaned_new();
//	gtk_container_add(GTK_CONTAINER(main_box), paned);
	gtk_paned_add2(GTK_PANED(panedup), paned);
	gtk_paned_set_position(GTK_PANED(paned), gdk_screen_width() * 0.35);
	gtk_widget_show(paned);

	
	/* setup two scrolled windows (local/left) and (remote/right). */
	/* setup a container for the scrolled windows. */
	hbox = gtk_hbox_new(FALSE, 20);
//	gtk_container_add(GTK_CONTAINER(main_box), hbox);
	gtk_paned_pack1(GTK_PANED(paned), hbox, TRUE, FALSE);
	
	/* the left one */
	vbox_left = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(hbox), vbox_left);
	gtk_widget_show(vbox_left);

	paths_left = gui_paths_setup(1);
	gtk_box_pack_start(GTK_BOX(vbox_left), paths_left, FALSE, FALSE, 0);
	gtk_widget_show(paths_left);
	wd = getcwd(NULL, 0);
	if (wd != NULL) {
		gui_paths_text(1, wd);
		free(wd);
	}
	scrolled_left =
		gui_setup_scrolled_window(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(vbox_left), scrolled_left, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_left);

	/* setup a tview in the left scrolled window */
	lview = gui_setup_tview(scrolled_left);
	gtk_widget_show(lview);

	gui_insert_widget("left_tview", lview);
	/* Setup a popup menu for the "local" treeview to be displayed on right clicks. */
	g_signal_connect(G_OBJECT(lview), "button_press_event",
			G_CALLBACK(gui_menu_rightclick_cb),
			gui_tview_popup_init_local(GTK_WIDGET(lview)));
	
	/* the right one */
	vbox_right = gtk_vbox_new(FALSE, FALSE);
	gtk_container_add(GTK_CONTAINER(hbox), vbox_right);
	gtk_widget_show(vbox_right);

	hbox_sub = gtk_hbox_new(FALSE, FALSE);
	paths_right = gui_paths_setup(0);
	gtk_box_pack_start(GTK_BOX(hbox_sub), paths_right, TRUE, TRUE, 0);
	gtk_widget_show(paths_right);
	minimizer = gui_minimizer_new();
	gtk_box_pack_start(GTK_BOX(hbox_sub), minimizer, FALSE, TRUE, 0);
	gtk_widget_show(minimizer);
	gtk_box_pack_start(GTK_BOX(vbox_right), hbox_sub, FALSE, TRUE, 0);
	gtk_widget_show(hbox_sub);
	
	scrolled_right =
		gui_setup_scrolled_window(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(vbox_right), scrolled_right, TRUE, TRUE, 0);
	gtk_widget_show(scrolled_right);

	/* setup a tview in the right window. */
	rview = gui_setup_tview(scrolled_right);
	gtk_widget_show(rview);
	gui_insert_widget("right_tview", rview);

	/* Setup a popup menu for the "remote" treeview to be displayed on right click. */
	g_signal_connect(G_OBJECT(rview), "button_press_event",
		G_CALLBACK(gui_menu_rightclick_cb),
		gui_tview_popup_init_remote(GTK_WIDGET(rview)));

	
	gui_tview_dnd_setup(GTK_TREE_VIEW(lview), GTK_TREE_VIEW(rview));
	
	/* setup the queue window */
	qwin = gui_qwin_setup(window);
//	gtk_box_pack_start(GTK_BOX(main_box), qwin, FALSE, FALSE, 0);
	gtk_paned_pack2(GTK_PANED(paned), qwin, TRUE, FALSE);
	gtk_widget_show_all(qwin);

	//gui_qwin_dnd_setup(GTK_TREE_VIEW(lview), GTK_TREE_VIEW(rview));

	/* setup the status bar */
	statusbar = gui_statusbar_setup();
	gtk_box_pack_start(GTK_BOX(main_box), statusbar, FALSE, TRUE, 0);

	gtk_widget_show(hbox);
	gtk_widget_show(main_box);

	return window;
}


#ifdef HAVE_SOUND
/*
 * play a little song for me. :)
 */
void
gui_main_sound_init_thread(void *arg)
{
	char *str = NULL;
	
	sound_init();
	sound_play_event(SOUND_STARTUP);
	if (config_read_bool("sound_startup_enable")) {
		config_read_value("sound_startup_path", T_STR, (void **)&str);
		if (str)
		        sound_play_path(str, 0, NULL);

		free(str);
	}

	pthread_exit(NULL);
	return;
}
#endif

int main(int argc, char **argv)
{
	GtkWidget *window = NULL, *startup_window = NULL;
	struct _list *list_dir = NULL;
	char buf[MED_SIZE + 1], *theme = NULL;
#ifdef HAVE_SOUND
	pthread_t thread;
#endif

	gtk_init(&argc, &argv);

	startup_window = gui_startup(NULL);
	gui_hammering_setup();

	/* load the config file */
	gui_startup_write_text(startup_window, "Loading config file.\n");
	config_init();

	/* initialise the theme */
	config_read_value("theme", T_STR, (void **)&theme);
	if (theme == NULL) {
		DEBUG_MSG(DBG_INFO, "no theme installed");
		return 1;
	}

	snprintf(buf, MED_SIZE, "Loading theme: %s\n", theme);
	gui_startup_write_text(startup_window, buf); 

	DEBUG_MSG(DBG_INFO, "loading theme %s", theme);
	if (gui_theme_load(theme) == -1) 
		DEBUG_MSG(DBG_WEIRD, "gui_theme, theme not found");

	/* Setup the gui logging function as the main log function. */
	log_setup(gui_log_wrapper);
	
	gui_startup_write_text(startup_window, "Setting up a beautiful window.\n");
	window = gui_setup();

	gui_medium_setup();

	/* add some more fancy logs.  */
	MISC_LOG(NULL, "    Started up junkie\n");

	if (! (list_dir = local_list()) ) {
		g_print("Could not open your local directory");
//		return 1;
	}

//	gui_tview_fill(GTK_TREE_VIEW(gui_lookup_widget("left_tview")), list_dir, ftp_free_list, gui_fill_tview_cb);
	gui_tree_model_fill(gtk_tree_view_get_model(GTK_TREE_VIEW(gui_lookup_widget("left_tview"))), 
				list_dir, ftp_free_list, gui_tree_model_fill_cb);

	if (config_read_bool("cache-enable")) {
		gui_startup_write_text(startup_window, "Loading the elite superman cache.\n");
		cache_init();
	}

	gui_startup_write_text(startup_window, "Finished\n");
	gui_startup_end(startup_window, window);

#ifdef HAVE_SOUND
	/* calls sound_init and start playing. */
	pthread_create(&thread, NULL, (void *)gui_main_sound_init_thread, (void *)NULL);
#endif

	/* For Yo Momma randomization */
	srandom(time(NULL));
	gtk_main();

	if (config_read_bool("cache-enable")) {
		cache_save();
	}

	DEBUG_MSG(DBG_WEIRD, "do i ever get here?");
	config_sync();
	gui_theme_close();
#ifdef HAVE_SOUND
	sound_play_event(SOUND_QUIT);
#endif

	return 0;
}
