/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_menu.c,v 1.19 2003/03/03 02:16:33 tgucio Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
#include <configure.h>
/* local headers */
#include <gui.h>
#include <gui_widgets.h>
#include <gui_statusbar.h>
#include <gui_siteman.h>
#include <gui_treestore.h>
#include <gui_io.h>
#include <gui_ftp.h>
#include <gui_ftp_lookup.h>
#include <ftp.h>
#include <ftp_net.h>
#include <local.h>
#include <gui_qwin.h>
#include <queue.h>
#include <config.h>
#include <cache.h>
#include <error.h>
#ifdef HAVE_SOUND
#include <sound.h>
#endif


extern void gui_yomomma_dialog();

static GtkItemFactoryEntry menu_items[] = {
	
{	"/_File",                       NULL,        
	NULL,                           0,	                   
	"<Branch>",                     0           },

{	"/File/_Connect",               "<control>C", 
	(void(*)())gui_connect_dialog,	0,	                   
	NULL,                           0           },

{	"/File/S_ite manager",          "<control>I", 
	(void(*)())gui_siteman_startup,	0,	                   
	NULL,                           0           },

{	"/File/_Quit",                  "<control>Q", 
	gui_quit,		                0,	                   
	NULL,                           0           },

{	"/_Edit",                       NULL,         
	NULL,			                0,	                   
	"<Branch>",                     0           },

{	"/Edit/_Preferences",           NULL,         
	gui_preferences_dialog,	        0,
	NULL,                           0           },

#ifdef FUN
{	"/Important Info",                     NULL,         
	gui_yomomma_dialog,	            0,	                   
	NULL,                           0           },
{   "/Cute things",                 NULL,         
	gui_cute_dialog,                0,
	NULL,                           0           },
#endif

{	"/_Help",                       NULL,         
	NULL,			                0,
	"<Branch>",                     0           },

{	"/Help/About",                  NULL,
	gui_about_dialog,	            0,
	NULL,                           0           },
};



/* XXX maybe ok, maybe not. */
void gui_quit(gpointer ptr)
{
	gui_ftp_lookup *lookup = gui_ftp_lookup_widget(gui_lookup_widget("right_tview"));
	
	if (lookup && lookup->site) {
		/* XXX ftp_main_close fix argument 2.. */
		ftp_main_close(lookup->site->fd, 0);
	}

	if (config_read_bool("cache-enable")) {
		cache_save();
	}
	
	config_sync();
#ifdef HAVE_SOUND
	sound_play_event(SOUND_QUIT);
	sound_shutdown();
#endif

	lookup = gui_ftp_lookup_get_head();
	while(lookup) {
		queue_save(gui_treestore_get_list(
				gtk_tree_view_get_model(GTK_TREE_VIEW(gui_qwin_treeview()))),
				lookup->site->addr);
	lookup = lookup->next;
	}
	gtk_main_quit();
	gtk_exit(0);
}

/*
 * make a new tearoff menu
 */ 
GtkWidget *
gui_menu_new_tearoff(char* name)
{
	GtkWidget *menu = 0, *item = 0;

	menu = gtk_menu_new();
	item = gtk_tearoff_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), item);
	gtk_widget_show(menu);
	
	g_object_set_data(G_OBJECT(menu), "name", strdup(name));
	
	return menu;
}

/*
 * add a submenu to a menu
 */
GtkWidget *
gui_menu_add_menu(GtkWidget *menu, const gchar *name)
{
	GtkWidget *submenu = 0, *menu_item = 0;

	menu_item = gtk_menu_item_new_with_label(name);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_widget_show(menu_item);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
	
	return submenu;
}

/*
 * add an item to the menu, setup activation callback such
 */
void
gui_menu_add_item(GtkWidget *menu, const gchar *name, GtkSignalFunc callback, gpointer user_pointer)
{
	GtkWidget *item = 0;

	item = gtk_menu_item_new_with_label(name);
	
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			G_CALLBACK(callback), user_pointer);
	
	gtk_menu_append(GTK_MENU(menu), item);
	
	gtk_widget_show(item);
	
	return;
}



GtkWidget *
gui_menu_setup(GtkWidget *window)
{
	GtkWidget *menu = NULL;
	GtkItemFactory *item_factory = NULL;
	GtkAccelGroup *accel_group = NULL;


	accel_group = gtk_accel_group_new();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_group);

	/* creates the menu. */
	gtk_item_factory_create_items(item_factory, sizeof(menu_items) / sizeof(menu_items[0]), menu_items, NULL);
	gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
	
	/* get the menu widget */
	menu = gtk_item_factory_get_widget(item_factory, "<main>");
	
	return menu;
}

