
/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_minimizer.c,v 1.21 2003/03/03 02:16:33 tgucio Exp $
 */

#include <gtk/gtk.h>
#include <stdio.h>
/* local includes */
#include <gui_widgets.h>
#include <gui_ftp_lookup.h>
#include <gui_tview.h>
#include <gui_minimizer.h>
#include <gui_paths.h>
#include <gui_logwin.h>
#include <gui_qwin.h>
#include <gui_statusbar.h>
#include <gui_toolkit.h>

GtkWidget *
gui_minimizer_new()
{
	GtkWidget *mini;
	mini = gtk_button_new_with_label(":)");
	g_signal_connect(G_OBJECT(mini), "pressed",
			G_CALLBACK(gui_minimize_tview), NULL);
	return mini;
}

void
gui_minimize_tview(GtkWidget *button, gpointer data)
{
	GtkTreeView    *tree;
	GtkTreeModel   *model_old, *model_new;
	status_bar     *sbar;
	GtkWidget      *mini;
	gui_ftp_lookup *lookup;
	int            freeze;

	tree = GTK_TREE_VIEW(gui_lookup_widget("right_tview"));
	lookup = gui_ftp_lookup_widget(GTK_WIDGET(tree));
	if(!lookup)	return;
	
	model_old = gtk_tree_view_get_model(tree);
	model_new = gui_tview_model_init();
	gtk_tree_view_set_model(tree, model_new);

	sbar = gui_lookup_widget("statusbar");
	mini = gtk_button_new_with_label(lookup->site->addr);
	gtk_box_pack_end(GTK_BOX(sbar->hbox), mini, FALSE, FALSE, 0);
	gtk_widget_show(mini);

	gui_paths_text(0, "");
	g_object_set_data(G_OBJECT(mini), "gui_ftp_lookup_site", lookup->site);
	g_object_set_data(G_OBJECT(mini), "model", model_old);
	g_object_set_data(G_OBJECT(mini), "list",
			g_object_get_data(G_OBJECT(model_old), "list"));
	g_object_set_data(G_OBJECT(model_old), "list", NULL);
	g_object_set_data(G_OBJECT(model_old), "view", NULL);
	g_object_set_data(G_OBJECT(model_new), "view", NULL);
	lookup->widget = NULL;
	gui_log_ftp_show_buffer(NULL);
	gui_qwin_model_show(NULL);

	freeze = (int)g_object_get_data(G_OBJECT(model_old), "freeze");
	DEBUG_MSG(DBG_INFO, "when minimizing %p is set to %d", model_old, freeze);
	g_object_set_data(G_OBJECT(mini), "freeze", (gpointer)freeze);
	if(freeze)
		g_signal_handler_unblock(G_OBJECT(tree),
			(int)g_object_get_data(G_OBJECT(tree), "handler"));

	
	g_signal_connect(G_OBJECT(mini), "pressed",
			G_CALLBACK(gui_maximize_tview), NULL);

	gui_connect_button_set_connect();
	DEBUG_MSG(DBG_INFO, "updating the statusbar");
	gui_statusbar_update(NULL);
}

void
gui_maximize_tview(GtkWidget *button, gpointer data)
{
	GtkTreeView    *tree;
	GtkTreeModel   *model_old, *model_new;
	status_bar     *sbar;
	gui_ftp_lookup *lookup;
	int             freeze;
	
	tree = GTK_TREE_VIEW(gui_lookup_widget("right_tview"));
	lookup = gui_ftp_lookup_widget(GTK_WIDGET(tree));
	if(lookup)	gui_minimize_tview(NULL, 0);

	model_old = gtk_tree_view_get_model(tree);
	model_new = g_object_get_data(G_OBJECT(button), "model");
	gtk_tree_view_set_model(tree, model_new);
	g_object_unref(model_old);


	g_object_set_data(G_OBJECT(model_new), "view", tree);
	lookup = gui_ftp_lookup_site(g_object_get_data(G_OBJECT(button), 
				                 "gui_ftp_lookup_site"));		
	if (lookup) {
		if (lookup->site && lookup->site->dirinfo)
			gui_paths_text(0, lookup->site->dirinfo->pwd);
			
		lookup->widget = GTK_WIDGET(tree);
		g_object_set_data(G_OBJECT(model_new), "list",
				g_object_get_data(G_OBJECT(button), "list"));
		
		gui_log_ftp_show_buffer(lookup->site);
		gui_qwin_model_show(lookup->site);
	}

	freeze = (int)g_object_get_data(G_OBJECT(button), "freeze");
	g_object_set_data(G_OBJECT(model_new), "freeze", (gpointer)freeze);
	DEBUG_MSG(DBG_INFO, "%p SET TO: %d <-----------for(%d)---------", 
			model_new,
			freeze,
			(int)g_object_get_data(G_OBJECT(tree), "handler"));
	if (freeze)
		g_signal_handler_block(G_OBJECT(tree),
			(int)g_object_get_data(G_OBJECT(tree), "handler"));


	sbar = gui_lookup_widget("statusbar");
	gtk_container_remove(GTK_CONTAINER(sbar->hbox), button);
	if (lookup) gui_statusbar_update(lookup->site);

	gui_connect_button_set_disconnect();
}	
