/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_paths.c,v 1.19 2003/02/24 04:10:40 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
/* local headers */
#include <gui.h>
#include <ftp_request.h>
#include <ftp_cmd.h>
#include <ftp.h>
#include <local.h>
#include <error.h>
#include <medium.h>
#include <log.h>
#include <gui_ftp_callback.h>
#include <gui_widgets.h>
#include <gui_io.h>
#include <gui_treestore.h>
#include <gui_ftp.h>
#include <gui_ftp_lookup.h>
#include <ftp_misc.h>
#include <gui_tview.h>
#include <gui_tree_model.h>

GtkWidget *entry_local = NULL;
GtkWidget *entry_remote = NULL;


void
gui_paths_text(int local, gchar *path)
{
	GtkWidget *entry = ((local) ? (entry_local) : (entry_remote));

	if (entry == NULL)
		return;
	
	gtk_entry_set_text(GTK_ENTRY(entry), path);

	return;
}

int
gui_medium_paths_text(MEDIUM_ARGS)
{
	char *path = NULL;
	_ftp_item *ftp = NULL;

	ftp = MEDIUM_ARG(0);
	path = MEDIUM_ARG(1);

	gui_paths_text(0, path);

	return 1;
}

void
gui_paths_cb(GtkWidget *widget, GtkWidget *entry)
{
	GtkWidget *rtview = NULL, *ltview = NULL;
	gui_ftp_lookup *lookup = NULL;
	ftp_request *req = NULL;
	struct _list	*list_dir = NULL;
	const gchar	*path = NULL;
	int		local = (entry == entry_local) ? 1 : 0;

		
	path = gtk_entry_get_text(GTK_ENTRY(entry));

	if (local) {
		if (!(ltview = gui_lookup_widget("left_tview")))
			return;
					   
		if (chdir(path) == 0) {
			list_dir = local_list();

//		         gui_tview_fill(GTK_TREE_VIEW(ltview), list_dir,
 //                           ftp_free_list, gui_fill_tview_cb);
			gui_tree_model_fill(gtk_tree_view_get_model(GTK_TREE_VIEW(ltview)), list_dir,
					ftp_free_list, gui_tree_model_fill_cb);
		} else 
			MISC_LOG(NULL, "    Local - No such directory %s\n", path);
		
	} else {
		if (!(rtview = gui_lookup_widget("right_tview")))
			return;

		if (!(lookup = gui_ftp_lookup_widget(rtview)))
			return;


		if (lookup->site) {
			req = ftp_cwd(lookup->site, path);
			req = ftp_pwd_with_req(lookup->site, req);
			gui_ftp_list_with_req(lookup->site, req, gui_ftp_callback_list, 0, (char *)path, "widget", rtview);
		}
	}

	return;
}

GtkWidget      *
gui_paths_setup(int local)
{
	GtkWidget      *entry = NULL;
	GtkWidget      *hbox = NULL;

	hbox = gtk_hbox_new(FALSE, 0);

	gtk_container_set_border_width(GTK_CONTAINER(hbox), 0);
	entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
	gtk_widget_show(entry);

	if (local)
		entry_local = entry;
	else 
		entry_remote = entry;

	g_signal_connect(G_OBJECT(entry), "activate", G_CALLBACK(gui_paths_cb), entry);

	return (hbox);
}
