/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_popup.c,v 1.6 2003/03/03 02:16:33 tgucio Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
/* local headers */
#include <gui.h>
#include <gui_popup.h>
#include <ftp.h>
#include <ftp_net.h>
#include <local.h>
#include <config.h>
#include <cache.h>
#include <error.h>


/*
** New popup menu (teared off),  Will usually, if not always, be displayed on right-clicks.
*/
GtkWidget *
gui_popup_new_tearoff(cb callback)
{
	GtkWidget *menu = NULL, *item = NULL;

	menu = gtk_menu_new();
	item = gtk_tearoff_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), item);
	gtk_widget_show(menu);

		g_object_set_data(G_OBJECT(menu), "callback", callback);
	
	return menu;
}


/*
** Add a menu onto a popup menu
*/
GtkWidget *
gui_popup_add_menu(GtkWidget *menu, const gchar *name)
{
	GtkWidget *submenu = NULL, *menu_item = NULL;

	menu_item = gtk_menu_item_new_with_label(name);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_widget_show(menu_item);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
	
	return submenu;
}

/*
** Add an item to a popup menu
*/
void
gui_popup_add_item(GtkWidget *menu, const gchar *name, GtkSignalFunc callback, gpointer user_pointer)
{
	GtkWidget *item = NULL;

	item = gtk_menu_item_new_with_label(name);
	
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			G_CALLBACK(callback), user_pointer);
	
	gtk_menu_append(GTK_MENU(menu), item);
	
	gtk_widget_show(item);
	
	return;
}

gboolean
gui_menu_rightclick_cb(GtkWidget *tree, GdkEvent *event, gpointer user_data)
{
	GtkMenu *queue_menu = 0;
	GtkTreePath* path = 0;

	if (event == 0 || tree == 0)
		return FALSE;
	if (event->type != GDK_BUTTON_PRESS || event->button.button != 3 || 
		user_data == 0)
			return FALSE;
	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(tree),
			event->button.x, event->button.y,
			&path, NULL, 0, 0);
	if (!path)
		return FALSE;
	gtk_tree_path_free(path);

	/* disable/enable the menu XXX (this might not be used later). */
	queue_menu = user_data;
	gui_popup_sensitive_update(GTK_WIDGET(queue_menu));

	/* pop up the queue menu. */
	gtk_menu_popup(GTK_MENU(queue_menu), 0, 0, 0, 0,
			((GdkEventButton *)event)->button,
			((GdkEventButton *)event)->time);

	return TRUE;
}

void
gui_popup_sensitive_update(GtkWidget *menu)
{
	cb callback = g_object_get_data(G_OBJECT(menu), "callback");	
	callback(menu);
}
