/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_preferences.c,v 1.23 2003/03/07 17:48:12 thepin Exp $
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <dirent.h>
#include <sys/types.h>
/* local headers */
#include <configure.h>
#include <config.h>
#include <gui.h>
#include <gui_tview.h>
#include <gui_preferences.h>
#include <gui_preferences_filler.h>
#include <gui_treestore.h>
#include <gui_misc.h>
#include <mem.h>

/*
** ItemFactory fashioned.. :)
** This is just the setup stuff, it will then be loaded and put into a linked list?
*/
struct gui_prefsie prefs_list[] = {
	{	"/General",			NULL, 						PREF_TREE	},
	{	"/General/Settings",		gui_preferences_filler_general_settings,	PREF_NORMAL	},
	{	"/General/Login Settings",	gui_preferences_filler_general_login,		PREF_NORMAL	},
	{	"/General/Transfer Settings",	gui_preferences_filler_general_ftp,		PREF_NORMAL	},
	{	"/General/Editor",		gui_preferences_filler_general_editors,		PREF_NORMAL	},
	
	{	"/Queue",			NULL,						PREF_TREE},
	{	"/Queue/Preferences",	gui_preferences_filler_queue_prefs,         PREF_NORMAL },
	
	{	"/Interface",			NULL,						PREF_TREE	},
	{	"/Interface/Colours",		gui_preferences_filler_interface_colours,	PREF_NORMAL	},
	{	"/Interface/Junkie Themes",	gui_preferences_filler_interface_themes,	PREF_NORMAL	},
#ifdef HAVE_SOUND
	{	"/Interface/Sound",		gui_preferences_filler_interface_sound,		PREF_NORMAL	},
#endif

	{	"/Advanced",			NULL,						PREF_TREE	},
	{	"/Advanced/Cache",		gui_preferences_filler_advanced_cache,		PREF_NORMAL	},

	{ NULL, NULL, 0 }
};

static GtkWidget *notebook;
	

/*
** Find an entry in a category linked list.
*/
struct gui_pref_list *
gui_find_category(char *entry, struct gui_pref_list *list)
{
	struct gui_pref_list *plist = NULL;

	plist = list;
	while (plist) {
		if (!strcasecmp(prefs_list[plist->pindex].name, entry))
			break;
			
		plist = plist->next;
	}

	return plist;
}


/*
** The sole purpose of this little function is to add an entry to a category list and pass
** the values given as argument to the new list entry.
*/
void
gui_preferences_category_append(struct gui_pref_list **plist, const char *name, char *treepath, int index)
{
	struct gui_pref_list **ppnt = NULL;

	if (!plist)
		return;
	
	/* go all the way to the last entry. */
	ppnt = plist;
	while (*ppnt)
		ppnt = &(*ppnt)->next;

	/* allocate some space for the new entry. */
	if (!(*ppnt = j_malloc(sizeof(struct gui_pref_list))))
		return;

	memset(*ppnt, 0, sizeof(struct gui_pref_list));
	if (name)
		(*ppnt)->name = j_strdup(name);
		
	if (treepath)
		(*ppnt)->treepath = j_strdup(treepath);
		
	(*ppnt)->pindex = index;
	(*ppnt)->slaves = 0;
	(*ppnt)->next = NULL;
	
	return;
}

/*
 * free a gui_pref_list object
 */
struct gui_pref_list *
gui_preferences_free_list(struct gui_pref_list *ptree)
{
	struct gui_pref_list *alpha = NULL, *beta = NULL;
	
	beta = ptree;
	while (beta) {
		alpha = beta;
		beta = beta->next;
		/* free here */
		j_free(alpha->treepath);
		j_free(alpha->name);
		j_free(alpha);
	}
	
	return NULL;
}

/*
** This function will use the prefs_table declared at the top of this file and turn it
** into a linked list and also parse some values, such as the GtkTreePath for each entry,
** which is important for tree depths (ie. creating a tree in a tree in another tree...etc :))
*/
struct gui_pref_list *
gui_preferences_parse_table()
{
	char *name = NULL, *parent = NULL, *treepath = NULL;
	struct gui_pref_list *plist_root = NULL, *pparent = NULL, *ppnt = NULL;
	int i;

	/* add a root entry, "/" */
	gui_preferences_category_append(&plist_root, "/", NULL, -1);
	for (i = 0; prefs_list[i].name; i++) {
		/* parse the parent tree and the name of the new entry. */
		parent = j_strdup(prefs_list[i].name);
		if (!(name = strrchr(parent, '/'))) {
			j_free(parent);
			continue;
		}

		*(name++) = 0;

		/* find the parent of the new entry. */
		pparent = gui_find_category(parent, plist_root->next);

		/* setup the treepath for the new entry. */
		if (pparent) {
			treepath = j_asprintf("%s:%d",
						pparent->treepath,
						pparent->slaves++);
		} else {
			/*
			 * no parent, add to the root
			 * ; which is the first entry of the list.
			 */
			treepath = j_asprintf("%d", plist_root->slaves++);
		}
		
		/* attach the new entry to the parent. */
		gui_preferences_category_append(&plist_root, name, treepath, i);
		j_free(treepath);
		j_free(parent);
	}

	/*
	 * remove the first entry which is the "/" entry 
	 * and is not desired in the output.
	 */
	ppnt = plist_root;
	plist_root = plist_root->next;
	j_free(ppnt);
	
	return plist_root;
}


/*
** Setup the tview for the preferences, and also call a function to parse the list of preferences entries for the notebook.
** The result will be put into the plist in the arguments.
*/
GtkWidget *
gui_preferences_tview_setup(GtkWidget *window, struct gui_pref_list **plist)
{
	GtkWidget *scrolled = NULL;
	GtkTreeSelection *select = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeViewColumn *column = NULL;
	GtkCellRenderer *renderer = NULL;
	GtkTreeView *tree = NULL;
	struct gui_pref_list *ptree = NULL;

	scrolled = gui_setup_scrolled_window(GTK_POLICY_AUTOMATIC,
			                             GTK_POLICY_AUTOMATIC);
	model = GTK_TREE_MODEL(gtk_tree_store_new(1, G_TYPE_STRING));
	tree = (GtkTreeView *)gtk_tree_view_new_with_model(model);
	
	gtk_tree_selection_set_mode(
			gtk_tree_view_get_selection(tree),
			GTK_SELECTION_SINGLE);

	/* maybe add an icon for the first stuff too? :) */
	/* Setup the single column. */
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
				"Category", renderer, "text", 0, NULL);
	gtk_tree_view_append_column(tree, column);
	
	/* add to the model. */
	ptree = gui_preferences_category_tview_initialize(GTK_WIDGET(tree));
	if (plist)
		*plist = ptree;

	/* add containers and this stuff. */
	gtk_container_add(GTK_CONTAINER(scrolled), GTK_WIDGET(tree));

	/* add some event handler */
	select = gtk_tree_view_get_selection(tree);
	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
	g_signal_connect(G_OBJECT(select), "changed",
			G_CALLBACK(gui_preferences_category_tree_row_select_cb),
			tree);

	return GTK_WIDGET(scrolled);
}



/*
 * This function is called when a row in the treeview 
 * for the prefs (on the left) is selected.
 * If the row contains a normal entry (not a tree)
 * it will bring up the notebook entry for that entry.
 */
void
gui_preferences_category_tree_row_select_cb(GtkTreeSelection *select, GtkTreeView *tree)
{
	struct gui_pref_list *plist = NULL;

	plist = gui_treestore_get_list_from_selection(tree);
	if (plist) {
		if (prefs_list[plist->pindex].type == PREF_NORMAL) {
			gtk_notebook_set_current_page(
					GTK_NOTEBOOK(notebook), 
					plist->notebook);
		}
	}

	return;
}


/*
 * The purpose of this function is to take a tree 
 * that has been setup and add all categories that
 * are supposed to be in the tree into it. :)
 */
struct gui_pref_list *
gui_preferences_category_tview_initialize(GtkWidget *tree)
{
	struct gui_pref_list *plist = NULL, *proot = NULL;
	char *p = NULL;

	proot = gui_preferences_parse_table();
	if (!proot)
		return NULL;

	/* add the entries to the treestore model for the tview. */
	plist = proot;
	while (plist) {
		if ( (p = strrchr(plist->treepath, ':')))
			*p = 0;
		
		if (p) {
			gui_treestore_append_list(GTK_TREE_VIEW(tree), 
						plist, plist->treepath);
			gui_treestore_append(GTK_TREE_VIEW(tree), 
						plist, plist->treepath, 
 				     gui_preferences_category_list_append_cb);
		} else {
			p = j_strdup("");
			gui_treestore_append_list(GTK_TREE_VIEW(tree), plist, p);
			gui_treestore_append(GTK_TREE_VIEW(tree), plist, p, 
				     gui_preferences_category_list_append_cb);
			j_free(p);
		}
		
		plist = plist->next;
	}
	
	return proot;
}

/*
 * Callback for gui_treestore_append_list, 
 * simply sets texts specified by plist->name for that node.
 */
void
gui_preferences_category_list_append_cb(GtkTreeModel *model, 
			struct gui_pref_list *plist, GtkTreeIter *iter)
{
	gtk_tree_store_set(GTK_TREE_STORE(model), iter, 0, plist->name, -1); 
	return;
}


/*
 * Sets up the notebook.  Sets up all entries..
 */
void
gui_preferences_notebook_setup(GtkWidget *box, struct gui_pref_list *proot)
{
	GtkWidget *frame = NULL;
	GtkWidget *label = NULL;
	struct gui_pref_list *plist = NULL;
	int nbooks;
	
	notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_LEFT);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);

	/* let's create some frames... */
	plist = proot;
	for (nbooks = 0; plist; plist = plist->next) {
		if (prefs_list[plist->pindex].type == PREF_NORMAL) {
			/* setup new frame */
			frame = gtk_frame_new(plist->name);
			gtk_container_set_border_width(GTK_CONTAINER(frame), 5);

			/* make a label and add the frame to the notebook. */
			label = gtk_label_new(plist->name);
			gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);

			/* fill the frame. (set it up).  */
			prefs_list[plist->pindex].filler(frame);
			
			/*
			 * store the number of the notebook entry
			 */
			plist->notebook = nbooks++;
		}
	}
	
	/* pack the notebook into the box specified in the arguments. */
        gtk_box_pack_start(GTK_BOX(box), notebook, FALSE, FALSE, 0);

	return;
}


/*
** Setup the preferences dialog.
*/
GtkWidget *
gui_preferences_dialog_setup()
{
	GtkWidget *window = NULL;

	/* setup a dialog. */
	window = gtk_dialog_new_with_buttons(
		"Preferences:",
		NULL,
		GTK_DIALOG_MODAL,
		GTK_STOCK_APPLY, GTK_RESPONSE_YES,
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
		NULL);

	/* setup some default options. */
	gtk_dialog_set_default_response(GTK_DIALOG(window), GTK_RESPONSE_OK);
	gtk_container_set_border_width(GTK_CONTAINER(window), 2);
	gtk_widget_set_size_request(GTK_WIDGET(window),
						gdk_screen_width() * 0.52,
						gdk_screen_height() * 0.45);
	
	gtk_window_set_default_size(GTK_WINDOW(window),
					gdk_screen_width() * 0.52,
					gdk_screen_height() * 0.45);
	
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(window)->vbox), 5);

	return window;
}


/*
 * The main preferences function, creates the dialog, 
 * calls some functions, all to the end.
 * (when the user closes the prefs).
 */
void
gui_preferences_dialog(void)
{
	gint result;
	GtkWidget *window;
	GtkWidget *scrolled = NULL;
	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
	struct gui_pref_list *plist = NULL;


	/* setup a dialog window. */
	window = gui_preferences_dialog_setup();

	/* pack a hbox into the window. */
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(window)->vbox), hbox);

	/* Setup the treeview interface for swapping between notebooks, and pack it into the window. */
	scrolled = gui_preferences_tview_setup(GTK_WIDGET(window), &plist);
	gtk_box_pack_start(GTK_BOX(hbox), scrolled, TRUE, TRUE, 4);
	gtk_widget_show(scrolled);
	
	/* Setup the notebook, and pack it into the hbox.  plist contains info about what should be added. */
	gui_preferences_notebook_setup(hbox, plist);

	/* let's display all this cool stuff */
	gtk_widget_show_all(window);

	/* run the window, until gets closed. */
	result = gtk_dialog_run(GTK_DIALOG(window));

	/* if the response was "yes", then settings will be saved. */
	if (result == GTK_RESPONSE_YES) 
		gui_preferences_save_options();

	/* destroy the window. quit the preferences. */
	gtk_widget_destroy(window);

	/* free the linked list containing preferences categories */
	gui_preferences_free_list(plist);
	
	return;
}

