/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_preferences_filler.c,v 1.27 2003/03/07 21:22:10 thepin Exp $
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <dirent.h>
#include <sys/types.h>
#include <math.h>
/* local headers */
#include <configure.h>
#include <config.h>
#include <gui.h>
#include <gui_tview.h>
#include <gui_preferences.h>
#include <gui_preferences_filler.h>
#include <gui_treestore.h>
#include <gui_theme.h>
#include <gui_player.h>
#include <mem.h>
#ifdef HAVE_SOUND
#include <sound.h>
#endif

#ifdef HAVE_CONFIG_H
#include <configure.h>
#endif

#ifdef HAVE_BASENAME
#include <libgen.h>
#endif

static GtkWidget *conn_menu, *trans_menu, *exist_menu, *limit_spin, *hammering_spin, *antiidle_spin;
static GtkWidget *user_entry, *pass_entry;
static GtkWidget *port_spin;
static GtkWidget *cache_check, *cache_session_check, *cache_days_spin;
static GtkWidget *cache_check, *cache_days_spin;
static GtkWidget *queue_keep;
static GtkWidget *txtedit_entry, *imgedit_entry;
static GtkWidget *txtview_entry, *imgview_entry;
#ifdef HAVE_SOUND
static GtkWidget *sound_check, *show_player,
                 *sound_startup_check, *sound_startup_path_entry;
#endif
static GtkWidget *logwin_colour_entry;
static GtkWidget *logwin_misc_colour_entry;
static GtkWidget *logwin_ftp_misc_colour_entry, *logwindow_ftp_sent_colour_entry;
static GtkWidget *logwin_error_colour_entry, *logwindow_ftp_recv_colour_entry;
static GtkWidget *list_method_option_menu;
static GtkWidget *list_custom_entry;
static gchar *global_theme;

void
gui_preferences_filler_general_ftp(GtkWidget *frame)
{
	const char *r_name[] = { "Passive mode (PASV)",
				 "Active mode (ACTV) ",
				 NULL,
				 "       Binary      ",
				 "       ASCII       ",
				 NULL,
				 "      Resume     ",
				 "     Overwrite   ",
				 "       Skip      ",
				 "       Ask       ",
				 NULL };
	char **p_name;
	GtkWidget *menu = NULL, *menu_item = NULL, *label = NULL;
	GtkWidget *vbox = NULL; 
	GtkWidget *hbox1, *hbox2, *hbox3, *hbox4, *hbox5;
	long limit;
	GtkObject *adj = NULL;

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox);

	/* Connection mode option menu */
	hbox1 = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
	label = gtk_label_new("Connection mode");
	gtk_box_pack_start(GTK_BOX(hbox1), label, TRUE, FALSE, 0);
	menu = gtk_menu_new();
	for(p_name=(char **)&r_name[0]; *p_name; p_name++) {
		menu_item = gtk_menu_item_new_with_label(*p_name);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	}
	p_name++;

	conn_menu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(conn_menu), menu);
	gtk_box_pack_start(GTK_BOX(hbox1), conn_menu, TRUE, FALSE, 0);

	if (config_read_bool("usepassive")) {
		gtk_option_menu_set_history(GTK_OPTION_MENU(conn_menu), 0);
	} else {
		gtk_option_menu_set_history(GTK_OPTION_MENU(conn_menu), 1);
	}

	/* Transfer mode option menu */
	hbox2 = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
	label = gtk_label_new("Transfer mode  ");
	gtk_box_pack_start(GTK_BOX(hbox2), label, TRUE, FALSE, 0);
	menu = gtk_menu_new();
	for(; *p_name; p_name++) {
		menu_item = gtk_menu_item_new_with_label(*p_name);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	}
	p_name++;

	trans_menu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(trans_menu), menu);
	gtk_box_pack_start(GTK_BOX(hbox2), trans_menu, TRUE, FALSE, 0);

	if (config_read_bool("usebinary")) {
		gtk_option_menu_set_history(GTK_OPTION_MENU(trans_menu), 0);
	} else {
		gtk_option_menu_set_history(GTK_OPTION_MENU(trans_menu), 1);
	}

	/* if file exist option menu */
	hbox5 = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox5, FALSE, FALSE, 0);
	label = gtk_label_new("if file exist:");
	gtk_box_pack_start(GTK_BOX(hbox5), label, TRUE, FALSE, 0);
	menu = gtk_menu_new();
	for(; *p_name; p_name++) {
		menu_item = gtk_menu_item_new_with_label(*p_name);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	}

	exist_menu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(exist_menu), menu);
	gtk_box_pack_start(GTK_BOX(hbox5), exist_menu, TRUE, FALSE, 0);

	if (config_read_num("file-exist") == 4)
		gtk_option_menu_set_history(GTK_OPTION_MENU(exist_menu), 0);
	else if(config_read_num("file-exist") == 2)
		gtk_option_menu_set_history(GTK_OPTION_MENU(exist_menu), 1);
	else if(config_read_num("file-exist") == 8)
		gtk_option_menu_set_history(GTK_OPTION_MENU(exist_menu), 2);
	else if(config_read_num("file-exist") == 16)
		gtk_option_menu_set_history(GTK_OPTION_MENU(exist_menu), 3);
	

	/* Spin Button: Transfer limit */
	hbox3 = gtk_hbox_new(FALSE, 0);
	hbox4 = gtk_hbox_new(FALSE, 0);
	limit = (gdouble)config_read_num("transfer-limit");
	adj = gtk_adjustment_new(limit/1024, 0.0, 1048576.0, 1.0, 3.0, 0.0);
	label = gtk_label_new("Transfer limit (in kB/s) ");
	limit_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
	gtk_box_pack_start(GTK_BOX(hbox3), label, FALSE, FALSE, 0); 
	gtk_box_pack_start(GTK_BOX(hbox3), limit_spin, FALSE, FALSE, 0);
	label = gtk_label_new(" Kbytes (0: off)");
	gtk_box_pack_start(GTK_BOX(hbox3), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox4), hbox3, TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox4, FALSE, FALSE, 20);
  
	return;
}

void
gui_preferences_filler_general_login(GtkWidget *frame)
{
	GtkWidget *label;
	GtkWidget *vbox, *hbox1, *hbox2;
	char *str;
	long limit;
	GtkObject *adj = NULL;
	
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	hbox1 = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
	hbox2 = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);

	label = gtk_label_new("Default username:                  ");
	gtk_box_pack_start(GTK_BOX(hbox1), label, TRUE, FALSE, 0);
	user_entry = gtk_entry_new();
	config_read_value("defuser", T_STR, (void **)&str);
	if (str != NULL) {
		gtk_entry_set_text(GTK_ENTRY(user_entry), str);
		j_free(str);
	}
	gtk_box_pack_start(GTK_BOX(hbox1), user_entry, TRUE, FALSE, 0);

	label = gtk_label_new("Anonymous password (email address):");
	gtk_box_pack_start(GTK_BOX(hbox2), label, TRUE, FALSE, 0);
	pass_entry = gtk_entry_new();
	config_read_value("anonpass", T_STR, (void **)&str);
	if (str != NULL) {
		gtk_entry_set_text(GTK_ENTRY(pass_entry), str);
		j_free(str);
	}
	gtk_box_pack_start(GTK_BOX(hbox2), pass_entry, TRUE, FALSE, 0);

	hbox2 = gtk_hbox_new(FALSE, 0);
	label = gtk_label_new("     Hammering interval (Seconds) ");
	limit = (gdouble)config_read_num("hammering");
	adj = gtk_adjustment_new(limit, 0.0, 1073741824.0, 1.0, 3.0, 0.0);
	hammering_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), hammering_spin, FALSE, FALSE, 0);
	label = gtk_label_new(" (0: off)");
	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
	
	return;
}



int
gui_preferences_cache_session_toggle_cb(GtkSpinButton *spinbutton, gpointer data)
{
	GtkWidget *checkbox = NULL;
	gint shown;
	gint active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cache_check));

	if (!(checkbox = GTK_WIDGET(data))) {
		DEBUG_MSG(DBG_ERROR, "gui_preferences_cache_spin_change_value_cb, checkbox == NULL");
		return 1; 
	}

	shown = (gint)g_object_get_data(G_OBJECT(checkbox), "is_shown");
	if (active == FALSE) {
		if (!shown) {
			gtk_widget_show(checkbox);
			shown = 1;
		}
	} else {
		if (shown) {
			gtk_widget_hide(checkbox);
			shown = 0;
		}
	}
	
	g_object_set_data(G_OBJECT(checkbox), "is_shown", (void *)shown);

	return 0;
}


int
gui_preferences_cache_session_call_first_cb(GtkObject *obj, gpointer data)
{
	if (config_read_bool("cache-enable")) {
		gtk_widget_hide(GTK_WIDGET(data));
		g_object_set_data(G_OBJECT(data), "is_shown", (void *)0);
	} else 
		g_object_set_data(G_OBJECT(data), "is_shown", (void *)1);
	
	gtk_signal_disconnect_by_func(obj, G_CALLBACK(gui_preferences_cache_session_call_first_cb), data);

	return 0;
}


void
gui_preferences_filler_queue_prefs(GtkWidget *frame)
{
	queue_keep = gtk_check_button_new_with_label(
			"keep items in queue after transfer is done?");
	if(config_read_bool("queue-keep"))
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(queue_keep), TRUE);
	else
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(queue_keep), FALSE);
	gtk_container_add(GTK_CONTAINER(frame), queue_keep);
}


void
gui_preferences_filler_advanced_cache(GtkWidget *frame)
{
	GtkWidget *vbox = NULL;
	GtkWidget *hbox1 = NULL, *hbox2 = NULL, *hbox3 = NULL;
	GtkWidget *label = NULL;
	GtkObject *adj = NULL;
	gdouble exp;
	int cache_enable = config_read_bool("cache-enable");

	vbox = gtk_vbox_new(FALSE, 0);
	hbox1 = gtk_hbox_new(FALSE, 0);

	/* Checkbox: Use cache? */
	cache_check = gtk_check_button_new_with_label("Use cache?");
	if (cache_enable) {
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cache_check), TRUE);
	} else {
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cache_check), FALSE);
	}
	gtk_box_pack_start(GTK_BOX(hbox1), cache_check, FALSE, FALSE, 0);

	/* Spin Button: Expire after */
	hbox2 = gtk_hbox_new(FALSE, 0);
	hbox3 = gtk_hbox_new(FALSE, 0);
	exp = (gdouble)config_read_num("cache-days");
	adj = gtk_adjustment_new(exp, 0.0, 356.0, 1.0, 3.0, 0.0);
	label = gtk_label_new("Expire after (0-365) ");
	cache_days_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
	
	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), cache_days_spin, FALSE, FALSE, 0);
	label = gtk_label_new(" days (0: never)");
	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox3), hbox2, TRUE, FALSE, 0);

	/* pack all hboxes into the vbox */
	gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox3, FALSE, FALSE, 0);

	/* if cache-days is 0, should it cache for each session (not storing the data). */
	hbox2 = gtk_hbox_new(FALSE, 0);
	cache_session_check = gtk_check_button_new_with_label("Use cache during an FTP session: ");
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cache_session_check),
			((config_read_bool("cache-enable")) ? (TRUE) : (FALSE)));
	gtk_box_pack_start(GTK_BOX(hbox2), cache_session_check, FALSE, FALSE, 0);

	/* enable callbacks. */
	g_signal_connect(G_OBJECT(cache_check), "toggled",
		G_CALLBACK(gui_preferences_cache_session_toggle_cb), cache_session_check);
	gtk_signal_connect_after(GTK_OBJECT(frame), "show", 
		G_CALLBACK(gui_preferences_cache_session_call_first_cb), cache_session_check);

	/* pack into vbox */
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	
	return;
}

/*
 * Called when the LIST method is changed
 */
void
gui_preferences_filler_list_method_changed_cb(GtkOptionMenu *option_menu, gpointer data)
{
	GtkWidget *menu = NULL, *menu_item = NULL;
	GtkWidget *custom_entry = NULL, *custom_label = NULL;
	int list_id, is_shown;

	menu = gtk_option_menu_get_menu(option_menu);
	custom_entry = g_object_get_data(G_OBJECT(menu), "list_custom_entry");
	custom_label = g_object_get_data(G_OBJECT(menu), "list_custom_label");
	
	is_shown = (int)g_object_get_data(G_OBJECT(menu), "list_custom_is_shown");
	menu_item = gtk_menu_get_active(GTK_MENU(menu));
	list_id = (int)g_object_get_data(G_OBJECT(menu_item), "list_method_id");
	
	if (list_id == FTP_LIST_CUSTOM) {
		if (!is_shown) {
			gtk_widget_show(custom_entry);
			gtk_widget_show(custom_label);
			is_shown = 1;
		}
	} else {
		if (is_shown) {
			gtk_widget_hide(custom_entry);
			gtk_widget_hide(custom_label);
			is_shown = 0;
		}
	}
	
	g_object_set_data(G_OBJECT(menu), "list_custom_is_shown", (void *)is_shown);

	return;
}

/*
 * when the preferences are launched, depending on what method is list selected
 * decide whether or not to show the custom options
 */
void
gui_preferences_general_settings_list_show_cb(GtkObject *obj, gpointer data)
{
	GtkWidget *option_menu = NULL, *menu = NULL;
	GtkWidget *label = NULL, *entry = NULL;
	int list_method, is_shown = 0;

	if (!(option_menu = data))
		return;
	
	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(option_menu));
	entry = g_object_get_data(G_OBJECT(menu), "list_custom_entry");
	label = g_object_get_data(G_OBJECT(menu), "list_custom_label");
	is_shown = 1;
        if ((list_method = config_read_num("list_method")) != FTP_LIST_CUSTOM) {		
		gtk_widget_hide(entry);
		gtk_widget_hide(label);
		is_shown = 0;
	}
	g_object_set_data(G_OBJECT(menu), "list_custom_is_shown", (void *)is_shown);

	return;
}



int
gui_preferences_list_method_get_id(GtkOptionMenu *option_menu)
{
	GtkWidget *menu = NULL, *menu_item = NULL;

	menu = gtk_option_menu_get_menu(option_menu);
	menu_item = gtk_menu_get_active(GTK_MENU(menu));
	return (int)g_object_get_data(G_OBJECT(menu_item), "list_method_id");
}

void
gui_preferences_filler_general_settings(GtkWidget *frame)
{
	struct {
		char *tag;
		int id;
	} list_methods[] = {
		{ "         LIST -al         ",	FTP_LIST_DEFAULT	},
		{ "          LIST            ",	FTP_LIST_PLAIN		},
		{ "         Custom           ",	FTP_LIST_CUSTOM		},
		{NULL,0}
	};
	register int i;
	char *str = NULL;
	int conf_list_method, limit;
	int show_index = 0;
	GtkWidget *label;
	GtkObject *adj;
	GtkWidget *vbox		= gtk_vbox_new(FALSE, 0);
	GtkWidget *hbox1 = NULL, *hbox2 = NULL;
	GtkWidget *hbox3 = NULL, *hbox4 = NULL;
	GtkWidget *vbox2 = NULL;
	GtkWidget *list_frame = NULL;
	GtkWidget *menu = NULL, *menu_item = NULL;
	gdouble val;

	/*
	 * Default port (21)
	 */
	hbox1 = gtk_hbox_new(FALSE, 0);
	val = (gdouble)config_read_num("defport");
	adj = gtk_adjustment_new(val, 1.0, 65536.0, 1.0, 10.0, 0.0);
	label = gtk_label_new("Default port (1-65536):");
	port_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), label, TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), port_spin, TRUE, FALSE, 0);

	/*
	 * Anti Idler.
	 */
	hbox2 = gtk_hbox_new(FALSE, 0);
	label = gtk_label_new("Anti Idler (Seconds (0: off)):  ");
	limit = (gdouble)config_read_num("anti-idler");
	adj = gtk_adjustment_new(limit, 0.0, 1073741824.0, 1.0, 3.0, 0.0);
	antiidle_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);


	gtk_box_pack_start(GTK_BOX(hbox2), label, TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), antiidle_spin, TRUE, FALSE, 0);

	/*
	 * List method
	 */
	conf_list_method = config_read_num("list_method");
	hbox3 = gtk_hbox_new(FALSE, 0);
	vbox2 = gtk_vbox_new(FALSE, 0);
	menu = gtk_menu_new();
	list_frame = gtk_frame_new("List method");
	for (i = 0; list_methods[i].tag; i++) {
		if (list_methods[i].id == conf_list_method)
			show_index = i;
			
		menu_item = gtk_menu_item_new_with_label(list_methods[i].tag);
		g_object_set_data(G_OBJECT(menu_item), "list_method_id", (void *)list_methods[i].id);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	}
	
	list_method_option_menu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(list_method_option_menu), menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(list_method_option_menu), show_index);
	gtk_signal_connect(GTK_OBJECT(list_method_option_menu), "changed",
			G_CALLBACK(gui_preferences_filler_list_method_changed_cb), NULL);

	hbox4 = gtk_hbox_new(FALSE, 0);
        label = gtk_label_new("Custom list method: ");
	list_custom_entry = gtk_entry_new_with_max_length(5);
	gtk_entry_set_width_chars(GTK_ENTRY(list_custom_entry), 5);
	config_read_value("list_method_custom", T_STR, (void **)&str);
	if (str) {
		gtk_entry_set_text(GTK_ENTRY(list_custom_entry), str);
		j_free(str);
	}

	gtk_box_pack_start(GTK_BOX(hbox4), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox4), list_custom_entry, FALSE, FALSE, 0);
	
	g_object_set_data(G_OBJECT(menu), "list_custom_entry", list_custom_entry);
	g_object_set_data(G_OBJECT(menu), "list_custom_label", label);
	
	gtk_box_pack_start(GTK_BOX(vbox2), list_method_option_menu, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox4, FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(list_frame), vbox2);
	gtk_box_pack_start(GTK_BOX(hbox3), list_frame, TRUE, FALSE, 0);
	gtk_signal_connect_after(GTK_OBJECT(frame), "show",
		G_CALLBACK(gui_preferences_general_settings_list_show_cb), list_method_option_menu);

	gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox3, FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	return;
}

void
gui_preferences_filler_general_editors(GtkWidget *frame)
{
	GtkWidget *label = NULL;
	GtkWidget *align = NULL;
	GtkTable *table = NULL;
	char *str = NULL;

	align = gtk_alignment_new(0, 0, 1, 0);
   
	table = (GtkTable *)gtk_table_new(4, 2, FALSE);
	gtk_table_set_row_spacings(table, 3);
   
	/* Text editor to use with edit-on-the-fly */
	label = gtk_label_new("Text editor:");
	gtk_table_attach(table, label, 0, 1, 0, 1, 0,0,1,0);
	txtedit_entry = gtk_entry_new();
	config_read_value("txteditor", T_STR, (void **)&str);
	if (str != NULL) {
		gtk_entry_set_text(GTK_ENTRY(txtedit_entry), str);
		j_free(str);
	}
	gtk_table_attach_defaults(table, txtedit_entry, 1, 2, 0, 1);

	/* Image editor to use with edit-on-the-fly */
	label = gtk_label_new("Image editor:");
	gtk_table_attach(table, label, 0, 1, 1, 2, 0,0,1,0);
	imgedit_entry = gtk_entry_new();
	config_read_value("imgeditor", T_STR, (void **)&str);
	if (str != NULL) {
		gtk_entry_set_text(GTK_ENTRY(imgedit_entry), str);
		j_free(str);
	}
	gtk_table_attach_defaults(table, imgedit_entry, 1, 2, 1, 2);

	/* Text viewer to use with view-on-the-fly */
	label = gtk_label_new("Text viewer:");
	gtk_table_attach(table, label, 0, 1, 2, 3, 0,0,1,0);
	txtview_entry = gtk_entry_new();
	config_read_value("txtviewer", T_STR, (void **)&str);
	if (str != NULL) {
		gtk_entry_set_text(GTK_ENTRY(txtview_entry), str);
		j_free(str);
	}
	gtk_table_attach_defaults(table, txtview_entry, 1, 2, 2, 3);

	/* Image viewer to use with view-on-the-fly */
	label = gtk_label_new("Image viewer:");
	gtk_table_attach(table, label, 0, 1, 3, 4, 0,0,1,0);
	imgview_entry = gtk_entry_new();

	config_read_value("imgviewer", T_STR, (void **)&str);
	if (str != NULL) {
		gtk_entry_set_text(GTK_ENTRY(imgview_entry), str);
		j_free(str);
	}
	
	gtk_table_attach_defaults(table, imgview_entry, 1, 2, 3, 4);
   
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(align));
	gtk_container_add(GTK_CONTAINER(align), GTK_WIDGET(table));
	
	return;
}




void
gui_colours_button_select_colour_cb(GtkEventBox *box, GdkEvent *event, gpointer data)
{
	GtkWidget *colour_selection_dialog = NULL;
	GtkColorSelection *colour_selection = NULL;
	GdkColor color;
	int response;

	/* prepare the colour selection dialog  */
	colour_selection_dialog = gtk_color_selection_dialog_new ("Select colour");
	colour_selection = GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colour_selection_dialog)->colorsel);
	gtk_color_selection_set_has_palette(colour_selection, FALSE);
	gtk_color_selection_set_has_opacity_control(colour_selection, FALSE);

	/* run the dialogue */
	response = gtk_dialog_run(GTK_DIALOG(colour_selection_dialog));
	if (response == GTK_RESPONSE_OK) {
		GtkWidget **param = NULL;
		GtkWidget *entry = NULL;
		GtkWidget *eventbox = NULL;
		char buf[SMALL_SIZE+1];

		if (!(param = data))
			return;

		// get the arguments
		entry = param[0];
		eventbox = param[1];
		
		// put the new colour into the entry
		gtk_color_selection_get_current_color (colour_selection, &color);
		g_snprintf(buf, SMALL_SIZE, "#%.2X%.2X%.2X", color.red >> 8, color.green >> 8, color.blue >> 8);
		gtk_entry_set_text(GTK_ENTRY(entry), buf);

		// set the eventbox bg
		gtk_widget_modify_bg(GTK_WIDGET(eventbox), GTK_STATE_NORMAL, &color);
	}
	
	gtk_widget_hide(colour_selection_dialog);


	return;
}

void
gui_colours_button_cleanup_cb(GObject *obj, gpointer data)
{
	j_free(data);
	return;
}


void
gui_preferences_colour_setup(GtkTable *table, int row, 
					const char *text, const char *confopt, GtkWidget **entry)
{
	GtkWidget *label = NULL;
	GtkWidget *colabel = NULL;
	GtkWidget *eventbox = NULL;
	GtkWidget **param = NULL;
	GdkColor color;
	char *value = NULL;
	char *entry_text = NULL;

	if (!entry)
		return;
		
	label = gtk_label_new(text);
	*entry = gtk_entry_new();
	config_read_value((char *)confopt, T_STR, (void *)&value);
	if (value != NULL) {
	gtk_entry_set_text(GTK_ENTRY(*entry), value);
		j_free(value);
	}

	colabel = gtk_label_new(" ");
	eventbox = gtk_event_box_new();
	
	entry_text = (void *)gtk_entry_get_text(GTK_ENTRY(*entry));
	DEBUG_MSG(DBG_INFO, "entry text is %s", entry_text);
		
	gdk_color_parse((entry_text && strlen(entry_text) > 3) ? entry_text : "black", &color);
	gdk_colormap_alloc_color(gdk_colormap_get_system(), &color, 0, 1);
	gtk_widget_modify_bg(GTK_WIDGET(eventbox), GTK_STATE_NORMAL, &color);

	param = j_malloc(sizeof(char *) * 3);  // freed by gui_colours_button_cleanup_cb()
	param[0] = *entry;
	param[1] = eventbox;
	param[2] = NULL;
	
	gtk_container_add(GTK_CONTAINER(eventbox), colabel);
	g_signal_connect(G_OBJECT(eventbox), "button_press_event", G_CALLBACK(gui_colours_button_select_colour_cb), param);
	g_signal_connect(G_OBJECT(eventbox), "destroy", G_CALLBACK(gui_colours_button_cleanup_cb), param);
	
	gtk_table_attach(table, label, 0, 1, row, row+1, 0, 0, 1, 0);
	gtk_table_attach_defaults(table, *entry, 1, 2, row, row+1);
	gtk_table_attach_defaults(table, eventbox, 2, 3, row, row+1);
	
	return;
}

void
gui_preferences_filler_interface_colours(GtkWidget *frame)
{
	GtkWidget *align = NULL;
	GtkTable *table = NULL;
	

	align = gtk_alignment_new(0, 0, 1, 0);
	table = (GtkTable *)gtk_table_new(6, 3, FALSE);
	gtk_table_set_row_spacings(table, 3);

	
	/* bg colour */
	gui_preferences_colour_setup(table, 0,
					"Background colour: ",
					"logwin_colour", &logwin_colour_entry);
	
	
	/* miscellaneous colour */
	gui_preferences_colour_setup(table, 1,
					"Colour of messages: ",
					"logwindow_colour_misc", &logwin_misc_colour_entry);
	
	/* ftp misc */
	gui_preferences_colour_setup(table, 2,
					"Colour of ftp misc messages: ",
					"logwindow_colour_ftp_misc", &logwin_ftp_misc_colour_entry);
	
	/* ftp sent */
	gui_preferences_colour_setup(table, 3,
					"Colour of sent ftp messages: ",
					"logwindow_colour_ftp_sent", &logwindow_ftp_sent_colour_entry);
	
	/* ftp recv */
	gui_preferences_colour_setup(table, 4,
					"Colour of received ftp messages: ",
					"logwindow_colour_ftp_recv", &logwindow_ftp_recv_colour_entry);
	

	/* error colour */
	gui_preferences_colour_setup(table, 5,
					"Colour of error messages: ",
					"logwindow_colour_error", &logwin_error_colour_entry);

	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(align));
	gtk_container_add(GTK_CONTAINER(align), GTK_WIDGET(table));

	return;
}



void
gui_preferences_filler_interface_themes(GtkWidget *frame)
{
	GtkTreeSelection  *select   = 0;
	GtkTreeStore      *store    = 0;
	GtkCellRenderer   *renderer = 0;
	GtkTreeModel      *model    = 0;
	GtkTreeView       *tree     = 0;
	GtkTreeViewColumn *column   = 0;
	GtkTreeIter        iter;
	GtkWidget         *vbox     = gtk_vbox_new(0, 0);
	char *theme = NULL;
	
	store = gtk_tree_store_new(1, G_TYPE_STRING);
	model = GTK_TREE_MODEL(store);
	tree  = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
	gtk_tree_view_set_headers_visible(tree, 0);
	
	/* column for the name */
	renderer = gtk_cell_renderer_text_new();
	column   = gtk_tree_view_column_new_with_attributes(
				"Themes", renderer, "text", 0, NULL);
	gtk_tree_view_append_column(tree, column);

	/* selection changed */
	select = gtk_tree_view_get_selection(tree);
	g_signal_connect(G_OBJECT(select), "changed",
			G_CALLBACK(gui_prefs_theme_select_cb), vbox);

	config_read_value("theme", T_STR, (void **)&theme);

	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(tree), 0, 0, 0);
	
	{
		char          *j;
		DIR           *dir = NULL;
		struct dirent *ent = NULL;

		if(! (dir = opendir(DATADIR "/junkie/themes/")) ) {
			j_free(theme);
 			return;
		}
			
		while( (ent = readdir(dir)) ) {
			if(*ent->d_name == '.')	continue;
			j = strrchr(ent->d_name, '.');
			if(j)	*j = 0;

			gtk_tree_store_append(store, &iter, NULL);
			gtk_tree_store_set(store, &iter, 0, ent->d_name, -1);

			/* Select the current theme. */
			if (theme && !strcasecmp(theme, ent->d_name))
				gtk_tree_selection_select_iter(select, &iter);

		}
		closedir(dir);
	}
		
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	j_free(theme);
	return;
}


#ifdef HAVE_SOUND
/*
 * "toogle" signal
 */
int
gui_preferences_sound_startup_toggle_cb(GtkObject *check, gpointer func_data)
{
	GtkWidget **upnt = (GtkWidget **)func_data;
	GtkWidget *path_entry = upnt[0];
	GtkWidget *label = upnt[1];
	int mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));

	if (mode == TRUE) {
		/* use a startup sound. */
		gtk_widget_show(path_entry);
		gtk_widget_show(label);
	} else {
		/* disable startup sound. */
		gtk_widget_hide(path_entry);
		gtk_widget_hide(label);
	}

	return 0; 
}

/*
 * callback that frees some allocated data when a widget is destroyed.
 */
int
gui_preferences_sound_free(GtkObject *obj, gpointer data)
{
	GtkWidget **upnt = data;

	if (upnt) 
		j_free(upnt);
	
	return 0;
}


/*
 * this is a "hack"  this function is called when the check widget is first shown, and we call
 * a function to hide it if appropriate.  if we would hide it in gui_preferences_filler_interface_sound, it would
 * be shown later so it would not have any effect.
 * This function is called only once, cause it also removes the signal.
 */
int
gui_preferences_sound_call_first_cb(GtkObject *obj, gpointer data)
{
	GtkWidget **upnt = data;
	
        gui_preferences_sound_startup_toggle_cb(GTK_OBJECT(sound_startup_check), upnt);
	gtk_signal_disconnect_by_func(obj, G_CALLBACK(gui_preferences_sound_call_first_cb), data);

	return 0;
}

void
gui_preferences_filler_interface_sound(GtkWidget *frame)
{

	GtkWidget *vbox = NULL;
	GtkWidget *label = NULL;
	GtkWidget *hbox1 = NULL, *hbox2 = NULL, *hbox3 = NULL, *hbox4 = NULL;
	GtkWidget **upnt = NULL;
	int sound_enable = config_read_bool("sound");
	char *str = NULL;

	vbox = gtk_vbox_new(FALSE, 0);
	hbox1 = gtk_hbox_new(FALSE, 0);

	/* Checkbox: Use sounde? */
	sound_check = gtk_check_button_new_with_label("Enable Sound?");
	if (sound_enable) {
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sound_check), TRUE);
	} else {
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sound_check), FALSE);
	}

	gtk_box_pack_start(GTK_BOX(hbox1), sound_check, FALSE, FALSE, 0);

	/* setup startup sound stuff */
	hbox2 = gtk_hbox_new(FALSE, 0);
	hbox3 = gtk_hbox_new(FALSE, 0);
	sound_startup_check = gtk_check_button_new_with_label("Use a startup sound?");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sound_startup_check),
		config_read_bool("sound_startup_enable") ? TRUE : FALSE);

	label = gtk_label_new("Startup Song: ");
	sound_startup_path_entry = gtk_entry_new();
	
	config_read_value("sound_startup_path", T_STR, (void **)&str);
	if (str != NULL) {
		gtk_entry_set_text(GTK_ENTRY(sound_startup_path_entry), str);
		j_free(str);
	}
	
	gtk_box_pack_start(GTK_BOX(hbox2), sound_startup_check, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox3), label, TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox3), sound_startup_path_entry, TRUE, FALSE, 0);

	// setup a toggle cb
	upnt = j_malloc(3 * sizeof(GtkWidget *));
	if (!upnt)
		return;
		
	upnt[0] = sound_startup_path_entry;
	upnt[1] = label;
	upnt[2] = NULL;	
	gtk_signal_connect(GTK_OBJECT(sound_startup_check), "toggled", G_CALLBACK(gui_preferences_sound_startup_toggle_cb), upnt);
	gtk_signal_connect(GTK_OBJECT(sound_startup_check), "destroy", G_CALLBACK(gui_preferences_sound_free), upnt);
	gtk_signal_connect_after(GTK_OBJECT(frame), "show", G_CALLBACK(gui_preferences_sound_call_first_cb), upnt);

	/* Checkbox: show sound playr? */
	hbox4 = gtk_hbox_new(FALSE, 0);
	show_player = gtk_check_button_new_with_label("show sound player?");
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(show_player),
			                    config_read_bool("show_player"));
	gtk_box_pack_start(GTK_BOX(hbox4), show_player, FALSE, FALSE, 0);
	
	
	/* pack all hboxes into the vbox */
	gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox3, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox4, FALSE, FALSE, 0);
	
	gtk_container_add(GTK_CONTAINER(frame), vbox);

	return;
}
#endif /* eof HAVE_SOUND*/

void gui_prefs_theme_select_cb(GtkTreeSelection *select, GtkWidget *box)
{
	gui_theme_header *current_theme;
	GtkTreeIter   iter;
	GtkTreeModel *model;
	GList        *slist;
	int i;
	gchar     *theme;
	GtkWidget *hbox, *vbox;
	GtkWidget *img;
	gchar     *list_menu[] = { "connect_before.png", "connect_after.png",
		                       "preferences.png", "siteman.png", 
						       "refresh_before.png", "refresh_after.png",
							   "stop.png" };
	gchar     *list_tview[] = { "folder.png", "file.png", 
						        "file_package.png", "file_movie.png",
						        "file_doc.png", "file_sound.png", 
								"file_image.png", "cdup.png"};
	gchar     *list_site[] = { "group.png", "site.png" };

	gtk_tree_selection_get_selected(select, &model, &iter);
	gtk_tree_model_get(model, &iter, 0, &theme, -1);

	current_theme = gui_theme_get_current();
	gui_theme_load(theme);
	global_theme = theme;	
	
	slist = gtk_container_get_children(GTK_CONTAINER(box));
	slist = g_list_nth(slist, 1);
	if (slist)
		gtk_container_remove(GTK_CONTAINER(box), slist->data);
	
	vbox = gtk_vbox_new(0, 0);
	
	hbox = gtk_hbox_new(0, 4);
	for(i = 0; i < 7; i++) {
		img = gui_theme_image_get(list_menu[i]);
		if(img)
			gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(img), 0, 0, 0);
	}	
	gtk_box_pack_end(GTK_BOX(vbox), hbox, 0, 0, 1);

	hbox = gtk_hbox_new(0, 4);
	for(i = 0; i < 8; i++) {
		img = gui_theme_image_get(list_tview[i]);
		if(img)
			gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(img), 0, 0, 0);
	}
	gtk_box_pack_end(GTK_BOX(vbox), hbox, 0, 0, 1);
	
	hbox = gtk_hbox_new(0, 4);
	for(i = 0; i < 2; i++) {
		img = gui_theme_image_get(list_site[i]);
		if(img)
			gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(img), 0, 0, 0);
	}
	gtk_box_pack_end(GTK_BOX(vbox), hbox, 0, 0, 1);
	
	gui_theme_close();
	gui_theme_set_current(current_theme);
		
// XXX something not being freed?
	gtk_container_add(GTK_CONTAINER(box), vbox);
	gtk_widget_show_all(vbox);
	return;
}


void
gui_preferences_save_options()
{
	/* save settings through the config lib. */
	int cval;

	config_set_bool("usebinary", gtk_option_menu_get_history(GTK_OPTION_MENU(trans_menu)) ? 0 : 1);
	config_set_bool("usepassive", gtk_option_menu_get_history(GTK_OPTION_MENU(conn_menu)) ? 0 : 1);

	
	config_set_bool("cache-enable", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cache_check)) ? 1 : 0);
	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cache_check))) 
		config_set_bool("cache-session", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cache_session_check)) ? 1 : 0);
	else
		config_set_bool("cache-session", 0);
	
	config_set_bool("queue-keep",
			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(queue_keep)));
	
	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sound_check)))
	        sound_stop();
#ifdef HAVE_SOUND
	config_set_bool("sound", 
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sound_check)) ? 1 : 0);
	config_set_bool("sound_startup_enable", 
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sound_startup_check)) ? 1 : 0);
	config_set_value("sound_startup_path",  T_STR, 
		(void *)gtk_entry_get_text(GTK_ENTRY(sound_startup_path_entry)));
	cval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(show_player));
	config_set_bool("show_player", cval);
	cval ? gui_player_show() : gui_player_hide();
#endif
	
	config_set_num("transfer-limit",
			gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(limit_spin))*1024);
	cval = gtk_option_menu_get_history(GTK_OPTION_MENU(exist_menu));
	config_set_num("file-exist", cval == 0 ? 4 : cval == 1 ? 2 : cval == 2 ? 8 : 16);
	config_set_value("defuser", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(user_entry)));
	config_set_value("anonpass", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(pass_entry)));
	config_set_num("hammering", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(hammering_spin)));
	config_set_num("anti-idler", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(antiidle_spin)));

	config_set_value("imgeditor", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(imgedit_entry)));
	config_set_value("txteditor", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(txtedit_entry)));
	config_set_value("imgviewer", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(imgview_entry)));
	config_set_value("txtviewer", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(txtview_entry)));
	
	/* list method */
	config_set_num("list_method", gui_preferences_list_method_get_id(GTK_OPTION_MENU(list_method_option_menu)));
	config_set_value("list_method_custom", T_STR, 
		(void *)gtk_entry_get_text(GTK_ENTRY(list_custom_entry))); 
	config_set_num("defport", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(port_spin)));
	config_set_num("cache-days", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(cache_days_spin)));
	config_set_value("theme", T_STR, global_theme);
	config_set_value("logwin_colour", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(logwin_colour_entry)));
	config_set_value("logwindow_colour_error", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(logwin_error_colour_entry)));
	config_set_value("logwindow_colour_ftp_misc", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(logwin_ftp_misc_colour_entry)));
	config_set_value("logwindow_colour_ftp_sent", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(logwindow_ftp_sent_colour_entry)));
	config_set_value("logwindow_colour_ftp_recv", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(logwindow_ftp_recv_colour_entry)));
	config_set_value("logwindow_colour_misc", T_STR, (void *)gtk_entry_get_text(GTK_ENTRY(logwin_misc_colour_entry)));
	
	return;
}


