/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_qwin.c,v 1.53 2003/03/07 17:48:13 thepin Exp $
 */


#define _GNU_SOURCE

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
/* local headers */
#include <gui_qwin.h>
#include <gui_qwin_popup.h>
#include <gui_popup.h>
#include <gui_treestore.h>
#include <gui_theme.h>
#include <gui_misc.h>
#include <ftp_transfer.h>
#include <queue.h>
#include <mem.h>
#include <configure.h>
#include <misc.h>
/* icons */
#include <icons/queue_si.xpm>
#include <icons/queue_ni.xpm>
#include <icons/queue_so.xpm>
#include <icons/queue_no.xpm>
#include <icons/queue_dir.xpm>


static GtkWidget *q_window = NULL;

GtkWidget *
gui_qwin_treeview()
{
	return q_window;
}

void
gui_qwin_model_set(_ftp_item *ftp, GtkTreeView *tree)
{
	GtkTreeModel *model = NULL;

	if(!ftp)	return;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
	ftp_object_set_data(FTP_OBJECT(ftp), "gui_qwin_model", model);
}

GtkTreeModel *
gui_qwin_model_get(_ftp_item *ftp)
{
	if(!ftp)	return NULL;

	return ftp_object_get_data(FTP_OBJECT(ftp), "gui_qwin_model");
}

void
gui_qwin_model_show(_ftp_item *ftp)
{
	GtkTreeModel *model;
	
	if(ftp)	model = gui_qwin_model_get(ftp);
	else	model = gui_qwin_model_new();

	gtk_tree_view_set_model(GTK_TREE_VIEW(q_window), model);
}
	
GtkTreeModel *
gui_qwin_model_new()
{
	GtkTreeStore      *lst = 0;

	lst = gtk_tree_store_new(7,
			GDK_TYPE_PIXBUF, /* status      */
			G_TYPE_STRING,   /* type        */
			G_TYPE_STRING,   /* local file  */
			G_TYPE_INT,      /* size        */
			G_TYPE_STRING,   /* remote file */
			G_TYPE_STRING,   /* site:port   */
			G_TYPE_STRING);  /* username    */
	return GTK_TREE_MODEL(lst);
}

GtkWidget *
gui_qwin_setup(GtkWidget *window)
{
	GtkWidget         *scrolled = 0, *qmenu = 0;
	GtkTreeModel      *model = 0;
	GtkTreeView       *tree = 0;
	GtkTreeViewColumn *column = 0;
	GtkCellRenderer   *renderer = 0;

	model = gui_qwin_model_new();
	tree = (GtkTreeView *)gtk_tree_view_new_with_model(model);

	
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(tree),
			                    GTK_SELECTION_MULTIPLE);

	renderer = gtk_cell_renderer_pixbuf_new();
	/* column for queue status                      */
	column = gtk_tree_view_column_new_with_attributes(
			"",         renderer, "pixbuf", 0, NULL);
	gtk_tree_view_append_column(tree, column);
	
	renderer = gtk_cell_renderer_text_new();
	/* column for transfer type (upload | download) */
	column = gtk_tree_view_column_new_with_attributes(
			"Transfer", renderer, "text",   1, NULL);
	gtk_tree_view_append_column(tree, column);
	/* column for local file path                   */
	column = gtk_tree_view_column_new_with_attributes(
			"Local",    renderer, "text",   2, NULL);
	gtk_tree_view_append_column(tree, column);
	gtk_tree_view_column_set_resizable(column, 1);
	/* column for size                              */
	column = gtk_tree_view_column_new_with_attributes(
			"Size",     renderer, "text",   3, NULL);
	gtk_tree_view_append_column(tree, column);
	gtk_tree_view_column_set_resizable(column, 1);
	/* column for remote file path                  */
	column = gtk_tree_view_column_new_with_attributes(
			"Remote",   renderer, "text",   4, NULL);
	gtk_tree_view_append_column(tree, column);
	gtk_tree_view_column_set_resizable(column, 1);
	/* column for site:port                         */
	column = gtk_tree_view_column_new_with_attributes(
			"Site",     renderer, "text",   5, NULL);
	gtk_tree_view_append_column(tree, column);
	gtk_tree_view_column_set_resizable(column, 1);
	/* column for username                          */
	column = gtk_tree_view_column_new_with_attributes(
			"Username", renderer, "text",   6, NULL);
	gtk_tree_view_append_column(tree, column);
	gtk_tree_view_column_set_resizable(column, 1);

	/* setup menu on right click */
	qmenu = gui_qwin_popup_init();
	g_signal_connect(G_OBJECT(tree), "button_press_event", 
			G_CALLBACK(gui_menu_rightclick_cb), qmenu);
	
	/* setup editing on double left click */
	g_signal_connect(G_OBJECT(tree), "row_activated",
			G_CALLBACK(gui_qwin_double_click_cb), NULL);
	
	scrolled = gui_setup_scrolled_window(GTK_POLICY_AUTOMATIC,
			                             GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(scrolled), GTK_WIDGET(tree));
	q_window = GTK_WIDGET(tree);

//	head = queue_init();
//	queue_debug_list(head);
//	if(head)	gui_treestore_fill(gtk_tree_view_get_model(tree), 
//			                       head, NULL, gui_qwin_fill_cb); 
	
	return GTK_WIDGET(scrolled);
}

int
gui_qwin_queue_add(_ftp_item *ftp, struct _list *ent, int local, int trfr)
{
	char cwd[PATH_MAX + 1];
	_queue_item *qitem = 0;
	ftp_request *req = 0;

	if(!ent)                	return -1;
	if(!getcwd(cwd, PATH_MAX))	return -1;

	if(J_ISLINK(ent->type));
	/* XXX link XXX
	 | decide how to handle links,
	 | they might go outt of the directory...
	 | thus pwd would not do much good.
	 */
	else if(J_ISFILE(ent->type)) {
		qitem = queue_item_new(ftp, ent, 
				               ftp ? ftp->dirinfo->pwd : 0, cwd,
				               NULL, 
							   trfr);
		gui_treestore_append_list(GTK_TREE_VIEW(q_window), qitem, "");
		gui_treestore_append(GTK_TREE_VIEW(q_window), qitem, "",
				             gui_qwin_fill_cb);
	}
	else if(J_ISDIR(ent->type)) { /* adding a whole directory */
		if(!strcmp(ent->file, ".") || !strcmp(ent->file, ".."))	return -1;
		
		if(local) {
			qitem = queue_dir_local_new(ftp, 
					                    cwd, ftp ? ftp->dirinfo->pwd : NULL,
										ent->file,
										trfr);
			gui_treestore_append_list(GTK_TREE_VIEW(q_window), qitem, "");
			gui_qwin_refresh(gtk_tree_view_get_model(GTK_TREE_VIEW(q_window)));
		}
		else {
			qitem = queue_dir_remote_new(ftp,
					                     cwd, ftp->dirinfo->pwd,
										 ent->file,
										 trfr);
			gui_treestore_append_list(GTK_TREE_VIEW(q_window), qitem, "");
			req = ftp_request_add(ftp, RP_END, NULL, 0, 
					        gui_qwin_recurse_remote_cb, 0);
			ftp_object_set_data(FTP_OBJECT(req), "model", 
					gtk_tree_view_get_model(GTK_TREE_VIEW(q_window)));
		}
	}		
	
	return 0;
}
					     
FTP_CB_CMD(gui_qwin_recurse_remote_cb)
{
	gui_qwin_refresh(ftp_object_get_data(obj, "model"));
}

void
gui_qwin_fill_cb(GtkTreeModel *model, _queue_item *qiter, GtkTreeIter *iter)
{
	gchar *items[7] = {0, 0, 0, 0, 0, 0, 0};
	GdkPixbuf *pix,
	          *pixdir, *pixnone, *pixqueued, *pixtransferring, *pixtransfered;
	
	if( !(pixdir          = gui_theme_pixbuf_get("queue_dir.png")) )
		pixdir          = gdk_pixbuf_new_from_xpm_data(queue_dir_xpm);
	if( !(pixnone         = gui_theme_pixbuf_get("queue_no.png")) )
		pixnone         = gdk_pixbuf_new_from_xpm_data(queue_no_xpm);
	if( !(pixqueued       = gui_theme_pixbuf_get("queue_ni.png")) )
		pixqueued       = gdk_pixbuf_new_from_xpm_data(queue_ni_xpm);
	if( !(pixtransferring = gui_theme_pixbuf_get("queue_so.png")) )
		pixtransferring = gdk_pixbuf_new_from_xpm_data(queue_so_xpm);
	if( !(pixtransfered   = gui_theme_pixbuf_get("queue_si.png")) )
		pixtransfered   = gdk_pixbuf_new_from_xpm_data(queue_si_xpm);
	
	switch (qiter->t_mode) {
		case FTP_TRANSFER_UPLOAD:
			items[1] = "Upload";
			break;
		case FTP_TRANSFER_DOWNLOAD:
			items[1] = "Download";
			break;
		default:
			items[1] = "?";
			break;
	}
	items[2] = qiter->local_path;
	items[3] = (char *)qiter->size;
	items[4] = qiter->remote_path;
	if (qiter->site)
		items[5] = j_asprintf("%s:%d", qiter->site, qiter->port);

	items[6] = qiter->user;
	
	if (J_ISDIR(qiter->type))
		pix = pixdir;
	else switch(qiter->status) {
		case Q_NONE:
			pix = pixnone;
			break;
		case Q_QUEUED:
			pix = pixqueued;
			break;
		case Q_TRANSFERRING:
			pix = pixtransferring;
			break;
		case Q_TRANSFERED:
			pix = pixtransfered;
			break;
	}
	
	gtk_tree_store_set(GTK_TREE_STORE(model), iter,
			0, pix,
			1, items[1],
			2, items[2],
			3, items[3],
			4, items[4],
			5, items[5],
			6, items[6],
			-1);
	
	free(items[5]);
	g_object_unref(pixdir); g_object_unref(pixnone); g_object_unref(pixqueued);
	g_object_unref(pixtransferring); g_object_unref(pixtransfered);
}

void
gui_qwin_pixbuf_update(GtkTreeModel *model, _queue_item *root, GtkTreeIter *iter)
{
	GdkPixbuf    *pix,
	             *pixdir, *pixnone, *pixqueued, *pixtransferring, *pixtransfered;

	if( !(pixdir          = gui_theme_pixbuf_get("queue_dir.png")) )
		pixdir          = gdk_pixbuf_new_from_xpm_data(queue_dir_xpm);
	if( !(pixnone         = gui_theme_pixbuf_get("queue_no.png")) )
		pixnone         = gdk_pixbuf_new_from_xpm_data(queue_no_xpm);
	if( !(pixqueued       = gui_theme_pixbuf_get("queue_ni.png")) )
		pixqueued       = gdk_pixbuf_new_from_xpm_data(queue_ni_xpm);
	if( !(pixtransferring = gui_theme_pixbuf_get("queue_so.png")) )
		pixtransferring = gdk_pixbuf_new_from_xpm_data(queue_so_xpm);
	if( !(pixtransfered   = gui_theme_pixbuf_get("queue_si.png")) )
		pixtransfered   = gdk_pixbuf_new_from_xpm_data(queue_si_xpm);
	

	if(!iter) {
		iter = j_malloc(sizeof(GtkTreeIter));
		gtk_tree_model_get_iter_first(model, iter);
	}

	if(!root)	root = gui_treestore_get_list(model);
	while(root) {
		if(J_ISDIR(root->type)) pix = pixdir;
		else switch(root->status) {
			case Q_NONE:
				pix = pixnone;
				break;
			case Q_QUEUED:
				pix = pixqueued;
				break;
			case Q_TRANSFERRING:
				pix = pixtransferring;
				break;
			case Q_TRANSFERED:
				pix = pixtransfered;
				break;
		}
		gtk_tree_store_set(GTK_TREE_STORE(model), iter, 0, pix, -1);
		
		if(root->child) {
			GtkTreeIter *child = j_malloc(sizeof(GtkTreeIter));
			gtk_tree_model_iter_children(model, child, iter);
			gui_qwin_pixbuf_update(model, root->child, child);
		}
		
		root = root->next;
		gtk_tree_model_iter_next(model, iter);
	}

	g_object_unref(pixdir); g_object_unref(pixnone); g_object_unref(pixqueued);
	g_object_unref(pixtransferring); g_object_unref(pixtransfered);
	j_free(iter);
}


void
gui_qwin_refresh(GtkTreeModel *model)
{

	gtk_tree_store_clear(GTK_TREE_STORE(model));
	gui_treestore_fill(model, gui_treestore_get_list(model),
						   NULL, gui_qwin_fill_cb);
}


/*
 | for each entry selected, loads all selected items into a linked list
 */
void
gui_qwin_return_selected(GtkTreeModel *model, 
		                 GtkTreePath *path, GtkTreeIter *iter,
						 gpointer data)
{
	GtkTreeRowReference *row     = NULL;
	char                *index   = data;

	row = gtk_tree_row_reference_new(model, path);
	g_object_set_data(G_OBJECT(model), index,
			g_slist_append(g_object_get_data(G_OBJECT(model), index), row));

	return;
}

void
gui_qwin_double_click_cb(GtkTreeView *tree, 
		                 GtkTreePath *path, 
						 GtkTreeViewColumn *column,
						 gpointer data)
{
	GList *list, *lpnt;
	GtkCellRenderer *cell = NULL;
	int i;
	
	list = gtk_tree_view_get_columns(GTK_TREE_VIEW(q_window));
	for(i = 0, lpnt = list; lpnt; lpnt = g_list_next(lpnt), i++)
		if(lpnt->data == column)    break;
	g_list_free(list);
	
	if(i == 0 || i == 1 || i == 3)	return;
	
	cell = gtk_tree_view_column_get_cell_renderers(column)->data;
	g_object_set(G_OBJECT(cell), "editable", TRUE, NULL);
	g_signal_connect(G_OBJECT(cell), "edited",
			G_CALLBACK(gui_qwin_cell_edited), (gpointer)i);
	gtk_tree_view_set_cursor(tree, path, column, 1);
}

void
gui_qwin_cell_edited(GtkCellRendererText *cell, 
		             gchar *path_string,
		             gchar *new_text, 
					 gpointer data)
{
	GtkTreeModel *model;
	GtkTreePath  *path;
	GtkTreeIter   iter;
	int i, handler;
	char *j;
	_queue_item *item;
	
	g_object_set(G_OBJECT(cell), "editable", FALSE, NULL);
	handler = g_signal_handler_find(G_OBJECT(cell), G_SIGNAL_MATCH_FUNC, 
			           0, 0, 0, gui_qwin_cell_edited, 0);
	g_signal_handler_disconnect(G_OBJECT(cell), handler);

	path = gtk_tree_path_new_from_string(path_string);
	item = gui_treestore_get_list_from_path(GTK_TREE_VIEW(q_window), path);
	
	i = (int)data;	
	switch(i) {
		case 2:
			j_free(item->local_path);
			item->local_path = strdup(new_text);
			break;
		case 4:
			j_free(item->remote_path);
			item->remote_path = strdup(new_text);
			break;
		case 5:
			j_free(item->site);
			j = strrchr(new_text, ':');
			if(j) {
				*j = 0;
				item->port = atol(j+1);
			}
			item->site = strdup(new_text);
			if(j)	*j = ':';
			break;
		case 6:
			j_free(item->user);
			item->user = strdup(new_text);
			break;
	}
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(q_window));
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_store_set(GTK_TREE_STORE(model), &iter, i, new_text, -1);
}
			
			
	
void
gui_qwin_remove_if_transfered(_queue_item **item, 
							  t_iter *iter,
							  int value,
							  GtkTreeModel *model)
{
	_queue_item  *parent, *next;
	_queue_item  *head = gui_treestore_get_list(model);
	
	DEBUG_MSG(DBG_INFO, "removing TRANSFERED %s", (*item)->local_path);
	
	if( ((*item)->status == Q_TRANSFERED) || value ) { 
		parent = (*item)->parent;
		next   = (*item)->next;
		g_object_set_data(G_OBJECT(model), "list",
				jtree_remove(head, *item, queue_free));
	DEBUG_MSG(DBG_INFO, "print %p, %p", parent, parent ? parent->child : NULL);
	
//		if(parent && !parent->child) {
		//	t_iter *parent_iter = jtree_iter_parent(iter);
		//	DEBUG_MSG(DBG_INFO, "die");
		//	*item = parent;
		//	gui_qwin_remove_if_transfered(item, parent_iter, (gpointer)0x1);
		//	jtree_iter_free(parent_iter);
//		}

		if(*item) {
			if(next) {
				DEBUG_MSG(DBG_INFO, "setting to 'next'");
			}
			else if (parent) {
				DEBUG_MSG(DBG_INFO, "setting to 'parent'");
				*item = parent;
			}
			else {
				DEBUG_MSG(DBG_INFO, "setting to NULL");
				*item = NULL;
			}
		}
	}

	queue_debug_list(gui_treestore_get_list(model));
}
