/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_qwin_popup.c,v 1.14 2003/03/03 02:16:33 tgucio Exp $
 */


#include <gtk/gtk.h>
/* local headers */
#include <gui_qwin_popup.h>
#include <gui_qwin.h>
#include <gui_popup.h>
#include <gui_treestore.h>
#include <gui_tview.h>
#include <gui_ftp.h>
#include <gui_ftp_lookup.h>
#include <gui_widgets.h>
#include <jlist.h>
#include <mem.h>

GtkWidget *
gui_qwin_popup_init()
{
	GtkWidget *popup_menu = NULL;

	popup_menu = gui_popup_new_tearoff(gui_qwin_popup_update);
	
	gui_popup_add_item(popup_menu, "  Remove Entry   ",
			G_CALLBACK(gui_qwin_popup_remove), NULL);
	gui_popup_add_item(popup_menu, "  Submit All Queue   ",
			G_CALLBACK(gui_qwin_popup_submit_queue), NULL);
	gui_popup_add_item(popup_menu, "  Submit Selected",
			G_CALLBACK(gui_qwin_popup_submit_selected), NULL);

	return popup_menu;
}

void
gui_qwin_popup_update(GtkWidget *menu)
{
	GtkMenuItem *item1 = 0, *item2 = NULL;
	GList       *children;

	
	children = GTK_MENU(menu)->menu_shell.children;
	item1 = g_list_nth_data(children, 2);
	item2 = g_list_nth_data(children, 3);
	if(!item1 || !item2)	return;

	if(!g_object_get_data(G_OBJECT(gtk_tree_view_get_model(
						   GTK_TREE_VIEW(gui_lookup_widget("right_tview")))),
				         "freeze")) {
		gtk_widget_set_sensitive(GTK_WIDGET(item1), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(item2), TRUE);
	}
	else {
		gtk_widget_set_sensitive(GTK_WIDGET(item1), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(item2), FALSE);
	}
}

void
gui_qwin_popup_remove(GtkWidget *item, gpointer upointer)
{
	GSList           *glist     = NULL;
	GtkWidget        *q_window  = NULL;
	GtkTreeModel     *model     = NULL;
	GtkTreeSelection *selection = NULL;

	q_window = gui_qwin_treeview();
	model    = gtk_tree_view_get_model(GTK_TREE_VIEW(q_window));
	
	g_slist_free(g_object_get_data(G_OBJECT(model), "queue_rem_list"));
	g_object_set_data(G_OBJECT(model), "queue_rem_list", NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(q_window));
	gtk_tree_selection_selected_foreach(selection,
			(GtkTreeSelectionForeachFunc) gui_qwin_return_selected,
			"queue_rem_list");

	glist = g_object_get_data(G_OBJECT(model), "queue_rem_list");
	for(; glist; glist = g_slist_next(glist)) {
		GtkTreeIter iter;
		GtkTreePath *path = gtk_tree_row_reference_get_path(glist->data);
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
		gui_treestore_remove_list(GTK_TREE_VIEW(q_window), path,
				queue_free);
	}
}

void
gui_qwin_popup_submit_queue(GtkWidget *item, gpointer upointer)
{
	int           totsize  = 0;
	GtkTreeModel *model    = NULL;
	GtkWidget    *q_window = NULL;
	jslist       *jlist    = NULL;
	_queue_item  *list     = NULL,
	             *q        = NULL;
	
	q_window = gui_qwin_treeview();
	model    = gtk_tree_view_get_model(GTK_TREE_VIEW(q_window));
	list     = gui_treestore_get_list(model);
	
	for(q = list; q; q = q->next) {
		jlist = queue_list_append(jlist, q, 0);
		totsize += q->size;
	}
	gui_qwin_pixbuf_update(model, list, 0);
	gui_qwin_popup_submit_items(jlist, totsize);
}

void
gui_qwin_popup_submit_selected(GtkWidget *item, gpointer upointer)
{
	int               totsize   = 0;
	GSList           *glist     = NULL;
	jslist           *jlist     = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreePath      *path      = NULL;
	GtkWidget        *q_window  = NULL;
	GtkTreeModel     *model     = NULL;
	
	q_window = gui_qwin_treeview();
	model    = gtk_tree_view_get_model(GTK_TREE_VIEW(q_window));
			
	g_slist_free(g_object_get_data(G_OBJECT(model), "selected"));
	g_object_set_data(G_OBJECT(model), "selected", NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(q_window));
	gtk_tree_selection_selected_foreach(selection,
			(GtkTreeSelectionForeachFunc)gui_qwin_return_selected, 
			"selected");
	glist = g_object_get_data(G_OBJECT(model), "selected");
	for(; glist; glist = g_slist_next(glist)) {
		_queue_item *q;
		path = gtk_tree_row_reference_get_path(glist->data);
		q = gui_treestore_get_list_from_path(GTK_TREE_VIEW(q_window), path);
		jlist = queue_list_append(jlist, q, 0);
		totsize += q->size;
		gtk_tree_path_free(path);
	}
	gui_qwin_pixbuf_update(model, gui_treestore_get_list(model), 0);

	g_print("submiting %p --> %p\n", jlist, jlist->data);
	gui_qwin_popup_submit_items(jlist, totsize);
}

void
gui_qwin_popup_submit_items(jslist *list, int totsize)
{
	GtkWidget      *q_window = NULL;
	_queue_item    *q        = NULL;
	gui_ftp_lookup *lookup   = NULL;
	double         *now      = NULL;

	lookup = gui_ftp_lookup_widget(gui_lookup_widget("right_tview"));
	
	now = j_malloc(sizeof(double));
	*now = time_usec();
	ftp_object_set_data(FTP_OBJECT(lookup->site), 
			"queue_now", now);
	ftp_object_set_data(FTP_OBJECT(lookup->site), 
			"queue_total_size", (void*)totsize);
	ftp_object_set_data(FTP_OBJECT(lookup->site), 
			"queue_current_size",(void*)totsize);
	
	q_window = gui_qwin_treeview();
	q = gui_treestore_get_list(gtk_tree_view_get_model(GTK_TREE_VIEW(q_window)));
	if(!q)	return;
	
	/* block the tview until the queue submission is done */
	gui_tview_handler_block(gui_lookup_widget("right_tview"));
	
	queue_submit(lookup ? lookup->site : NULL, list);
}
	
