/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_siteman.c,v 1.68 2003/03/07 17:48:13 thepin Exp $
 */

#define _GNU_SOURCE
#define _XOPEN_SOURCE

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <string.h>
#include <strings.h>
#include <time.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

/* local headers */
#include <gui_tview.h>
#include <gui.h>
#include <gui_misc.h>
#include <gui_toolkit.h>
#include <gui_widgets.h>
#include <gui_io.h>
#include <gui_ftp_callback.h>
#include <gui_theme.h>
#include <gui_tree_model.h>
#include <gui_ftp.h>
#include <ftp_request.h>
#include <ftp_cmd.h>
#include <ftp.h>
#include <config.h>
#include <error.h>
#include <jfork.h>
#include <misc.h>
#include <gui_siteman.h>
#include <gui_dnd.h>
#include <gui_dnd_tview.h>
#include <gui_qwin.h>
#include <gui_treestore.h>
#include <gui_ftp_lookup.h>
#include <blowfish.h>
#include <jio.h>
#include <mem.h>
/* default icons */
#include <icons/site.xpm>
#include <icons/group.xpm>

//XXX global  ->static
struct slist   *siteaddr;


static GtkWidget *connect;
static gchar     *key;

void
gui_siteman_startup(GtkButton *b, char *lb)
{
	GtkWidget *win = NULL,   *vbox = NULL;
	GtkWidget *label = NULL, *entry = NULL, *button = NULL;
	
	if(! config_read_bool("encrypt"))
		gui_siteman_init();
	
	else {	
//		win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
//		gtk_window_set_transient_for(GTK_WINDOW(win), 
//				GTK_WINDOW(gui_lookup_widget("mainwin")));
//		gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
//		gtk_window_set_modal(GTK_WINDOW(win), TRUE);
//		gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_MOUSE);
//		gtk_window_set_title(GTK_WINDOW(win), "site manager protection");
		win = gui_setup_popup_window("site manager protection", 1, 0);

		
		vbox = gtk_vbox_new(FALSE, 5);
		label = gtk_label_new(lb ? lb : "enter password:");
		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
		entry = gtk_entry_new_with_max_length(20);
		gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
		gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);

		button = gtk_button_new_with_label("ok");
		gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

		gtk_container_set_border_width(GTK_CONTAINER(win), 5);
		gtk_container_add(GTK_CONTAINER(win), vbox);

		g_signal_connect(G_OBJECT(button), "clicked",
				G_CALLBACK(gui_siteman_check_pass), entry);
		g_signal_connect(G_OBJECT(button), "clicked",
				G_CALLBACK(gui_widget_destroy_cb), win);
		gtk_widget_show_all(win);
	}
}

void
gui_siteman_password_handler(GtkWidget *but, char *lbl)
{
	GtkWidget *win = NULL,   *vbox = NULL;
	GtkWidget *label = NULL, *entry = NULL, *button = NULL;
	int encrypt =  config_read_bool("encrypt");

//	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
//	gtk_window_set_modal(GTK_WINDOW(win), TRUE);
//	gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_MOUSE);
//	gtk_window_set_title(GTK_WINDOW(win), "site manager protection");
	win = gui_setup_popup_window("site manager protection", 1, 0);
	vbox = gtk_vbox_new(FALSE, 5);
	label = gtk_label_new(
			lbl ? lbl :
			encrypt ? 
			"enter the current password:" : 
			"enter the new password:");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	entry = gtk_entry_new_with_max_length(20);
	gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);

	button = gtk_button_new_with_label("ok");
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

	gtk_container_set_border_width(GTK_CONTAINER(win), 5);
	gtk_container_add(GTK_CONTAINER(win), vbox);

	if(!encrypt)
		g_signal_connect(G_OBJECT(button), "clicked",
				G_CALLBACK(gui_siteman_set_pass), entry);
	else
		g_signal_connect(G_OBJECT(button), "clicked",
				G_CALLBACK(gui_siteman_change_pass), entry);
	g_signal_connect(G_OBJECT(button), "clicked",
			G_CALLBACK(gui_widget_destroy_cb), win);
	gtk_widget_show_all(win);
}

void
gui_siteman_set_pass(GtkWidget *button, GtkEntry *entry)
{
	char *pass = strdup(gtk_entry_get_text(entry));
	if(*pass) {
		config_set_bool("encrypt", 1);
		config_set_value("id", T_STR, 
				crypt(pass, "pe"));
		key = pass;
	}
	else
		config_set_bool("encrypt", 0);
	gui_siteman_write_sites_cb(NULL, NULL);

}

void
gui_siteman_change_pass(GtkWidget *button, GtkEntry *entry)
{
	char *test;
	char *pass = (char*) gtk_entry_get_text(entry);

	DEBUG_MSG(DBG_INFO, "pass: %s", pass);
		if(!*pass) {	
			gui_siteman_password_handler(button, "wrong password, try again:");
			return;
		}
	
		config_read_value("id", T_STR, (void *)&test);
		
		if(!strcmp(test, crypt(pass, "pe"))) {
			config_set_bool("encrypt", 0);
			gui_siteman_password_handler(button, NULL);
		}
		else
			gui_siteman_password_handler(button, "wrong password, try again:");

}



void
gui_siteman_check_pass(GtkWidget *button, GtkEntry *entry)
{	
	char *test;
	const char *pass = gtk_entry_get_text(entry);
	
	config_read_value("id", T_STR, (void *)&test);
	if(!strcmp(test, crypt(pass, "pe"))) {
		key = strdup(pass);
		gui_siteman_init();
	}
	else
		gui_siteman_startup(0, "wrong password, try again:");
}


	
/*
 * gui_siteman_tree_row_select_cb() modifies the status bar everytime a new
 * selection is made
 */

void
gui_siteman_tree_row_select_cb(GtkTreeSelection * select, GtkWidget * status_bar)
{
	char           *buf;
	struct slist   *site;

	site = gui_treestore_get_list_from_selection(siteaddr->tree);

	if (site) {
		if(site->type == SITEMAN_SITE) {
			gtk_widget_set_sensitive(connect, TRUE);
			buf = j_asprintf("%s@%s:%s", 
					site->userid ? site->userid : FTP_ANON_LOGIN,
					site->hostaddr ? site->hostaddr : "???", 
					site->port ? site->port : "21");
		}
		else {
			gtk_widget_set_sensitive(connect, FALSE);
			buf = j_asprintf("[%s]",
					site->profile);
		}
	}

	gtk_statusbar_push(GTK_STATUSBAR(status_bar), GPOINTER_TO_INT(0), site ? buf : "nothing selected");
}



void
gui_siteman_connect_site_connected_cb(_ftp_item *ftp)
{
	GtkWidget *rtview = gui_lookup_widget("right_tview");

	DEBUG_MSG(DBG_INFO, "gui_siteman_connect_site_connected\n");

	if (!ftp) {
		gui_connect_button_set_connect();
		return;
	}
		
	gui_ftp_lookup_add(ftp, rtview, FTP_MODE_FTP, FTP_STATE_CONNECTED);
	gui_tree_model_set(ftp, GTK_TREE_VIEW(rtview));
	gui_qwin_model_set(ftp, GTK_TREE_VIEW(gui_qwin_treeview()));
	gui_connect_update_button();
	
	/* Send list request und fill the appropriate tview when the response has been retreived. :) */
	gui_ftp_list(ftp, gui_ftp_callback_list, 0, NULL, "widget", rtview);

	return;
}



/*
 * gui_siteman_connect_site_cb() is called when either a row is doubble
 * clicked or the connect button is hit. it basically checks the selected row
 * and acts accordingly.. connecting to the chosen site
 */
gboolean
gui_siteman_connect_site_cb(GtkWidget * widget, gpointer data)
{
//	_ftp_item      *ftp = NULL;
	struct slist   *site = NULL;

	site = gui_treestore_get_list_from_selection(siteaddr->tree);
	if (!site)
		return FALSE;
		
	if ((site == siteaddr) || (site->type == SITEMAN_GROUP)) {
		gtk_tree_view_expand_row(siteaddr->tree,
				gui_tview_get_selected_path(siteaddr->tree), 0);
		if(data)	g_signal_stop_emission_by_name(widget, "row_activated");
		return TRUE;	}
	
	gui_ftp_connect((site->hostaddr && *site->hostaddr ? site->hostaddr   : FTP_LOCALHOST),
	                (site->port     && *site->port     ? atol(site->port) : 21),
			(config_read_bool("usepassive") ? passive : active),
			(site->userid   && *site->userid   ? site->userid     : FTP_ANON_LOGIN),
			(site->passwd   && *site->passwd   ? site->passwd     : FTP_ANON_PASSWD),
			gui_siteman_connect_site_connected_cb);

	return FALSE;
}

gboolean
gui_siteman_keypress_cb(GtkWidget *widget, GdkEventKey *event, GtkWidget *w)
{
	if(event->keyval == 65293)
		if(!gui_siteman_connect_site_cb(widget, NULL))
			gtk_widget_destroy(w);
	return FALSE;
}


/*
 * gui_siteman_tview_list_append_cb() is simply a callback function for
 * gui_siteman_tview_list_append(): it was required since g_signal_connect
 * annoyingly passes the calling widget as the first argument
 */

void
gui_siteman_tview_list_append_cb(gpointer * o, struct slist * site, char *p)
{
	gui_treestore_append(siteaddr->tree, site, p,
                            gui_siteman_list_append_cb);
}


/*
 * gui_siteman_tview_list_append() appends elements visibly onto the tree: it
 * is called by various functions
 */

void
gui_siteman_list_append_cb(GtkTreeModel *model, 
                                 struct slist * site,
                                 GtkTreeIter  *iter)
{
	GdkPixbuf      *pixbuf1, *pixbuf2;


	if ( !(pixbuf1 = gui_theme_pixbuf_get("site.png")) )
		pixbuf1 = gdk_pixbuf_new_from_xpm_data(site_xpm);

	if ( !(pixbuf2 = gui_theme_pixbuf_get("group.png")) )
		pixbuf2 = gdk_pixbuf_new_from_xpm_data(group_xpm);
	
	gtk_tree_store_set(GTK_TREE_STORE(model), iter,
			   0, site->type == SITEMAN_SITE ? pixbuf1 : pixbuf2,
			   1, site->profile,
			   -1);

	g_object_unref(pixbuf1);
	g_object_unref(pixbuf2); 
}

char *
gui_siteman_parse_paths(char **buffer)
{
	char *ptr = NULL, *path = NULL;

	if (!buffer)
		return NULL;
		
	ptr = strchr(*buffer,'*');
	if (ptr) {
		*ptr = 0;
		path = strdup(*buffer);
		*buffer = ptr + 1;
	} else
		 return NULL;

	return path;
}


/*
 * gui_siteman_parse_sites() is called by gui_siteman_read_sites(): it parses
 * the output of "$HOME/.junkie/sites" placing it in the appropriate char*'s
 */

struct slist   *
gui_siteman_parse_sites(char *buffer)
{
//	int             i;
	char           *ptr;
	struct slist   *site;

	site = (struct slist *) j_malloc(sizeof(struct slist));
	if (!site) {
		MEM_ERR("couldn't alloc new_site", ERR_SHOW);
		return gui_siteman_error(site, 0);
	}
	memset(site, 0, sizeof(struct slist));

	ptr = strrchr(buffer, '\n');

	if (ptr)
		*ptr = 0;
	else 
		return gui_siteman_error(site, 1);

	site->type = *buffer - '0'; 
	buffer++;
	
	ptr = strchr(buffer, '=');

	if (ptr) {
		*ptr = 0;
		site->profile = strdup(buffer);
		buffer = ptr + 1;
	} else 
		return gui_siteman_error(site, 2);

	ptr = strchr(buffer, '@');

	if (ptr) {
		*ptr = 0;
		if(*buffer)	site->userid = strdup(buffer);
		buffer = ptr + 1;
	} else
		return gui_siteman_error(site, 3);

	ptr = strchr(buffer, 9);

	if (ptr) {
		*ptr = 0;
		if(*buffer) {
			if(config_read_bool("encrypt"))
				site->passwd = strdup(decrypt_string(key, buffer));
			else
				site->passwd = strdup(buffer);
		 //ENCRYTION 
		//	DEBUG_MSG(DBG_INFO, "reading PASS = %s", key);
		 //for(i = 0; site->passwd[i]; i++)
			//   site->passwd[i] ^= i;
      }
		buffer = ptr + 1;
	} else
		return gui_siteman_error(site, 35);

	ptr = strchr(buffer, ':');

	if (ptr) {
		*ptr = 0;
		site->hostaddr = strdup(buffer);
	} else
		return gui_siteman_error(site, 4);

	if (*(ptr + 1))
		site->port = strdup(ptr + 1);
	else
		site->port = NULL;

	return site;
}

void *
gui_siteman_error(struct slist *site, int error)
{
	if(site) {
		free(site->profile);
		free(site->hostaddr);
		free(site->userid);
		free(site->passwd);
		free(site->port);
		free(site);
	}
	DEBUG_MSG(DBG_ERROR, "WARNING: unable to parse the sites configuration file");
	return NULL;
}

/*
 * gui_siteman_read_sites() is an initialization function. It reads from
 * "$HOME/.junkie/sites" into the linked list appending it visibly onto the
 * tree
 */

int
gui_siteman_read_sites()
{
	FILE           *fhd = NULL;
	char           buffer[MAX_SIZE+1], *buf, *path;
	struct slist   *new_site = NULL;
	GtkTreePath    *treepath;
	GtkTreeIter    iter;
	int            expand;
	GSList         *rowlist = NULL, *slist = NULL, *link;
	GtkTreeRowReference *row;
	
	if (getenv("HOME") != 0) {
		snprintf(buffer, MAX_SIZE, "%s/.junkie/sites", getenv("HOME"));
	} else {
		DEBUG_MSG(DBG_WEIRD, "WARNING: $HOME isn't set, trying .");
		snprintf(buffer, MAX_SIZE, "./.junkie/sites");
	}

	fhd = fopen(buffer, "a+");

	if (!fhd) {
		DEBUG_MSG(DBG_WEIRD, "WARNING: unable to open file %s\n", buffer);
		return 1;
	}

	fseek(fhd, 0L, SEEK_SET);
	
	fgets(buffer, MAX_SIZE, fhd);
	while (!feof(fhd)) {
		
		buf = buffer;
		path     = gui_siteman_parse_paths(&buf);
		if(!path)
			return 1;
		expand = *buf;

		new_site = gui_siteman_parse_sites(++buf);
		if (!new_site)
			return 1;

		new_site->tree = siteaddr->tree;
  
		gui_treestore_append_list(siteaddr->tree, new_site, path);
		iter = gui_treestore_append(siteaddr->tree, new_site, path, 
				                    gui_siteman_list_append_cb);
		if(expand == '+') {
			treepath = gtk_tree_model_get_path(
						gtk_tree_view_get_model(siteaddr->tree),
						&iter);
			slist = g_slist_append(slist, treepath);
		}
		fgets(buffer, MAX_SIZE, fhd);
	}
	
	for(link = slist; link; link = link->next) {
	row = 0;rowlist = 0;
		gtk_tree_view_expand_row(siteaddr->tree, link->data, 0);
		free(link->data);
	}
	g_slist_free(slist);
	return 0;
}


/*
 * gui_siteman_entry_cb() is called to store the information of entries into
 * the appropreate char*'s
 * 
 * XXX	think about changing this to gui_siteman_entry_cb(GtkWidget * button,
 * GtkWidget *entry); and passing this entry by entry
 * g_signal_connect(GTK_ENTRY(entry[0...4]));
 * g_signal_connect(gui_siteman_tview_list_append(site)); this might be more
 * elegant? and gui_tollbox.c appropreate
 */

void
gui_siteman_entry_cb(GtkWidget * button, GtkEntry ** entry)
{
	GtkTreePath    *path;
	struct slist   *site;
	struct slist   *tmp;
	char           *p;

	site = (struct slist *) j_malloc(sizeof(struct slist));

	site->tree = siteaddr->tree;
	site->type = SITEMAN_SITE;
	site->profile = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry[0])));
	site->hostaddr = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry[1])));
	site->userid = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry[2])));
	site->passwd = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry[3])));
	site->port = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry[4])));
	site->next   = NULL;
	site->child  = NULL;
	site->parent = NULL;

	path = gui_tview_get_selected_path(site->tree);
	tmp = gui_treestore_get_list_from_selection(siteaddr->tree);
	
   if(tmp) {	
       if(tmp->type == SITEMAN_GROUP)
		   gtk_tree_path_down(path);
	    if(!gtk_tree_path_up(path)) path = NULL;
	}
   
	p = path ? gtk_tree_path_to_string(path) : strdup("");

   gui_treestore_append_list(siteaddr->tree, site, p ? p : ""); 
	gui_treestore_append(siteaddr->tree, site, p ? p : "",
                             gui_siteman_list_append_cb);

	free(p);

	gui_siteman_write_sites_cb(NULL, NULL);
}


/*
 * gui_siteman_create_site_cb() creates a new element in the linked list
 * chain by opening up a window which allows you to enter the site
 * information
 * 
 * it has some strange implementation such as a static GtkWidget double array
 * and a for loop for creating the entry widgets
 * 
 * XXX find some example for reading entries on button click.. this is fucking
 * crap
 */

void
gui_siteman_create_site_cb(GtkWidget * button, int edit)
{
	gint            entrycount = 0;
	char           *entrylabels[] = {
		"Site Profilename:",
		"Host Name/Address:",
		"User ID:",
		"Password:",
		"Port:",
	NULL};
	GtkWidget      *window;
	GtkWidget      *main_vbox;
	GtkWidget      *hbox;
	GtkWidget      *main_label;
	/* XXX */
	static
	GtkWidget      *entry[5];
	/* XXX */
	GtkWidget      *addbutton;
	struct slist   *site = NULL;

	if(edit)
		if(!(site = gui_treestore_get_list_from_selection(siteaddr->tree)))	
         return;
	
	if(site && site->type == SITEMAN_GROUP) {
		gui_siteman_create_grp(button, edit);
		return;
	}
	
//	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
//	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
//	if (edit)
//		gtk_window_set_title(GTK_WINDOW(window), "Edit Site");
//	else
//		gtk_window_set_title(GTK_WINDOW(window), "Create a New Site");
	window = gui_setup_popup_window(edit ? "edit site" : "create a new site",
			                        1, 0);
	
	main_vbox = gtk_vbox_new(FALSE, 5);

	for (; entrylabels[entrycount]; entrycount++) {
		hbox = gtk_hbox_new(FALSE, 5);
		main_label = gtk_label_new(entrylabels[entrycount]);
		gtk_widget_set_size_request(main_label, 170, -1);
		gtk_box_pack_start(GTK_BOX(hbox), main_label, FALSE, FALSE, 0);
		entry[entrycount] = gtk_entry_new_with_max_length(60);
		gtk_box_pack_start(GTK_BOX(hbox), entry[entrycount], FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, FALSE, 0);
		if(entrycount == 3)
			gtk_entry_set_visibility(GTK_ENTRY(entry[entrycount]), FALSE);
	}

	if (edit)
		addbutton = gtk_button_new_with_label("Apply Changes");
	else
		addbutton = gtk_button_new_with_label("Add Site");

	gtk_box_pack_start(GTK_BOX(main_vbox), addbutton, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(window), main_vbox);

	if (edit) {
		gtk_entry_set_text(GTK_ENTRY(entry[0]), site->profile ? site->profile : "Connection");
		gtk_entry_set_text(GTK_ENTRY(entry[1]), site->hostaddr ? site->hostaddr : "");
		gtk_entry_set_text(GTK_ENTRY(entry[2]), site->userid ? site->userid : FTP_ANON_LOGIN);
		gtk_entry_set_text(GTK_ENTRY(entry[3]), site->passwd ? site->passwd : FTP_ANON_PASSWD);
		gtk_entry_set_text(GTK_ENTRY(entry[4]), site->port ? site->port : "21");
	}
	g_signal_connect(G_OBJECT(addbutton), "clicked",
			G_CALLBACK(edit ? gui_siteman_edit_site_cb : gui_siteman_entry_cb), entry);
	g_signal_connect(G_OBJECT(addbutton), "clicked", G_CALLBACK(gui_widget_destroy_cb), window);

	gtk_widget_show_all(window);
}


void
gui_siteman_widget_destroy_cb(GtkWidget *data, gpointer object)
{
   gui_siteman_free(gui_treestore_get_list(gtk_tree_view_get_model(siteaddr->tree)));
	gui_widget_destroy_cb(data, object);
}


void
gui_siteman_edit_site_cb(GtkWidget *button, GtkEntry **entry)
{
	struct slist *site;
	GtkTreeModel *model;
	GtkTreePath  *path;
	GtkTreeIter   iter;

	site = gui_treestore_get_list_from_selection(siteaddr->tree);
	free(site->profile);
	free(site->hostaddr);
	free(site->userid);
	free(site->passwd);
	free(site->port);
	site->profile  = g_strdup(gtk_entry_get_text(entry[0]));
	site->hostaddr = g_strdup(gtk_entry_get_text(entry[1]));
	site->userid   = g_strdup(gtk_entry_get_text(entry[2]));
	site->passwd   = g_strdup(gtk_entry_get_text(entry[3]));
	site->port     = g_strdup(gtk_entry_get_text(entry[4]));
	
	model = gtk_tree_view_get_model(siteaddr->tree);
	path  = gui_tview_get_selected_path(site->tree);
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
			1, site->profile, -1);
	g_signal_emit_by_name(G_OBJECT(gtk_tree_view_get_selection(siteaddr->tree)), 
			"changed");

	gui_siteman_write_sites_cb(NULL, NULL);
}

gboolean
gui_siteman_create_grp(GtkWidget *button, int edit)
{
	GtkWidget *win;
	GtkWidget *label;
	GtkWidget  *entry;
	GtkWidget *addbutton;
	GtkWidget *vbox;
	struct slist *site;

//	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
//	gtk_window_set_modal(GTK_WINDOW(win), TRUE);
//	gtk_window_set_title(GTK_WINDOW(win), edit ? "Edit Group" : "Create a New Group");
	win = gui_setup_popup_window(edit ? "edit group" : "create a new group", 
			                     1, 0);
	
	vbox = gtk_vbox_new(FALSE, 5);
	label = gtk_label_new("Enter the name of the group:");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	entry = gtk_entry_new_with_max_length(60);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
	if (edit) {
		site = gui_treestore_get_list_from_selection(siteaddr->tree);
		gtk_entry_set_text(GTK_ENTRY(entry), site->profile);
	}

	addbutton = gtk_button_new_with_label(edit ? "Modify" : "Add Group");
	gtk_box_pack_start(GTK_BOX(vbox), addbutton, FALSE, FALSE, 0);

	gtk_container_set_border_width(GTK_CONTAINER(win), 5);
	gtk_container_add(GTK_CONTAINER(win), vbox);

	g_signal_connect(G_OBJECT(addbutton), "clicked",
			G_CALLBACK(edit ? gui_siteman_edit_grp_cb : gui_siteman_create_grp_cb), entry);
	g_signal_connect(G_OBJECT(addbutton), "clicked",
			G_CALLBACK(gui_widget_destroy_cb), win);

	gtk_widget_show_all(win);
	return FALSE;
}

void
gui_siteman_edit_grp_cb(GtkWidget *button, GtkEntry *entry)
{
	struct slist *site;
	GtkTreeModel *model;
	GtkTreePath  *path;
	GtkTreeIter   iter;
	
	site = gui_treestore_get_list_from_selection(siteaddr->tree);
	free(site->profile);
	site->profile = g_strdup(gtk_entry_get_text(entry));
	
	model = gtk_tree_view_get_model(siteaddr->tree);
	path = gui_tview_get_selected_path(site->tree);
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_store_set(GTK_TREE_STORE(model), &iter, 1, site->profile, -1);
   
   gui_siteman_write_sites_cb(NULL, NULL);
}
	
	
void
gui_siteman_create_grp_cb(GtkWidget *button, GtkEntry *entry)
{
	GtkTreePath  *path;
	char         *p;
	struct slist *tmp;
	struct slist *site = j_malloc(sizeof(struct slist));
	site->tree = siteaddr->tree;
	site->type = SITEMAN_GROUP;
	site->profile = g_strdup(gtk_entry_get_text(entry));
	site->hostaddr = site->userid = site->passwd = site->port = NULL;
	site->next   = NULL;
	site->child  = NULL;
	site->parent = NULL;
	
	 path = gui_tview_get_selected_path(site->tree);
	 tmp = gui_treestore_get_list_from_selection(siteaddr->tree);

    if(tmp) {
		if(tmp->type == SITEMAN_GROUP)
			gtk_tree_path_down(path);
		if(!gtk_tree_path_up(path)) path = NULL;;
	 }

	 p = path ? gtk_tree_path_to_string(path) : NULL;

    gui_treestore_append_list(siteaddr->tree, site, p ? p : "");
	 gui_treestore_append(siteaddr->tree, site, p ? p : "",
                              gui_siteman_list_append_cb);

		free(p);
   gui_siteman_write_sites_cb(NULL, NULL);
}


#ifdef DEBUG
/*
 * gui_test_cb() is a temporary debugging function for displaying the
 * contence of the linked list
 */

void
gui_test_cb(GtkWidget *button, struct slist *s)
{
	struct slist   *site;
   if(!s) s = gui_treestore_get_list(gtk_tree_view_get_model(siteaddr->tree));

	DEBUG_MSG(DBG_INFO, "> Debugging START");

	for (site = s; site; site = site->next) {
		DEBUG_MSG(DBG_INFO, "--------%p-----------\n" \
			"site->profile:\t%s\n"
			"site->hostaddr:\t%s\n"
			"site->userid:\t%s\n"
			"site->passwd:\t%s\n"
			"site->port:\t%s\n"
			"site->next:\t%p\n"
			"site->child:\t%p\n"
			"site->parent:\t%p",
			site,
			site->profile,
			site->hostaddr, 
			site->userid, 
			site->passwd, 
			site->port,
			site->next,
			site->child,
			site->parent);
		if(site->child)
			gui_test_cb(NULL, site->child);
	}

	DEBUG_MSG(DBG_INFO, "< Debugging END");
}
#endif

/*
 * gui_siteman_delete_site_cb() removes and free()s single elements of the
 * linked list by resetting the global "struct slist *siteaddr" pointer
 */

void
gui_siteman_delete_site_cb(GtkWidget * widget, GtkTreeView * tree)
{
	struct slist   *site;
	GtkTreePath    *path;

	site = gui_treestore_get_list_from_selection(siteaddr->tree);
	path = gui_tview_get_selected_path(tree);

	if (site == siteaddr) {
		DEBUG_MSG(DBG_ERROR, "WARNING: cannot delete this gadget!!");
		return;
	}

	if (!site)
		return;

	free(site->profile);
	free(site->hostaddr);
	free(site->userid);
	free(site->passwd);
	free(site->port);
	gui_siteman_free_node(site->child);
	
   gui_treestore_remove_list(tree, path, 0);
	free(site);


      gui_treestore_remove(tree, path);   

	gui_siteman_write_sites_cb(NULL, NULL);
}

void
gui_siteman_free_node(struct slist *site)
{
	if (!site)	return;
	free(site->profile);
	free(site->hostaddr);
	free(site->userid);
	free(site->passwd);
	free(site->port);
	gui_siteman_free_node(site->child);
	gui_siteman_free_node(site->next);
}

/*
 * gui_siteman_setup_tree() sets up the tree for the site managment and then
 * returns it to the calling function (in this case gui_siteman_init())
 */
GtkTreeView    *
gui_siteman_setup_tree(GtkWidget * scrolled, GtkWidget * status_bar)
{
	GtkTreeView    *tree = 0;
	GtkTreeStore   *store = 0;
	GtkTreeModel   *model = 0;
	GtkCellRenderer *renderer = 0;
	GtkTreeSelection *select = 0;
	GtkTreeViewColumn *column = 0;

	store = gtk_tree_store_new(2, GDK_TYPE_PIXBUF,
				   G_TYPE_STRING);

	model = GTK_TREE_MODEL(store);
	tree = (GtkTreeView *) gtk_tree_view_new_with_model(model);

	/* add pixmap column */
	renderer = gtk_cell_renderer_pixbuf_new();
	column = gtk_tree_view_column_new_with_attributes(
					   "", renderer, "pixbuf", 0, NULL);

	gtk_tree_view_set_headers_visible(tree, 0);

	/* set the column to a fixed sizing of 27 pixels */
//	gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column), GTK_TREE_VIEW_COLUMN_FIXED);
//	gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 55);
	gtk_tree_view_append_column(tree, column);

	/* column for site name */
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
					 "site", renderer, "text", 1, NULL);
	gtk_tree_view_append_column(tree, column);

	/* setup the selection handler */
	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

	g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(gui_siteman_tree_row_select_cb), status_bar);

	/* setup the expand/collapse handler */
	g_signal_connect(G_OBJECT(tree), "row-collapsed",
			G_CALLBACK(gui_siteman_node_collapsed_cb), NULL);
	 g_signal_connect(G_OBJECT(tree), "row-expanded",
			G_CALLBACK(gui_siteman_node_expanded_cb), NULL);
	
	gtk_widget_set_size_request(GTK_WIDGET(tree), 300, 400);
	gtk_container_add(GTK_CONTAINER(scrolled), GTK_WIDGET(tree));

   g_object_set_data(G_OBJECT(tree), "list", siteaddr->next);
   
	return tree;
}


void
gui_siteman_node_collapsed_cb(GtkTreeView *tree, 
							  GtkTreeIter *iter, GtkTreePath *path,
							  gpointer data)
{
	GSList *slist, *link;
	gchar *p;
	GtkTreeRowReference *row;
	GtkTreeModel        *model;
	p = gtk_tree_path_to_string(path);
	model = gtk_tree_view_get_model(tree);
	
	slist = g_object_get_data(G_OBJECT(tree), "rows");
	if(!slist)	return;
	DEBUG_MSG(DBG_INFO, "COLLAPSE:");
	
	for(link = slist; link; link = link->next) {
		DEBUG_MSG(DBG_INFO, "***[%p] %s <--", link,
	            gtk_tree_path_to_string(gtk_tree_row_reference_get_path(link->data)));
	}
	
	for(link = slist; link; link = link->next) {
		DEBUG_MSG(DBG_INFO, "link: %p - link->data: %p", link, link->data);
		row = link->data;
		DEBUG_MSG(DBG_INFO, "checking... %s VS %s",
				gtk_tree_path_to_string(gtk_tree_row_reference_get_path(row)),
				p);
		if(!strcmp(p,gtk_tree_path_to_string(gtk_tree_row_reference_get_path(row))))
			{
				slist = g_slist_remove_link(slist, link);
				g_slist_free_1(link);
				DEBUG_MSG(DBG_INFO, "deleted!");
				g_object_set_data(G_OBJECT(tree), "rows", slist);
				DEBUG_MSG(DBG_INFO, "left---->");
				for(link = slist; link; link = link->next) {
					DEBUG_MSG(DBG_INFO, "[%p] %s <--", link,
						gtk_tree_path_to_string(
							gtk_tree_row_reference_get_path(link->data)));
				}
				break;
			}
	}
	gui_siteman_write_sites_cb(NULL, NULL);	
}	

void
gui_siteman_node_expanded_cb(GtkTreeView *tree,
							 GtkTreeIter *iter, GtkTreePath *path,
							 gpointer data)
{
	gchar               *p;
	GSList              *slist, *link;
	GtkTreeIter         itr;
	GtkTreePath         *plink;
	GtkTreeRowReference *row;
	GtkTreeModel        *model;
	model = gtk_tree_view_get_model(tree);
	row = gtk_tree_row_reference_new(model, path);

	DEBUG_MSG(DBG_INFO, "EXPAND:");
	
	slist = g_object_get_data(G_OBJECT(tree), "rows");
	slist = g_slist_append(slist, row);
	g_object_set_data(G_OBJECT(tree), "rows", slist);
	gtk_tree_model_iter_nth_child(model, &itr, iter, 0);
	do {
		if(gtk_tree_model_iter_has_child(model, &itr)) {
			path = gtk_tree_model_get_path(model, &itr);
			p = gtk_tree_path_to_string(path);
			for(link = slist; link; link = link->next) {
				row = link->data;
				plink = gtk_tree_row_reference_get_path(row);
				if(!strcmp(p, gtk_tree_path_to_string(plink))) {
					slist = g_slist_remove_link(slist, link);
					g_slist_free_1(link);
					
					for(link = slist; link; link = link->next) {
						DEBUG_MSG(DBG_INFO, "::::::::::::[%p] %s <--",
						link,
						gtk_tree_path_to_string(
							gtk_tree_row_reference_get_path(link->data)));
					}
					
					g_object_set_data(G_OBJECT(tree), "rows", slist);
					gtk_tree_view_expand_row(tree, path, 0);
					break;
				}
			}
		}
	} while(gtk_tree_model_iter_next(model, &itr));
	//g_object_set_data(G_OBJECT(tree), "rows", slist);
	
	gui_siteman_write_sites_cb(NULL, NULL);
}

/*
 * gui_siteman_write_sites_cb() is called to deinitialize the Site Manager:
 * 
 * - It flushes the linked list into the sites file and cleans up the linked
 * list, unlinking and freeing its elements
 */
void
gui_siteman_write_sites_cb(GtkWidget * button, gpointer data)
{
	jio           *fhd;
	char           *file;
	GtkTreeModel   *model;
	GtkTreeIter     iter;

	if (getenv("HOME") != 0)
		file = j_asprintf("%s/.junkie/sites", getenv("HOME"));
	else {
		DEBUG_MSG(DBG_WEIRD, "WARNING: $HOME isn't set, trying .");
		file = j_asprintf("./.junkie/sites");
	}

	fhd = j_open(file, O_WRONLY | O_CREAT | O_TRUNC, 0600);	
	j_free(file);
	if (!fhd) 
		return;
	
	model = gtk_tree_view_get_model(siteaddr->tree);
	gtk_tree_model_get_iter_first(model, &iter);
	gui_siteman_fprintf(fhd, gui_treestore_get_list(gtk_tree_view_get_model(siteaddr->tree)), iter);
	j_close(fhd);
}

void
gui_siteman_fprintf(jio *fhd, struct slist *sptr, GtkTreeIter iter)
{
	GSList *slist = NULL, *link;
	char         *path, *j, *passwd, expand;
	GtkTreeModel *model;
	GtkTreePath  *p, *plink;
	GtkTreeIter  child;
	GtkTreeRowReference *row;
	slist = g_object_get_data(G_OBJECT(siteaddr->tree), "rows");
	model = gtk_tree_view_get_model(siteaddr->tree);
	while(sptr) {
		 p = gtk_tree_model_get_path(model, &iter);
		 path = gtk_tree_path_to_string(p);
		 expand = '-';
		 if(gtk_tree_model_iter_has_child(model, &iter)) {
			 for(link = slist; link; link = link->next) {
					row = link->data;
					plink = gtk_tree_row_reference_get_path(row);
					if(!plink) {
						DEBUG_MSG(DBG_ERROR, "plink == NULL");
						break;
					}
					if(!strcmp(path, gtk_tree_path_to_string(plink))) {
						expand = '+';
						break;
					}
			 }																					}
		 j = strrchr(path, ':');
		 if(j)	*j = '\0';
		 else    path[0] = '\0';
		 if(sptr->passwd)	passwd = strdup(sptr->passwd);
		 else			passwd = strdup(FTP_ANON_PASSWD);
		 //ENCRYPTION
		// DEBUG_MSG(DBG_INFO, "writing PASS=%s", key);
		 //for(i = 0; passwd[i]; i++)
		//	 passwd[i] ^= i;
		 if(config_read_bool("encrypt"))
			passwd = encrypt_string(key, passwd);
		 j_printf(fhd, "%s*%c%d%s=%s@%s\t%s:%s\n", 
					path,
					expand,
					sptr->type,
					sptr->profile, 
					sptr->userid   ? sptr->userid   : "",
					passwd         ? passwd         : "",
				   sptr->hostaddr ? sptr->hostaddr : "", 
					sptr->port     ? sptr->port     : "");
		 free(path);
		 free(passwd);
		 if(sptr->child)	{
			 gtk_tree_model_iter_children(model, &child, &iter);
			 gui_siteman_fprintf(fhd, sptr->child, child);
		 }
		 gtk_tree_model_iter_next(model, &iter);
		 sptr = sptr->next;
	}
}

void
gui_siteman_write_slist()
{
	jio  *fhd;
	char *file;
	char  path[20] = { 0 };
	
	if(getenv("HOME") != 0)
		file = j_asprintf("%s/.junkie/sites", getenv("HOME"));
	else {
		DEBUG_MSG(DBG_WEIRD, "WARNING: $HOME isn't set, trying .");
		file = j_asprintf("./.junkie/sites");
	}

	fhd = j_open(file, O_CREAT|O_WRONLY|O_TRUNC, 0600);
	j_free(file);
	if(!fhd) 
		return;

	gui_siteman_fprintf_slist(fhd, gui_treestore_get_list(gtk_tree_view_get_model(siteaddr->tree)), path);
	j_close(fhd);
}

void
gui_siteman_fprintf_slist(jio *fhd, struct slist *sptr, char *path)
{
	int i = 0;
	
	while(sptr) {
		j_printf(fhd, "%s*%d%s=%s@%s:%s\n",
				path,
				sptr->type,
				sptr->profile,
				sptr->userid   ? sptr->userid   : "",
				sptr->hostaddr ? sptr->hostaddr : "",
				sptr->port     ? sptr->port     : "");
		if(sptr->child) {
			if(!strcmp(path, "")) sprintf(path, "%d", i);
			else	                snprintf(path, 19, "%s:%d", path, i);
			gui_siteman_fprintf_slist(fhd, sptr->child, path);
		}
		i++;
		sptr = sptr->next;
	}
}

void
gui_siteman_free(struct slist *sptr)
{
	while(sptr) {
		struct slist *tmp;
			free(sptr->profile);
			free(sptr->hostaddr);
			free(sptr->userid);
			free(sptr->passwd);
			free(sptr->port);
		if(sptr->child)	gui_siteman_free(sptr->child);
		tmp = sptr->next;
		free(sptr);
		sptr = tmp;
	}
	
}

/*
 * gui_siteman_init() initializes the Site Manager: - The Site Manager
 * bookmark sites by storing them into a linked list and writing them into
 * "$HOME/.junkie/sites" enabeling you to quickly connect to your favourite
 * ftp sites
 * 
 * - Each Site can (or will be able to) be stored in a group of sites. This will
 * allow the User to categorize his Sites and manage them efficiently
 * 
 * - The Site Manager has a little Statusbar below to display the connect
 * information of the selected site
 */
void
gui_siteman_init(void)
{
	GtkWidget      *window;
	GtkWidget      *label;
	GtkWidget      *main_vbox;
	GtkWidget      *hbox;
	GtkWidget      *vbox;
	GtkWidget      *frame;
	GtkWidget      *status_bar;
	/* buttons */
	GtkWidget      *create_site;
	GtkWidget      *create_grp;
	GtkWidget      *edit;
	GtkWidget      *delete;
	GtkWidget      *close;
	GtkWidget      *password;
	/* debug */
#ifdef DEBUG
	GtkWidget      *debug;
#endif

//	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
//	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
//	gtk_window_set_title(GTK_WINDOW(window), "Site manager: Connect to Remote Host");
	window = gui_setup_popup_window("site manager: connect to remote host", 
			                        1, 0);
	
	main_vbox = gtk_vbox_new(FALSE, 0);
	hbox = gtk_hbox_new(FALSE, 0);
	vbox = gtk_vbox_new(FALSE, 0);
	status_bar = gtk_statusbar_new();

	siteaddr = (struct slist *)j_malloc(sizeof(struct slist));
	if (!siteaddr) {
		MEM_ERR("couldn't alloc siteaddr", ERR_SHOW);
		return;
	}
	memset(siteaddr, 0, sizeof(struct slist));

	/* Setup the tree for the sitemanager */
	siteaddr->tree = gui_siteman_setup_tree(hbox, status_bar);
	if (!siteaddr->tree)
		return;
	
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);


	gtk_container_set_border_width(GTK_CONTAINER(window), 0);
	gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 10);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

	/* XXX read up in the tutorial of www.gtk.org about linewarped labels */
	label = gtk_label_new(
			      "Welcome to the Site Manager of junkie.\n"
			      "Select a server to connect to from\n"
			      "the list below or add a new site with\n"
			      "the \"Create Site...\" button.");

	frame = gtk_frame_new("Remote Host Labels:");
	connect = gtk_button_new_with_label("Connect");
	create_site = gtk_button_new_with_label("Create Site...");
	create_grp = gtk_button_new_with_label("Create Group...");
	edit = gtk_button_new_with_label("Edit...");
	delete = gtk_button_new_with_label("Delete");
	close = gtk_button_new_with_label("Close");
	password = gtk_button_new_with_label("Password");
#ifdef DEBUG
	debug = gtk_button_new_with_label("Debug Llist");
#endif

	g_signal_connect(G_OBJECT(connect), "clicked", G_CALLBACK(gui_siteman_connect_site_cb), NULL);
	g_signal_connect(G_OBJECT(connect), "clicked", G_CALLBACK(gui_siteman_widget_destroy_cb), window);
	g_signal_connect(G_OBJECT(create_site), "clicked", G_CALLBACK(gui_siteman_create_site_cb), (gpointer) FALSE);
	g_signal_connect(G_OBJECT(create_grp), "clicked",
			G_CALLBACK(gui_siteman_create_grp), NULL);
	g_signal_connect(G_OBJECT(edit), "clicked", G_CALLBACK(gui_siteman_create_site_cb), (gpointer) TRUE);
	g_signal_connect(G_OBJECT(delete), "clicked", G_CALLBACK(gui_siteman_delete_site_cb), siteaddr->tree);
	g_signal_connect(G_OBJECT(close), "clicked", G_CALLBACK(gui_siteman_widget_destroy_cb), window);
	g_signal_connect(G_OBJECT(password), "clicked", 
			G_CALLBACK(gui_siteman_password_handler), NULL);
	g_signal_connect(G_OBJECT(siteaddr->tree), "row_activated", 
			G_CALLBACK(gui_siteman_connect_site_cb), NULL);
	g_signal_connect(G_OBJECT(siteaddr->tree), "row_activated",
			G_CALLBACK(gui_siteman_destroy_cb), window);
#ifdef DEBUG
	g_signal_connect(G_OBJECT(debug), "clicked", 
			G_CALLBACK(gui_test_cb), NULL);
#endif
	g_signal_connect(G_OBJECT(siteaddr->tree), "key_press_event",
			G_CALLBACK(gui_siteman_keypress_cb), window);
	
	gtk_container_add(GTK_CONTAINER(vbox), label);

	/* dummy line */
	gtk_statusbar_push(GTK_STATUSBAR(status_bar), GPOINTER_TO_INT(0), "No Server Selected");

	gtk_box_pack_start(GTK_BOX(vbox), connect, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), create_site, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), create_grp, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), edit, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), delete, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), close, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), password, FALSE, FALSE, 5);
#ifdef DEBUG
	gtk_box_pack_start(GTK_BOX(vbox), debug, FALSE, FALSE, 5);
#endif

	gtk_container_add(GTK_CONTAINER(window), main_vbox);
	gtk_container_add(GTK_CONTAINER(main_vbox), hbox);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_container_add(GTK_CONTAINER(hbox), frame);

	/* And finally the statusbar */
	gtk_box_pack_start(GTK_BOX(main_vbox), status_bar, FALSE, TRUE, 0);

	gtk_widget_show_all(window);
	gui_siteman_dnd_setup();

	 /* read the other sites from $HOME/.junkie/sites */
	    if (gui_siteman_read_sites())
			        return;
}

gboolean
gui_siteman_destroy_cb(GtkTreeView *t, GtkTreePath *p, GtkTreeViewColumn *c, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(data));
	return FALSE;
}


void 
gui_siteman_dnd_setup()
{
	GtkTreeView *tree = siteaddr->tree;
	gui_dnd_tview_enable(tree);
	gui_dnd_tview_scroll_set(tree, 1);
	gui_dnd_tview_interaction(tree, tree, gui_siteman_move, GDK_ACTION_MOVE);
}

void
gui_siteman_move(GtkWidget *widget, GdkDragContext *dc,
		           gint x, gint y, guint t, GSList *glist)
{
	GtkTreePath  *path,  *pathsrc;
	dnd_row       dndrow;
	struct slist *list,  *listsrc;
	GSList       *slist, *link;
	
	/* find path at which the drop has occured */
	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
			                        x, y, &path, NULL, 0, 0);
	if(path == 0)	return;
	dndrow.rows  = gtk_tree_path_get_indices(path);
	dndrow.depth = gtk_tree_path_get_depth(path);
	list = gui_treestore_get_list_row(gtk_tree_view_get_model(siteaddr->tree), &dndrow);
	if(!list)	   return;
	listsrc = glist->data;
	if(!listsrc)	return;
	pathsrc = gui_tview_get_selected_path(GTK_TREE_VIEW(widget));

	slist = g_object_get_data(G_OBJECT(widget), "rows");
	if(slist) {
		for(link = slist; link; link = link->next) {
			GtkTreeRowReference *row;
			GtkTreePath         *rowpath;
			char                *p;
			row = link->data;
			rowpath = gtk_tree_row_reference_get_path(row);
			p = gtk_tree_path_to_string(rowpath);
			if(!strcmp(p, gtk_tree_path_to_string(pathsrc))) {
				slist = g_slist_remove_link(slist, link);
				g_slist_free_1(link);
				break;
			}
		}
		g_object_set_data(G_OBJECT(widget), "rows", slist);
	}
	
	gui_treestore_move(siteaddr->tree, gui_siteman_move_cb, 
                      listsrc, list, pathsrc, path);
//	gui_siteman_refresh();
//	gui_siteman_move_tview(pathsrc, path);
   gui_siteman_write_sites_cb(NULL, NULL);
}


void
gui_siteman_move_cb(GtkTreeModel *model, GtkTreeIter src,
                                         GtkTreeIter *dst,
                                         GtkTreeIter *aft)
{
   GtkTreeIter iter;
   GdkPixbuf *pixbuf;
   gchar     *profile;
   
   gtk_tree_model_get(model, &src,
                      0, &pixbuf, 1, &profile, -1);
   gtk_tree_store_insert_after(GTK_TREE_STORE(model), &iter, dst, aft);
   gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                      0, pixbuf, 1, strdup(profile), -1);
   
}

		
