/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_startup.c,v 1.14 2003/03/03 02:16:33 tgucio Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>
/* local includes */
#include <error.h>
#include <mem.h>
#include <gui_startup.h>
#include <misc.h>
#ifdef HAVE_CONFIG_H
#include <configure.h>
#endif

GtkWidget *
gui_startup()
{
	GtkWidget *win = NULL;
	GtkWidget *logo = NULL;
	GtkWidget *vbox = NULL, *frame = NULL, *label = NULL;
   
	win = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_window_set_resizable(GTK_WINDOW(win), FALSE);
	gtk_window_set_decorated(GTK_WINDOW(win), FALSE);
	gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER);
	/*   gtk_window_set_transient_for(GTK_WINDOW(win), GTK_WINDOW(mainwin));*/
	/*   gtk_window_set_modal(GTK_WINDOW(win), TRUE);*/

	frame = gtk_frame_new(NULL);
	gtk_container_add(GTK_CONTAINER(win), frame);

	vbox = gtk_vbox_new(FALSE,FALSE);
	gtk_container_add(GTK_CONTAINER(frame), vbox);

	logo = gtk_image_new_from_file(DATADIR "/junkie/" "logo.png");
	gtk_box_pack_start(GTK_BOX(vbox), logo, FALSE, FALSE, 0);

	gtk_widget_show(logo);
	label = gtk_label_new("Starting up Junkie" VERSION);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	
	label = gtk_label_new("\n");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	g_object_set_data(G_OBJECT(win), "label", label);

	gtk_widget_show_all(win);

	return win;
}


void
gui_startup_write_text(GtkWidget *win, const char *text)
{
	GtkWidget *label = NULL;
	gui_startup_item *root = NULL;
	int id;

	label = g_object_get_data(G_OBJECT(win), "label");
	if (!label)
		return;

	id = (int)g_object_get_data(G_OBJECT(win), "timeout_id");
	if (!id) {
		id = gtk_timeout_add(60, gui_startup_timeout_iteration, win);
		g_object_set_data(G_OBJECT(win), "timeout_id", (void *)id);
	}

	root = g_object_get_data(G_OBJECT(win), "gui_startup_item_root");
	gui_startup_add_queue(&root, text);
	g_object_set_data(G_OBJECT(win), "gui_startup_item_root", root);
	
	return;
}


void
gui_startup_add_queue(gui_startup_item **root, const char *text)
{
	gui_startup_item **pnt = NULL;

	if (!root)
		return;

	pnt = root;
	while (*pnt)
		pnt = &(*pnt)->next;
	
	if ( ((*pnt) = j_malloc(sizeof(gui_startup_item))) == NULL) {
		MEM_ERR("malloc", ERR_SHOW);
		return;
	}

	
	(*pnt)->text = NULL;
	if (text)
		(*pnt)->text = j_strdup(text);

	(*pnt)->next = NULL;
	
	return;
}



gint
gui_startup_timeout_iteration(gpointer data)
{
	GtkWidget *window = NULL, *mainwin = NULL, *label = NULL;
	gui_startup_item *pnt = NULL;
	int id;

	window = GTK_WIDGET(data);
	if (!window)
		return 0;
	
	pnt = g_object_get_data(G_OBJECT(window), "gui_startup_item_root");
	if (!pnt) 
		return 0;
	
	g_object_set_data(G_OBJECT(window), "gui_startup_item_root", pnt ? pnt->next : NULL);

	if (pnt->text == NULL) {
		id = (int)g_object_get_data(G_OBJECT(window), "timeout_id");
		gtk_timeout_remove(id);
		mainwin = g_object_get_data(G_OBJECT(window), "gui_main_win");
		gtk_widget_destroy(GTK_WIDGET(window));
		if (mainwin)
			gtk_widget_show(mainwin);
	} else {
		label = g_object_get_data(G_OBJECT(window), "label");
		gtk_label_set_text(GTK_LABEL(label), pnt->text);
		j_free(pnt->text);
		j_free(pnt);
	}

	return 1;
}

void
gui_startup_end(GtkWidget *win, GtkWidget *mainwin)
{
	gui_startup_item *root = NULL;

	root = g_object_get_data(G_OBJECT(win), "gui_startup_item_root");
	g_object_set_data(G_OBJECT(win), "gui_main_win", mainwin);
	gui_startup_add_queue(&root, NULL);
	
	return;
}




