/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_statusbar.c,v 1.39 2003/03/07 17:48:13 thepin Exp $
 */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <math.h>
/* local headers */
#include <gui.h>
#include <ftp.h>
#include <ftp_transfer.h>
#include <gui_widgets.h>
#include <gui_io.h>
#include <gui_statusbar.h>
#include <gui_ftp.h>
#include <gui_ftp_lookup.h>
#include <medium.h>
#include <error.h>
#include <config.h>
#include <mem.h>
#include <misc.h>

status_bar *sbar = NULL;

void
gui_statusbar_update_connect(int connected)
{
	GtkWidget *rtview = NULL;
	gui_ftp_lookup *lookup = NULL;
	char buf[GOOD_SIZE+1], *file = NULL;
	
	rtview = gui_lookup_widget("right_tview");
	lookup = gui_ftp_lookup_widget(rtview);
	
	if (sbar->state == NULL)
		return;

	memset(buf, 0, sizeof(buf));
	if (connected == 1) { 
		if (lookup && lookup->mode == FTP_MODE_FTP && lookup->site != NULL) 
			snprintf(buf, GOOD_SIZE, "      Connected to %s:%d", lookup->site->addr, (int)lookup->site->port);
	} else if (connected == 2) {
		if (lookup && lookup->mode == FTP_MODE_FTP && lookup->site != NULL) 
			snprintf(buf, GOOD_SIZE, "      Logging into %s:%d", lookup->site->addr, (int)lookup->site->port);
	} else if (connected == 3) {
		if (lookup && lookup->mode == FTP_MODE_FTP && lookup->site != NULL) {
			file = ftp_object_get_data(FTP_OBJECT(lookup->site), "ftp_transfer_file");
			if (ftp_transfer_get_state(FTP_TRANSFER(lookup->site)) != FTP_TRANSFER_NONE && file)
				snprintf(buf, GOOD_SIZE, "      %s %s from %s",
					ftp_transfer_get_state(FTP_TRANSFER(lookup->site)) == FTP_TRANSFER_DOWNLOAD ? "Downloading" : "Uploading",
					file, lookup->site->addr);
			else
				snprintf(buf, GOOD_SIZE, "      Logged into %s:%d", lookup->site->addr, (int)lookup->site->port);
		}
	} else {
		strncpy(buf, "      Not Connected", GOOD_SIZE);
	}

	gtk_label_set_text(GTK_LABEL(sbar->state), buf);
	
	return;
}



gint
gui_statusbar_update_pbar(gpointer data)
{
	_ftp_item *ftp = data;
	gui_ftp_lookup *lookup = NULL;
	gdouble frac;
	char buf[128];

	if (sbar == NULL)
		return 0;

	lookup = gui_ftp_lookup_site(ftp);
	if (lookup && lookup->widget && sbar->pbar && ftp_transfer_get_state(FTP_TRANSFER(lookup->site)) != FTP_TRANSFER_NONE) {
		frac = (gdouble)ftp_transfer_get_position(FTP_TRANSFER(lookup->site))
				/ (gdouble)ftp_transfer_get_size(FTP_TRANSFER(lookup->site));
				
		if (frac <= 1.0) {
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(sbar->pbar), frac);
			snprintf(buf, 128, "%3.0f%%", frac * 100.0);
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(sbar->pbar), buf);
		}
	} else if (lookup && lookup->widget) {
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(sbar->pbar), 0.0);
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(sbar->pbar), "");
		gtk_label_set_text(GTK_LABEL(sbar->label), "");
		gtk_label_set_text(GTK_LABEL(sbar->label2),"");
		gtk_label_set_text(GTK_LABEL(sbar->label3),"");
	}
	
	return 1;
}

gint
gui_statusbar_update_label(gpointer data)
{
	_ftp_item *ftp = NULL;
	gui_ftp_lookup *lookup = NULL;
	char buf[SMALL_SIZE + 1];
	double speed, eta;
	const char *type = NULL;
	unsigned long q_totalsize;
	
	if (!(ftp = data) || !sbar)
		return 0;

	if (!(lookup = gui_ftp_lookup_site(ftp)))
		return 0;
	
	if (!lookup->widget)
		return 0;

	if (ftp_transfer_get_state(FTP_TRANSFER(ftp)) == FTP_TRANSFER_NONE)
		return 0;

	/* update the transfer speed label (showing current speed) */
	speed = ftp_transfer_get_average_speed(FTP_TRANSFER(ftp));
	if (speed < 1024)
		type = "B/s";
	else if (speed < 1024*1024) {
		speed /= 1024;
		type = "kB/s";
	}
	else {
		speed /= 1024*1024;
		type = "MB/s";
	}
	
	snprintf(buf, SMALL_SIZE, "%0.2f%s", speed, type);
	gtk_label_set_text(GTK_LABEL(sbar->label), buf);

	/* eta beta */
	speed = ftp_transfer_get_average_speed(FTP_TRANSFER(ftp));
	if (speed > 0) {
		eta = ((double)ftp_transfer_get_size(FTP_TRANSFER(ftp)) - (double)ftp_transfer_get_position(FTP_TRANSFER(ftp)))
		     / (double) speed;
		snprintf(buf, SMALL_SIZE, "%d:%.2d:%.2d", (int)eta/(60*60), (int)(eta/60)%60, (int)eta % 60);
		gtk_label_set_text(GTK_LABEL(sbar->label2), buf);
	}

	/* queue eta */
	q_totalsize = (unsigned long)ftp_object_get_data(FTP_OBJECT(ftp), "queue_current_size")
		      + ftp_transfer_get_resume_position(FTP_TRANSFER(ftp))
		      - ftp_transfer_get_position(FTP_TRANSFER(ftp));
		      
	if (ftp_object_get_data(FTP_OBJECT(ftp), "queue_total_size")) {
		double q_currentsize, q_currentfiled;

		q_currentfiled = ftp_transfer_get_position(FTP_TRANSFER(ftp))
				- ftp_transfer_get_resume_position(FTP_TRANSFER(ftp));
				
		q_currentsize = (long)ftp_object_get_data(FTP_OBJECT(ftp), "queue_current_size")
					- q_currentfiled;

		speed = ftp_transfer_get_average_speed(FTP_TRANSFER(ftp));
		eta = (double) q_currentsize / (double) speed;
		snprintf(buf, SMALL_SIZE, "%d:%.2d:%.2d", (int)eta/(60*60), (int)(eta/60)%60, (int)eta % 60);
		gtk_label_set_text(GTK_LABEL(sbar->label3), buf);
	}

	return 0;
}

void
gui_statusbar_update(_ftp_item *ftp)
{
	int connected;

	connected = 0;
	if (ftp && ftp->fd > 0) {
		if (ftp->state == FTP_STATE_CONNECTED
		    || ftp->state == FTP_STATE_GREETED)
			connected = 1;
		else if (ftp->state == FTP_STATE_LOGGED_IN)
			connected = 3;
	}
	
	gui_statusbar_update_connect(connected);
	gui_statusbar_update_pbar(ftp);
	gui_statusbar_update_label(ftp);
	return;
}

/*
 * (ftp)
 *
 */
int
gui_medium_statusbar_trfr_update(MEDIUM_ARGS)
{
	_ftp_item *ftp = MEDIUM_ARG(0);
	gui_ftp_lookup *lookup = gui_ftp_lookup_site(ftp);
	
	if (lookup && lookup->widget) {
		gui_statusbar_update_pbar(ftp);
		gui_statusbar_update(ftp);
		gui_statusbar_update_connect(3);
	}
	return gui_statusbar_update_label(ftp);
}

GtkWidget *
gui_statusbar_setup()
{
	GtkWidget *frame = NULL;
	GtkWidget *eventbox, *eventbox1, *eventbox2, *eventbox3;
	
	if (sbar == NULL) {
		if ( (sbar =(status_bar *)j_malloc(sizeof(status_bar))) == NULL) {
			MEM_ERR("malloc", ERR_SHOW);
			return NULL;
		}
	}

	sbar->frame = gtk_frame_new(NULL);
	sbar->hbox = gtk_hbox_new(FALSE, 0);	
	sbar->state = gtk_label_new("      Not connected");
	sbar->pbar = gtk_progress_bar_new();
	sbar->label = gtk_label_new("        ");
	sbar->label2 = gtk_label_new("        ");
	sbar->label3 = gtk_label_new("        ");

	gtk_container_set_border_width(GTK_CONTAINER(sbar->hbox), 1);
	
	frame = gtk_frame_new(NULL);
//	gtk_widget_set_size_request(frame, 300, -1);
	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(eventbox), sbar->state);
	gtk_container_add(GTK_CONTAINER(frame), eventbox);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	gtk_box_pack_start(GTK_BOX(sbar->hbox), frame, TRUE, TRUE, 0);
	gtk_misc_set_alignment(GTK_MISC(sbar->state), 0, 0.5);
	gtk_widget_show_all(frame);
	frame = gtk_frame_new(NULL);
	gtk_container_add(GTK_CONTAINER(frame), sbar->pbar);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_box_pack_start(GTK_BOX(sbar->hbox), frame, FALSE, FALSE, 1);
	gtk_widget_show(frame);
	frame = gtk_frame_new(NULL);
	eventbox1 = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(eventbox1), sbar->label);
	gtk_container_add(GTK_CONTAINER(frame), eventbox1);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_box_pack_start(GTK_BOX(sbar->hbox), frame, FALSE, FALSE, 1);
	gtk_widget_show(frame);
	frame = gtk_frame_new(NULL);
	eventbox2 = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(eventbox2), sbar->label2);
	gtk_container_add(GTK_CONTAINER(frame), eventbox2);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_box_pack_start(GTK_BOX(sbar->hbox), frame, FALSE, FALSE, 1);
	gtk_widget_show(frame);
	frame = gtk_frame_new(NULL);
	eventbox3 = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(eventbox3), sbar->label3);
	gtk_container_add(GTK_CONTAINER(frame), eventbox3);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_box_pack_start(GTK_BOX(sbar->hbox), frame, FALSE, FALSE, 1);
	gtk_widget_show(frame);
	
	
	gtk_frame_set_shadow_type(GTK_FRAME(sbar->frame), GTK_SHADOW_OUT);
	gtk_widget_set_size_request(sbar->frame, -1, 30);
	gtk_container_add(GTK_CONTAINER(sbar->frame), sbar->hbox);
	
	g_signal_connect(G_OBJECT(eventbox1), "button_press_event",
			G_CALLBACK(gui_statusbar_label_change), sbar->label);
	g_signal_connect(G_OBJECT(eventbox2), "button_press_event",
			G_CALLBACK(gui_statusbar_label_change), sbar->label2);
	g_signal_connect(G_OBJECT(eventbox3), "button_press_event",
			G_CALLBACK(gui_statusbar_label_change), sbar->label3);
	
	gtk_widget_show(eventbox1);
	gtk_widget_show(eventbox2);
	gtk_widget_show(eventbox3);
	gtk_widget_show(sbar->state);
	gtk_widget_show(sbar->pbar);
	gtk_widget_show(sbar->label);
	gtk_widget_show(sbar->label2);
	gtk_widget_show(sbar->label3);
	gtk_widget_show(sbar->hbox);
	gtk_widget_show(sbar->frame);

	gui_insert_widget("statusbar", sbar);
	
	return sbar->frame;
}


void
gui_statusbar_del_timeout()
{
	if (sbar->timeout)
		gtk_timeout_remove(sbar->timeout);

	return;
}



void
gui_statusbar_label_change(GtkEventBox *box, GdkEvent *event, GObject *label)
{
	int i = (int)g_object_get_data(label, "label_property");
	g_object_set_data(label, "label_property", (void *)!i);
	return;
}

