/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_theme.c,v 1.19 2003/03/03 03:05:33 thepin Exp $
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <errno.h>
#include <gtk/gtk.h>

/* local headers */
#ifdef HAVE_CONFIG_H
#include <configure.h>
#endif
#include <config.h>
#include <error.h>
#include <gui_theme.h>
#include <mem.h>

#ifdef HAVE_BASENAME
#include <libgen.h>
#endif

gui_theme_files gui_theme_list[] = {
	{ "cdup.png", T_PRELOAD },
	{ "file.png", T_PRELOAD },
	{ "folder.png", T_PRELOAD },

	{ "file_movie.png", T_PRELOAD },
	{ "file_image.png", T_PRELOAD },
	{ "file_sound.png", T_PRELOAD },
	
	{ "queue_ni.png", T_PRELOAD },
	{ "queue_no.png", T_PRELOAD },
	{ "queue_si.png", T_PRELOAD },
	{ "queue_so.png", T_PRELOAD },

	{ "site.png", T_PRELOAD },
	
	{ "connect_before.png", T_PRELOAD },
	{ "connect_after.png", T_PRELOAD },
	{ "refresh_before.png", T_PRELOAD },
	{ "refresh_after.png", T_PRELOAD },
	
	{ "preferences.png", T_LOAD },
	{ "siteman.png", T_LOAD },
	{ "stop.png", T_LOAD },

	/* resource file */
	{ "rcfile", T_RCFILE },
	{ "theme.conf", T_CONFIG },
	
	{ NULL, 0 },
};

gui_theme_header *root_theme_header;

/*
 * changes a string with an octal number to a decimal number (returns unsigned long).
 * Excludes spaces in the beginning of the string.
 * String Octal to Decimal Long.
 */
long
so2dl(char *str)
{
	char *pnt = NULL;
	long oct = 0;

	pnt = str;
	while (*pnt == ' ') pnt++;

	while (isdigit(*pnt)) 
		oct = (oct << 3) + *(pnt++) - '0';
	
	return oct;
}

void
tar_skip_file(int size, FILE *f)
{
	int turns;

	turns = ((size - (size % RECORDSIZE)) / RECORDSIZE);
	if ((size % RECORDSIZE) > 0)
		turns++;

	fseek(f, turns * RECORDSIZE, SEEK_CUR);	
	return;
}

int
tar_read_header(gui_theme_entry *entry, FILE *f)
{
	if (!entry)
		return 0;

	/* read the header in */
	if (fread(entry->rec.charptr, sizeof(char), RECORDSIZE, f) <= 0)
		return 0;

	if (strcmp(entry->rec.header.magic, TMAGIC)) {
		/* XXX invalid magic ? */
		return 0;
	}

	/* parse the header into more useful units */

	entry->info.file = NULL;
	if (*entry->rec.header.name)	/* neccessry ? */
		entry->info.file = strdup(entry->rec.header.name);

	entry->info.size = so2dl(entry->rec.header.size);
	entry->info.type = entry->rec.header.linkflag;
	entry->pos = ftell(f);

	return 1;
}

#ifndef HAVE_BASENAME
char *
basename(char *name)
{
	char *pnt = NULL;

	if (!name)
		return NULL;

	pnt = strrchr(name, '/');
	return ((pnt) ?  (pnt + 1) : (name));
}
#endif

gui_theme_header *
gui_theme_get_current(void)
{
	return root_theme_header;
}

void
gui_theme_set_current(gui_theme_header *theme)
{
	root_theme_header = theme;
}


GdkPixbuf *
gui_theme_pixbuf_get(const char *icon)
{
	GdkPixbufLoader *loader = NULL;
	GdkPixbuf *pix = NULL;
	GError *gerr = NULL;
	gui_theme_entry *epnt = NULL;
	FILE *f = NULL;
	char buf[GOOD_SIZE + 1], *data = NULL;
	
	/* load the icon into pix and return it */
	if (!root_theme_header || !root_theme_header->theme || !icon)
		return NULL;

	epnt = root_theme_header->entry;
	while (epnt) {
		if (epnt->info.file && !strcasecmp(basename(epnt->info.file), icon)) {
			if (!epnt->data) {
				/* Read and load the image into a pixbuf.  */
				snprintf(buf, GOOD_SIZE, DATADIR "/junkie/themes/%s.jt", root_theme_header->theme);
				if ( (f = fopen(buf, "r")) == NULL) {
					DEBUG_MSG(DBG_ERROR, "failed to open the theme at %s", buf);
					return NULL;
				}
	
				if ( !(data = j_malloc(epnt->info.size + 1)) ) 
					return NULL;
			
				fseek(f, epnt->pos, SEEK_SET);
				if (fread(data, sizeof(char), epnt->info.size, f) <= 0)
					return NULL;
			} else {
				/* Preloaded icon. */
				data = epnt->data;
			}
		

			loader = gdk_pixbuf_loader_new();
			gdk_pixbuf_loader_write(loader, data, epnt->info.size, &gerr);
			pix = gdk_pixbuf_copy(gdk_pixbuf_loader_get_pixbuf(loader));
			if (pix == NULL) {
				DEBUG_MSG(DBG_ERROR, "pixbufloader error, pix == NULL");
				return NULL;
			}
			gdk_pixbuf_loader_close(loader, &gerr);
			g_object_unref(G_OBJECT(loader));	
			
			if (!epnt->data) {	
				j_free(data);
				fclose(f);
			}
			
			return pix;
		}
		
		epnt = epnt->next;
	}
	
	return NULL;
}


GtkWidget *
gui_theme_image_get(const char *image)
{
	GdkPixbuf *pix = NULL;
	GtkWidget  *img = NULL;

	pix = gui_theme_pixbuf_get(image);
	if (!pix)
		return NULL;
	
	img = gtk_image_new_from_pixbuf(pix);
	g_object_unref(pix);
	return img;
}


int
theme_get_list_index(char *filename)
{
	int i;

	for (i = 0; gui_theme_list[i].file; i++)  {
		if (gui_theme_list[i].file && filename && !strcasecmp(gui_theme_list[i].file, basename(filename))) 
			return i;
	}
	
	return -1;
}

/*
 * load a junkie theme and return the first entry of the linked list.
 * loads it from the junkie share dir, i.e. /usr/local/share/junkie/themes/something.jt
 */
int
gui_theme_load(const char *theme)
{
	FILE *f = NULL;
	gui_theme_entry **epnt = NULL;
	char buf[GOOD_SIZE + 1];
	int i;
	
	snprintf(buf, GOOD_SIZE, DATADIR "/junkie/themes/%s.jt", theme);

	if ( (f = fopen(buf, "r")) == NULL) {
		DEBUG_MSG(DBG_ERROR, "failed to open %s", buf);
		return -1;
	}

	if ( !(root_theme_header = j_malloc(sizeof(gui_theme_header))) ) 
		return -1;

	epnt = &root_theme_header->entry;
	
	while (1) {
		if ( !(*epnt = j_malloc(sizeof(gui_theme_entry))) ) 
			return -1;
			
		(*epnt)->data = NULL;
		memset(&(*epnt)->info, 0, sizeof(gui_theme_stat));
		
		if (!tar_read_header(*epnt, f))  {
			(*epnt)->next = NULL;
			break;
		}

		DEBUG_MSG(DBG_INFO, "%s (%s) (size: %d)", (*epnt)->info.file, (((*epnt)->info.type == LF_NORMAL) ? "file" : "dir?"), (*epnt)->info.size);

		if ((*epnt)->info.type == LF_NORMAL) {
			i = theme_get_list_index((*epnt)->info.file);

			if (i < 0 || gui_theme_list[i].type == T_LOAD) {
				tar_skip_file((*epnt)->info.size, f);
				(*epnt)->data = NULL;
			} else if (gui_theme_list[i].type == T_PRELOAD) {
				if ( !((*epnt)->data = j_malloc(sizeof(char) * (*epnt)->info.size+1)) ) 
					return -1;

				memset((*epnt)->data, 0, sizeof(char) * (*epnt)->info.size+1);
				DEBUG_MSG(DBG_INFO, "preloading %s", (*epnt)->info.file);

				fread((*epnt)->data, sizeof(char), (*epnt)->info.size, f);
				if (((*epnt)->info.size % 512) > 0) {
					fseek(f, RECORDSIZE - ((*epnt)->info.size % RECORDSIZE), SEEK_CUR);
				}
			} else if (gui_theme_list[i].type == T_RCFILE) {
				(*epnt)->data = j_malloc(sizeof(char) * (*epnt)->info.size + 1);
				memset((*epnt)->data, 0, sizeof(char) * (*epnt)->info.size + 1);
				DEBUG_MSG(DBG_INFO, "loading resource file %s", (*epnt)->info.file);
				fread((*epnt)->data, sizeof(char), (*epnt)->info.size, f);

				if (((*epnt)->info.size % 512) > 0) {
					fseek(f, RECORDSIZE - ((*epnt)->info.size % RECORDSIZE), SEEK_CUR);
				}

				DEBUG_MSG(DBG_INFO, "parsing the resource file: %s",
					(*epnt)->data);
				gtk_rc_parse_string((*epnt)->data);
				
				j_free((*epnt)->data);
				(*epnt)->data = NULL;
			} else if (gui_theme_list[i].type == T_CONFIG) {
				(*epnt)->data = j_malloc(sizeof(char) * (*epnt)->info.size + 1);
				memset((*epnt)->data, 0, sizeof(char) * (*epnt)->info.size + 1);
				DEBUG_MSG(DBG_INFO, "loading theme config file %s", (*epnt)->info.file);
				fread((*epnt)->data, sizeof(char), (*epnt)->info.size, f);

				if (((*epnt)->info.size % 512) > 0) 
					fseek(f, RECORDSIZE - ((*epnt)->info.size % RECORDSIZE), SEEK_CUR);
				DEBUG_MSG(DBG_INFO, "parsing the config file: %s", (*epnt)->data);
				config_getvars_str((*epnt)->data);
				j_free((*epnt)->data);
				(*epnt)->data = NULL;
			}
		}

		(*epnt)->next = NULL;
		epnt = &(*epnt)->next;
	}
		
	fclose(f);

	root_theme_header->theme = NULL;
	if (theme)
		root_theme_header->theme = j_strdup(theme);
	
	return 1;
}


void
gui_theme_close(void)
{
	gui_theme_entry *pnt = NULL, *pnt2 = NULL;
	
	if (root_theme_header == NULL)
		return;

	pnt = root_theme_header->entry;
	while (pnt) {
		pnt2 = pnt;
		pnt = pnt->next;

		j_free(pnt2->data);
		j_free(pnt2->info.file);
		j_free(pnt2);
	}

	j_free(root_theme_header);
	root_theme_header = NULL;

	return;
}





