/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_timeout.c,v 1.8 2003/03/03 02:16:33 tgucio Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <gtk/gtk.h>
#include <sys/socket.h>
#include <netinet/in.h>
/* local headers */
#include <gui.h>
#include <gui_timeout.h>
#include <medium.h>
#include <config.h>
#include <error.h>
#include <log.h>
#include <mem.h>
#include <defaults.h>


/*
** This then actually calls the callback and kills the timer hopefully by returning FALSE.
*/
gint
gui_timeout_handler(gpointer data)
{
	timeout_args *targs = NULL;

	DEBUG_MSG(DBG_INFO, "gui_timeout_handler reached, calling callback with data");
	/* call the callback :) */
	if ((targs = data) && targs->callback) {
		targs->callback(targs->data);
		free(targs);
	}

	return FALSE;
}

/*
 * medium M_TIMEOUT_REMOVE function
 * takes 1 arg (id)
 */
int
gui_timeout_remove(MEDIUM_ARGS)
{
	gtk_timeout_remove((int)MEDIUM_ARG(0));
	return 0;
}

/* 
 * medium M_TIMEPUT_ADD function
 * takes 2 arguments (time, callback)
 * /callback/ will be called in /time/ seconds.
 */
int
gui_timeout_add(MEDIUM_ARGS)
{
	int time;
	void (*callback)() = NULL, *data = NULL;
	timeout_args *targs = NULL;

	/* Get the arguments to the function. */
	time = (int)MEDIUM_ARG(0);		/* time in seconds */
	callback = MEDIUM_ARG(1);		/* function to be called after the specified time. */
	data = MEDIUM_ARG(2);

	/* install argument for the timeout callback. */
	if (!(targs = j_malloc(sizeof(timeout_args))))
		return -1;

	targs->callback = callback;
	targs->data = data;

	DEBUG_MSG(DBG_INFO, "the timeout callback to be called in %d seconds", time);
	/* setup the callback. (time * 1000) - seconds to milliseconds conversion. */
	return gtk_timeout_add(time, gui_timeout_handler, targs);
}


