/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_toolkit.c,v 1.56 2003/03/07 17:48:13 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <strings.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
/* local headers */
#include <gui.h>
#include <gui_widgets.h>
#include <gui_statusbar.h>
#include <gui_paths.h>
#include <gui_ftp_callback.h>
#include <gui_io.h>
#include <ftp.h>
#include <ftp_net.h>
#include <ftp_cmd.h>
#include <ftp_misc.h>
#include <ftp_object.h>
#include <ftp_request.h>
#include <ftp_transfer.h>
#include <telnet.h>
#include <local.h>
#include <config.h>
#include <cache.h>
#include <error.h>
#include <log.h>
#include <gui_toolkit.h>
#include <gui_anti_idler.h>
#include <gui_theme.h>
#include <gui_treestore.h>
#include <gui_tview.h>
#include <gui_tree_model.h>
#include <gui_ftp.h>
#include <gui_ftp_lookup.h>
#include <gui_toolbar.h>
#include <hammering.h>
#include <sound.h>
#include <sound_mixer.h>
#include <gui_siteman.h>
/* default icons */
#include <icons/connect_before.xpm>
#include <icons/connect_after.xpm>
#include <icons/refresh_before.xpm>
#include <icons/preferences.xpm>
#include <icons/siteman.xpm>
#include <icons/stop.xpm>

static GtkWidget *connect_button;

GtkWidget *
gui_new_pixmap(GtkWidget * root, const char **xpm)
{
	GdkPixmap *pix = NULL;
	GdkBitmap *bit = NULL;
	GtkWidget *pixmap = NULL;
	
	pix = gdk_pixmap_create_from_xpm_d(root->window, &bit, 0, (char **)xpm);
	pixmap = gtk_pixmap_new(pix, bit);
	gdk_pixmap_unref(pix);
	gdk_bitmap_unref(bit);
	
	return pixmap;
}

void
gui_button_pixmap_set(GtkButton* button, gpointer data)
{
	GList          *children = NULL;
	GtkWidget      *pixmap = NULL;
	GtkWidget      *child = gtk_bin_get_child(GTK_BIN(button));

	children = gtk_container_get_children(GTK_CONTAINER(child));
	gtk_container_remove(GTK_CONTAINER(child), g_list_nth_data(children, 0));
	pixmap = gui_new_pixmap(GTK_WIDGET(button), data);
	gtk_widget_show(pixmap);
	gtk_box_pack_end(GTK_BOX(child), pixmap, 0, 0, 0);

	return;
}

int
gui_button_image_set(GtkButton *button, gpointer data)
{
	GList *children = NULL;
	GtkWidget *child = NULL;
	GtkWidget *image = NULL;

	if (!data)
		return -1;
		
	if (!(image = gui_theme_image_get(data)))
		return -1;
   
	/* remove the old icon */
	child = gtk_bin_get_child(GTK_BIN(button));
	children = gtk_container_get_children(GTK_CONTAINER(child));
	gtk_container_remove(GTK_CONTAINER(child), g_list_nth_data(children, 0));

	/* add the new icon :)) */
	gtk_widget_show(image);
	gtk_box_pack_end(GTK_BOX(child), image, 0, 0, 0);

	return 0;
}



void
gui_refresh_all()
{
	gui_local_refresh();
	gui_remote_refresh();
}

void
gui_refresh_all_forced()
{
	gtk_widget_grab_focus(gui_lookup_widget("left_tview"));
	gui_local_refresh();
	gtk_widget_grab_focus(gui_lookup_widget("right_tview"));
	gui_remote_refresh();
}

void
gui_refresh_local_forced()
{
	gtk_widget_grab_focus(gui_lookup_widget("left_tview"));
	gui_local_refresh();
}

void
gui_refresh_remote_forced()
{
	gtk_widget_grab_focus(gui_lookup_widget("right_tview"));
	gui_remote_refresh();
}	

void
gui_local_refresh() {
	GtkWidget *ltview = NULL;
	struct _list *list_dir = NULL;

	/* find the widget of the local treeview (on the left). */
	if (!(ltview = gui_lookup_widget("left_tview")))
		return;

	if (gtk_widget_is_focus(ltview)) {
		list_dir = local_list();
//		gui_tview_fill(GTK_TREE_VIEW(ltview), list_dir, ftp_free_list, gui_fill_tview_cb);
		gui_tree_model_fill(gtk_tree_view_get_model(GTK_TREE_VIEW(ltview)), list_dir,
				ftp_free_list, gui_tree_model_fill_cb);
	}
	return;
}

void
gui_remote_refresh()
{
	GtkWidget *ltview = NULL, *rtview = NULL;
	gui_ftp_lookup *lookup = NULL;

	ltview = gui_lookup_widget("left_tview");	/* local (left) tview.   */
	rtview = gui_lookup_widget("right_tview");	/* remote (right) tview. */
	
	if (!(lookup = gui_ftp_lookup_widget(rtview)))
		return;
		
	if (!lookup->site)
		return;
	switch (lookup->mode) {
		case FTP_MODE_FTP:
			if (gtk_widget_is_focus(rtview)) {
				gui_ftp_list(lookup->site, gui_ftp_callback_list, 1, NULL, "widget", rtview);
			}
			break;
		
		case FTP_MODE_FXP:
			if (gtk_widget_is_focus(ltview)) {
				ftp_list_with_cb(lookup->site, gui_ftp_callback_list, 1, 0);
				gui_ftp_list(lookup->site, gui_ftp_callback_list, 1, NULL, "widget", rtview);
				/* XXX hm, not neccessarily rtview... */
			}
			break;
	}

	return;
}

void
gui_stop()
{
	int ret;
	char tcmd[3];
	gui_ftp_lookup *lookup = NULL;
	ftp_request *req = NULL;

	/* Check if connected.... (make more abstract later)... for fxp and stuff. */
	if (!(lookup = gui_ftp_lookup_widget(gui_lookup_widget("right_tview")))) 
		return;		
		
	if (lookup->site && ftp_transfer_get_state(FTP_TRANSFER(lookup->site)) != FTP_TRANSFER_NONE) {
		/* Send Interrupt. */
		tcmd[0] = _IAC_;
		tcmd[1] = _IP_;
		tcmd[2] = 0;
		ret = send(lookup->site->fd, tcmd, 2, MSG_OOB);
		if (ret != 2)
			DEBUG_MSG(DBG_ERROR, "ABORT: did NOT succeed to send the IAC, IP");
			
		/* Synch Signal. */
		tcmd[0] = _IAC_;
		tcmd[1] = _DM_;
		tcmd[2] = 0;
		ret = send(lookup->site->fd, tcmd, 2, MSG_OOB);
		if (ret != 2)
			DEBUG_MSG(DBG_ERROR, "ABORT: did NOT succeed to send the IAC, DM");


		FTP_MISC_LOG(lookup->site, "    Sent abort (ABOR) request for %s\n",
		    (ftp_transfer_get_state(FTP_TRANSFER(lookup->site)) == FTP_TRANSFER_UPLOAD) ? "upload" : "download");

		ftp_send_command(lookup->site, "ABOR\r\n");


		/* Synch signal.  */
		tcmd[0] = _IAC_;
		tcmd[1] = _DM_;
		tcmd[2] = 0;
		ret = send(lookup->site->fd, tcmd, 2, MSG_OOB);
		if (ret != 2)         
			DEBUG_MSG(DBG_ERROR, "ABORT: did NOT succeed to send the IAC, DM");

		if (lookup->site->dataconn) {
			/* Close the dataconn fd and set it to -1 so other parts of junkie will notice that it is not valid anymore. */
			close(lookup->site->dataconn->fd);
			lookup->site->dataconn->fd = -1;
		}
	}
	/* unblock the tview ? */
	req = ftp_request_add(lookup->site, RP_END, 0, R_ALWAYS,
			gui_tview_handler_unblock, 0);

	ftp_object_set_data(FTP_OBJECT(req), "tree",
			gtk_tree_view_get_model(
				        GTK_TREE_VIEW(gui_lookup_widget("right_tview"))));

	return;
}

/*
** Enforce disconnect.
*/
gboolean
gui_disconnect_force_timeout(gpointer data)
{	
	GtkWidget *rtview = NULL;
	struct _list *list = NULL;
	_ftp_item *ftp = data;

	DEBUG_MSG(DBG_INFO, "gui_disconnect_force_timeout");
	DEBUG_MSG(DBG_INFO, "ftp: %p", ftp);
	if (!ftp) {
		DEBUG_MSG(DBG_WEIRD, "ftp not set!");
		goto the_end;
	}
	
	DEBUG_MSG(DBG_INFO, "clearing the right tree view");
	/* fix the right treeview up. */
	rtview = gui_lookup_widget("right_tview");
	list = g_object_get_data(G_OBJECT(rtview), "list");
	ftp_free_list(list);
	g_object_set_data(G_OBJECT(rtview), "list", NULL);
	
	
	DEBUG_MSG(DBG_INFO, "closing the connection and such");
	/* close the connection. */
	close(ftp->fd); 
	ftp->fd = -1;
	gui_anti_idler_toggle(ftp, FALSE);
	gui_ftp_setup(ftp, 0);
	gui_ftp_setup_write(ftp, 0);

	DEBUG_MSG(DBG_INFO, "eof gui_disconnect_force_timeout");

the_end:
	return FALSE;
}



void
gui_disconnect()
{
	GtkWidget *rtview = NULL;
	ftp_request *req = NULL;
	gui_ftp_lookup *lookup = NULL;

	DEBUG_MSG(DBG_INFO, "gui_disconnect");
	rtview = gui_lookup_widget("right_tview");
	if (!(lookup = gui_ftp_lookup_widget(rtview)))
		return;

	switch (lookup->mode) {
		case FTP_MODE_FXP:
			ftp_quit(lookup->site);
			close(lookup->site->fd);
			gui_ftp_lookup_remove(lookup);

			break;
		case FTP_MODE_FTP:
			req = ftp_quit_with_cb(lookup->site, gui_ftp_callback_quit);
			ftp_object_set_data(FTP_OBJECT(req), "widget", rtview);
			break;
	}
	
	gtk_timeout_add(1500, gui_disconnect_force_timeout, (void *)lookup->site);
	gui_paths_text(0, "");

	return;
}

void 
gui_connect(GtkButton *button)
{
	GList          *children = NULL;
	GtkWidget      *c = NULL;
	GtkWidget      *child = gtk_bin_get_child(GTK_BIN(button));
	gui_ftp_lookup *lookup = gui_ftp_lookup_widget(gui_lookup_widget("right_tview"));
	
	children = gtk_container_get_children(GTK_CONTAINER(child));
	c = g_list_nth_data(children, 1);

	if (lookup && hammering_enabled(lookup->site))	/* XXX lame, fix in 0.4 :) */
		hammering_stop(lookup->site);
	else if (!lookup || !lookup->site) 
		gui_connect_dialog();
	 else 
		gui_disconnect();

	return;
}


/*
** Set the icon and label for the connection button.
** For example, change the label (and icon) from Connect to Disconnect after connecting to a site.
** ARguments: icon is the path of the icon to be set, if that one does not exist,
** the xpm_sec will be used instead.  text is the text to write on the label.
*/
void
gui_connect_button_set(const char *text, const char *icon, const char **xpm_sec)
{
	GtkButton *button = GTK_BUTTON(connect_button);
	GList *children = NULL;
	GtkWidget *label = NULL, *child;
	GtkWidget *rtview = NULL;
	gui_ftp_lookup *lookup = NULL;

	rtview = gui_lookup_widget("right_tview"); 
	lookup = gui_ftp_lookup_widget(rtview);
	if (lookup && hammering_enabled(lookup->site))	/* XXX :)) lame fix in 0.4 */
		return;

	child = gtk_bin_get_child(GTK_BIN(button));
	children = gtk_container_get_children(GTK_CONTAINER(child));
	label = g_list_nth_data(children, 1);
	
	if (gui_button_image_set(button, (char *)icon) && xpm_sec)
		gui_button_pixmap_set(button, xpm_sec);

	if (text)
		gtk_label_set_text(GTK_LABEL(label), text);

	DEBUG_MSG(DBG_INFO, "boo boo:) setting button with label %s und icon %s", text, icon);
	
	return;
}

/*
** Set the connected button to disconnect (the user will disconnect if they press that button).i
*/
void
gui_connect_button_set_disconnect(void)
{
	gui_connect_button_set("Disconnect", "connect_after.png", connect_after_xpm);
	return;
}

/*
** Set the connected button to connect (the user will get the connect dialog if they press).
*/
void
gui_connect_button_set_connect(void)
{
	gui_connect_button_set("Connect", "connect_before.png", connect_before_xpm);
	return;
}

/*
** Bring the connect button "up to date", set it to "Connect" if not connected and "Disconnect" if connected,
** with the appropriate icons.
*/
void
gui_connect_update_button()
{
	gui_ftp_lookup *lookup = gui_ftp_lookup_widget(gui_lookup_widget("right_tview"));

	DEBUG_MSG(DBG_INFO, "gui _connect udpdate button :)");
	DEBUG_MSG(DBG_INFO, "lookup is %s (%p)", lookup ? "not NULL" : "NULL", lookup);
	if (lookup && lookup->site) {
		/* Is connected. */
		gui_connect_button_set_disconnect();
	} else {
		/* not connected. */
		gui_connect_button_set_connect();
	}

	return;
}

	
void
gui_menubar_setup(GtkWidget *menubar)
{
	GList          *children = NULL;
	GtkWidget      *button = NULL;
	GtkWidget *icon = NULL;
	gui_toolbar *toolbar = NULL;
	gui_toolbar_child *refresh = NULL, *connect = NULL;


	/* Connect before icon */
	if ( !(icon = gui_theme_image_get("connect_before.png")) )
		icon = gui_new_pixmap(menubar, connect_before_xpm);
	
	gui_toolbar_append_item(menubar, GUI_JUSTIFY_LEFT, "Connect", icon, NULL, NULL);

	/* Preferences icon */
	if ( !(icon = gui_theme_image_get("preferences.png")) )
		icon = gui_new_pixmap(menubar, preferences_xpm);

	gui_toolbar_append_item(menubar, GUI_JUSTIFY_LEFT, "Preferences",
				icon, gui_preferences_dialog, NULL);


	/* Site manager */
	if ( !(icon = gui_theme_image_get("siteman.png")) )
		icon = gui_new_pixmap(menubar, siteman_xpm);
		
	gui_toolbar_append_item(menubar, GUI_JUSTIFY_LEFT, "Site manager",
				icon, (GtkSignalFunc)gui_siteman_startup, NULL);


	/* Refresh (before) */
	if ( !(icon = gui_theme_image_get("refresh_before.png")) )
		icon = gui_new_pixmap(menubar, refresh_before_xpm);

	gui_toolbar_append_item(menubar, GUI_JUSTIFY_LEFT, "Refresh",
				icon, gui_refresh_all, menubar);
	/* Stop icon */
	if ( !(icon = gui_theme_image_get("stop.png")) )
		icon = gui_new_pixmap(menubar, stop_xpm);

	gui_toolbar_append_item(menubar, GUI_JUSTIFY_LEFT, "Stop",
				icon, gui_stop, NULL);

	toolbar = g_object_get_data(G_OBJECT(menubar), "gui_toolbar");
	if (!toolbar)
		return;
	
	children = toolbar->children;
	refresh = g_list_nth_data(children, 3);
	connect = g_list_nth_data(children, 0);

	/* XXX preload the images for refresh after and before */
	button = refresh->widget;
	g_signal_connect(G_OBJECT(button), "pressed",
		      G_CALLBACK(gui_button_image_set), "refresh_after.png");
	
	g_signal_connect(G_OBJECT(button), "released",
		     G_CALLBACK(gui_button_image_set), "refresh_before.png");

	connect_button = connect->widget;
	g_signal_connect(G_OBJECT(connect->widget), "pressed",  G_CALLBACK(gui_connect), NULL);

	return;

}

/* destroy dialogs and windows (used by g_signal) */
void
gui_widget_destroy_cb(GtkWidget * data, gpointer object)
{
	gtk_widget_destroy(GTK_WIDGET(object));
}

GtkWidget *
gui_create_arrow_button(GtkArrowType arrow_type,
			GtkShadowType shadow_type)
{
	GtkWidget      *button;
	GtkWidget      *arrow;

	button = gtk_button_new();
	arrow = gtk_arrow_new(arrow_type, shadow_type);

	gtk_container_add(GTK_CONTAINER(button), arrow);

	gtk_widget_show(button);
	gtk_widget_show(arrow);

	return button;
}





