/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_tree_model.c,v 1.12 2003/03/07 17:48:13 thepin Exp $
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <strings.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
/* local headers */
#include <gui_tview.h>
#include <gui_treestore.h>
#include <gui_toolkit.h>
#include <gui.h>
#include <gui_menu.h>
#include <gui_widgets.h>
#include <gui_paths.h>
#include <gui_io.h>
#include <gui_qwin.h>
#include <gui_theme.h>
#include <gui_ftp.h>
#include <gui_ftp_callback.h>
#include <gui_ftp_lookup.h>
#include <gui_dnd_tview.h>
#include <ftp_request.h>
#include <ftp.h>
#include <ftp_cmd.h>
#include <ftp_misc.h>
#include <local.h>
#include <config.h>
#include <error.h>
#include <queue.h>
#include <list_sort.h>
#include <meta.h>
#include <mem.h>
#include <misc.h>
/* icons */
#include <icons/file.xpm>
#include <icons/folder.xpm>
#include <icons/movie.xpm>
#include <icons/image.xpm>
#include <icons/sound.xpm>
#include <icons/doc.xpm>
#include <icons/package.xpm>
#include <icons/cdup.xpm>




void
gui_tree_model_show(_ftp_item *ftp)
{
	GtkTreeModel *model = NULL;	
	GtkWidget *tview = NULL;

	if (!ftp)
		return;
		
	if (!(model = ftp_object_get_data(FTP_OBJECT(ftp), "gui_tree_model")))
		return;

	if (!(tview = gui_lookup_widget("right_tview")))
		return;
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(tview), model);
	
	return;
}

GtkTreeModel *
gui_tree_model_get(_ftp_item *ftp)
{
	if (!ftp)	return NULL;

	return ftp_object_get_data(FTP_OBJECT(ftp), "gui_tree_model");
}

void
gui_tree_model_set(_ftp_item *ftp, GtkTreeView *tree)
{
	GtkTreeModel *model = NULL;
	
	if (!ftp)
		return;
		
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
	ftp_object_set_data(FTP_OBJECT(ftp), "gui_tree_model", model);
	g_object_set_data(G_OBJECT(model), "view", tree);
	return;
}



struct _list *
gui_tree_model_list_sort(GtkTreeModel *model, struct _list *list)
{
	int  *order;
	compare_func compare;
	
	compare = g_object_get_data(G_OBJECT(model), "compare");
	order   = g_object_get_data(G_OBJECT(model), "order");
	
	if(!compare && !order) {
		compare = (compare_func) list_sort_type;
		order = j_malloc(sizeof(int));
		order[0] = 0;
	}
	
	list = list_sort(list, 1, compare, order, 0);
	if(compare == (compare_func) list_sort_type)
		list = gui_sort_type_by_name(list, order);
	
	j_free(order);
	return list;
}





void
gui_tree_model_fill(GtkTreeModel *model, struct _list *list, cb freelist, cb callback)
{
	struct _list *lista = NULL;
	int   *hide;
  
	if (freelist)
		freelist(gui_treestore_get_list(model));

	/*
	 | sort the list
	*/
	list = gui_tree_model_list_sort(model, list);
	
	/*
	 | check if hide or show the hidden files
	 */
	hide = g_object_get_data(G_OBJECT(model), "hidden");
	if(hide && list) {
		if(*list->file == '.') {
			for(;  list->next && *list->next->file == '.'; list = list->next);
			list = list->next;
		}
		
		lista = list;
		while(lista) {
			if (lista->next && *lista->next->file == '.')	
				lista->next = lista->next->next;
			else
				lista = lista->next;
		}
	}
	   
	/*
	 | insert a ".." at the top
	 | done to ensure that the user can do a CDUP
	 | since some directories do not have .. (denied)
	*/
	if ( (lista = (struct _list *)j_malloc(sizeof(struct _list))) == NULL) {
		MEM_ERR("malloc", ERR_SHOW);
		return;
	}
	
	memset(lista, 0, sizeof(struct _list));
	lista->file = j_strdup("..");
	lista->type = _LIST_DIR;
	lista->next = list;
	gtk_tree_store_clear(GTK_TREE_STORE(model));
	gui_treestore_fill(model, lista, NULL, callback);
	
	return;
}

   

void
gui_tree_model_fill_cb(GtkTreeModel *model, struct _list *list, GtkTreeIter *iter)
{
	size_t n;
	struct tm *tp = NULL;
	gchar *item_buf[7] = { 0, 0, 0, 0, 0, 0 };
	char datebuf[32];
	static GdkPixbuf *pixfile = NULL, *pixdir = NULL, *pixmovie = NULL, 
	                 *pixcdup = NULL, *piximg = NULL, *pixsnd = NULL, 
					 *pixpack = NULL, *pixdoc = NULL;
	GdkPixbuf *pix = NULL;
	
	if(!pixfile)
		if ( !(pixfile = gui_theme_pixbuf_get("file.png")) )
		pixfile = gdk_pixbuf_new_from_xpm_data(file_xpm);
	if(!pixdir)
		if ( !(pixdir = gui_theme_pixbuf_get("folder.png")) )
		pixdir  = gdk_pixbuf_new_from_xpm_data(folder_xpm);
	if(!pixcdup)
		if ( !(pixcdup = gui_theme_pixbuf_get("cdup.png")) )
		pixcdup = gdk_pixbuf_new_from_xpm_data(cdup_xpm);
	if(!pixmovie)
		if ( !(pixmovie = gui_theme_pixbuf_get("file_movie.png")) )
		pixmovie = gdk_pixbuf_new_from_xpm_data(movie_xpm);
	if(!piximg)
		if ( !(piximg = gui_theme_pixbuf_get("file_image.png")) )
		piximg = gdk_pixbuf_new_from_xpm_data(image_xpm);
	if(!pixsnd)
		if ( !(pixsnd = gui_theme_pixbuf_get("file_sound.png")) )
		pixsnd = gdk_pixbuf_new_from_xpm_data(sound_xpm);
	if(!pixpack)
		if ( !(pixpack = gui_theme_pixbuf_get("file_package.png")) )
		pixpack = gdk_pixbuf_new_from_xpm_data(package_xpm);
	if(!pixdoc)
		if ( !(pixdoc = gui_theme_pixbuf_get("file_doc.png")) )
		pixdoc = gdk_pixbuf_new_from_xpm_data(doc_xpm);
	
//	DEBUG_MSG(DBG_INFO, "lil, list->file, %s", list->file);
	item_buf[1] = gui_convert_utf8(list->file);
	item_buf[2] = list->size > 0 ? (char*)list->size : 0;

	if (list->mtime > 0) {
		tp = localtime(&list->mtime);
		n = strftime(datebuf, 32, "%b %e %R %Y", tp);
		item_buf[3] = n != 0 ? datebuf : "?";
	} else {
		item_buf[3] = NULL;
		if (list->mtime_string) {
			strncpy(datebuf, list->mtime_string, sizeof(datebuf)-1);
			item_buf[3] = gui_convert_utf8(datebuf);
			
		}
	}
	
   	item_buf[4] = list->perm;
	item_buf[5] = list->owner;
	item_buf[6] = list->group;
		
	if (!strcmp(list->file, ".."))
		pix = pixcdup;
	else if (J_ISFILE(list->type)) {
		switch (meta_get_file_type(list->file)) {
			case META_MOVIE:
				pix = pixmovie;
				break;
			case META_IMG:
				pix = piximg;
				break;
			case META_SND:
				pix = pixsnd;
				break;
			case META_PACKAGE:
				pix = pixpack;
				break;
			case META_DOC:
				pix = pixdoc;
				break;
			case META_UNKNOWN:
				pix = pixfile;
				break;
		}
	} else 
		pix = pixdir;
		
      gtk_tree_store_set(GTK_TREE_STORE(model), iter,
				0, pix,
				1, item_buf[1],
				2, item_buf[2],
				3, item_buf[3],
				4, item_buf[4],
				5, item_buf[5],
				6, item_buf[6],
				-1);

	g_free(item_buf[1]);

	return;
}

