/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_treestore.c,v 1.19 2003/03/07 17:48:13 thepin Exp $
 */


#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
/* local includes */
#include <gui_treestore.h>
#include <gui_tview.h>
#include <gui_dnd.h>
#include <gui_dnd_tview.h>
#include <list_sort.h>
#include <queue.h>
#include <mem.h>

void *
gui_treestore_get_list(GtkTreeModel *model)
{
	void *list;
	list = g_object_get_data(G_OBJECT(model), "list");
	return list;
}

void *
gui_treestore_get_list_from_dndrow(void *list, dnd_row *dnd)
{
	register int i, j;	
	t_tree *lpnt = list;
	
	for(i = 0; i < dnd->depth; i++) {
		for(j = 0; j < dnd->rows[i]; j++)
			if(lpnt) lpnt = lpnt->next;
			
		if(i + 1 < dnd->depth)
			if(lpnt) lpnt = lpnt->child;
	}
	
	return lpnt;
}

void *
gui_treestore_get_list_row(GtkTreeModel *model, dnd_row *dnd)
{
   return gui_treestore_get_list_from_dndrow(gui_treestore_get_list(model), dnd);
}

void *
gui_treestore_get_list_from_selection(GtkTreeView *tree)
{
   GtkTreePath *path;
   void        *list;

   path = gui_tview_get_selected_path(tree);
   if(!path) return NULL;
   list = gui_treestore_get_list_from_path(tree, path);

   gtk_tree_path_free(path);
   return list;
}

void *
gui_treestore_get_list_from_path(GtkTreeView *tree, GtkTreePath *path)
{
   gint        *indices, i, j, z;
   t_tree      *lpnt = gui_treestore_get_list(gtk_tree_view_get_model(tree));

   indices = gtk_tree_path_get_indices(path);
   z = gtk_tree_path_get_depth(path);
   for(i = 0; i < z; i++) {
      for(j = 0; j < indices[i]; j++)  if(lpnt) lpnt = lpnt->next;
      if(i + 1 < z)
         if(lpnt) lpnt = lpnt->child;
   }
   return lpnt;
}

GtkTreeIter
gui_treestore_append_model(GtkTreeModel *model, void *list, gchar *p, cb callback)
{
   GtkTreePath    *path;
   GtkTreeIter     iter;
   GtkTreeIter     parent;

   if(*p) {
      path = gtk_tree_path_new_from_string(p);
      gtk_tree_model_get_iter(model, &parent, path);
      gtk_tree_store_append(GTK_TREE_STORE(model), &iter, &parent);
   }
   else
      gtk_tree_store_append(GTK_TREE_STORE(model), &iter, NULL);
   
   
    callback(model, list, &iter);

	return iter;
}

GtkTreeIter
gui_treestore_append(GtkTreeView *tree, void *list, gchar *p, cb callback)
{
	GtkTreeModel   *model;
	GtkTreePath    *path;
	GtkTreeIter     iter;
	GtkTreeIter     parent;

	model = gtk_tree_view_get_model(tree);

	if(*p) {
		path = gtk_tree_path_new_from_string(p);
		gtk_tree_model_get_iter(model, &parent, path);
		gtk_tree_store_append(GTK_TREE_STORE(model), &iter, &parent);
		gtk_tree_path_free(path);
	}
	else
		gtk_tree_store_append(GTK_TREE_STORE(model), &iter, NULL);

	callback(model, list, &iter);

	return iter;
}


  
void
gui_treestore_append_list_model(GtkTreeModel *model, void *l, gchar *p)
{
  g_object_set_data(G_OBJECT(model), "list", 
		  jtree_append_from_path(gui_treestore_get_list(model), l, p));
  
#if 0
   int          row,   i;
   char        *pather, *path, *j;
   t_tree      *tmp,  *lpnt, *list = l, *head = gui_treestore_get_list(tree);
   lpnt = head;
   
   pather = j_strdup(p);
   path   = pather;
   if(!*path) {
      for(; lpnt && lpnt->next; lpnt = lpnt->next);
      if(lpnt)
         lpnt->next = list, list->parent = lpnt->parent;
      else {
         if(head)
            head->next = list, list->parent = head->parent;
         else
            head = list;
      }
   }
   else {
      do {
         j = strchr(path, ':');
         if(j) *j = '\0';
         row = atoi(path);
         if(j) path = j + 1;
         for(i = 0; i < row; i++)   lpnt = lpnt->next;
         if(j)                      lpnt = lpnt->child;
      } while(j);
      
		for(tmp = lpnt->child; tmp && tmp->next; tmp = tmp->next);

		if(tmp)
			tmp->next   = list, list->parent = tmp->parent;
		else
			lpnt->child = list, list->parent = lpnt;
	}

	free(pather);
	g_object_set_data(G_OBJECT(model), "list", head);
	return;
#endif
}

void
gui_treestore_append_list(GtkTreeView *tree, void *l, gchar *p)
{
	gui_treestore_append_list_model(gtk_tree_view_get_model(tree), l, p);
	return;
}

void
gui_treestore_remove(GtkTreeView *tree, GtkTreePath *p)
{  
	GtkTreeIter   iter;
   GtkTreeModel *model = gtk_tree_view_get_model(tree);
	gtk_tree_model_get_iter(model, &iter, p);
	gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
}

void
gui_treestore_remove_list(GtkTreeView *tree, GtkTreePath *p, cb freecb)
{
	GtkTreeModel *model = NULL;
	t_tree *list, *plist, *blist, *head;
	GtkTreePath *parentpath;
   
   	model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
	head = gui_treestore_get_list(model);
	list = gui_treestore_get_list_from_path(tree, p);
  
   
   parentpath = gtk_tree_path_copy(p);
   gtk_tree_path_up(parentpath);
   if(gtk_tree_path_to_string(parentpath)) {
      plist = gui_treestore_get_list_from_path(tree, parentpath);
      if(plist->child == list)   plist->child = list->next;
      else {
         for(blist = plist->child; blist->next != list; blist = blist->next);
         blist->next = list->next;
      }
   }
   else {
      if(list == head) g_object_set_data(G_OBJECT(model), "list", list->next);
      else {
         for(blist=head; blist->next && blist->next!=list; blist=blist->next);
         blist->next = list->next;
      }
   }
   
   if(freecb)	freecb(list);
}

void
gui_treestore_move(GtkTreeView *tree, cb callback,
                   void *source, void *destination, 
                   GtkTreePath *psrc, GtkTreePath *pdst)
{
   GtkTreeModel *model;
   GtkTreeRowReference *rowsrc, *rowdst;
   GtkTreeIter   iter;
   t_tree *src = source, *dst = destination;
   t_tree *parent, *bro, *head;
   parent = src->parent;

   head = gui_treestore_get_list(gtk_tree_view_get_model(tree));
   
   if(gtk_tree_path_is_descendant(pdst, psrc)) return;

   if(parent) {
      if(parent->child == src)   parent->child = src->next;
      else {
         for(bro = parent->child; bro->next != src; bro = bro->next);
         if(bro->next)  bro->next = src->next;
      }
   }
   else {
      bro = head;
      for(; bro->next &&  bro->next != src; bro = bro->next);
      if(bro->next)  bro->next = src->next;
      if(src == head)   head = src->next;
   }

   if(dst->type == GUI_TREESTORE_ITEM)
      src->next = dst->next,  dst->next = src,  src->parent = dst->parent;
   else
      src->next = dst->child, dst->child = src, src->parent = dst;


   model = gtk_tree_view_get_model(tree);
   rowsrc = gtk_tree_row_reference_new(model, psrc);
   rowdst = gtk_tree_row_reference_new(model, pdst);
   if(dst->type == GUI_TREESTORE_ITEM)
      gui_treestore_move_rowref(tree, callback, rowsrc, NULL, rowdst, 0);
   else
      gui_treestore_move_rowref(tree, callback, rowsrc, rowdst, NULL, 0);
   psrc = gtk_tree_row_reference_get_path(rowsrc);
   gtk_tree_model_get_iter(model, &iter, psrc);
   gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);

   g_object_set_data(G_OBJECT(tree), "list", head);
}


void
gui_treestore_move_rowref(GtkTreeView *tree,
                          cb callback,
                          GtkTreeRowReference *rsrc,
                          GtkTreeRowReference *rdst,
                          GtkTreeRowReference *rchd,
                          int child)
{
   GtkTreeModel *model;
   GtkTreePath  *next;
   GtkTreePath  *src, *dst;
   GtkTreeIter itersrc, iternext;
   GtkTreeIter  *iterdst;

   iterdst = j_malloc(sizeof(GtkTreeIter));
   
   rdst = rdst ? rdst : rchd;
   src = gtk_tree_row_reference_get_path(rsrc);
   dst = gtk_tree_row_reference_get_path(rdst);
   /* copy the current node */
   model = gtk_tree_view_get_model(tree);
   gtk_tree_model_get_iter(model, &itersrc, src);
   gtk_tree_model_get_iter(model, iterdst, dst);
   callback(model, itersrc, rchd ? 0 : iterdst, rchd ? iterdst : 0);
   /* check if the node has a child */
   if(gtk_tree_model_iter_has_child(model, &itersrc)) {
      src = gtk_tree_row_reference_get_path(rsrc);
      dst = gtk_tree_row_reference_get_path(rdst);
      gtk_tree_path_down(src);
      if(rchd)
         gtk_tree_path_next(dst);
      if(!rchd)
         gtk_tree_path_down(dst);
      rsrc = gtk_tree_row_reference_new(model, src);
      rdst = gtk_tree_row_reference_new(model, dst);
      gui_treestore_move_rowref(tree, callback, rsrc, rdst, NULL, 1);
      gtk_tree_path_up(src);
      gtk_tree_path_up(dst);
   }
   /* check if the child has next */
   if(child) {
      iternext = itersrc;
      if(gtk_tree_model_iter_next(model, &iternext)) {
         next = gtk_tree_model_get_path(model, &iternext);
         rsrc = gtk_tree_row_reference_new(model, next);
         rdst = gtk_tree_row_reference_new(model, dst);
         gui_treestore_move_rowref(tree, callback, rsrc, rdst, NULL, 1);
         gtk_tree_path_free(next);
      }
   }
   j_free(iterdst);
}

void
gui_treestore_fill(GtkTreeModel *model, void *list, GtkTreeIter *pa, cb callback)
{
   GtkTreeIter  *iter;
   register void *lista = NULL;
   
   iter = j_malloc(sizeof(GtkTreeIter));
  
   
   lista = list;
   while(lista) {
      register t_tree *lpnt = lista;
      gtk_tree_store_append(GTK_TREE_STORE(model), iter, pa);
      callback(model, lista, iter);
      if(lpnt->child)   {
         gui_treestore_fill(model, lpnt->child, iter, callback);
      }
      lista = lpnt->next;
   }
   
	g_object_set_data(G_OBJECT(model), "list", list);
	j_free(iter);
}
