/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_tview.c,v 1.98 2003/03/07 17:48:13 thepin Exp $
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <strings.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
/* local headers */
#include <gui_tview.h>
#include <gui_treestore.h>
#include <gui_toolkit.h>
#include <gui.h>
#include <gui_misc.h>
#include <gui_menu.h>
#include <gui_widgets.h>
#include <gui_paths.h>
#include <gui_io.h>
#include <gui_qwin.h>
#include <gui_theme.h>
#include <gui_ftp.h>
#include <gui_ftp_callback.h>
#include <gui_ftp_lookup.h>
#include <gui_dnd_tview.h>
#include <gui_tree_model.h>
#include <ftp_request.h>
#include <ftp.h>
#include <ftp_cmd.h>
#include <ftp_misc.h>
#include <local.h>
#include <config.h>
#include <error.h>
#include <queue.h>
#include <list_sort.h>
#include <meta.h>
#include <misc.h>
#include <mem.h>


void
gui_tree_double_click_cb(GtkTreeView *tree, GtkTreePath *path, GtkTreeViewColumn *arg2, gpointer data)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	GtkWidget *ltview = NULL;
	gui_ftp_lookup *lookup = NULL;
	struct _list *list_dir = NULL;
	char wd[255];
	ftp_request *req = NULL;
	int handler;
	
	DEBUG_MSG(DBG_INFO, "start");
	
	model = gtk_tree_view_get_model(tree);
	gtk_tree_model_get_iter(model, &iter, path);
	

//	handler = g_signal_handler_find(G_OBJECT(tree),G_SIGNAL_MATCH_FUNC,
//			0, 0, 0, gui_tree_double_click_cb, 0);
//	g_signal_handler_block(G_OBJECT(tree),handler);

	handler = gui_tview_handler_block(GTK_WIDGET(tree));
	
	list_dir = gui_treestore_get_list_from_path(tree, path);
	if (!list_dir)
		return;
	
	lookup = gui_ftp_lookup_widget(GTK_WIDGET(tree));
	DEBUG_MSG(DBG_INFO, "called as %s", lookup ? "remote" : "local");
	if (!lookup) {
		/* local. */

		if(J_ISDIR(list_dir->type)) {
			/* local cwd */
			local_cwd(list_dir->file);
			getcwd(wd, 255);
			gui_paths_text(1, wd);
			list_dir = local_list();
//			gui_tview_fill(GTK_TREE_VIEW(tree), list_dir, 
//					ftp_free_list, gui_fill_tview_cb);
			gui_tree_model_fill(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)), list_dir,
					ftp_free_list, gui_tree_model_fill_cb);
		}
		else if (J_ISFILE(list_dir->type)) {
			/* Upload file */
			if (!(lookup = gui_ftp_lookup_widget(gui_lookup_widget("right_tview"))))
				return;
				
			ftp_stor(lookup->site, RP_END, NULL, 0, NULL, list_dir, list_dir->file, 0); 
			req = ftp_list_with_cb(lookup->site, gui_ftp_callback_list, 1, 0);
			ftp_object_set_data(FTP_OBJECT(req), "widget", lookup->widget);
		}
	} else {
		/* remote. */
		g_object_set_data(G_OBJECT(lookup->widget), "freeze", (gpointer)1);
		if (J_ISDIR(list_dir->type) && lookup->site) {
			/* CWD to a dir and list it */
			DEBUG_MSG(DBG_INFO, "i'm about to to cwd to %s (%p)",
					list_dir->file, lookup->site);
			req = ftp_cwd(lookup->site, list_dir->file);
			ftp_pwd_with_req(lookup->site, req);
			/* gui_paths_text(0, wd); XXX fix paths */
			req = ftp_list_with_cb(lookup->site, gui_ftp_callback_list, 0, 0);
			ftp_object_set_data(FTP_OBJECT(req), "widget", tree);
		}
		else if (J_ISFILE(list_dir->type) && lookup->site) {
			/* Download file */
			req = ftp_retr(lookup->site, RP_END, NULL, 0, gui_ftp_callback_local_list, list_dir, NULL, TRFR_DEFAULT);
			ltview = gui_lookup_widget("left_tview");
			ftp_object_set_data(FTP_OBJECT(req), "widget", ltview);
		}
		else if (J_ISLINK(list_dir->type) && lookup->site) {
			/* CWD to a dir and list it */
			req = ftp_cwd_wrapper(lookup->site, RP_END, NULL, R_SUCCESS | R_FAILURE, gui_ftp_link_handler_callback, list_dir->file);
			ftp_object_set_data(FTP_OBJECT(req), "link_file", strdup(list_dir->file));
			ftp_object_set_data(FTP_OBJECT(req), "link_file_size", (void *)list_dir->size);
		}
	}

	DEBUG_MSG(DBG_INFO, "ploudf? (c) 2003 ya");
	if (!req)
		g_signal_handler_unblock(G_OBJECT(tree),handler);
	else {
		req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
				        gui_tview_handler_unblock, 0);
		ftp_object_set_data(FTP_OBJECT(req), "tree", 
				gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	}
}
	
int
gui_tview_handler_block(GtkWidget *tview)
{
	GObject *tree = G_OBJECT(tview);

	int handler = g_signal_handler_find(tree, G_SIGNAL_MATCH_FUNC,
			0, 0, 0, gui_tree_double_click_cb, 0);
	g_signal_handler_block(tree,handler);

	DEBUG_MSG(DBG_INFO, "setting 1 to %p", gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	g_object_set_data(G_OBJECT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree))), 
			          "freeze", (gpointer)1);

	return handler;
}


FTP_CB_CMD(gui_tview_handler_unblock)
{
	GtkTreeView  *tree;  
	GtkTreeModel *model;
	int handler;

	model = ftp_object_get_data(obj, "tree");
	DEBUG_MSG(DBG_INFO, "model ----> %p", model);
	tree = g_object_get_data(G_OBJECT(model), "view");
	DEBUG_MSG(DBG_INFO, "tree ----> %p", tree);

	if(!g_object_get_data(G_OBJECT(model), "freeze"))	return;
	
	g_object_set_data(G_OBJECT(model),
			                      "freeze", (gpointer)0);
	DEBUG_MSG(DBG_INFO, "setting 0 to %p", gtk_tree_view_get_model(tree));
	
	
	if(model != gtk_tree_view_get_model(tree))	return;
	handler = (int)g_object_get_data(G_OBJECT(tree), "handler");
	DEBUG_MSG(DBG_INFO, "now! unblocking the tree");
	g_signal_handler_unblock(tree, handler);
}

	

void gui_tree_sort_cb(GtkTreeSortable *tlist, gpointer data)
{
	gint id, last_id;
	static int asc[] = { 0 };
	static int dec[] = { 1 };
	int *order, *last_order;
	compare_func compare = 0;
	GtkTreeView *tree;
	GtkTreeViewColumn *column, *column_old;
	struct _list *list;
	
	id = (int)g_object_get_data(G_OBJECT(tlist),"id");
	
	if ( (tree = g_object_get_data(G_OBJECT(tlist), "view")) == NULL)
		return;

	column = gtk_tree_view_get_column(tree, id);
	
	last_id    = (int)g_object_get_data(G_OBJECT(tlist),"last");
	last_order = g_object_get_data(G_OBJECT(tree), "order");

	if(last_id == id) {
		order = last_order == asc ? dec : asc;
		gtk_tree_view_column_set_sort_order(column, 
				order == asc ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
	}
	else {
		order = asc;
		column_old = gtk_tree_view_get_column(tree, last_id);
		gtk_tree_view_column_set_sort_indicator(column_old, FALSE);
		gtk_tree_view_column_set_sort_order(column, GTK_SORT_ASCENDING);
		gtk_tree_view_column_set_sort_indicator(column, TRUE);
	}
	
	switch(id) {
		case 6:
			compare = (compare_func) list_sort_group;
			break;
		case 5:
			compare = (compare_func) list_sort_owner;
			break;
		case 4:
			compare = (compare_func) list_sort_perm;
			break;
		case 3:
			compare = (compare_func) list_sort_date;
			break;
		case 2:
			compare = (compare_func) list_sort_size;
			break;
		case 1:
			compare = (compare_func) list_sort_file;
			break;
		case 0:
			compare = (compare_func) list_sort_type;
			break;
	}
	
	g_object_set_data(G_OBJECT(tree),"compare",compare);
	g_object_set_data(G_OBJECT(tree),"order", order);
	g_object_set_data(G_OBJECT(tlist),"last", (gpointer)id);
	list = (struct _list *)list_sort(gui_treestore_get_list(gtk_tree_view_get_model(tree)), 
                                    1, compare, order, 0);
	if(id == 0)  list = gui_sort_type_by_name(list,order);
	gtk_tree_store_clear(GTK_TREE_STORE(
				gtk_tree_view_get_model(GTK_TREE_VIEW(tree))));
	gui_treestore_fill(gtk_tree_view_get_model(tree), list,
                      NULL, gui_tree_model_fill_cb );
}

struct _list *
gui_sort_type_by_name(struct _list *list, int* order)
{
	/* lame code, fix me :) */

		struct _list *t,  *dirs, *files;
		int type;

		if(!list)	return NULL;
		
		type = *order ? _LIST_FILE : _LIST_DIR;
		dirs = list->type == type ? list : NULL;

		
		if(dirs) {
			t = dirs->next;
			for(; t && t->type == type; t = t->next) {
				dirs->next = t;
				dirs = t;
			}
			dirs->next = NULL;
			dirs = list_sort(list, 1, (compare_func)list_sort_file, order, 0);
		
			list = files = t;
			t = t ? t->next : 0;
			for(; t; t = t->next) {
				files->next = t;
				files = t;
			}

			files = list_sort(list, 1, (compare_func)list_sort_file, order, 0);
		
			for(list = dirs; dirs->next; dirs = dirs->next);
			dirs->next = files;
		}
		else
			list = list_sort(list, 1, (compare_func)list_sort_file, order, 0);
		
		return list;
}


void
header_clicked_cb(GtkButton *button, gpointer ptr) {
	struct { GtkTreeView* widget; int id; } *data = ptr;
	GtkTreeStore *store = GTK_TREE_STORE(gtk_tree_view_get_model(data->widget));
	g_object_set_data(G_OBJECT(store), "id", (gpointer)data->id);
	g_signal_emit_by_name(store,"sort-column-changed", 0);
}

void
gui_tview_add_sighandler(GtkTreeView *tview, GtkTreeViewColumn *column, int id)
{
	struct foo { GtkTreeView* widget; int id; } * data;
	GtkButton *button;

	data = j_malloc(sizeof(struct foo));
	if (!data) {
		MEM_ERR("couldn't alloc data", ERR_SHOW);
		return;
	}

	gtk_tree_view_column_set_clickable(column, TRUE);
	button = GTK_BUTTON(column->button);
	data->widget = tview;
	data->id = id;
	g_signal_connect(G_OBJECT(button),"pressed",
		G_CALLBACK(header_clicked_cb), (gpointer)data);
}

GtkTreeModel *
gui_tview_model_init()
{
	GtkTreeStore *store = 0;
	store = gtk_tree_store_new(7, GDK_TYPE_PIXBUF,
			G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING,
			G_TYPE_STRING, G_TYPE_STRING);

	/* set up the signal "sort-column-changed" handler */
	g_signal_connect(G_OBJECT(store), "sort-column-changed",
	    	            G_CALLBACK(gui_tree_sort_cb), NULL);
		
	
	return GTK_TREE_MODEL(store);
}
	


GtkTreeModel *
gui_tview_model_setup(GtkTreeView *tree)
{
	GtkCellRenderer *renderer = 0;
	GtkTreeModel *model = 0;
	GtkTreeViewColumn *column = 0;

	
	model = gui_tview_model_init();
	
	g_object_set_data(G_OBJECT(model),"view",tree); //first setup

	/* add pixmap column */
	renderer = gtk_cell_renderer_pixbuf_new();
	column = gtk_tree_view_column_new_with_attributes(
			"", renderer, "pixbuf", 0, NULL);
	
	/* set the column to a fixed sizing of 27 pixels */
	gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column), GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 27);
	gtk_tree_view_append_column(tree, column);
	gui_tview_add_sighandler(tree, column, 0);

	/* column for name */
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
			"Name", renderer, "text", 1, NULL);
	gtk_tree_view_append_column(tree, column);
	gui_tview_add_sighandler(tree, column, 1);
	
	/* column for size */
	column = gtk_tree_view_column_new_with_attributes(
			"Size", renderer, "text", 2, NULL);
	gtk_tree_view_append_column(tree, column);
	gui_tview_add_sighandler(tree, column, 2);
	
	/* column for date */
	column = gtk_tree_view_column_new_with_attributes(
			"Date", renderer, "text", 3, NULL);
	gtk_tree_view_append_column(tree, column);
	gui_tview_add_sighandler(tree, column, 3);
	
	/* column for permissions */
	column = gtk_tree_view_column_new_with_attributes(
			"Permissions", renderer, "text", 4, NULL);
	gtk_tree_view_append_column(tree, column);
	gui_tview_add_sighandler(tree, column, 4);
	
	/* column for owner */
	column = gtk_tree_view_column_new_with_attributes(
			"Owner", renderer, "text", 5, NULL);
	gtk_tree_view_append_column(tree, column);
	gui_tview_add_sighandler(tree, column, 5);
	
	/* column for group */
	column = gtk_tree_view_column_new_with_attributes(
			"Group", renderer, "text", 6, NULL);
	gtk_tree_view_append_column(tree, column);
	gui_tview_add_sighandler(tree, column, 6);

	
	return model;
}

	
GtkWidget *
gui_setup_tview(GtkWidget *scrolled)
{
	GtkTreeView      *tree;
	GtkTreeModel     *model;
	GtkTreeSelection *select;
	
	/*setup the tree model */
	tree  = GTK_TREE_VIEW(gtk_tree_view_new());
	model = gui_tview_model_setup(tree);
	gtk_tree_view_set_model(tree, model);
	
	/* setup the selection handler */
	select = gtk_tree_view_get_selection(tree);
	gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
	
	/* simple click */
/* XXX any need for this?
	g_signal_connect(G_OBJECT(select), "changed", 
			G_CALLBACK(gui_tree_selection_changed_cb), NULL);
*/
	
	/* double click */
	DEBUG_MSG(DBG_INFO, "setting up the signals");
	g_object_set_data(G_OBJECT(tree), "handler", (gpointer)
			g_signal_connect(G_OBJECT(tree), "row_activated",
				G_CALLBACK(gui_tree_double_click_cb), NULL));

	/* connect to keypress so to select the first item starting with that char */
	g_signal_connect(G_OBJECT(tree), "key_press_event",
			G_CALLBACK(gui_treeview_select_bychar), NULL);
	
	gtk_container_add(GTK_CONTAINER(scrolled), GTK_WIDGET(tree));

	return GTK_WIDGET(tree);
}


gint *
gui_tview_get_selected_row(GtkTreeView *tree)
{
	gint *indices = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	GtkTreePath *path = NULL;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
		 return NULL;
		 
	if ( !(path = gtk_tree_model_get_path(GTK_TREE_MODEL(model), &iter)) )
		 return NULL;
		 
	if ( !(indices = gtk_tree_path_get_indices(path)) )
		 return NULL;
		 
	return indices;
}

GtkTreePath *
gui_tview_get_selected_path(GtkTreeView *tree)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
		return NULL;
		
	return gtk_tree_model_get_path(GTK_TREE_MODEL(model), &iter);
}

/*
** Note that gtk2.1.0 has added a function that will replace this function in later
** versions gtk_tree_selection_count_selected_rows()
*/
int
gui_tview_selected_n(GtkTreeView *tree)
{
	int n = 0;
	GtkTreeSelection *selection = NULL;
	
	selection = gtk_tree_view_get_selection(tree);
	gtk_tree_selection_selected_foreach(selection, (GtkTreeSelectionForeachFunc)increment, &n);
	DEBUG_MSG(DBG_INFO, "n = %d\n", n);

	return n;
}

void
increment(GtkTreeModel *model, GtkTreePath  *path, GtkTreeIter  *iter, int *n) 
{
	(*n)++;
	return;
}

GSList *
gui_tview_get_selected_row_refs(GtkTreeView *tree)
{
	GtkTreeSelection *selection = NULL;
	
	g_slist_free(g_object_get_data(G_OBJECT(tree), "selected"));
	g_object_set_data(G_OBJECT(tree), "selected", NULL);
	selection = gtk_tree_view_get_selection(tree);
	gtk_tree_selection_selected_foreach(selection,
			(GtkTreeSelectionForeachFunc) gui_tview_return_selected_cb, tree);
	
	return g_object_get_data(G_OBJECT(tree), "selected");
}

void
gui_tview_return_selected_cb(GtkTreeModel *model,
		                       GtkTreePath  *path,
									  GtkTreeIter  *iter,
									  GtkTreeView  *tree)
{
	GtkTreeRowReference *row = NULL;
	row = gtk_tree_row_reference_new(model, path);
	g_object_set_data(G_OBJECT(tree), "selected",
			g_slist_append(
				g_object_get_data(G_OBJECT(tree), "selected"), row));
}
	


int
gui_treeview_select_bychar(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
//	gchar *text = NULL;
//	GtkTreeIter iter;
//	GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	
//	if (!model)
//		return;
		
	if(event->keyval == 65293) {
		GtkTreePath *path = NULL;
		
		DEBUG_MSG(DBG_INFO, "you pressed RETURN");
		gtk_tree_view_get_cursor(GTK_TREE_VIEW(widget), &path, 0);
		gui_tree_double_click_cb(GTK_TREE_VIEW(widget), path, 0, 0);
		return 1;
	}
/*XXX segfaulter */
/*	gtk_tree_model_get_iter_first (model, &iter);
	do {
		gtk_tree_model_get(model, &iter, 1, &text, -1);
		if(text && !strncmp(event->string, text, 1)) {
			GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
			gtk_tree_view_set_cursor(GTK_TREE_VIEW(widget), path, 0, 0);
			break;
		}
	} while(gtk_tree_model_iter_next(model, &iter));
*/
	return 0;
}
		

void
gui_tview_dnd_setup(GtkTreeView *left, GtkTreeView *right)
{
	 gui_dnd_tview_enable(left);
	 gui_dnd_tview_scroll_set(left, 1);
	 gui_dnd_tview_highlight_add(left, _LIST_DIR,  "black",  DNDTVIEW_HIRECT);

	 gui_dnd_tview_enable(right);
	 gui_dnd_tview_scroll_set(right, 1);
	 gui_dnd_tview_highlight_add(right, _LIST_DIR,  "black",  DNDTVIEW_HIRECT);

	 gui_dnd_tview_interaction(left,  left,   gui_tview_move,    GDK_ACTION_COPY);
	 gui_dnd_tview_interaction(left,  right,  gui_tview_upload,  GDK_ACTION_COPY);
	 gui_dnd_tview_interaction(right, left,   gui_tview_dwload,  GDK_ACTION_COPY);
	 gui_dnd_tview_interaction(right, right,  gui_tview_ftpmove, GDK_ACTION_COPY);
}


void
gui_tview_dwload(GtkWidget *widget, GdkDragContext *dc,
					  gint x, gint y, guint t, GSList *glist)
{
	struct _list *list = NULL;
	GtkWidget *source = NULL;
	ftp_request *req = NULL;
	gui_ftp_lookup *lookup = NULL;
	GtkWidget *tree = gui_lookup_widget("right_tview");

	DEBUG_MSG(DBG_INFO, "gui_tview_dwload");
	source = gtk_drag_get_source_widget(dc);
	if (!(lookup = gui_ftp_lookup_widget(tree)))
		return;

	if(glist)	gui_tview_handler_block(tree);
	
	while (glist) {
		list = glist->data;
		
		if(!list) {
			gtk_drag_finish(dc, 0, 0, t);
			return;
		 }
			
		
		if(J_ISFILE(list->type)) {
			DEBUG_MSG(DBG_INFO, "that one is a file... lookup is %s and lookup->site is %s",
				lookup ? "set" : "not set", lookup ? (lookup->site ? "set" : "not set") : "not set");
			if (lookup->site) {
				req = ftp_retr(lookup->site, RP_END, NULL, 0,
						gui_ftp_callback_local_list, list, NULL, TRFR_DEFAULT);
				ftp_object_set_data(FTP_OBJECT(req), "widget", widget);
			}
		}
		else if(J_ISDIR(list->type)) {
			DEBUG_MSG(DBG_INFO, "that one is a whooole fucking directory");
			if (lookup->site) {
				localdir_retr(lookup->site, list);
				req = ftp_list_with_cb(lookup->site,
						gui_ftp_callback_local_list, 1, 0);
				ftp_object_set_data(FTP_OBJECT(req), "widget", widget);
			}
		
		}

		glist = g_slist_next(glist);
	}

	if(req) {
		req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
			                        gui_tview_handler_unblock, 0);
	    ftp_object_set_data(FTP_OBJECT(req), "tree",
				                gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	}	
	
	return;
}

void
gui_tview_upload(GtkWidget *widget, GdkDragContext *dc,
		           gint x, gint y, guint t, GSList *glist)
{
	struct _list *list = NULL;
	GtkWidget *source = NULL;
	gui_ftp_lookup *lookup = NULL;
	ftp_request *req = NULL;
	GtkWidget *tree = gui_lookup_widget("right_tview");
	
	source = gtk_drag_get_source_widget(dc);
	if (!(lookup = gui_ftp_lookup_widget(tree)))
		return;
 
	if(glist)	gui_tview_handler_block(tree);
	
	while(glist) {
		list = glist->data;
			
		if (J_ISFILE(list->type)) {
			if (lookup->site) {
				req = ftp_stor(lookup->site, RP_END, NULL, 0, NULL, 
						list, NULL, 0);
				req = gui_ftp_list(lookup->site, 
						gui_ftp_callback_list, 1, NULL, "widget", widget);
			}
		}
		else {
			if (lookup->site) {
				localdir_stor(lookup->site, list);
				//ftp_cwd(lookup->site, "..");
				req = ftp_list_with_cb(lookup->site, 
						gui_ftp_callback_list, 1, 0);
				ftp_object_set_data(FTP_OBJECT(req), "widget", widget);

				/* gui_fill_tview(GTK_WIDGET(widget), list, gtk_tview_local->tview_mode); XXX */
			}
		}

		glist = g_slist_next(glist);
	}

	if(req) {
		req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
				                        gui_tview_handler_unblock, 0);
	    ftp_object_set_data(FTP_OBJECT(req), "tree",
				                gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	}
	
	return;
}

void
gui_tview_move(GtkWidget *widget, GdkDragContext *dc,
					gint x, gint y, guint t, GSList *glist)
{
	struct _list *list = NULL;
	GtkTreePath *path = NULL;
	int tx, ty;
	struct _list *list_drop = NULL;
	GSList *g = NULL;
	
	for(g = glist; g; g=g_slist_next(g))
		DEBUG_MSG(DBG_INFO, "glist = %s", ((struct _list*)g->data)->file);
	
		
	/* calculate row and column at which the drop has occured */
	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
			x, y, &path, NULL, &tx, &ty);
	
	gtk_tree_path_prev(path);
	list_drop = gui_treestore_get_list_from_path(GTK_TREE_VIEW(widget), path);
	if(!list_drop || !list_drop->file)
		return;

	DEBUG_MSG(DBG_INFO, "list_drop is %s", list_drop->file);
	while(glist) {
		list = glist->data;

		DEBUG_MSG(DBG_INFO, "list is %s",list->file);
      
		if(!list || !list->file)	return;
		local_mv(list->file, list_drop->file);

		glist = g_slist_next(glist);
	}
	gui_refresh_all();
}

void
gui_tview_ftpmove(GtkWidget *widget, GdkDragContext *dc,
		            gint x, gint y, guint t, GSList *glist)
{
	struct _list *list = NULL, *list_drop = NULL;
	GtkTreePath  *path = NULL;
	gui_ftp_lookup *lookup = NULL;
	char *file = NULL, *new = NULL;
	ftp_request *req = NULL;
	
	/* calculate row and column at which the drop has occured */
	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget), x, y, 
			&path, NULL, 0, 0);
	if(!path)	return;
	gtk_tree_path_prev(path);
	if(!path)	return;
		
	list_drop = gui_treestore_get_list_from_path(GTK_TREE_VIEW(widget), path);
	if (!(lookup = gui_ftp_lookup_widget(widget)))
		return;
		
	if(!list_drop || !list_drop->file)	return;
	if(!J_ISDIR(list_drop->type))		return;
	
	if(glist)	gui_tview_handler_block(widget);
	
   while(glist) {
		list = glist->data;

		if(!list || !list->file)	return;
		file = strdup(list->file);
		new = j_asprintf("%s/%s", list_drop->file, file);
		req = ftp_move_wrapper(lookup->site, RP_END,
						NULL, 0, NULL, file, new);

		glist = g_slist_next(glist);
	}
   
	if(req) {
		req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
				                 gui_tview_handler_unblock, 0);
		
		ftp_object_set_data(FTP_OBJECT(req), "tree",
	       			gtk_tree_view_get_model(GTK_TREE_VIEW(widget)));
	}
   
	gui_refresh_remote_forced();
}
