/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gui_tview_popup.c,v 1.37 2003/03/07 17:48:13 thepin Exp $
 */


#define _GNU_SOURCE

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <time.h>
/* local headers */
#include <gui.h>
#include <gui_misc.h>
#include <gui_widgets.h>
#include <gui_ftp.h>
#include <gui_ftp_lookup.h>
#include <gui_ftp_callback.h>
#include <gui_treestore.h>
#include <gui_toolkit.h>
#include <gui_tview.h>
#include <gui_popup.h>
#include <gui_tview_popup.h>
#include <gui_qwin.h>
#include <gui_player.h>
#include <ftp.h>
#include <ftp_net.h>
#include <ftp_request.h>
#include <ftp_misc.h>
#include <ftp_cmd.h>
#include <ftp_transfer.h>
#include <queue.h>
#include <local.h>
#include <log.h>
#include <config.h>
#include <cache.h>
#include <error.h>
#include <defaults.h>
#include <meta.h>
#include <mem.h>

/*
** Setup a popup menu for right clicks on the "remote" treeview
*/
GtkWidget *
gui_tview_popup_init_remote(GtkWidget *tview)
{
	GtkWidget *popup_menu = NULL;
	
	popup_menu = gui_popup_new_tearoff(gui_tview_popup_update_remote);
	gui_popup_add_item(popup_menu, "   Download      ", 
		G_CALLBACK(gui_tview_popup_download), tview);
	gui_popup_add_item(popup_menu, "   Edit on the fly",
			G_CALLBACK(gui_tview_popup_edit_fly), tview);
	gui_popup_add_item(popup_menu, "   View ",
			G_CALLBACK(gui_tview_popup_view_fly), tview);
	gui_popup_add_item(popup_menu, "   Add To Queue  ", 
			G_CALLBACK(gui_tview_popup_add_queue), tview);
	gui_popup_add_item(popup_menu, "   Delete  ",
			G_CALLBACK(gui_tview_popup_delete), tview);
	gui_popup_add_item(popup_menu, "   Rename        ",
			G_CALLBACK(gui_tview_popup_rename), tview);
	gui_popup_add_item(popup_menu, "   MkDir         ",
			G_CALLBACK(gui_tview_popup_mkdir), tview);
	gui_popup_add_item(popup_menu, "   Raw Comand    ",
			G_CALLBACK(gui_tview_popup_raw), tview);
	gui_popup_add_item(popup_menu, "   Show/Hide hidden files ",
			G_CALLBACK(gui_tview_hidden), tview);
	gui_popup_add_item(popup_menu, "   Refresh ",
			G_CALLBACK(gui_refresh_remote_forced), tview);
	
	return popup_menu;
}


/*
** Setup the popup menu for right clicks on the "local" treeview
*/
GtkWidget *
gui_tview_popup_init_local(GtkWidget *tview)
{
	GtkWidget *popup_menu = NULL;

	popup_menu = gui_popup_new_tearoff(gui_tview_popup_update_local);
	gui_popup_add_item(popup_menu, "   Upload      ",
			G_CALLBACK(gui_tview_popup_download), tview);
	gui_popup_add_item(popup_menu, "   Add To Queue  ",
			G_CALLBACK(gui_tview_popup_add_queue), tview);
	gui_popup_add_item(popup_menu, "   Delete  ",
			G_CALLBACK(gui_tview_popup_delete), tview);
	gui_popup_add_item(popup_menu, "   Rename  ", 
			G_CALLBACK(gui_tview_popup_rename), tview);
	gui_popup_add_item(popup_menu, "   Chmod   ", 
			G_CALLBACK(gui_tview_popup_chmod), tview);
	gui_popup_add_item(popup_menu, "   MkDir   ", 
			G_CALLBACK(gui_tview_popup_mkdir), tview);
	gui_popup_add_item(popup_menu, "   Show/Hide hidden files ",
	        G_CALLBACK(gui_tview_hidden), tview);
	gui_popup_add_item(popup_menu, "   Refresh ",
			G_CALLBACK(gui_refresh_local_forced), tview);
#ifdef HAVE_SOUND
	gui_popup_add_item(popup_menu, "   Play ",
			G_CALLBACK(gui_tview_popup_play), tview);
#endif
	return popup_menu; 
}

void
gui_tview_popup_update_local(GtkWidget *menu)
{
//	 GtkTreeRowReference *row;
//	 GSList *rows;
//	 struct _list *list;
	 gui_ftp_lookup *lookup =
		 gui_ftp_lookup_widget(gui_lookup_widget("right_tview"));
	 GList *children = GTK_MENU(menu)->menu_shell.children;
	 GtkMenuItem *item1 = g_list_nth_data(children, 1);
	 GtkMenuItem *item2 = g_list_nth_data(children, 2);
//	 GtkMenuItem *item3 = g_list_nth_data(children, 9);
	 
	 if (!lookup || !lookup->site) {
		 gtk_widget_set_sensitive(GTK_WIDGET(item1),  FALSE);
		 gtk_widget_set_sensitive(GTK_WIDGET(item2),  FALSE);
	 }
	 else { 
		 int freeze = (int)
			 g_object_get_data(G_OBJECT(gtk_tree_view_get_model(
									GTK_TREE_VIEW(lookup->widget))),
					           "freeze");
			 
				 gtk_widget_set_sensitive(GTK_WIDGET(item1),  !freeze);
				 gtk_widget_set_sensitive(GTK_WIDGET(item2),  !freeze);
	}

//	gtk_widget_set_sensitive(GTK_WIDGET(item3), FALSE);
#if 0	
	 rows = gui_tview_get_selected_row_refs(
			 GTK_TREE_VIEW(gui_lookup_widget("left_tview")));
	 if(!rows)	return;
	 row = g_slist_last(rows)->data;
	 if(!row)	return;
	 list = gui_treestore_get_list_from_path(
			              GTK_TREE_VIEW(gui_lookup_widget("left_tview")),
			              gtk_tree_row_reference_get_path(row));
	 if(meta_get_file_type(list->file) == META_SND)
		 gtk_widget_set_sensitive(GTK_WIDGET(item3), TRUE);
#endif
}

void
gui_tview_popup_update_remote(GtkWidget *menu)
{
	GtkWidget *tree = gui_lookup_widget("right_tview");
	GList *children = GTK_MENU(menu)->menu_shell.children;

	int freeze = !(int)g_object_get_data(
			G_OBJECT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree))), "freeze");

	
	
	{
		GtkMenuItem 
			*item1, *item2, *item3, *item5, *item6, *item7, *item8, *item0;
		item1 = g_list_nth_data(children, 1);
		item2 = g_list_nth_data(children, 2);
		item3 = g_list_nth_data(children, 3);
		item5 = g_list_nth_data(children, 5);
		item6 = g_list_nth_data(children, 6);
		item7 = g_list_nth_data(children, 7);
		item8 = g_list_nth_data(children, 8);
		item0 = g_list_nth_data(children, 10);
		gtk_widget_set_sensitive(GTK_WIDGET(item1),  freeze);
		gtk_widget_set_sensitive(GTK_WIDGET(item2),  freeze);
		gtk_widget_set_sensitive(GTK_WIDGET(item3),  freeze);
		gtk_widget_set_sensitive(GTK_WIDGET(item5),  freeze);
		gtk_widget_set_sensitive(GTK_WIDGET(item6),  freeze);
		gtk_widget_set_sensitive(GTK_WIDGET(item7),  freeze);
		gtk_widget_set_sensitive(GTK_WIDGET(item8),  freeze);
		gtk_widget_set_sensitive(GTK_WIDGET(item0),  freeze);
	}
}





/*
** Download
*/

/*
** Called when the user select download in the right-click popup for either local or remote treeview.
** Will call gui_tree_download for each selected item.
*/
void
gui_tview_popup_download(GtkWidget *item, gpointer upointer)
{
	GtkWidget *tview  = GTK_WIDGET(upointer);
	GtkTreeSelection *selection = NULL;
	
	/* Determine which items have been selected. */
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview));
	
	/* For each item selected, call gui_popup_download which will retrieve the file. */
	gtk_tree_selection_selected_foreach(selection, (GtkTreeSelectionForeachFunc)gui_popup_download, NULL);
	return;
}


/*
** This function will be called through gtk_tree_selection_selected_foreach() which will call it for all selected entries in the treeview
** the download request was issued in.  This function will download the selected files through the ftp lib.
*/
void
gui_popup_download(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter iter, gpointer data)
{	
	ftp_request *req = NULL;
	GtkWidget *tree = NULL;
	gui_ftp_lookup *lookup = NULL;
	struct _list *list = NULL;
	_ftp_item *site = NULL;
	
	tree = g_object_get_data(G_OBJECT(model), "view");
	if ( !(list = gui_treestore_get_list_from_path(GTK_TREE_VIEW(tree), path)) )
		return;
	
	lookup = gui_ftp_lookup_widget(tree);
	if (lookup) {
		/*
		** Remote site!
		*/
	
		int handler = gui_tview_handler_block(tree);
		
		if (J_ISFILE(list->type)) 
			gui_ftp_retr(lookup->site, list, 0/*gui_ftp_callback_local_list*/, 
					     "widget", gui_lookup_widget("left_tview"));
		else if (J_ISDIR(list->type)) 
			localdir_retr(lookup->site, list);
		
		req = ftp_request_add(lookup->site, RP_END, NULL, 0,
				                gui_ftp_callback_local_list, 0);
		ftp_object_set_data(FTP_OBJECT(req), "widget",
				                gui_lookup_widget("left_tview"));
		req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
				gui_tview_handler_unblock, 0);
		ftp_object_set_data(FTP_OBJECT(req), "tree", 
				gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
		ftp_object_set_data(FTP_OBJECT(req), "handler", (gpointer)handler);
		
//		list = local_list();
//		gui_tview_fill(GTK_TREE_VIEW(tree), list, ftp_free_list, gui_fill_tview_cb);
	} else { 
		/*
		** Local !
		*/
		/*
		** Find out which site is currently associated with the local tview.
		*/
//		site = g_object_get_data(G_OBJECT(tree), "remote_site");
		GtkWidget *widget = gui_lookup_widget("right_tview");
		lookup = gui_ftp_lookup_widget(widget);
		site = lookup->site;
		DEBUG_MSG(DBG_INFO, "the site is: %p", site);
		if (site) {
			if (J_ISFILE(list->type)) {
				req = gui_ftp_stor(site, list, 
						//gui_ftp_callback_get_fresh_list, 
						gui_ftp_callback_add_file,
						"widget", widget);
				ftp_object_set_data(FTP_OBJECT(req), "item", list);
			}
			else if (J_ISDIR(list->type)) {
				localdir_stor(site, list);
				gui_ftp_list(site, gui_ftp_callback_list, 1, NULL, 
						"widget", widget);
			}
		}
	}

	return;
}


/* Delete selected items from the treeview. */
void
gui_tview_popup_delete(GtkWidget *item, gpointer upointer)
{
	GtkWidget *tview = GTK_WIDGET(upointer);
	GtkTreeSelection *selection = NULL;

	/* Get the selected items. */
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview));

	/* For each item selected, call gui_popup_delete which will delete the item. */
	gtk_tree_selection_selected_foreach(selection, (GtkTreeSelectionForeachFunc)gui_popup_delete, NULL);

	/* Force a local refresh. */
	gui_refresh_local_forced();

	return;
}

#ifdef HAVE_SOUND
/* play selected sound files */
void
gui_tview_popup_play(GtkWidget *item, GtkTreeView *tview)
{
	GtkWidget           *player;
	GSList              *rows, *j;
	GList               *children;
	GtkTreeRowReference *row;
	struct _list        *list;

	player     = gui_lookup_widget("player");
	children   = gtk_container_get_children(GTK_CONTAINER(player));
	player     = children->data;
	children   = gtk_container_get_children(GTK_CONTAINER(player));
	player     = children->data;
	
	/* get the selected items */
	j = rows = gui_tview_get_selected_row_refs(tview);
	while(j) {
		row = j->data;
	    list = gui_treestore_get_list_from_path(tview,
                         gtk_tree_row_reference_get_path(row));
		if(meta_get_file_type(list->file) == META_SND)
			gui_treestore_append_list(GTK_TREE_VIEW(player),
				gui_player_list_song_new(list->file), "");
		j = g_slist_next(j);
	}

		
	gtk_tree_store_clear(
			GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(player))));	
	gui_treestore_fill(gtk_tree_view_get_model(GTK_TREE_VIEW(player)),
			           gui_treestore_get_list(
						   gtk_tree_view_get_model(GTK_TREE_VIEW(player))),
					   NULL,
					   gui_player_list_fill_cb);
	
}
#endif

/* Add selected items into the queue. */
void
gui_tview_popup_add_queue(GtkWidget *item, gpointer upointer)
{
	GtkWidget *tview = GTK_WIDGET(upointer);
	GtkTreeSelection *selection = NULL;

	/*
	** Get the selected items into selection,
	** For each selected item, call gui_menu_add_queue
	*/
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview));
	gtk_tree_selection_selected_foreach(selection, (GtkTreeSelectionForeachFunc)gui_popup_add_queue, NULL);

	return;
}



/*
** For each selected item, add it to the queue.
*/
void
gui_popup_add_queue(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter iter, gpointer data)
{
	GtkWidget *tview = NULL;
	gui_ftp_lookup *lookup = NULL;
	_ftp_item *site = NULL;
	struct _list *fent = NULL;
	
	tview = g_object_get_data(G_OBJECT(model), "view");
	
	if ( !(fent = gui_treestore_get_list_from_path(GTK_TREE_VIEW(tview), path)) )
		return;

	lookup = gui_ftp_lookup_widget(tview);
	if (lookup)
		gui_qwin_queue_add(lookup->site, fent, 0, FTP_TRANSFER_DOWNLOAD);	/* remote ftp. */
	else {
		/* local. */
		site = gui_ftp_current_connection(); 
		gui_qwin_queue_add(site, fent, 1, FTP_TRANSFER_UPLOAD);
	}
		
	return;
	
}

/* Edit a file on the fly */
void
gui_tview_popup_edit_fly(GtkWidget *item, gpointer upointer)
{
	GtkWidget *tview = upointer;
	GtkTreeSelection *selection = NULL;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview));
	gtk_tree_selection_selected_foreach(selection, (GtkTreeSelectionForeachFunc)gui_popup_edit_fly, NULL);

	return;
}

/* View a file on the fly. */
void 
gui_tview_popup_view_fly(GtkWidget *item, gpointer upointer)
{
	GtkWidget *tview = upointer;
	GtkTreeSelection *selection = NULL;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview));
	gtk_tree_selection_selected_foreach(selection, (GtkTreeSelectionForeachFunc)gui_popup_view_fly, (void *)1);

	return;
}


/*
** Setup a window, asking the user about the name of the desired directory.
** Will call gui_popup_mkdir_(remote/local) with the name of the desired directory when the user presses "Create"
*/
void
gui_tview_popup_mkdir(GtkWidget *item, gpointer upointer)
{
	GtkWidget *win = NULL, *label = NULL, *entry = NULL, *button = NULL, *vbox = NULL;
	gui_ftp_lookup *lookup = NULL;

	/* Setup a window where the user can write in the name of the directory to create. */
//	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
//	gtk_window_set_modal(GTK_WINDOW(win), TRUE);
//	gtk_window_set_title(GTK_WINDOW(win), "Make up a whooole new directory\n");
	win = gui_setup_popup_window("make up a whole new directory\n", 1, 0);
	
	vbox = gtk_vbox_new(FALSE, 5);
	label = gtk_label_new("Enter the name of the new dir:");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	entry = gtk_entry_new_with_max_length(20);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);

	button = gtk_button_new_with_label("Create");
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

	gtk_container_set_border_width(GTK_CONTAINER(win), 5);
	gtk_container_add(GTK_CONTAINER(win), vbox);

	if (upointer) {
		lookup = gui_ftp_lookup_widget(upointer);
		if (lookup)
			g_object_set_data(G_OBJECT(button), "remote_site", lookup->site);			
	}
	/* When the button is clicked, call gui_popup_mkdir_(local/remote).  */
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(lookup ? gui_popup_mkdir_remote : gui_popup_mkdir_local), entry);
	/* Then destroy the window. */	
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(gui_widget_destroy_cb), win);
	
	gtk_widget_show_all(win);
	
	return;
}

/*
** Make a local directory
*/
void
gui_popup_mkdir_local(GtkWidget *button, GtkEntry *entry)
{
	local_mkdir(gtk_entry_get_text(entry));
	gui_refresh_local_forced();
	return;
}

/*
** Make a remote directory
*/
void
gui_popup_mkdir_remote(GtkWidget *button, GtkEntry *entry)
{
	gui_ftp_lookup *lookup = NULL;
	_ftp_item *site = NULL;
	char *dir = NULL;
	ftp_request *req;
	int handler;
	GtkWidget *tree;
	
	if (!(site = g_object_get_data(G_OBJECT(button), "remote_site")))
		return;
	
	if (!(lookup = gui_ftp_lookup_site(site)))
		return;
	tree = lookup->widget;
	
	if ( (dir = (char *)gtk_entry_get_text(entry))) {
		handler = gui_tview_handler_block(tree);
		
		req = gui_ftp_mkdir(site, gui_ftp_callback_get_fresh_list, dir, 
				"widget", tree);
		req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
				gui_tview_handler_unblock, 0);
		ftp_object_set_data(FTP_OBJECT(req), "tree", 
				gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
		ftp_object_set_data(FTP_OBJECT(req), "handler", (gpointer)handler);
	}
	return;
}


/*
** Calls a function (gui_popup_rename) that will rename all selected entries.
*/
void
gui_tview_popup_rename(GtkWidget *item, gpointer upointer)
{
	GtkWidget *tview = GTK_WIDGET(upointer);
	GtkTreeSelection *selection = NULL;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview));
	gtk_tree_selection_selected_foreach(selection, (GtkTreeSelectionForeachFunc)gui_popup_rename, NULL);

	return;
}


/*
** Change file modes
*/
void
gui_popup_chmod(GtkWidget *tview, int m)
{
	GtkTreeSelection *selection = NULL;
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview));
	gtk_tree_selection_selected_foreach(selection, (GtkTreeSelectionForeachFunc)gui_popup_chmod_cb, (gpointer)m);
	gui_local_refresh();

	return;
}


/*
** XXX wowowow, fix this chmod stuff, looks rather ugly :)
** maybe make gui_window.c and abstract this stuff
*/
void
gui_tview_popup_chmod(GtkWidget *item, gpointer upointer)
{
	int i, j;
	GtkTreeView *tview = upointer;
	GtkWidget *win = NULL, *label = NULL, *entry = NULL;
	GtkWidget *vbox, *vtable = NULL, *htable = NULL;
	GtkWidget *checks[3][8];
	t_toggle  *togs[3][8];
	int owner_1 = 0, owner_2 = 0, owner_3 = 0;
	int group_1 = 0, group_2 = 0, group_3 = 0;
	int other_1 = 0, other_2 = 0, other_3 = 0;
	char *pnt = NULL;

	memset(togs, 0, (sizeof(t_toggle)*3*8));
	
	win = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(win), "Change Modes \n");
	gtk_container_set_border_width(GTK_CONTAINER(win), 5);
	
	vbox = gtk_vbox_new(FALSE, 5);
	label = gtk_label_new("Enter the new modes (integer) for the selected files:");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	entry = gtk_entry_new_with_max_length(20);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
	label = gtk_label_new("Or check the buttons to set the new modes:");
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	htable = gtk_hbox_new(FALSE, 0);
	vtable = gui_popup_chmod_box(0, checks[0], togs[0], &owner_1, &owner_2, &owner_3);
	gtk_box_pack_start(GTK_BOX(htable), vtable, FALSE, FALSE, 0);
	vtable = gui_popup_chmod_box(1, checks[1], togs[1], &group_1, &group_2, &group_3);
	gtk_box_pack_start(GTK_BOX(htable), vtable, FALSE, FALSE, 0);
	vtable = gui_popup_chmod_box(2, checks[2], togs[2], &other_1, &other_2, &other_3);
	gtk_box_pack_start(GTK_BOX(htable), vtable, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), htable, FALSE, FALSE, 0);
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(win)->vbox), vbox);

	gtk_dialog_add_button(GTK_DIALOG(win), "Cancel", 0);
	gtk_dialog_add_button(GTK_DIALOG(win), "Apply", GTK_RESPONSE_ACCEPT);

	if (gui_tview_selected_n(tview) == 1) {
		char *c = NULL;
		struct _list *list = NULL;
		GtkTreeRowReference *row = NULL;
		GtkTreePath *path = NULL;

		row = gui_tview_get_selected_row_refs(tview)->data;
		path = gtk_tree_row_reference_get_path(row);
		list = gui_treestore_get_list_from_path(tview, path);
		for(i = 0, c = list->perm + 1; i < 3; i++) {
			for(j = 0; j < 3; j++, c++) {
				switch(*c) {
					case '-':
						break;
					case 'r':
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checks[i][0]), TRUE);
						break;
					case 'w':
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checks[i][1]), TRUE);
						break;
					case 'x':
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checks[i][3]), TRUE);
						break;
					case 'S':
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checks[i][4]), TRUE);
						break;
					case 's':
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checks[i][5]), TRUE);
						break;
					case 'T':
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checks[i][6]), TRUE);
						break;
					case 't':
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checks[i][7]), TRUE);
						break;
				}
			}
		}
	}
		
	gtk_widget_show_all(win);

	if(gtk_dialog_run(GTK_DIALOG(win))) {
		if(*(pnt = (char *)gtk_entry_get_text(GTK_ENTRY(entry))))
			gui_popup_chmod(upointer, atol(pnt));
		else
			gui_popup_chmod(upointer, owner_1 | owner_2 | owner_3 |
					         group_1 | group_2 | group_3 |
						 other_1 | other_2 | other_3);
	}
	
	gtk_widget_destroy(win);
	for(i = 0; i < 3; i++)
		for(j = 0; j < 8; j++)
			j_free(togs[i][j]);

	return;
}

/*
** Setup the chmod box where the user can adjust the file modes through an interface
** Returns a vertical box container, containing the interface through which the user
** can pick file modes.
*/
GtkWidget *
gui_popup_chmod_box(int t, GtkWidget **checks, t_toggle **togles, int *read, int *write, int *exec)
{
	GSList *grp = NULL;
	GtkWidget *label = NULL, *vbox = NULL;
	char *labels[] = { "Readable", "Writeable", "None", "Executable", "Suid file (exec)", "Suid file", "Sticky bit (exec)", "Sticky bit", NULL };
	register int i;
	int max;

	/* Setup the chmod vbox and add a label declaring the group owner of the file. */
	vbox = gtk_vbox_new(FALSE, 5);
	label = gtk_label_new((t == 0) ? "Owner:" : ((t == 1) ? "Group:" : "Others"));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

	togles[0] = toggle_new(read, t == 0 ? S_IRUSR : t == 1 ? S_IRGRP : S_IROTH);
	togles[1] = toggle_new(write, t == 0 ? S_IWUSR : t == 1 ? S_IWGRP : S_IWOTH);
	togles[2] = toggle_new(exec, 0);
	togles[3] = toggle_new(exec, t == 0 ? S_IXUSR : t == 1 ? S_IXGRP : S_IXOTH);
	togles[4] = toggle_new(exec, t == 0 ? S_ISUID : t == 1 ? S_ISGID : 0);
	togles[5] = toggle_new(exec, t == 0 ? S_ISUID : t == 1 ? S_ISGID : 0);
	
	if (t == 2) {
		/* Those two modes only apply to files owned by the group of others. */
		togles[6] = toggle_new(exec, S_ISVTX);
		togles[7] = toggle_new(exec, S_ISVTX);
	}

	/*
	** Add checkboxes and radio buttons, for deterining which file modes should be
	** enabled and which ones should not enabled and which ones should not.
	*/
	max = ((t == 2) ? 7 : 5);
	for (i = 0; i < max; i++) {
		checks[i] = ((i < 2) ? (gtk_check_button_new_with_label(labels[i])) : (gtk_radio_button_new_with_label(grp, labels[i])));
		
		if (i > 1)
			grp = gtk_radio_button_group(GTK_RADIO_BUTTON(checks[i]));
			
		gtk_box_pack_start(GTK_BOX(vbox), checks[i], FALSE, FALSE, 0);
		gtk_signal_connect(GTK_OBJECT(checks[i]), "toggled", G_CALLBACK(((i < 2) ? check_cb : toggle_cb)), togles[i]);
	}
	
	return vbox;
}

void
gui_popup_chmod_cb(GtkTreeModel *model, GtkTreePath  *path, GtkTreeIter  *iter, int modes)
{
	GtkTreeView *tview = NULL;
	struct _list *fent = NULL;
	char cwd[MED_SIZE + 1], *file = NULL;

	if(!getcwd(cwd, MED_SIZE))
		return;
	
	tview = g_object_get_data(G_OBJECT(model), "view");
	
	if( !(fent = gui_treestore_get_list_from_path(GTK_TREE_VIEW(tview), path)) )
		return;
	
	file = j_asprintf("%s/%s", cwd, fent->file);
	if( chmod(file, modes) )
		perror("chmod");
			
	j_free(file);
	return;
}

void
gui_popup_edit_fly(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter iter, gpointer data)
{
	/* XXX get those char** from the config lib */
	static char *argv[5] = { "/bin/sh", "-c", NULL, NULL, NULL };
	GtkWidget *tview = NULL;
	gui_ftp_lookup *lookup = NULL;
	ftp_request *req = NULL;
	struct _list *fent = NULL;
	char *ext = NULL, *pnt = NULL, *buf = NULL, *pnt2 = NULL;
	int size, fd;
	int handler;
	
	tview = g_object_get_data(G_OBJECT(model), "view");
	if ( !(fent = gui_treestore_get_list_from_path(GTK_TREE_VIEW(tview), path)) )
		return;

	if (!(lookup = gui_ftp_lookup_widget(tview)))
		return;

	if (J_ISDIR(fent->type))
		return;
	
	handler = gui_tview_handler_block(tview);	
	
	if ( (ext = strrchr(fent->file, '.')) ) {
		/*
		** IIIIYAK! fix :) XXX
		** Use meta in combination with the config lib to retrieve this stuff :)
		** lalalala
		*/
		if (!strcasecmp(ext, ".txt")) 
			config_read_value("txteditor", T_STR, (void **)&argv[2]);
		else if(!strcasecmp(ext, ".jpg") || !strcasecmp(ext, ".png") || !strcasecmp(ext, ".gif") ) 
			config_read_value("imgeditor", T_STR, (void **)&argv[2]);
	}
	
	if (!argv[2])
		config_read_value("txteditor", T_STR, (void **)&argv[2]);

	pnt2 = j_asprintf("/tmp/%s", fent->file);
	pnt = strstr(argv[2], "%f");
	if (pnt) {
		*pnt = 0;
		size = strlen(argv[2]) + strlen(pnt+1) + strlen(fent->file) + 20;
		if (!(buf = j_malloc(size + 1))) {
			MEM_ERR("malloc", ERR_SHOW);
			return;
		}
	
		snprintf(buf, size, "%s%s%s", argv[2], pnt2, pnt+2);
		j_free(argv[2]);
		argv[2] = buf;
	}
		
	fd = open("/dev/null", 0);
	req = ftp_retr(lookup->site, RP_END, NULL, 0, 0, fent, pnt2, TRFR_OVERWRT);
	req = ftp_request_add(lookup->site, RP_END, req, 0, gui_popup_edit_fly_pending, 0);
	ftp_object_set_data(FTP_OBJECT(req), "fd", (void *)fd);
	ftp_object_set_data(FTP_OBJECT(req), "argv", argv);
	ftp_object_set_data(FTP_OBJECT(req), "ffv_file_name", pnt2);
	
	ftp_object_set_data(FTP_OBJECT(req), "tree", tview);
	ftp_object_set_data(FTP_OBJECT(req), "handler", (gpointer)handler);
	return;						/* dum di dum */
}



FTP_CB_CMD(gui_popup_edit_fly_pending)
{
	struct stat stats;
	GIOChannel *source = NULL;
	char **argv = NULL, *file = NULL;
	int lilpipe[2];
	
	
	argv    = ftp_object_get_data(obj, "argv");
	file    = ftp_object_get_data(obj, "ffv_file_name");
	
	// g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, NULL, NULL, &fd, NULL, NULL);

	pipe (lilpipe);
	if (fork() == 0) {
		/* child. */
		register int cnt;

		for (cnt = 3; cnt < 255; cnt++)
			if (cnt != lilpipe[1])
				close(cnt);
		
		close(lilpipe[0]);
		dup2(lilpipe[1], 1);
		close(lilpipe[1]);
		
		execv(argv[0], argv);
		_exit(0);
	} 

	/* parent. */
	close(lilpipe[1]);
	source = g_io_channel_unix_new(lilpipe[0]);
	g_io_add_watch_full(source, G_PRIORITY_HIGH, G_IO_HUP, (GIOFunc)gui_popup_edit_fly_done, argv, 0);

	stat(file, &stats);
	g_dataset_set_data(source, "mtime", (void *)stats.st_mtime);
	g_dataset_set_data(source, "ffv_file_name", file);
	g_dataset_set_data(source, "site", ftp);
	g_dataset_set_data(source, "tree", ftp_object_get_data(obj, "tree"));
	g_dataset_set_data(source, "handler", ftp_object_get_data(obj, "handler"));
		
	return;
}

void
gui_popup_edit_fly_done(GIOChannel *source, GIOCondition condition, char **argv)
{
	_ftp_item *site = NULL;
	int t_time;
	struct stat stats;
	char *file = NULL, *pnt = NULL;
	struct _list fent;
	ftp_request *req = 0;
	int handler;
	GtkWidget *tree;

	site = g_dataset_get_data(source, "site");
	t_time = (int)g_dataset_get_data(source, "mtime");
	file = g_dataset_get_data(source, "ffv_file_name");
	if (!file)
		return;

	handler = (int)g_dataset_get_data(source, "handler");
	tree    = g_dataset_get_data(source, "tree");

	stat(file, &stats);
	if (site != NULL && stats.st_mtime != t_time) {
		char *j = strrchr(file, '/');
		if(j) {
			pnt = j_asprintf("%s", j + 1);
			memset(&fent, 0, sizeof(struct _list));
			fent.file = pnt;
			fent.size = stats.st_size;
			req = ftp_stor(site, RP_END, NULL, 0, 0, &fent, file, TRFR_OVERWRT);
		}
	}
	j_free(argv[2]);
	argv[2] = NULL;

	j_free(file); j_free(pnt);
	g_io_channel_shutdown(source, 0, NULL);
	
	req = ftp_request_add(site, RP_END, req, R_ALWAYS,
			gui_tview_handler_unblock, 0);
	ftp_object_set_data(FTP_OBJECT(req), "tree", 
			gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	ftp_object_set_data(FTP_OBJECT(req), "handler", (gpointer)handler);
	return;
}
		 

void
gui_popup_view_fly(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter iter, gpointer data)
{
	/* XXX get those char** from the config lib */
	static char *argv[5] = { "/bin/sh", "-c", NULL, NULL, NULL };
	GtkWidget *tview = NULL;
	gui_ftp_lookup *lookup = NULL;
	ftp_request *req = NULL;
	struct _list *fent = NULL;
	char *ext = NULL, *buf = NULL, *pnt = NULL, *pnt2 = NULL;
	int size;
	int handler;

	tview  = g_object_get_data(G_OBJECT(model), "view");
	if ( !(fent = gui_treestore_get_list_from_path(GTK_TREE_VIEW(tview), path)) )
		return;
	
	if (!(lookup = gui_ftp_lookup_widget(tview)))
		return;

	if (J_ISDIR(fent->type))
		return;
	
	if ((ext = strrchr(fent->file, '.'))) {
		/* XXX recode using a new feature of the config lib, arrays of dimension n. */
		if (!strcasecmp(ext, ".txt"))
			config_read_value("txtviewer", T_STR, (void *)&argv[2]);
		else if(!strcasecmp(ext, ".jpg") || !strcasecmp(ext, ".gif") || !strcasecmp(ext, ".png") )
			config_read_value("imgviewer", T_STR, (void *)&argv[2]);
	}
	
	if (!argv[2])
		config_read_value("txtviewer", T_STR, (void *)&argv[2]);

	pnt2 = j_asprintf("/tmp/%s", fent->file);

	if ((pnt = strstr(argv[2], "%f"))) {
		*pnt = 0;
		size = strlen(argv[2]) + strlen(pnt+1) + strlen(fent->file) + 20;
		if (!(buf = j_malloc(size + 1))) {
			MEM_ERR("malloc", ERR_SHOW);
			return;
		}       

		snprintf(buf, size, "%s%s%s", argv[2], pnt2, pnt + 2);
		j_free(argv[2]);
		argv[2] = buf;
	}

	handler = gui_tview_handler_block(tview);	
	
	req = ftp_retr(lookup->site, RP_END, NULL, 0, 0, fent, pnt2, TRFR_OVERWRT);
	req = ftp_request_add(lookup->site, RP_END, req, 0, gui_popup_view_fly_done, 0);
	ftp_object_set_data(FTP_OBJECT(req), "argv", argv);	

	req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
			gui_tview_handler_unblock, 0);
	ftp_object_set_data(FTP_OBJECT(req), "tree", 
			gtk_tree_view_get_model(GTK_TREE_VIEW(tview)));
	ftp_object_set_data(FTP_OBJECT(req), "handler", (gpointer)handler);
	return;
}



FTP_CB_CMD(gui_popup_view_fly_done)
{ 
	char **argv = ftp_object_get_data(obj, "argv");

	if (fork() == 0) {
		register int cnt;
		
		// g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH,  NULL, NULL, NULL, NULL, NULL, NULL, NULL);
		for (cnt = 3; cnt < 255; cnt++)
			close(cnt);
		
		execv(argv[0], argv);
		_exit(0);
	}

	j_free(argv[2]);
	argv[2] = NULL;

	return;
}


/*
** This function will be called for each selected entry that is to be deleted,
** it will remove all selected files/directories.
*/
void
gui_popup_delete(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter iter, gpointer data)
{
	GtkWidget *tview = NULL;
	gui_ftp_lookup *lookup = NULL;
	struct _list *fent = NULL;
	char *file = NULL;
	char cwd[MED_SIZE + 1];
	ftp_request *req = NULL;

	tview = g_object_get_data(G_OBJECT(model), "view");
	if ( !(fent = gui_treestore_get_list_from_path(GTK_TREE_VIEW(tview), path)) )
		return;

	lookup = gui_ftp_lookup_widget(tview);
	if (lookup && lookup->site) {
		/* remote */
		GtkWidget *tree = lookup->widget;
		int handler = gui_tview_handler_block(tree);
		
		file = j_asprintf("%s/%s", lookup->site->dirinfo->pwd, fent->file);

		if(J_ISFILE(fent->type))
			ftp_remove(lookup->site, file);
		else
			ftp_remdir(lookup->site, file);

		gui_ftp_list(lookup->site, gui_ftp_callback_list, 1, NULL, 
				"widget", lookup->widget);
		req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
				gui_tview_handler_unblock, 0);
		ftp_object_set_data(FTP_OBJECT(req), "tree", 
				gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
		ftp_object_set_data(FTP_OBJECT(req), "handler", (gpointer)handler);
		
	} else {
		/* local */
		if (!getcwd(cwd, MED_SIZE))
			return;
		
		file = j_asprintf("%s/%s", cwd, fent->file);
		local_remove(file);
	}

	return;
}

gint
gui_tview_cell_edited(GtkCellRendererText *cell, gchar *path_string, gchar *new_text, gpointer data)
{
	GtkWidget *tview = NULL;
	GtkTreeModel *model = (GtkTreeModel *)data;
	GtkTreePath *path = NULL;
	GtkTreeIter iter;
	gui_ftp_lookup *lookup = NULL;
	int handler, row, *indices = NULL;
	char *file = NULL, *new = NULL;
	char cwd[MED_SIZE + 1];
	ftp_request *req = NULL;
	
	if (!(tview = g_object_get_data(G_OBJECT(model), "view")))
		return 0;

	g_object_set (G_OBJECT (cell), "editable", FALSE, NULL);
	handler = g_signal_handler_find(G_OBJECT(cell), G_SIGNAL_MATCH_FUNC, 0, 0, 0, gui_tview_cell_edited, 0);
	g_signal_handler_disconnect(G_OBJECT(cell), handler);

	path = gtk_tree_path_new_from_string(path_string);
	indices = gtk_tree_path_get_indices(path);
	row = indices[0];
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(model, &iter, 1, &file, -1);

	if(!strcmp(file, new_text))
		return 1;

	if(tview == gui_lookup_widget("right_tview")) {
		/* remote ftp site. */
		lookup = gui_ftp_lookup_widget(tview);
		if (lookup->site && lookup->site->dirinfo) {

			handler = gui_tview_handler_block(tview);	
			
			file = j_asprintf("%s/%s", lookup->site->dirinfo->pwd, file);
			new = j_asprintf("%s/%s", lookup->site->dirinfo->pwd, new_text);
			ftp_move_wrapper(lookup->site, RP_END, NULL, 0, NULL, file, new);
			gui_ftp_list(lookup->site, gui_ftp_callback_list, 1, NULL, 
					"widget", lookup->widget);
			
			req = ftp_request_add(lookup->site, RP_END, req, R_ALWAYS,
					gui_tview_handler_unblock, 0);
			ftp_object_set_data(FTP_OBJECT(req), "tree", 
					gtk_tree_view_get_model(GTK_TREE_VIEW(tview)));
			ftp_object_set_data(FTP_OBJECT(req), "handler", (gpointer)handler);
		}
	} else {
		/* locally. */
		if(!getcwd(cwd, MED_SIZE))
			return -1;
			
		file = j_asprintf("%s/%s", cwd, file);
		new = j_asprintf("%s/%s", cwd, new_text);
		
		if(rename(file, new) == -1) {
			FTP_MISC_LOG(NULL, "    Local rename failed\n");
			DEBUG_MSG(DBG_INFO, "Local Rename failed");
		}
			
		j_free(file); j_free(new);
		gtk_widget_grab_focus(tview);
		gui_local_refresh();
	}

	return 0;
}



/*
** This function will be called for each selected item requested for renaming,
** however, it will only work for the last entry
*/
void
gui_popup_rename(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter iter, gpointer data)
{
	GtkWidget *tree = NULL;
	GtkTreeViewColumn *column = NULL;
	GtkCellRenderer *cell = NULL;

	tree = g_object_get_data(G_OBJECT(model), "view");
	column = gtk_tree_view_get_column(GTK_TREE_VIEW(tree), 1);

	cell = gtk_tree_view_column_get_cell_renderers(column)->data;
	g_object_set (G_OBJECT (cell), "editable", TRUE, NULL);
	g_signal_connect (G_OBJECT (cell), "edited", G_CALLBACK (gui_tview_cell_edited), model);
		  
	gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree), path, column, 1);

	return;
}

/*
 |  send raw commands to the ftp server
 */
void
gui_tview_popup_raw(GtkWidget *item, GtkWidget *tree)
{
	GtkWidget *dialog = 0;
	GtkWidget *label  = 0;
	GtkWidget *entry  = 0;
	GtkWidget *vbox   = 0;
	gint result;
	gchar *command = NULL;
	gui_ftp_lookup *lookup = NULL;
	
	dialog = gtk_dialog_new_with_buttons(
			"Send raw command:",
			NULL,
			GTK_DIALOG_MODAL,
			GTK_STOCK_OK,     GTK_RESPONSE_ACCEPT,
			GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
			NULL);
	gtk_dialog_set_default_response(GTK_DIALOG(dialog),
			GTK_RESPONSE_ACCEPT);
	gtk_container_set_border_width(GTK_CONTAINER(dialog), 2);
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 5);

	vbox = gtk_vbox_new(FALSE, 5);
	label = gtk_label_new("Raw Command:");
	entry = gtk_entry_new_with_max_length(100);
	gtk_widget_set_size_request(entry, 100, -1);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox);
	
	gtk_widget_grab_focus(entry);

	gtk_widget_show_all(dialog);
	
	result = gtk_dialog_run(GTK_DIALOG(dialog));
	command = g_strdup((gchar*)gtk_entry_get_text(GTK_ENTRY(entry)));
	gtk_widget_destroy(dialog);
	if(result != GTK_RESPONSE_ACCEPT)	return;
	lookup = gui_ftp_lookup_widget(tree);
	ftp_request_add(lookup->site, RP_END, NULL, 0, NULL, "%s", command);
	j_free(command);
	return;
}


/*
 | toggle the show hidden value for the given tview
 */
void 
gui_tview_hidden(GtkWidget *item, GtkWidget *tree)
{
	GtkTreeModel * model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
	int hidden = (int)g_object_get_data(G_OBJECT(model), "hidden");

	hidden = !hidden;
	g_object_set_data(G_OBJECT(model), "hidden", (gpointer)hidden);
	
	if(tree == gui_lookup_widget("left_tview"))
		gui_refresh_local_forced();
	else
		gui_refresh_remote_forced();
		
	return;		
}


