/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: jfork.c,v 1.6 2003/03/03 02:16:34 tgucio Exp $
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/resource.h>
/* local headers */
#include <jfork.h>
#include <medium.h>
#include <mem.h>
#include <ftp.h>

/*
0.4 rm data from jfork_create and use the jobject instead?
*/

jfork *
jfork_create(cb callback, void *data)
{
	jfork *item = NULL;
	register int i;
	int ret;
	
	/* make the pipe fd reading non blocking. */
	if (!(item = j_malloc(sizeof(jfork))))
		return NULL;

	item->objects = NULL;

	/* create a pipe:
	** 	item->pipe[0] is the reading fd.
	**	item->pipe[1] is the writing fd
	*/
	if (pipe(item->pipe) == -1)
		return NULL;

	/*
	** set the CLOEXEC flag on the fds so they'll be closed upon fork.
	** prevents blocking
	*/
	for (i = 3; i < 255; i++)
		if (i != item->pipe[0] && i != item->pipe[1])
			fcntl(i, F_SETFD, FD_CLOEXEC);

	/* set callback and data. */
	item->callback = callback;
	item->data = data;

	/* fork */
        if ((ret = fork()) == 0) {
		item->type = JFORK_CHILD;
		close(item->pipe[0]); 
	} else if (ret == -1) {
		return NULL;
	} else {
		item->type = JFORK_PARENT;
		item->child_pid = ret;
		close(item->pipe[1]);
		
		if (!medium_cmd(M_IO_WATCH, &ret, item->pipe[0], IO_WATCH_HUP, jfork_cb, item)) 
			j_free(item), item = NULL;	/* xxx ? */
	}

	return item;
}


void
jfork_cb(int fd, jfork *item)
{
	int status;
	
	if (!item)
		return;
		
	if (item->callback) {
		/* calling the callback */
		item->callback(item, item->data);
	}

	if (jfork_get_type(item) == JFORK_PARENT) {
		waitpid(item->child_pid, &status, WNOHANG);
	}
	
	j_free(item);

	close(fd);
	return;
}

int
jfork_get_type(jfork *item)
{
	if (!item) return -1;
	return item->type;
}

int
jfork_read(jfork *item, void *buf, int len)
{
	if (jfork_get_type(item) != JFORK_PARENT)
		return 0;
	
	return read(item->pipe[0], buf, len);
}

int
jfork_write(jfork *item, void *buf, int len)
{
	if (jfork_get_type(item) != JFORK_CHILD)
		return 0;

	return write(item->pipe[1], buf, len);
}

void
jfork_close(jfork *item)
{
	if (!item)
		return;

	if (jfork_get_type(item) == JFORK_CHILD) {
		int i;
		for (i = 0; i < 255; i++)
			close(i);

		close(item->pipe[1]);
//		j_free(item);
		_exit(0);
	}
	
//	j_free(item);
	return;
}
