/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: jlist.c,v 1.8 2003/02/24 04:10:42 thepin Exp $
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* local headers */
#include <jlist.h>
#include <mem.h>

/*
 | junkie single linked list 
 */


/* return a new element of a jslist, filled with passed data */
jslist *
jslist_new(void *item)
{
	jslist *j;
	j = j_malloc(sizeof(jslist));
	memset(j, 0, sizeof(jslist));
	j->data = item;
	return j;
}


/* free an element of a jslist, not the full list */
void
jslist_free(jslist *j)
{
	j_free(j);
}

/* free the whole list. */
jslist *
jslist_free_list(jslist *root, cb callback)
{
	jslist *j = NULL, *k = NULL;

	j = root;
	while (j) {
		k = j;
		j = j->next;
		/* free the data. */
		if (callback)
			callback(k);
			
		jslist_free(k);
	}
	
	return NULL;
}

/* append a new element to a jslist, and return the first member of the list */
jslist *
jslist_append(jslist *root, void *item)
{
	jslist *pnt;
	
	if(!root) {
		root = jslist_new(item);
		return root;
	}
	for(pnt = root; pnt->next; pnt = pnt->next);
	pnt->next = jslist_new(item);
	return root;
}



/*
 | junkie abstracted tree
 */


/* return a new toplevel jtree iter */
t_iter *
jtree_iter_new()
{
	t_iter *iter;
	iter = j_malloc(sizeof(t_iter));
	memset(iter, 0, sizeof(t_iter));
	return iter;
}

/* free a jtree iter */
void
jtree_iter_free(t_iter *iter)
{
	j_free(iter);
}

/* return a new jtree iter pointing to the child node of the passed one */
t_iter *
jtree_iter_child(t_iter *parent)
{
	t_iter *iter;
	iter = j_malloc(sizeof(t_iter));
	memcpy(iter, parent, sizeof(t_iter));
	iter->depth++;
	return iter;
}

/* return a new jtree iter pointing to the parent node of the passed one */
t_iter *
jtree_iter_parent(t_iter *child)
{
	t_iter *iter;
	iter = j_malloc(sizeof(t_iter));
	memcpy(iter, child, sizeof(t_iter));
	iter->n[iter->depth] = 0;
	iter->depth--;
	return iter;
}

/* iterate over the passed tree, calling the passed callback for each node */
void
jtree_iterate(void *root, t_iter *iter, cb callback, int value, void *data)
{
	t_tree *list = root;
	if(!iter)	iter = jtree_iter_new();

	
	while(list) {
		t_tree *old = list;
		callback(&list, iter, value, data);
		if(!list)		break;
		if(list != old)	continue;
		
		if(list->child) {
			jtree_iterate(list->child, jtree_iter_child(iter), callback, value, data);
			if(!list->child)	callback(&list, iter, 0x1, data);
		}
		if(!list)		break;
		if(list != old)	continue;
		list = list->next;
		iter->n[iter->depth]++;
	}
	
	jtree_iter_free(iter);
}

/* append the passed node to the given path of the passed tree */
void *
jtree_append_from_path(void *root, void *l, char *p)
{
	int     row,   i;
	char   *pather, *path, *j;
	t_tree *tmp,  *head = root, *list = l, *lpnt;
	lpnt = head;
	
	pather = strdup(p);
	path   = pather;
	if(!*path) {
		for(; lpnt && lpnt->next; lpnt = lpnt->next);
		if(lpnt)
			lpnt->next = list, list->parent = lpnt->parent;
		else {
			if(head)
				head->next = list, list->parent = head->parent;
			else
				head = list;
		}
	}
	else {
		do {
			j = strchr(path, ':');
			if(j) *j = '\0';
			row = atoi(path);
			if(j) path = j + 1;
			for(i = 0; i < row; i++)   lpnt = lpnt->next;
			if(j)                      lpnt = lpnt->child;
		} while(j);

		for(tmp = lpnt->child; tmp && tmp->next; tmp = tmp->next);

		if(tmp)
			tmp->next   = list, list->parent = tmp->parent;
		else
			lpnt->child = list, list->parent = lpnt;
	}
	free(pather);

	return head;
}

/* remove the passed node from the tree, calling the passed callback if any */
void *
jtree_remove(void *root, void *l, cb freecb)
{
	t_tree *list   = l;
	t_tree *parent = list->parent;
	t_tree *head   = root;
	t_tree *pnt;
	
	if(parent) {
		if(parent->child == list)	parent->child = list->next;
		else {
			for(pnt = parent->child; pnt->next != list; pnt = pnt->next);
			pnt->next = list->next;
		}
	}
	else {
		if(head == list)	head = list->next;
		else {
			for(pnt = head; pnt->next && pnt->next != list; pnt = pnt->next);
			pnt->next = list->next;
		}
	}

	if(freecb)	freecb(list);

	return head;
}

	
