/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: jtimer.c,v 1.4 2003/02/24 04:10:42 thepin Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
/* local headers */
#include <jtimer.h>
#include <medium.h>
#include <mem.h>
#include <ftp.h>

/*
0.4 use the jobject
*/
double
jtime_get_current_time(void)
{
	struct timeval tp;
	double tmp;
	
	gettimeofday(&tp, NULL);
	tmp = (double)tp.tv_sec + (double)tp.tv_usec / (double)1000000;
	return tmp;
}


jtimer *
jtimer_new(void)
{
	jtimer *timer = NULL;

	timer = j_malloc(sizeof(jtimer));
	memset(timer, 0, sizeof(jtimer));
	timer->status = JTIMER_INACTIVE;
	
	return timer;
}

jtimer *
jtimer_kill(jtimer *timer)
{
	j_free(timer);
	return NULL;
}

void
jtimer_start(jtimer *timer)
{
	timer->status = JTIMER_RUNNING;
	timer->time_stop = 0.0;
	timer->time_start = jtime_get_current_time();
	return;
}

void
jtimer_stop(jtimer *timer)
{
	if (!timer || timer->status != JTIMER_RUNNING)
		return;
	
	timer->time_stop = jtime_get_current_time();
	timer->status = JTIMER_STOPPED;
	
	return;
}


double
jtimer_get_elapsed_time(jtimer *timer)
{
	if (!timer)
		return 0.0;

	if (timer->status == JTIMER_RUNNING)
		return (double)jtime_get_current_time() - (double)timer->time_start;
	else if (timer->status == JTIMER_STOPPED)
		return (double)timer->time_stop - (double)timer->time_stop;

	return 0.0;
}
