/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: list_sort.c,v 1.5 2003/02/24 04:10:42 thepin Exp $
 */


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
/* local headers */
#include <ftp.h>
#include <list_sort.h>

int list_sort_date(struct _list *p, struct _list *q, void *pointer)
{
	int *reverse = pointer;
	int ret;
	if(!p->mtime || !q->mtime) {
		ret = p->mtime ? 1 : -1;
		if(*reverse) ret = -ret;
	}
	else
		ret = p->mtime > q->mtime ? 1 : -1;
	return *reverse ? -ret : ret;
}

int list_sort_group(struct _list *p, struct _list *q, void *pointer)
{
	int *reverse = pointer;
	int ret;
	if(!p->group || !q->group) {
		ret = p->group ? 1 : -1;
		if(*reverse) ret = -ret;
	}
	else
		ret = strcmp(p->group, q->group);
	return *reverse ? -ret : ret;
}

int list_sort_owner(struct _list *p, struct _list *q, void *pointer)
{
	int *reverse = pointer;
	int ret;
	if(!p->owner || !q->owner) {
		ret = p->owner ? 1 : -1;
		if(*reverse) ret = -ret;
	}
	else
		ret = strcmp(p->owner, q->owner);
	return *reverse ? -ret : ret;
}

int list_sort_perm(struct _list *p, struct _list *q, void *pointer)
{
	int *reverse = pointer;
	int ret;
	if(!p->perm || !q->perm) {
		ret = p->perm ? 1 : -1;
		if(*reverse) ret = -ret;
	}
	else
		ret = strcmp(p->perm, q->perm);
	return *reverse ? -ret : ret;
}

int list_sort_file(struct _list *p, struct _list *q, void *pointer)
{
	int *reverse = pointer;
	int ret;
	if     (!strcmp(p->file, "..")) ret = *reverse ?  1 : -1;
	else if(!strcmp(q->file, "..")) ret = *reverse ? -1 :  1;
	else ret = strcmp(p->file, q->file);
	return *reverse ? -ret : ret;
}

int list_sort_size(struct _list *p, struct _list *q, void *pointer)
{
	int *reverse = pointer;
	int ret;
	if     (!strcmp(p->file, "..")) ret = *reverse ?  1 : -1;
	else if(!strcmp(q->file, "..")) ret = *reverse ? -1 :  1;
	else ret = p->size > q->size ? 1 : -1;
	return *reverse ? -ret : ret;
}

int list_sort_type(struct _list *p, struct _list *q, void *pointer)
{
	int *reverse = pointer;
	int ret;
	if     (!strcmp(p->file, "..")) ret = *reverse ?  1 : -1;
	else if(!strcmp(q->file, "..")) ret = *reverse ? -1 :  1;
	else if(J_ISDIR(p->type))   ret = -1;
	else if(J_ISDIR(q->type))   ret =  1;
	else ret = 0;
	return *reverse ? -ret : ret;
}

// linked-list memory sort by philip j. erdelsky (pje@acm.org)
void *list_sort(void *p, unsigned index,
		compare_func compare,
		void *pointer, unsigned long *pcount)
{
	unsigned base;
	unsigned long block_size;

	struct record {
		struct record *next[1];
		// other members not directly accessed by this function
	};

	struct tape {
		struct record *first, *last;
		unsigned long count;
	} tape[4];

	
	// distribute the records alternately to tape[0] and tape[1]
	tape[0].count = tape[1].count = 0L;
	tape[0].first = NULL;
	base = 0;
	while(p != NULL) {
		struct record *next = ((struct record *)p)->next[index];
		((struct record *)p)->next[index] = tape[base].first;
		tape[base].first = ((struct record *)p);
		tape[base].count++;
		p = next;
		base ^= 1;
	}
	
	// if the list is empty or contains only a single record
	// tape[1].count == OL and this part is vacuous
	for(base = 0, block_size = 1L; tape[base+1].count != 0L;
			base ^= 2, block_size <<= 1)
	{
		int dest;
		struct tape *tape0, *tape1;
		tape0 = tape + base;
		tape1 = tape + base + 1;
		dest = base ^ 2;
		tape[dest].count = tape[dest+1].count = 0;
		for(; tape0->count != 0; dest ^= 1) {
			unsigned long n0, n1;
			struct tape *output_tape = tape + dest;
			n0 = n1 = block_size;
			while(1) {
				struct record *chosen_record;
				struct tape *chosen_tape;
				if(n0 == 0 || tape0->count == 0) {
					if(n1 == 0 || tape1->count == 0)	break;
					chosen_tape = tape1;
					n1--;
				}
				else if(n1 == 0 || tape1->count == 0) {
					chosen_tape = tape0;
					n0--;
				}
				else if((*compare)(tape0->first, tape1->first, pointer) > 0) {
					chosen_tape = tape1;
					n1--;
				}
				else {
					chosen_tape = tape0;
					n0--;
				}
				chosen_tape->count--;
				chosen_record = chosen_tape->first;
				chosen_tape->first = chosen_record->next[index];
				if(output_tape->count == 0)
					output_tape->first = chosen_record;
				else
					output_tape->last->next[index] = chosen_record;
				output_tape->last = chosen_record;
				output_tape->count++;
			}
		}
	}
	if(tape[base].count > 1L) {
		tape[base].last->next[index] = NULL;
	}
	if(pcount != NULL)
		*pcount = tape[base].count;
	return tape[base].first;
}
