/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: medium.c,v 1.15 2003/03/07 17:48:13 thepin Exp $
 */


/* Medium between the gui and the raw ftp library and possibly other additional "libs". */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
/* local headers */
#include <medium.h>
#include <error.h>
#include <mem.h>


static struct medium_list medium_list[] = {
	{	M_TRFR_UPDATE,	   	1	},
	{	M_PATHS_UPDATE,		2	},
	{	M_SELECT_WRITE,		1	},
	{	M_SELECT_WAIT,		1	},
	{	M_TIMEOUT_ADD,		3	},
	{	M_TIMEOUT_REMOVE,	1	},
	{	M_FTP_CONNECTING,	1	},
	{	M_FTP_CONNECTED,	2	},
	{	M_IO_WATCH,		4	},
	{ 0, 0 }
};


medium *medium_head = NULL;


int
medium_cmd(int type, int *ret, ...)
{
	medium *mpnt = NULL;
	int i, nargs = 0;
	void **args = NULL;
	va_list val;

	for (i = 0; medium_list[i].type != 0; i++) {
		if (medium_list[i].type == type) {
			nargs = medium_list[i].args;
			break;
		}
	}
	
	if (medium_list[i].type != type)
		return 0;


	if ( (args = j_malloc(sizeof(void *) * (nargs + 1))) == NULL) {
		MEM_ERR("malloc", ERR_SHOW);
		return -1;
	}
	
	va_start(val, ret);

	for (i = 0; i < nargs; i++) {
		args[i] = va_arg(val, void *);
	}
	
	args[i] = NULL;
	
	va_end(val);


	if (ret)
		*ret = 0;

	mpnt = medium_find(type);
	if (!mpnt) {
		j_free(args);
		return 0;
	}
		
	if (mpnt->function) {
		if (ret) {
			*ret = mpnt->function(args);
			
/*         printf("ret is %d\n", *ret); */
      }
		else
			mpnt->function(args);
      
	}
	
	j_free(args);
	return 1;
}

int
medium_set_function(int type, MEDIUM_FUNC(function))
{
	medium **mpnt = NULL;

	mpnt = &medium_head;
	while (*mpnt) {
		if ((*mpnt)->type == type)
			return 0;
			
		mpnt = &(*mpnt)->next;
	}

	
	if ( ((*mpnt) = j_malloc(sizeof(medium))) == NULL) {
		MEM_ERR("malloc", ERR_SHOW);
		return -1;
	}

	(*mpnt)->type = type;
	(*mpnt)->function = function;
	(*mpnt)->next = NULL;
	
	return 1;
}


void
medium_free(void)
{
	medium *mpnt = NULL;

	while (medium_head) {
		mpnt = medium_head;
		medium_head = medium_head->next;
		j_free(mpnt);
	}
	
	return;
}

medium *
medium_find(int type)
{
	medium *pnt = NULL;

	pnt = medium_head;
	while (pnt) {
		if (pnt->type == type)
			break;
			
		pnt = pnt->next;
	}

	return pnt;
}
