/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: meta.c,v 1.4 2003/02/24 04:10:42 thepin Exp $
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <strings.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
/* local headers */
#include <meta.h>
#include <misc.h>

meta_type meta_list[] = {
	/* movie format */
	{	"avi",		META_MOVIE	},
	{	"mpg",		META_MOVIE	},
	{	"mpeg",		META_MOVIE	},
	{	"divx",		META_MOVIE	},
	{	"mov",		META_MOVIE	},
	{	"asf",		META_MOVIE	},
	
	/* picture */
	{	"jpg",		META_IMG	},
	{	"gif",		META_IMG	},
	{	"png",		META_IMG	},
	{	"bmp",		META_IMG	},

	/* sound */
	{	"mp3",		META_SND	},
	{	"au",		META_SND	},
	{	"wav",		META_SND	},
	{	"mod",		META_SND	},
	{	"xm",		META_SND	},
	{	"ogg",		META_SND	},

	/* compresed (and or) archived files (packages) */
	{	"gz",		META_PACKAGE	},
	{	"zip",		META_PACKAGE	},
	{	"arj",		META_PACKAGE	},
	{	"rar",		META_PACKAGE	},
	{	"bzip2",	META_PACKAGE	},
	{	"tar",		META_PACKAGE	},
	{	"tgz",		META_PACKAGE	},
	{	"r(%d)(%d)",	META_PACKAGE	},	/* r00 r01 etc rar files  ((ext+1)[0] == 'r' && isdigit((ext+1)[1]) && isdigit((ext+1)[2] */
	{	"ace",		META_PACKAGE	},
	{	"bz2",		META_PACKAGE	},

	/* documents containing text (and or images). */
	{	"pdf",		META_DOC	},
	{	"ps",		META_DOC	},
	{	"doc",		META_DOC	},
	{	"txt",		META_DOC	},
	{	"rtf",		META_DOC	},
	{	"dvi",		META_DOC	},

	/* last entry */
	{ NULL, 0 }
};

int
meta_get_file_type(char *file)
{
	char *ext = NULL;
	register int i;

	if (!(ext = strrchr(file, '.')))
		return META_UNKNOWN;

/*
** XXX Remember to handle the (rXX) rar files...
*/
	for (i = 0; meta_list[i].ext ; i++)
		if (!strcasecmp(ext+1, meta_list[i].ext))
			return meta_list[i].type;

	

	return META_UNKNOWN;
	
}
