/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: misc.c,v 1.6 2003/03/07 17:48:13 thepin Exp $
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/time.h>
/* local headers */
#include <mem.h>
#include <misc.h>

#ifdef HAVE_CONFIG_H
#include <configure.h>
#endif


#ifndef HAVE_ASPRINTF
int
asprintf(char **strp, const char *format, ...)
{
	va_list varg;
	int ret, size = 100;

	/* allocate enough memory for *strp */
	if ( (*strp = (char *)j_malloc(size)) == NULL)
		return -1;
	
	while (1) {
		va_start(varg, format);
		ret = vsnprintf(*strp, size, format, varg);
		va_end(varg);
		
		if (ret > -1 && ret < size)
			break;
		
		if (ret > -1)
			size = ret + 1;
		else
			size *= 2;
		
		if ( (*strp = (char *)j_realloc(*strp, size)) == NULL)
			return -1;
	}
	

	return ret;
}
#endif

#ifndef HAVE_ATOL
long
atol(char *str)
{
	char *pnt = str;
	long ret = 0;
	
	if (pnt) {
		while (*pnt) {
			ret *= 10;
			ret += (*(pnt++) - '0');
		}
	} else return 0;

	return ret;
}
#endif


char *
readline(int fd)
{
	static size_t i;
	static struct stat sb;
	static char *buf = NULL, *p;
	int n;
	char *newlin, *q;

	if(buf == NULL) {             /* init some stuff */
		n = fstat(fd, &sb);
		if(n == -1) {
			warn("WARNING: fstat()");
			return NULL;
		}
#ifdef __FreeBSD__
		buf = mmap(NULL, (size_t)sb.st_size, PROT_READ,
				MAP_PRIVATE | MAP_NOCORE, fd, 0);
#else
		buf = mmap(NULL, (size_t)sb.st_size, PROT_READ,
				MAP_PRIVATE,              fd, 0);
#endif
		if(buf == MAP_FAILED) {
			warn("WARNING: mmap()");
			return NULL;
		}
		p = buf;
		i = 0;
	}
	if(i == (size_t)sb.st_size) { /* EOF */
		munmap(buf, (size_t)sb.st_size);
		buf = NULL;
		return NULL;
	}
	q = memchr(p, '\n', (size_t)sb.st_size - i);
	if(!q) {
		newlin = j_malloc(sb.st_size - (size_t)(p - buf) + 1);
		if(!newlin) {
			munmap(buf, (size_t)sb.st_size);
			buf = NULL;
			return NULL;
		}
		memcpy(newlin, p, sb.st_size - (size_t)(p - buf));
		i += sb.st_size - (size_t)(p - buf) + 1;
		newlin[sb.st_size - (p - buf)] = '\0';
		p = q + 1;
		return newlin;
	}
	else {
		newlin = j_malloc((size_t)(q - p) + 1);
		if(!newlin) {
			munmap(buf, (size_t)sb.st_size);
			buf = NULL;
			return NULL;
		}
		memcpy(newlin, p, (size_t)(q - p));
		i += (size_t)(q - p) + 1;
		newlin[q - p] = '\0';
		p = q + 1;
		return newlin;
	}
}

double
time_usec()
{
	double tmp;
	struct timeval tp;
	gettimeofday(&tp, NULL);
	tmp = (double)tp.tv_sec + (double)tp.tv_usec / (double)1000000;
	return tmp;
}

