/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: queue.c,v 1.68 2003/03/07 17:48:13 thepin Exp $
 */


#define _GNU_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
/* local headers */
#include <queue.h>
#include <gui_widgets.h>   //XXX die
#include <gui_qwin.h>      //XXX kill me
#include <gui_toolkit.h>   //XXX kill me too
#include <mem.h>
#include <gui_ftp.h>       //XXX me too :/
#include <gui_treestore.h> //XXX :(
#include <gui_tview.h>     //XXX :<
#include <ftp.h>
#include <ftp_net.h>
#include <ftp_request.h>
#include <ftp_transfer.h>
#include <ftp_cmd.h>
#include <jlist.h>
#include <local.h>
#include <config.h>
#include <jio.h>
#include <misc.h>
#include <configure.h>

/*
 | load the queue up into a linked list (if it exists)
 */
_queue_item *
queue_init(char *sitename)
{
	_queue_item *head = 0;
	char *buf = 0, *qline = 0;
	int   qfd;
	
	if(getenv("HOME"))
		buf = j_asprintf("%s/.junkie/queue/%s", getenv("HOME"), sitename);
	else {
		DEBUG_MSG(DBG_WEIRD, "$HOME isnt set, trying .");
		buf = j_asprintf("./.junkie/queue/%s", sitename);
	}

	qfd = open(buf, O_RDWR);
	if(qfd == -1) {
		DEBUG_MSG(DBG_WEIRD, "no queue file found");
		return NULL;
		j_free(buf);
	}
	
	while((qline = readline(qfd))) {
		_queue_item *qnew;
		char *buf = qline, *path;
		path = queue_parse_path(&buf);
		qnew = queue_parse_item(buf);
		head = jtree_append_from_path(head, qnew, path);
	}	
	
	j_free(buf);

	return head;
}

/*
 */
char *
queue_parse_path(char **buffer)
{
	char *ptr = NULL, *path = NULL;

	if (!buffer)
		return NULL;

	ptr = strchr(*buffer, '*');
	if(ptr)	{
		*ptr = 0;
		path = j_strdup(*buffer);
		*buffer = ptr + 1;
	}
	else
		return NULL;

	return path;
}

/*
 */
_queue_item *
queue_parse_item(char *buffer)
{
	char *j;
	_queue_item *q = 0;
	q = j_malloc(sizeof(_queue_item));
	memset(q, 0, sizeof(_queue_item));

	q->type = (int)atol(buffer++);
	
	j = strchr(buffer, '\t');
	if (!j)
		return NULL;
	*j = 0;
	q->remote_path = j_strdup(buffer);
	buffer = j + 1;

	j = strchr(buffer, '\t');
	if (!j)
		return NULL;
	*j = 0;
	q->local_path  = j_strdup(buffer);
	buffer = j + 1;

	j = strchr(buffer, '@');
	if (!j)
		return NULL;
	*j = 0;
	if (*buffer)
		q->user = j_strdup(buffer);
	buffer = j + 1;

	j = strchr(buffer, ':');
	if(!j)	return NULL;
	*j = 0;
	if (*buffer)
		q->site = j_strdup(buffer);
	buffer = j + 1;

	j = strchr(buffer, '\t');
	if(!j)	return NULL;
	*j = 0;
	if (*buffer)
		q->port = (int) atol(buffer);
	buffer = j + 1;

	j = strchr(buffer, ' ');
	if(!j)	return NULL;
	*j = 0;
	q->size = atol(buffer);
	buffer = j + 1;

	q->t_mode = *buffer++ - '0';

	q->status = *buffer - '0';

	return q;
}

void
queue_save(_queue_item *head, char *sitename)
{
	jio  *fhd;
	char  *file, *path;
	struct stat sb;

	DEBUG_MSG(DBG_INFO, "saving... head = %p", head);
	
	if (getenv("HOME"))
		path = j_asprintf("%s/.junkie/queue/", getenv("HOME"));
	else {
		DEBUG_MSG(DBG_WEIRD, "$HOME isn't set, trying .");
		path = j_asprintf("./.junkie/queue/");
	}
	if(stat(path, &sb) == -1) {
		printf("WARNING: trying to create %s\n", path);
		if(mkdir(path, 0755) == -1) {
			perror("WARNING: mkdir()");
			return;
		}
	}

	file = j_asprintf("%s%s", path, sitename);
	j_free(path);
	fhd = j_open(file, O_CREAT|O_WRONLY|O_TRUNC, 0600);
	j_free(file);
	if (!fhd)
		return;
	
	jtree_iterate(head, NULL, queue_save_printf, 0, fhd);
	
	j_close(fhd);
}

void
queue_save_printf(_queue_item **item_pnt, t_iter *jiter, int value, jio *fhd)
{
	int i;
	char *ptr, *path;
	_queue_item *item = *item_pnt;
	
	if (!jiter->depth)
		path = j_strdup("");
	else {
		path = j_malloc(sizeof(char *) * jiter->depth*8);
		memset(path, 0, sizeof(char *) * jiter->depth*8);

		ptr = path;
		for(i = 0; i < jiter->depth; i++) {
			DEBUG_MSG(DBG_INFO, "jiter is: %d", jiter->n[i]);
			sprintf(ptr, "%d:", jiter->n[i]);
			while(*ptr)
				ptr++;
		}
		path[strlen(path) - 1] = '\0';
	}
	DEBUG_MSG(DBG_INFO, " -> path = %s -- FHD: %p", path, fhd);
	
	j_printf(fhd, "%s*%d%s\t%s\t%s@%s:%d\t%d %d%d\n",
		path, 
		item->type, 
		item->remote_path, item->local_path,
		item->user ? item->user : "",
		item->site ? item->site : "",
		item->port,
		item->size, item->t_mode, item->status);
//	fflush(fhd);
	j_free(path);

	return;
}

void
queue_free(_queue_item *entry)
{
	if (entry)
		queue_free_entry(entry, 0);

	return;
}

void
queue_free_entry(_queue_item *entry, int all)
{
	if (!entry)
		return;

	if(entry->child)
		queue_free_entry(entry->child, 1);
	if(all)	
		queue_free_entry(entry->next, 1);

	j_free(entry->remote_path);
	j_free(entry->local_path);
	j_free(entry);

	return;
}

/*
 | add a file to the queue
 */
_queue_item *
queue_item_new(_ftp_item *ftp, struct _list *ent,
		       char *remote_root, char *local_root,
			   char *sub,
			   int trfr)
{
	char *local_path  = 0, *remote_path = 0;
	_queue_item *qent = 0;

	qent = j_malloc(sizeof(_queue_item));
	memset(qent, 0, sizeof(_queue_item));
	local_path  = queue_full_path(local_root,  sub, ent->file);
	remote_path = queue_full_path(remote_root, sub, ent->file);
	
	qent->remote_path = remote_path;
	qent->local_path  = local_path;
	qent->size        = ent->size;
	qent->t_mode      = trfr;
	qent->status      = Q_NONE;
	qent->type        = ent->type;
	if(ftp) {
		qent->site = ftp->addr;
		qent->port = ftp->port;
		qent->user = ftp->user;
		qent->pass = ftp->pass;
	}

	return qent;
}

_queue_item *
queue_dir_local_new(_ftp_item *ftp, char *cwd, char *pwd, char *folder, int trfr)
{
	gchar *local_root_dir, *remote_root_dir;
	_queue_item *q;
	struct _list list;
	list.file = folder;
	list.type = _LIST_DIR;

	DEBUG_MSG(DBG_INFO, "cwd: %s, pwd: %s, folder: %s\n", cwd, pwd, folder);
	
	local_root_dir  = j_strdup(cwd);
	remote_root_dir = ftp ? j_strdup(pwd) : NULL;
	
	q = queue_item_new(ftp, &list, remote_root_dir, local_root_dir, NULL, trfr);
	q->child = queue_recurse_local(ftp, q, trfr, 0, folder, 
			                       local_root_dir, remote_root_dir);
	return q;
}

_queue_item *
queue_recurse_local(_ftp_item *ftp, _queue_item *parent, 
					int trfr,
		            char *sub_dir, char *dir, 
					char *local_root_dir, char *remote_root_dir)
{
	char *tmp = NULL;
	struct _list *list_dir = NULL;
	_queue_item *item = NULL, *item_pnt = NULL;

	local_cwd(dir);
	
	/* update sub */
	tmp = j_asprintf("%s%s%s", sub_dir ? sub_dir : "",
			                 sub_dir ? "/"     : "",
							 dir);
	sub_dir = tmp;

	item = j_malloc(sizeof(_queue_item));
	memset(item, 0, sizeof(_queue_item));
	item_pnt = item;

	list_dir = local_list();
	while(list_dir) {
		if(J_ISDIR(list_dir->type)) {
			item_pnt->next = 
				queue_item_new(ftp, list_dir,
					remote_root_dir, local_root_dir, sub_dir, trfr);
			item_pnt->next->child = 
				queue_recurse_local(ftp, item_pnt->next,
						trfr, sub_dir, list_dir->file, 
						local_root_dir, remote_root_dir);
		}
		else if(J_ISFILE(list_dir->type))
			item_pnt->next =
				queue_item_new(ftp, list_dir,
					remote_root_dir, local_root_dir, sub_dir, trfr);
			
		else if(J_ISLINK(list_dir->type)) {
		/* XXX handle me */
			list_dir = list_dir->next;
			continue;
		}

		item_pnt->next->parent = parent;
		
		list_dir = list_dir->next;
		item_pnt = item_pnt->next;
	}
	local_cwd("..");

	item_pnt = item->next;
	j_free(item);
	j_free(tmp);	// XXX, added this, analytically, did not test
	return item_pnt;
}

_queue_item *
queue_dir_remote_new(_ftp_item *ftp, char *cwd, char *pwd, char *folder, int trfr)
{
	gchar *local_root_dir, *remote_root_dir;
	_queue_item *q;
	struct _list list;
	list.file = folder;
	list.type = _LIST_DIR;

	DEBUG_MSG(DBG_INFO, "cwd: %s, pwd: %s, folder: %s\n", cwd, pwd, folder);
							    
	local_root_dir  = j_strdup(cwd);
	remote_root_dir = j_strdup(pwd);

	q = queue_item_new(ftp, &list, remote_root_dir, local_root_dir, NULL, trfr);
	
	ftp_object_set_data(FTP_OBJECT(ftp), 
			"queue_transfer_type",   (void *)trfr);
	ftp_object_set_data(FTP_OBJECT(ftp), 
			"queue_remote_dir_root", remote_root_dir);
	ftp_object_set_data(FTP_OBJECT(ftp),
			"queue_local_dir_root",  local_root_dir);
	queue_recurse_remote(ftp, NULL, &q->child, q, folder);

	return q;
}

void
queue_recurse_remote(_ftp_item *ftp, ftp_request *mreq, 
		             _queue_item **head, _queue_item *parent,
					 char *dir)
{
	ftp_request *req = NULL;

	req = ftp_list_wrapper(ftp, (mreq ? RP_NEXT : RP_END), mreq, 0,
			               queue_recurse_list_cb, 1, dir);
	ftp_object_set_data(FTP_OBJECT(req), "queue_sub_dir",     dir);
	ftp_object_set_data(FTP_OBJECT(req), "queue_head",        head);
	ftp_object_set_data(FTP_OBJECT(req), "queue_head_parent", parent);
	ftp_object_set_data(FTP_OBJECT(req), "request",           req);

	return;
}

FTP_CB_CMD(queue_recurse_list_cb)
{
	char         *path_dir        = NULL;
	struct _list *list_dir        = NULL;
	ftp_request  *req             = NULL;
	char         *remote_root_dir = NULL,
	             *local_root_dir  = NULL,
				 *sub_dir         = NULL;
	_queue_item **item_pnt        = NULL,
	            **head            = NULL,
				*parent           = NULL;
	int           trfr;

	if (!ftp || !ftp->dirinfo || !ftp->dirinfo->list_dir)
		return;

	head    = ftp_object_get_data(obj, "queue_head");
	parent  = ftp_object_get_data(obj, "queue_head_parent");
	req     = ftp_object_get_data(obj, "request");
	sub_dir = ftp_object_get_data(obj, "queue_sub_dir");
	
	trfr =       (int)ftp_object_get_data(FTP_OBJECT(ftp), 
			                              "queue_transfer_type");
	remote_root_dir = ftp_object_get_data(FTP_OBJECT(ftp),
			                              "queue_remote_dir_root");
	local_root_dir  = ftp_object_get_data(FTP_OBJECT(ftp),
			                              "queue_local_dir_root");


	list_dir = ftp->dirinfo->list_dir;

	item_pnt = head;
	while(list_dir) {
		if(J_ISDIR(list_dir->type))  {
			*item_pnt = queue_item_new(ftp, list_dir,
					remote_root_dir, local_root_dir, sub_dir, trfr);
			(*item_pnt)->parent = parent;
			path_dir = j_asprintf("%s/%s", sub_dir ? sub_dir : "",
					                     list_dir->file);
			ftp_object_set_data(FTP_OBJECT(ftp), "queue_dir_sub", path_dir);
		
			/* doesnt recurse.. only adds a request.. */
			queue_recurse_remote(ftp, req, &(*item_pnt)->child, *item_pnt, 
					             path_dir);
		}
		else if(J_ISFILE(list_dir->type)) {
			*item_pnt = queue_item_new(ftp, list_dir,
					remote_root_dir, local_root_dir, sub_dir, trfr);
			(*item_pnt)->parent = parent;
		}
		else if(J_ISLINK(list_dir->type)) {
			/* handle me */
			list_dir = list_dir->next;
			continue;
		}

		list_dir = list_dir->next;
		item_pnt = &(*item_pnt)->next;
	}

	j_free(sub_dir);
	return;
}




	
char *
queue_full_path(char *root, char *subdir, char *file)
{
	char *ret = 0;
	int rootlen, sublen, filelen;

	rootlen = root    ? strlen(root)   : 0;
	sublen  = subdir  ? strlen(subdir) : 0;
	filelen = file    ? strlen(file)   : 0;

	ret = j_malloc(sizeof(char) * (rootlen + sublen + filelen + 5));
	memset(ret, 0, sizeof(char) * (rootlen + sublen + filelen + 5));

	if(rootlen) {
		strncpy(ret, root, rootlen);
		if(ret[strlen(ret)-1] != '/')
			ret[strlen(ret)] = '/';
	}
	if(sublen) {
		strncpy(ret+strlen(ret), subdir, sublen);
		if(ret[strlen(ret)-1] != '/')
			ret[strlen(ret)] = '/';
	}
	strncpy(ret+strlen(ret), file, filelen);

	return ret;
}


void
queue_submit(_ftp_item *ftp, jslist *list)
{
	_ftp_item   *q_ftp = NULL;
	ftp_request *req   = NULL;
	_queue_item *q     = NULL;
	struct _list fent;

	q = list->data;
	
	if (q->status == Q_TRANSFERED) {
		req = ftp_request_add(ftp, RP_END, NULL, R_ALWAYS, queue_submit_cb, 0);
		ftp_object_set_data(FTP_OBJECT(req), "queue_transfer_entry", list);
		return;
	}
	
	if (J_ISDIR(q->type)) {
		/* should never happen */
		return;
	}

	
	if (!q->site)
		q->site = ftp->addr;
	if (!q->port)
		q->port = ftp->port;
	if (!q->user)
		q->user = ftp->user;
	if (!q->pass)
		q->pass = ftp->pass;

	q_ftp = ftp;
/*
	if(strcasecmp(ftp->addr, q->site) || 
	   strcasecmp(ftp->user, q->user) || ftp->port != q->port) {
		DEBUG_MSG(DBG_INFO, "connecting to %s - %s(%s)\n",
			q->site, q->user, q->pass);
		q_ftp = ftp_connect(q->site, q->port, 
				    config_read_bool("usepassive"),
				    q->user, q->pass,
				    queue_submit_connected, q_ftp);

		return;
		
		if (!q_ftp)
			foo_bar(this_is);
		req = ftp_login(q_ftp);
		if (!req)
			foo_bar(really_bad);
		ftp_pwd_with_req(q_ftp, req);
		gui_ftp_setup(q_ftp, 1);       //XXX fucking gui
		gui_ftp_setup_events(q_ftp);   //XXX fucking gui

	}
*/

	fent.file = j_strdup(q->remote_path);
	fent.size = q->size;
	if(q->t_mode == FTP_TRANSFER_DOWNLOAD) {
		local_mkpath(fent.file);
		if (J_ISFILE(q->type))
			req = ftp_retr(q_ftp, RP_END, NULL, 0, queue_submit_cb,
					&fent, q->local_path, TRFR_DEFAULT);
	}
	else if(q->t_mode == FTP_TRANSFER_UPLOAD) {
		req = ftp_mkpath(ftp, NULL, fent.file);
		if (J_ISFILE(q->type))
			req = ftp_stor(q_ftp, RP_END, req, R_SUCCESS, queue_submit_cb,
					&fent, q->local_path, TRFR_DEFAULT);
	}

	q->status = Q_TRANSFERRING;
	gui_qwin_pixbuf_update(gui_qwin_model_get(ftp), 0, 0); //XXX kill me
	DEBUG_MSG(DBG_INFO, "[%p] storing %p list->data %p = %s",
		FTP_OBJECT(req), list, list->data, 
		((_queue_item*)list->data)->local_path);
	ftp_object_set_data(FTP_OBJECT(req), "queue_transfer_entry", list);

	if(ftp != q_ftp)
		ftp_quit_with_req(q_ftp, req);

	//XXX any more cleanup needed?

	return;
}

FTP_CB_CMD(queue_submit_cb) 
{
	jslist      *list = ftp_object_get_data(obj, "queue_transfer_entry");
	_queue_item *q    = list->data;
	ftp_request *req;
	
	DEBUG_MSG(DBG_INFO, "[%p] retrieved %p list->data %p", obj, list, q);
	ftp_object_set_data(FTP_OBJECT(ftp), "queue_current_size",
			ftp_object_get_data(FTP_OBJECT(ftp), "queue_current_size")
			- q->size);
	
		q->status = Q_TRANSFERED;
		gui_qwin_pixbuf_update(gui_qwin_model_get(ftp), 0, 0); //XXX kill me

	if(list->next) {
		queue_submit(ftp, list->next);
		jslist_free(list);
	}
	else { //XXX gui crap
		ftp_object_set_data(FTP_OBJECT(ftp), "queue_total_size", NULL);
		j_free(ftp_object_get_data(FTP_OBJECT(ftp), "queue_now"));
		if (!config_read_bool("queue-keep"))
		{
			jtree_iterate(gui_treestore_get_list(gui_qwin_model_get(ftp)),
					      NULL,
						  gui_qwin_remove_if_transfered,
							0x0,
							gui_qwin_model_get(ftp));
			gui_qwin_refresh(gui_qwin_model_get(ftp));
		}
		/* unblock the tview */
		req = ftp_request_add(ftp, RP_END, 0, R_ALWAYS,
				gui_tview_handler_unblock, 0);
		ftp_object_set_data(FTP_OBJECT(req), "tree",
				gtk_tree_view_get_model(
					GTK_TREE_VIEW(gui_lookup_widget("right_tview"))));
		
		gui_refresh_all_forced(); //XXX kill me too!
	}
}

jslist *
queue_list_append(jslist *list, _queue_item *q, int all)
{
	if (!q)
		return list;
	
	q->status = Q_QUEUED;
	
	if (J_ISFILE(q->type)) {
		list = jslist_append(list, q);
		if (all)
			list = queue_list_append(list, q->next, all);
	}
	else if (J_ISDIR(q->type)) {
		list = queue_list_append(list, q->child, 1);
		if (all)
			list = queue_list_append(list, q->next, all);
	}
	
	return list;
}
	

void
queue_debug_list(_queue_item *q)
{
	_queue_item *qiter;
	
	DEBUG_MSG(DBG_INFO, " -> Debugging START");
	for(qiter = q; qiter; qiter = qiter->next) {
		DEBUG_MSG(DBG_INFO, "--------------%p---------\n" \
			"qiter->local_path:\t%s\n" \
			"qiter->remote_path:\t%s\n" \
			"qiter->next:\t%p\n" \
			"qiter->child:\t%p\n" \
			"qiter->parent:\t%p",
			qiter,
			qiter->local_path,
			qiter->remote_path,
			qiter->next,
			qiter->child,
			qiter->parent);

		if(qiter->child)
			queue_debug_list(qiter->child);
	}

	DEBUG_MSG(DBG_INFO, " -> Debugging END");
}
	
		
