/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sound_mikmod.c,v 1.9 2003/03/03 02:16:34 tgucio Exp $
 */


#include <configure.h>

#ifdef HAVE_LIBMIKMOD
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <pthread.h>
#include <mikmod.h>
/* local includes */
#include <defaults.h>
#include <sound.h>
#include <sound_mikmod.h>

MODULE *module;
pthread_t thread;
int while_playing = 0;


/*
 * the thread will update the playback and keep it going till end of days
 */
void
sound_mikmod_update_thread(cb callback)
{
	while_playing = 1;
	while (Player_Active() && while_playing) {
		usleep(10000);
		MikMod_Update(); 
	}
	
	while_playing = 0;
	Player_Stop();
	Player_Free(module);
	MikMod_Exit();
	
	if(callback)
		callback();

	pthread_exit(NULL);
	
	return;
}


/*
 * stop playing for some reason
 */
int
sound_mikmod_stop()
{
	while_playing = 0;
	return 1;
}

/*
 * whatever
 */
int
sound_mikmod_play_file(char *path, cb callback)
{

	if (MikMod_Init("")) {
		DEBUG_MSG(DBG_ERROR, "mikmod initialisation failed");
		return 0;	/* mikmod failed. */
	}

	/* try to load up the module. */
	module = Player_Load((char *)path, 64, 0);
	if (!module) {
		MikMod_Exit();
		return 0;
	}

	Player_Start(module);

	DEBUG_MSG(DBG_INFO, "Player_start()ed");
	
	pthread_create(&thread, NULL, (void *)sound_mikmod_update_thread, 
			                       callback);

	return 1;
}

/*
 * speaks for itself
 */
int
sound_mikmod_init()
{
	/* Register drivers and loaders... Mikmod will detect everything that is
	 * available. */
	MikMod_RegisterAllDrivers();
	MikMod_RegisterAllLoaders();
	return 0;
}

#endif
